/********************************************************************************
 * Copyright (c) 2023 Contributors to the Gamma project
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * SPDX-License-Identifier: EPL-1.0
 ********************************************************************************/
grammar InternalFaultExtensionLanguage;

options {
	superClass=AbstractInternalContentAssistParser;
	backtrack=true;
}

@lexer::header {
package hu.bme.mit.gamma.fei.language.ide.contentassist.antlr.internal;

// Hack: Use our own Lexer superclass by means of import. 
// Currently there is no other way to specify the superclass for the lexer.
import org.eclipse.xtext.ide.editor.contentassist.antlr.internal.Lexer;
}

@parser::header {
package hu.bme.mit.gamma.fei.language.ide.contentassist.antlr.internal;

import java.io.InputStream;
import org.eclipse.xtext.*;
import org.eclipse.xtext.parser.*;
import org.eclipse.xtext.parser.impl.*;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.xtext.parser.antlr.XtextTokenStream;
import org.eclipse.xtext.parser.antlr.XtextTokenStream.HiddenTokens;
import org.eclipse.xtext.ide.editor.contentassist.antlr.internal.AbstractInternalContentAssistParser;
import org.eclipse.xtext.ide.editor.contentassist.antlr.internal.DFA;
import hu.bme.mit.gamma.fei.language.services.FaultExtensionLanguageGrammarAccess;

}
@parser::members {
	private FaultExtensionLanguageGrammarAccess grammarAccess;

	public void setGrammarAccess(FaultExtensionLanguageGrammarAccess grammarAccess) {
		this.grammarAccess = grammarAccess;
	}

	@Override
	protected Grammar getGrammar() {
		return grammarAccess.getGrammar();
	}

	@Override
	protected String getValueForTokenName(String tokenName) {
		return tokenName;
	}
}

// Entry rule entryRuleFaultExtensionInstructions
entryRuleFaultExtensionInstructions
:
{ before(grammarAccess.getFaultExtensionInstructionsRule()); }
	 ruleFaultExtensionInstructions
{ after(grammarAccess.getFaultExtensionInstructionsRule()); } 
	 EOF 
;

// Rule FaultExtensionInstructions
ruleFaultExtensionInstructions 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getFaultExtensionInstructionsAccess().getGroup()); }
		(rule__FaultExtensionInstructions__Group__0)
		{ after(grammarAccess.getFaultExtensionInstructionsAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleFaultSlice
entryRuleFaultSlice
:
{ before(grammarAccess.getFaultSliceRule()); }
	 ruleFaultSlice
{ after(grammarAccess.getFaultSliceRule()); } 
	 EOF 
;

// Rule FaultSlice
ruleFaultSlice 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getFaultSliceAccess().getGroup()); }
		(rule__FaultSlice__Group__0)
		{ after(grammarAccess.getFaultSliceAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleFaultMode
entryRuleFaultMode
:
{ before(grammarAccess.getFaultModeRule()); }
	 ruleFaultMode
{ after(grammarAccess.getFaultModeRule()); } 
	 EOF 
;

// Rule FaultMode
ruleFaultMode 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getFaultModeAccess().getGroup()); }
		(rule__FaultMode__Group__0)
		{ after(grammarAccess.getFaultModeAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleEffect
entryRuleEffect
:
{ before(grammarAccess.getEffectRule()); }
	 ruleEffect
{ after(grammarAccess.getEffectRule()); } 
	 EOF 
;

// Rule Effect
ruleEffect 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getEffectAccess().getAlternatives()); }
		(rule__Effect__Alternatives)
		{ after(grammarAccess.getEffectAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleStuckAtEffect
entryRuleStuckAtEffect
:
{ before(grammarAccess.getStuckAtEffectRule()); }
	 ruleStuckAtEffect
{ after(grammarAccess.getStuckAtEffectRule()); } 
	 EOF 
;

// Rule StuckAtEffect
ruleStuckAtEffect 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getStuckAtEffectAccess().getGroup()); }
		(rule__StuckAtEffect__Group__0)
		{ after(grammarAccess.getStuckAtEffectAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleFrozenEffect
entryRuleFrozenEffect
:
{ before(grammarAccess.getFrozenEffectRule()); }
	 ruleFrozenEffect
{ after(grammarAccess.getFrozenEffectRule()); } 
	 EOF 
;

// Rule FrozenEffect
ruleFrozenEffect 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getFrozenEffectAccess().getGroup()); }
		(rule__FrozenEffect__Group__0)
		{ after(grammarAccess.getFrozenEffectAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleNonDeterminismEffect
entryRuleNonDeterminismEffect
:
{ before(grammarAccess.getNonDeterminismEffectRule()); }
	 ruleNonDeterminismEffect
{ after(grammarAccess.getNonDeterminismEffectRule()); } 
	 EOF 
;

// Rule NonDeterminismEffect
ruleNonDeterminismEffect 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getNonDeterminismEffectAccess().getGroup()); }
		(rule__NonDeterminismEffect__Group__0)
		{ after(grammarAccess.getNonDeterminismEffectAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleNonDeterminismBooleanEffect
entryRuleNonDeterminismBooleanEffect
:
{ before(grammarAccess.getNonDeterminismBooleanEffectRule()); }
	 ruleNonDeterminismBooleanEffect
{ after(grammarAccess.getNonDeterminismBooleanEffectRule()); } 
	 EOF 
;

// Rule NonDeterminismBooleanEffect
ruleNonDeterminismBooleanEffect 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getNonDeterminismBooleanEffectAccess().getGroup()); }
		(rule__NonDeterminismBooleanEffect__Group__0)
		{ after(grammarAccess.getNonDeterminismBooleanEffectAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleConditionalEffect
entryRuleConditionalEffect
:
{ before(grammarAccess.getConditionalEffectRule()); }
	 ruleConditionalEffect
{ after(grammarAccess.getConditionalEffectRule()); } 
	 EOF 
;

// Rule ConditionalEffect
ruleConditionalEffect 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getConditionalEffectAccess().getGroup()); }
		(rule__ConditionalEffect__Group__0)
		{ after(grammarAccess.getConditionalEffectAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleRampDownEffect
entryRuleRampDownEffect
:
{ before(grammarAccess.getRampDownEffectRule()); }
	 ruleRampDownEffect
{ after(grammarAccess.getRampDownEffectRule()); } 
	 EOF 
;

// Rule RampDownEffect
ruleRampDownEffect 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getRampDownEffectAccess().getGroup()); }
		(rule__RampDownEffect__Group__0)
		{ after(grammarAccess.getRampDownEffectAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleInvertedEffect
entryRuleInvertedEffect
:
{ before(grammarAccess.getInvertedEffectRule()); }
	 ruleInvertedEffect
{ after(grammarAccess.getInvertedEffectRule()); } 
	 EOF 
;

// Rule InvertedEffect
ruleInvertedEffect 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getInvertedEffectAccess().getGroup()); }
		(rule__InvertedEffect__Group__0)
		{ after(grammarAccess.getInvertedEffectAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleStuckAtFixedEffect
entryRuleStuckAtFixedEffect
:
{ before(grammarAccess.getStuckAtFixedEffectRule()); }
	 ruleStuckAtFixedEffect
{ after(grammarAccess.getStuckAtFixedEffectRule()); } 
	 EOF 
;

// Rule StuckAtFixedEffect
ruleStuckAtFixedEffect 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getStuckAtFixedEffectAccess().getGroup()); }
		(rule__StuckAtFixedEffect__Group__0)
		{ after(grammarAccess.getStuckAtFixedEffectAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleRandomEffect
entryRuleRandomEffect
:
{ before(grammarAccess.getRandomEffectRule()); }
	 ruleRandomEffect
{ after(grammarAccess.getRandomEffectRule()); } 
	 EOF 
;

// Rule RandomEffect
ruleRandomEffect 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getRandomEffectAccess().getGroup()); }
		(rule__RandomEffect__Group__0)
		{ after(grammarAccess.getRandomEffectAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleErroneousEffect
entryRuleErroneousEffect
:
{ before(grammarAccess.getErroneousEffectRule()); }
	 ruleErroneousEffect
{ after(grammarAccess.getErroneousEffectRule()); } 
	 EOF 
;

// Rule ErroneousEffect
ruleErroneousEffect 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getErroneousEffectAccess().getGroup()); }
		(rule__ErroneousEffect__Group__0)
		{ after(grammarAccess.getErroneousEffectAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleDeltaOutEffect
entryRuleDeltaOutEffect
:
{ before(grammarAccess.getDeltaOutEffectRule()); }
	 ruleDeltaOutEffect
{ after(grammarAccess.getDeltaOutEffectRule()); } 
	 EOF 
;

// Rule DeltaOutEffect
ruleDeltaOutEffect 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getDeltaOutEffectAccess().getGroup()); }
		(rule__DeltaOutEffect__Group__0)
		{ after(grammarAccess.getDeltaOutEffectAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleDeltaInRandomEffect
entryRuleDeltaInRandomEffect
:
{ before(grammarAccess.getDeltaInRandomEffectRule()); }
	 ruleDeltaInRandomEffect
{ after(grammarAccess.getDeltaInRandomEffectRule()); } 
	 EOF 
;

// Rule DeltaInRandomEffect
ruleDeltaInRandomEffect 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getDeltaInRandomEffectAccess().getGroup()); }
		(rule__DeltaInRandomEffect__Group__0)
		{ after(grammarAccess.getDeltaInRandomEffectAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleDeltaInErroneousEffect
entryRuleDeltaInErroneousEffect
:
{ before(grammarAccess.getDeltaInErroneousEffectRule()); }
	 ruleDeltaInErroneousEffect
{ after(grammarAccess.getDeltaInErroneousEffectRule()); } 
	 EOF 
;

// Rule DeltaInErroneousEffect
ruleDeltaInErroneousEffect 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getDeltaInErroneousEffectAccess().getGroup()); }
		(rule__DeltaInErroneousEffect__Group__0)
		{ after(grammarAccess.getDeltaInErroneousEffectAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleTemplate
entryRuleTemplate
:
{ before(grammarAccess.getTemplateRule()); }
	 ruleTemplate
{ after(grammarAccess.getTemplateRule()); } 
	 EOF 
;

// Rule Template
ruleTemplate 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getTemplateAccess().getSelfFixTemplateParserRuleCall()); }
		ruleSelfFixTemplate
		{ after(grammarAccess.getTemplateAccess().getSelfFixTemplateParserRuleCall()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleSelfFixTemplate
entryRuleSelfFixTemplate
:
{ before(grammarAccess.getSelfFixTemplateRule()); }
	 ruleSelfFixTemplate
{ after(grammarAccess.getSelfFixTemplateRule()); } 
	 EOF 
;

// Rule SelfFixTemplate
ruleSelfFixTemplate 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getSelfFixTemplateAccess().getGroup()); }
		(rule__SelfFixTemplate__Group__0)
		{ after(grammarAccess.getSelfFixTemplateAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleGlobalDynamics
entryRuleGlobalDynamics
:
{ before(grammarAccess.getGlobalDynamicsRule()); }
	 ruleGlobalDynamics
{ after(grammarAccess.getGlobalDynamicsRule()); } 
	 EOF 
;

// Rule GlobalDynamics
ruleGlobalDynamics 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getGlobalDynamicsAccess().getGroup()); }
		(rule__GlobalDynamics__Group__0)
		{ after(grammarAccess.getGlobalDynamicsAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleFaultTransition
entryRuleFaultTransition
:
{ before(grammarAccess.getFaultTransitionRule()); }
	 ruleFaultTransition
{ after(grammarAccess.getFaultTransitionRule()); } 
	 EOF 
;

// Rule FaultTransition
ruleFaultTransition 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getFaultTransitionAccess().getGroup()); }
		(rule__FaultTransition__Group__0)
		{ after(grammarAccess.getFaultTransitionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleFaultModeStateReference
entryRuleFaultModeStateReference
:
{ before(grammarAccess.getFaultModeStateReferenceRule()); }
	 ruleFaultModeStateReference
{ after(grammarAccess.getFaultModeStateReferenceRule()); } 
	 EOF 
;

// Rule FaultModeStateReference
ruleFaultModeStateReference 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getFaultModeStateReferenceAccess().getGroup()); }
		(rule__FaultModeStateReference__Group__0)
		{ after(grammarAccess.getFaultModeStateReferenceAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleFaultTransitionTrigger
entryRuleFaultTransitionTrigger
:
{ before(grammarAccess.getFaultTransitionTriggerRule()); }
	 ruleFaultTransitionTrigger
{ after(grammarAccess.getFaultTransitionTriggerRule()); } 
	 EOF 
;

// Rule FaultTransitionTrigger
ruleFaultTransitionTrigger 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getFaultTransitionTriggerAccess().getGroup()); }
		(rule__FaultTransitionTrigger__Group__0)
		{ after(grammarAccess.getFaultTransitionTriggerAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleCommonCauses
entryRuleCommonCauses
:
{ before(grammarAccess.getCommonCausesRule()); }
	 ruleCommonCauses
{ after(grammarAccess.getCommonCausesRule()); } 
	 EOF 
;

// Rule CommonCauses
ruleCommonCauses 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getCommonCausesAccess().getGroup()); }
		(rule__CommonCauses__Group__0)
		{ after(grammarAccess.getCommonCausesAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleCommonCause
entryRuleCommonCause
:
{ before(grammarAccess.getCommonCauseRule()); }
	 ruleCommonCause
{ after(grammarAccess.getCommonCauseRule()); } 
	 EOF 
;

// Rule CommonCause
ruleCommonCause 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getCommonCauseAccess().getGroup()); }
		(rule__CommonCause__Group__0)
		{ after(grammarAccess.getCommonCauseAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleCommonCauseProbability
entryRuleCommonCauseProbability
:
{ before(grammarAccess.getCommonCauseProbabilityRule()); }
	 ruleCommonCauseProbability
{ after(grammarAccess.getCommonCauseProbabilityRule()); } 
	 EOF 
;

// Rule CommonCauseProbability
ruleCommonCauseProbability 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getCommonCauseProbabilityAccess().getGroup()); }
		(rule__CommonCauseProbability__Group__0)
		{ after(grammarAccess.getCommonCauseProbabilityAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleCommonCauseMode
entryRuleCommonCauseMode
:
{ before(grammarAccess.getCommonCauseModeRule()); }
	 ruleCommonCauseMode
{ after(grammarAccess.getCommonCauseModeRule()); } 
	 EOF 
;

// Rule CommonCauseMode
ruleCommonCauseMode 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getCommonCauseModeAccess().getGroup()); }
		(rule__CommonCauseMode__Group__0)
		{ after(grammarAccess.getCommonCauseModeAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleCommonCauseRange
entryRuleCommonCauseRange
:
{ before(grammarAccess.getCommonCauseRangeRule()); }
	 ruleCommonCauseRange
{ after(grammarAccess.getCommonCauseRangeRule()); } 
	 EOF 
;

// Rule CommonCauseRange
ruleCommonCauseRange 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getCommonCauseRangeAccess().getAlternatives()); }
		(rule__CommonCauseRange__Alternatives)
		{ after(grammarAccess.getCommonCauseRangeAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRulePrimaryExpression
entryRulePrimaryExpression
:
{ before(grammarAccess.getPrimaryExpressionRule()); }
	 rulePrimaryExpression
{ after(grammarAccess.getPrimaryExpressionRule()); } 
	 EOF 
;

// Rule PrimaryExpression
rulePrimaryExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getPrimaryExpressionAccess().getAlternatives()); }
		(rule__PrimaryExpression__Alternatives)
		{ after(grammarAccess.getPrimaryExpressionAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleTimeoutDeclaration
entryRuleTimeoutDeclaration
:
{ before(grammarAccess.getTimeoutDeclarationRule()); }
	 ruleTimeoutDeclaration
{ after(grammarAccess.getTimeoutDeclarationRule()); } 
	 EOF 
;

// Rule TimeoutDeclaration
ruleTimeoutDeclaration 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getTimeoutDeclarationAccess().getGroup()); }
		(rule__TimeoutDeclaration__Group__0)
		{ after(grammarAccess.getTimeoutDeclarationAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleTrigger
entryRuleTrigger
:
{ before(grammarAccess.getTriggerRule()); }
	 ruleTrigger
{ after(grammarAccess.getTriggerRule()); } 
	 EOF 
;

// Rule Trigger
ruleTrigger 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getTriggerAccess().getAlternatives()); }
		(rule__Trigger__Alternatives)
		{ after(grammarAccess.getTriggerAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleSimpleTrigger
entryRuleSimpleTrigger
:
{ before(grammarAccess.getSimpleTriggerRule()); }
	 ruleSimpleTrigger
{ after(grammarAccess.getSimpleTriggerRule()); } 
	 EOF 
;

// Rule SimpleTrigger
ruleSimpleTrigger 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getSimpleTriggerAccess().getAlternatives()); }
		(rule__SimpleTrigger__Alternatives)
		{ after(grammarAccess.getSimpleTriggerAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleComplexTrigger
entryRuleComplexTrigger
:
{ before(grammarAccess.getComplexTriggerRule()); }
	 ruleComplexTrigger
{ after(grammarAccess.getComplexTriggerRule()); } 
	 EOF 
;

// Rule ComplexTrigger
ruleComplexTrigger 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getComplexTriggerAccess().getBinaryTriggerParserRuleCall()); }
		ruleBinaryTrigger
		{ after(grammarAccess.getComplexTriggerAccess().getBinaryTriggerParserRuleCall()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleBinaryTrigger
entryRuleBinaryTrigger
:
{ before(grammarAccess.getBinaryTriggerRule()); }
	 ruleBinaryTrigger
{ after(grammarAccess.getBinaryTriggerRule()); } 
	 EOF 
;

// Rule BinaryTrigger
ruleBinaryTrigger 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getBinaryTriggerAccess().getEqualTriggerParserRuleCall()); }
		ruleEqualTrigger
		{ after(grammarAccess.getBinaryTriggerAccess().getEqualTriggerParserRuleCall()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleEqualTrigger
entryRuleEqualTrigger
:
{ before(grammarAccess.getEqualTriggerRule()); }
	 ruleEqualTrigger
{ after(grammarAccess.getEqualTriggerRule()); } 
	 EOF 
;

// Rule EqualTrigger
ruleEqualTrigger 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getEqualTriggerAccess().getGroup()); }
		(rule__EqualTrigger__Group__0)
		{ after(grammarAccess.getEqualTriggerAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleImplyTrigger
entryRuleImplyTrigger
:
{ before(grammarAccess.getImplyTriggerRule()); }
	 ruleImplyTrigger
{ after(grammarAccess.getImplyTriggerRule()); } 
	 EOF 
;

// Rule ImplyTrigger
ruleImplyTrigger 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getImplyTriggerAccess().getGroup()); }
		(rule__ImplyTrigger__Group__0)
		{ after(grammarAccess.getImplyTriggerAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleOrTrigger
entryRuleOrTrigger
:
{ before(grammarAccess.getOrTriggerRule()); }
	 ruleOrTrigger
{ after(grammarAccess.getOrTriggerRule()); } 
	 EOF 
;

// Rule OrTrigger
ruleOrTrigger 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getOrTriggerAccess().getGroup()); }
		(rule__OrTrigger__Group__0)
		{ after(grammarAccess.getOrTriggerAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleXorTrigger
entryRuleXorTrigger
:
{ before(grammarAccess.getXorTriggerRule()); }
	 ruleXorTrigger
{ after(grammarAccess.getXorTriggerRule()); } 
	 EOF 
;

// Rule XorTrigger
ruleXorTrigger 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getXorTriggerAccess().getGroup()); }
		(rule__XorTrigger__Group__0)
		{ after(grammarAccess.getXorTriggerAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleAndTrigger
entryRuleAndTrigger
:
{ before(grammarAccess.getAndTriggerRule()); }
	 ruleAndTrigger
{ after(grammarAccess.getAndTriggerRule()); } 
	 EOF 
;

// Rule AndTrigger
ruleAndTrigger 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getAndTriggerAccess().getGroup()); }
		(rule__AndTrigger__Group__0)
		{ after(grammarAccess.getAndTriggerAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleBinaryOperand
entryRuleBinaryOperand
:
{ before(grammarAccess.getBinaryOperandRule()); }
	 ruleBinaryOperand
{ after(grammarAccess.getBinaryOperandRule()); } 
	 EOF 
;

// Rule BinaryOperand
ruleBinaryOperand 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getBinaryOperandAccess().getAlternatives()); }
		(rule__BinaryOperand__Alternatives)
		{ after(grammarAccess.getBinaryOperandAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleUnaryTrigger
entryRuleUnaryTrigger
:
{ before(grammarAccess.getUnaryTriggerRule()); }
	 ruleUnaryTrigger
{ after(grammarAccess.getUnaryTriggerRule()); } 
	 EOF 
;

// Rule UnaryTrigger
ruleUnaryTrigger 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getUnaryTriggerAccess().getAlternatives()); }
		(rule__UnaryTrigger__Alternatives)
		{ after(grammarAccess.getUnaryTriggerAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleNotTrigger
entryRuleNotTrigger
:
{ before(grammarAccess.getNotTriggerRule()); }
	 ruleNotTrigger
{ after(grammarAccess.getNotTriggerRule()); } 
	 EOF 
;

// Rule NotTrigger
ruleNotTrigger 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getNotTriggerAccess().getGroup()); }
		(rule__NotTrigger__Group__0)
		{ after(grammarAccess.getNotTriggerAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleParenthesesTrigger
entryRuleParenthesesTrigger
:
{ before(grammarAccess.getParenthesesTriggerRule()); }
	 ruleParenthesesTrigger
{ after(grammarAccess.getParenthesesTriggerRule()); } 
	 EOF 
;

// Rule ParenthesesTrigger
ruleParenthesesTrigger 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getParenthesesTriggerAccess().getGroup()); }
		(rule__ParenthesesTrigger__Group__0)
		{ after(grammarAccess.getParenthesesTriggerAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleOnCycleTrigger
entryRuleOnCycleTrigger
:
{ before(grammarAccess.getOnCycleTriggerRule()); }
	 ruleOnCycleTrigger
{ after(grammarAccess.getOnCycleTriggerRule()); } 
	 EOF 
;

// Rule OnCycleTrigger
ruleOnCycleTrigger 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getOnCycleTriggerAccess().getGroup()); }
		(rule__OnCycleTrigger__Group__0)
		{ after(grammarAccess.getOnCycleTriggerAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleAnyTrigger
entryRuleAnyTrigger
:
{ before(grammarAccess.getAnyTriggerRule()); }
	 ruleAnyTrigger
{ after(grammarAccess.getAnyTriggerRule()); } 
	 EOF 
;

// Rule AnyTrigger
ruleAnyTrigger 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getAnyTriggerAccess().getGroup()); }
		(rule__AnyTrigger__Group__0)
		{ after(grammarAccess.getAnyTriggerAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleOpaqueTrigger
entryRuleOpaqueTrigger
:
{ before(grammarAccess.getOpaqueTriggerRule()); }
	 ruleOpaqueTrigger
{ after(grammarAccess.getOpaqueTriggerRule()); } 
	 EOF 
;

// Rule OpaqueTrigger
ruleOpaqueTrigger 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getOpaqueTriggerAccess().getTriggerAssignment()); }
		(rule__OpaqueTrigger__TriggerAssignment)
		{ after(grammarAccess.getOpaqueTriggerAccess().getTriggerAssignment()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleEventTrigger
entryRuleEventTrigger
:
{ before(grammarAccess.getEventTriggerRule()); }
	 ruleEventTrigger
{ after(grammarAccess.getEventTriggerRule()); } 
	 EOF 
;

// Rule EventTrigger
ruleEventTrigger 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getEventTriggerAccess().getEventReferenceAssignment()); }
		(rule__EventTrigger__EventReferenceAssignment)
		{ after(grammarAccess.getEventTriggerAccess().getEventReferenceAssignment()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleEventReference
entryRuleEventReference
:
{ before(grammarAccess.getEventReferenceRule()); }
	 ruleEventReference
{ after(grammarAccess.getEventReferenceRule()); } 
	 EOF 
;

// Rule EventReference
ruleEventReference 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getEventReferenceAccess().getAlternatives()); }
		(rule__EventReference__Alternatives)
		{ after(grammarAccess.getEventReferenceAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleClockTickReference
entryRuleClockTickReference
:
{ before(grammarAccess.getClockTickReferenceRule()); }
	 ruleClockTickReference
{ after(grammarAccess.getClockTickReferenceRule()); } 
	 EOF 
;

// Rule ClockTickReference
ruleClockTickReference 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getClockTickReferenceAccess().getClockAssignment()); }
		(rule__ClockTickReference__ClockAssignment)
		{ after(grammarAccess.getClockTickReferenceAccess().getClockAssignment()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleTimeoutEventReference
entryRuleTimeoutEventReference
:
{ before(grammarAccess.getTimeoutEventReferenceRule()); }
	 ruleTimeoutEventReference
{ after(grammarAccess.getTimeoutEventReferenceRule()); } 
	 EOF 
;

// Rule TimeoutEventReference
ruleTimeoutEventReference 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getTimeoutEventReferenceAccess().getGroup()); }
		(rule__TimeoutEventReference__Group__0)
		{ after(grammarAccess.getTimeoutEventReferenceAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleAnyPortEventReference
entryRuleAnyPortEventReference
:
{ before(grammarAccess.getAnyPortEventReferenceRule()); }
	 ruleAnyPortEventReference
{ after(grammarAccess.getAnyPortEventReferenceRule()); } 
	 EOF 
;

// Rule AnyPortEventReference
ruleAnyPortEventReference 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getAnyPortEventReferenceAccess().getGroup()); }
		(rule__AnyPortEventReference__Group__0)
		{ after(grammarAccess.getAnyPortEventReferenceAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRulePortEventReference
entryRulePortEventReference
:
{ before(grammarAccess.getPortEventReferenceRule()); }
	 rulePortEventReference
{ after(grammarAccess.getPortEventReferenceRule()); } 
	 EOF 
;

// Rule PortEventReference
rulePortEventReference 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getPortEventReferenceAccess().getGroup()); }
		(rule__PortEventReference__Group__0)
		{ after(grammarAccess.getPortEventReferenceAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRulePort
entryRulePort
:
{ before(grammarAccess.getPortRule()); }
	 rulePort
{ after(grammarAccess.getPortRule()); } 
	 EOF 
;

// Rule Port
rulePort 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getPortAccess().getGroup()); }
		(rule__Port__Group__0)
		{ after(grammarAccess.getPortAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleInterfaceRealization
entryRuleInterfaceRealization
:
{ before(grammarAccess.getInterfaceRealizationRule()); }
	 ruleInterfaceRealization
{ after(grammarAccess.getInterfaceRealizationRule()); } 
	 EOF 
;

// Rule InterfaceRealization
ruleInterfaceRealization 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getInterfaceRealizationAccess().getGroup()); }
		(rule__InterfaceRealization__Group__0)
		{ after(grammarAccess.getInterfaceRealizationAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleSynchronousStatechartDefinition
entryRuleSynchronousStatechartDefinition
:
{ before(grammarAccess.getSynchronousStatechartDefinitionRule()); }
	 ruleSynchronousStatechartDefinition
{ after(grammarAccess.getSynchronousStatechartDefinitionRule()); } 
	 EOF 
;

// Rule SynchronousStatechartDefinition
ruleSynchronousStatechartDefinition 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getGroup()); }
		(rule__SynchronousStatechartDefinition__Group__0)
		{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleAsynchronousStatechartDefinition
entryRuleAsynchronousStatechartDefinition
:
{ before(grammarAccess.getAsynchronousStatechartDefinitionRule()); }
	 ruleAsynchronousStatechartDefinition
{ after(grammarAccess.getAsynchronousStatechartDefinitionRule()); } 
	 EOF 
;

// Rule AsynchronousStatechartDefinition
ruleAsynchronousStatechartDefinition 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getGroup()); }
		(rule__AsynchronousStatechartDefinition__Group__0)
		{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleComponentAnnotation
entryRuleComponentAnnotation
:
{ before(grammarAccess.getComponentAnnotationRule()); }
	 ruleComponentAnnotation
{ after(grammarAccess.getComponentAnnotationRule()); } 
	 EOF 
;

// Rule ComponentAnnotation
ruleComponentAnnotation 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getComponentAnnotationAccess().getAlternatives()); }
		(rule__ComponentAnnotation__Alternatives)
		{ after(grammarAccess.getComponentAnnotationAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleStatechartContractAnnotation
entryRuleStatechartContractAnnotation
:
{ before(grammarAccess.getStatechartContractAnnotationRule()); }
	 ruleStatechartContractAnnotation
{ after(grammarAccess.getStatechartContractAnnotationRule()); } 
	 EOF 
;

// Rule StatechartContractAnnotation
ruleStatechartContractAnnotation 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getStatechartContractAnnotationAccess().getAlternatives()); }
		(rule__StatechartContractAnnotation__Alternatives)
		{ after(grammarAccess.getStatechartContractAnnotationAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleNegativeContractStatechartAnnotation
entryRuleNegativeContractStatechartAnnotation
:
{ before(grammarAccess.getNegativeContractStatechartAnnotationRule()); }
	 ruleNegativeContractStatechartAnnotation
{ after(grammarAccess.getNegativeContractStatechartAnnotationRule()); } 
	 EOF 
;

// Rule NegativeContractStatechartAnnotation
ruleNegativeContractStatechartAnnotation 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getNegativeContractStatechartAnnotationAccess().getGroup()); }
		(rule__NegativeContractStatechartAnnotation__Group__0)
		{ after(grammarAccess.getNegativeContractStatechartAnnotationAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleHasInitialoutputsBlockAnnotation
entryRuleHasInitialoutputsBlockAnnotation
:
{ before(grammarAccess.getHasInitialoutputsBlockAnnotationRule()); }
	 ruleHasInitialoutputsBlockAnnotation
{ after(grammarAccess.getHasInitialoutputsBlockAnnotationRule()); } 
	 EOF 
;

// Rule HasInitialoutputsBlockAnnotation
ruleHasInitialoutputsBlockAnnotation 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getHasInitialoutputsBlockAnnotationAccess().getGroup()); }
		(rule__HasInitialoutputsBlockAnnotation__Group__0)
		{ after(grammarAccess.getHasInitialoutputsBlockAnnotationAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleScenarioAllowedWaitingAnnotation
entryRuleScenarioAllowedWaitingAnnotation
:
{ before(grammarAccess.getScenarioAllowedWaitingAnnotationRule()); }
	 ruleScenarioAllowedWaitingAnnotation
{ after(grammarAccess.getScenarioAllowedWaitingAnnotationRule()); } 
	 EOF 
;

// Rule ScenarioAllowedWaitingAnnotation
ruleScenarioAllowedWaitingAnnotation 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getScenarioAllowedWaitingAnnotationAccess().getGroup()); }
		(rule__ScenarioAllowedWaitingAnnotation__Group__0)
		{ after(grammarAccess.getScenarioAllowedWaitingAnnotationAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleScenarioContractAnnotation
entryRuleScenarioContractAnnotation
:
{ before(grammarAccess.getScenarioContractAnnotationRule()); }
	 ruleScenarioContractAnnotation
{ after(grammarAccess.getScenarioContractAnnotationRule()); } 
	 EOF 
;

// Rule ScenarioContractAnnotation
ruleScenarioContractAnnotation 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getScenarioContractAnnotationAccess().getGroup()); }
		(rule__ScenarioContractAnnotation__Group__0)
		{ after(grammarAccess.getScenarioContractAnnotationAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleAdaptiveContractAnnotation
entryRuleAdaptiveContractAnnotation
:
{ before(grammarAccess.getAdaptiveContractAnnotationRule()); }
	 ruleAdaptiveContractAnnotation
{ after(grammarAccess.getAdaptiveContractAnnotationRule()); } 
	 EOF 
;

// Rule AdaptiveContractAnnotation
ruleAdaptiveContractAnnotation 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getAdaptiveContractAnnotationAccess().getGroup()); }
		(rule__AdaptiveContractAnnotation__Group__0)
		{ after(grammarAccess.getAdaptiveContractAnnotationAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleMissionPhaseAnnotation
entryRuleMissionPhaseAnnotation
:
{ before(grammarAccess.getMissionPhaseAnnotationRule()); }
	 ruleMissionPhaseAnnotation
{ after(grammarAccess.getMissionPhaseAnnotationRule()); } 
	 EOF 
;

// Rule MissionPhaseAnnotation
ruleMissionPhaseAnnotation 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getMissionPhaseAnnotationAccess().getGroup()); }
		(rule__MissionPhaseAnnotation__Group__0)
		{ after(grammarAccess.getMissionPhaseAnnotationAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleWrapperComponentAnnotation
entryRuleWrapperComponentAnnotation
:
{ before(grammarAccess.getWrapperComponentAnnotationRule()); }
	 ruleWrapperComponentAnnotation
{ after(grammarAccess.getWrapperComponentAnnotationRule()); } 
	 EOF 
;

// Rule WrapperComponentAnnotation
ruleWrapperComponentAnnotation 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getWrapperComponentAnnotationAccess().getGroup()); }
		(rule__WrapperComponentAnnotation__Group__0)
		{ after(grammarAccess.getWrapperComponentAnnotationAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleRunUponExternalEventAnnotation
entryRuleRunUponExternalEventAnnotation
:
{ before(grammarAccess.getRunUponExternalEventAnnotationRule()); }
	 ruleRunUponExternalEventAnnotation
{ after(grammarAccess.getRunUponExternalEventAnnotationRule()); } 
	 EOF 
;

// Rule RunUponExternalEventAnnotation
ruleRunUponExternalEventAnnotation 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getRunUponExternalEventAnnotationAccess().getGroup()); }
		(rule__RunUponExternalEventAnnotation__Group__0)
		{ after(grammarAccess.getRunUponExternalEventAnnotationAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleRunUponExternalEventOrInternalTimeoutAnnotation
entryRuleRunUponExternalEventOrInternalTimeoutAnnotation
:
{ before(grammarAccess.getRunUponExternalEventOrInternalTimeoutAnnotationRule()); }
	 ruleRunUponExternalEventOrInternalTimeoutAnnotation
{ after(grammarAccess.getRunUponExternalEventOrInternalTimeoutAnnotationRule()); } 
	 EOF 
;

// Rule RunUponExternalEventOrInternalTimeoutAnnotation
ruleRunUponExternalEventOrInternalTimeoutAnnotation 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getRunUponExternalEventOrInternalTimeoutAnnotationAccess().getGroup()); }
		(rule__RunUponExternalEventOrInternalTimeoutAnnotation__Group__0)
		{ after(grammarAccess.getRunUponExternalEventOrInternalTimeoutAnnotationAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleRegion
entryRuleRegion
:
{ before(grammarAccess.getRegionRule()); }
	 ruleRegion
{ after(grammarAccess.getRegionRule()); } 
	 EOF 
;

// Rule Region
ruleRegion 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getRegionAccess().getGroup()); }
		(rule__Region__Group__0)
		{ after(grammarAccess.getRegionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleTransition
entryRuleTransition
:
{ before(grammarAccess.getTransitionRule()); }
	 ruleTransition
{ after(grammarAccess.getTransitionRule()); } 
	 EOF 
;

// Rule Transition
ruleTransition 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getTransitionAccess().getGroup()); }
		(rule__Transition__Group__0)
		{ after(grammarAccess.getTransitionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleTransitionAnnotation
entryRuleTransitionAnnotation
:
{ before(grammarAccess.getTransitionAnnotationRule()); }
	 ruleTransitionAnnotation
{ after(grammarAccess.getTransitionAnnotationRule()); } 
	 EOF 
;

// Rule TransitionAnnotation
ruleTransitionAnnotation 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getTransitionAnnotationAccess().getTransitionIdAnnotationParserRuleCall()); }
		ruleTransitionIdAnnotation
		{ after(grammarAccess.getTransitionAnnotationAccess().getTransitionIdAnnotationParserRuleCall()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleTransitionIdAnnotation
entryRuleTransitionIdAnnotation
:
{ before(grammarAccess.getTransitionIdAnnotationRule()); }
	 ruleTransitionIdAnnotation
{ after(grammarAccess.getTransitionIdAnnotationRule()); } 
	 EOF 
;

// Rule TransitionIdAnnotation
ruleTransitionIdAnnotation 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getTransitionIdAnnotationAccess().getGroup()); }
		(rule__TransitionIdAnnotation__Group__0)
		{ after(grammarAccess.getTransitionIdAnnotationAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleStateNode
entryRuleStateNode
:
{ before(grammarAccess.getStateNodeRule()); }
	 ruleStateNode
{ after(grammarAccess.getStateNodeRule()); } 
	 EOF 
;

// Rule StateNode
ruleStateNode 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getStateNodeAccess().getAlternatives()); }
		(rule__StateNode__Alternatives)
		{ after(grammarAccess.getStateNodeAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleState
entryRuleState
:
{ before(grammarAccess.getStateRule()); }
	 ruleState
{ after(grammarAccess.getStateRule()); } 
	 EOF 
;

// Rule State
ruleState 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getStateAccess().getGroup()); }
		(rule__State__Group__0)
		{ after(grammarAccess.getStateAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleStateAnnotation
entryRuleStateAnnotation
:
{ before(grammarAccess.getStateAnnotationRule()); }
	 ruleStateAnnotation
{ after(grammarAccess.getStateAnnotationRule()); } 
	 EOF 
;

// Rule StateAnnotation
ruleStateAnnotation 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getStateAnnotationAccess().getAlternatives()); }
		(rule__StateAnnotation__Alternatives)
		{ after(grammarAccess.getStateAnnotationAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleSpecialStateAnnotation
entryRuleSpecialStateAnnotation
:
{ before(grammarAccess.getSpecialStateAnnotationRule()); }
	 ruleSpecialStateAnnotation
{ after(grammarAccess.getSpecialStateAnnotationRule()); } 
	 EOF 
;

// Rule SpecialStateAnnotation
ruleSpecialStateAnnotation 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getSpecialStateAnnotationAccess().getGroup()); }
		(rule__SpecialStateAnnotation__Group__0)
		{ after(grammarAccess.getSpecialStateAnnotationAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleStateContractAnnotation
entryRuleStateContractAnnotation
:
{ before(grammarAccess.getStateContractAnnotationRule()); }
	 ruleStateContractAnnotation
{ after(grammarAccess.getStateContractAnnotationRule()); } 
	 EOF 
;

// Rule StateContractAnnotation
ruleStateContractAnnotation 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getStateContractAnnotationAccess().getGroup()); }
		(rule__StateContractAnnotation__Group__0)
		{ after(grammarAccess.getStateContractAnnotationAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleMissionPhaseStateAnnotation
entryRuleMissionPhaseStateAnnotation
:
{ before(grammarAccess.getMissionPhaseStateAnnotationRule()); }
	 ruleMissionPhaseStateAnnotation
{ after(grammarAccess.getMissionPhaseStateAnnotationRule()); } 
	 EOF 
;

// Rule MissionPhaseStateAnnotation
ruleMissionPhaseStateAnnotation 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getMissionPhaseStateAnnotationAccess().getGroup()); }
		(rule__MissionPhaseStateAnnotation__Group__0)
		{ after(grammarAccess.getMissionPhaseStateAnnotationAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleVariableBinding
entryRuleVariableBinding
:
{ before(grammarAccess.getVariableBindingRule()); }
	 ruleVariableBinding
{ after(grammarAccess.getVariableBindingRule()); } 
	 EOF 
;

// Rule VariableBinding
ruleVariableBinding 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getVariableBindingAccess().getAlternatives()); }
		(rule__VariableBinding__Alternatives)
		{ after(grammarAccess.getVariableBindingAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleInstanceVariableReference
entryRuleInstanceVariableReference
:
{ before(grammarAccess.getInstanceVariableReferenceRule()); }
	 ruleInstanceVariableReference
{ after(grammarAccess.getInstanceVariableReferenceRule()); } 
	 EOF 
;

// Rule InstanceVariableReference
ruleInstanceVariableReference 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getInstanceVariableReferenceAccess().getGroup()); }
		(rule__InstanceVariableReference__Group__0)
		{ after(grammarAccess.getInstanceVariableReferenceAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRulePseudoState
entryRulePseudoState
:
{ before(grammarAccess.getPseudoStateRule()); }
	 rulePseudoState
{ after(grammarAccess.getPseudoStateRule()); } 
	 EOF 
;

// Rule PseudoState
rulePseudoState 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getPseudoStateAccess().getAlternatives()); }
		(rule__PseudoState__Alternatives)
		{ after(grammarAccess.getPseudoStateAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleMergeState
entryRuleMergeState
:
{ before(grammarAccess.getMergeStateRule()); }
	 ruleMergeState
{ after(grammarAccess.getMergeStateRule()); } 
	 EOF 
;

// Rule MergeState
ruleMergeState 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getMergeStateAccess().getGroup()); }
		(rule__MergeState__Group__0)
		{ after(grammarAccess.getMergeStateAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleChoiceState
entryRuleChoiceState
:
{ before(grammarAccess.getChoiceStateRule()); }
	 ruleChoiceState
{ after(grammarAccess.getChoiceStateRule()); } 
	 EOF 
;

// Rule ChoiceState
ruleChoiceState 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getChoiceStateAccess().getGroup()); }
		(rule__ChoiceState__Group__0)
		{ after(grammarAccess.getChoiceStateAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleJoinState
entryRuleJoinState
:
{ before(grammarAccess.getJoinStateRule()); }
	 ruleJoinState
{ after(grammarAccess.getJoinStateRule()); } 
	 EOF 
;

// Rule JoinState
ruleJoinState 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getJoinStateAccess().getGroup()); }
		(rule__JoinState__Group__0)
		{ after(grammarAccess.getJoinStateAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleForkState
entryRuleForkState
:
{ before(grammarAccess.getForkStateRule()); }
	 ruleForkState
{ after(grammarAccess.getForkStateRule()); } 
	 EOF 
;

// Rule ForkState
ruleForkState 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getForkStateAccess().getGroup()); }
		(rule__ForkState__Group__0)
		{ after(grammarAccess.getForkStateAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleEntryState
entryRuleEntryState
:
{ before(grammarAccess.getEntryStateRule()); }
	 ruleEntryState
{ after(grammarAccess.getEntryStateRule()); } 
	 EOF 
;

// Rule EntryState
ruleEntryState 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getEntryStateAccess().getAlternatives()); }
		(rule__EntryState__Alternatives)
		{ after(grammarAccess.getEntryStateAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleInitialState
entryRuleInitialState
:
{ before(grammarAccess.getInitialStateRule()); }
	 ruleInitialState
{ after(grammarAccess.getInitialStateRule()); } 
	 EOF 
;

// Rule InitialState
ruleInitialState 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getInitialStateAccess().getGroup()); }
		(rule__InitialState__Group__0)
		{ after(grammarAccess.getInitialStateAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleShallowHistoryState
entryRuleShallowHistoryState
:
{ before(grammarAccess.getShallowHistoryStateRule()); }
	 ruleShallowHistoryState
{ after(grammarAccess.getShallowHistoryStateRule()); } 
	 EOF 
;

// Rule ShallowHistoryState
ruleShallowHistoryState 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getShallowHistoryStateAccess().getGroup()); }
		(rule__ShallowHistoryState__Group__0)
		{ after(grammarAccess.getShallowHistoryStateAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleDeepHistoryState
entryRuleDeepHistoryState
:
{ before(grammarAccess.getDeepHistoryStateRule()); }
	 ruleDeepHistoryState
{ after(grammarAccess.getDeepHistoryStateRule()); } 
	 EOF 
;

// Rule DeepHistoryState
ruleDeepHistoryState 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getDeepHistoryStateAccess().getGroup()); }
		(rule__DeepHistoryState__Group__0)
		{ after(grammarAccess.getDeepHistoryStateAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleComponent
entryRuleComponent
:
{ before(grammarAccess.getComponentRule()); }
	 ruleComponent
{ after(grammarAccess.getComponentRule()); } 
	 EOF 
;

// Rule Component
ruleComponent 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getComponentAccess().getAlternatives()); }
		(rule__Component__Alternatives)
		{ after(grammarAccess.getComponentAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleSynchronousComponent
entryRuleSynchronousComponent
:
{ before(grammarAccess.getSynchronousComponentRule()); }
	 ruleSynchronousComponent
{ after(grammarAccess.getSynchronousComponentRule()); } 
	 EOF 
;

// Rule SynchronousComponent
ruleSynchronousComponent 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getSynchronousComponentAccess().getAlternatives()); }
		(rule__SynchronousComponent__Alternatives)
		{ after(grammarAccess.getSynchronousComponentAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleAbstractSynchronousComponent
entryRuleAbstractSynchronousComponent
:
{ before(grammarAccess.getAbstractSynchronousComponentRule()); }
	 ruleAbstractSynchronousComponent
{ after(grammarAccess.getAbstractSynchronousComponentRule()); } 
	 EOF 
;

// Rule AbstractSynchronousComponent
ruleAbstractSynchronousComponent 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getAbstractSynchronousComponentAccess().getAlternatives()); }
		(rule__AbstractSynchronousComponent__Alternatives)
		{ after(grammarAccess.getAbstractSynchronousComponentAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleCascadeCompositeComponent
entryRuleCascadeCompositeComponent
:
{ before(grammarAccess.getCascadeCompositeComponentRule()); }
	 ruleCascadeCompositeComponent
{ after(grammarAccess.getCascadeCompositeComponentRule()); } 
	 EOF 
;

// Rule CascadeCompositeComponent
ruleCascadeCompositeComponent 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getCascadeCompositeComponentAccess().getGroup()); }
		(rule__CascadeCompositeComponent__Group__0)
		{ after(grammarAccess.getCascadeCompositeComponentAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleSynchronousCompositeComponent
entryRuleSynchronousCompositeComponent
:
{ before(grammarAccess.getSynchronousCompositeComponentRule()); }
	 ruleSynchronousCompositeComponent
{ after(grammarAccess.getSynchronousCompositeComponentRule()); } 
	 EOF 
;

// Rule SynchronousCompositeComponent
ruleSynchronousCompositeComponent 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getSynchronousCompositeComponentAccess().getGroup()); }
		(rule__SynchronousCompositeComponent__Group__0)
		{ after(grammarAccess.getSynchronousCompositeComponentAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleAsynchronousComponent
entryRuleAsynchronousComponent
:
{ before(grammarAccess.getAsynchronousComponentRule()); }
	 ruleAsynchronousComponent
{ after(grammarAccess.getAsynchronousComponentRule()); } 
	 EOF 
;

// Rule AsynchronousComponent
ruleAsynchronousComponent 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getAsynchronousComponentAccess().getAlternatives()); }
		(rule__AsynchronousComponent__Alternatives)
		{ after(grammarAccess.getAsynchronousComponentAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleAbstractAsynchronousCompositeComponent
entryRuleAbstractAsynchronousCompositeComponent
:
{ before(grammarAccess.getAbstractAsynchronousCompositeComponentRule()); }
	 ruleAbstractAsynchronousCompositeComponent
{ after(grammarAccess.getAbstractAsynchronousCompositeComponentRule()); } 
	 EOF 
;

// Rule AbstractAsynchronousCompositeComponent
ruleAbstractAsynchronousCompositeComponent 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getAbstractAsynchronousCompositeComponentAccess().getAlternatives()); }
		(rule__AbstractAsynchronousCompositeComponent__Alternatives)
		{ after(grammarAccess.getAbstractAsynchronousCompositeComponentAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleAsynchronousCompositeComponent
entryRuleAsynchronousCompositeComponent
:
{ before(grammarAccess.getAsynchronousCompositeComponentRule()); }
	 ruleAsynchronousCompositeComponent
{ after(grammarAccess.getAsynchronousCompositeComponentRule()); } 
	 EOF 
;

// Rule AsynchronousCompositeComponent
ruleAsynchronousCompositeComponent 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getAsynchronousCompositeComponentAccess().getGroup()); }
		(rule__AsynchronousCompositeComponent__Group__0)
		{ after(grammarAccess.getAsynchronousCompositeComponentAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleScheduledAsynchronousCompositeComponent
entryRuleScheduledAsynchronousCompositeComponent
:
{ before(grammarAccess.getScheduledAsynchronousCompositeComponentRule()); }
	 ruleScheduledAsynchronousCompositeComponent
{ after(grammarAccess.getScheduledAsynchronousCompositeComponentRule()); } 
	 EOF 
;

// Rule ScheduledAsynchronousCompositeComponent
ruleScheduledAsynchronousCompositeComponent 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getGroup()); }
		(rule__ScheduledAsynchronousCompositeComponent__Group__0)
		{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleAsynchronousAdapter
entryRuleAsynchronousAdapter
:
{ before(grammarAccess.getAsynchronousAdapterRule()); }
	 ruleAsynchronousAdapter
{ after(grammarAccess.getAsynchronousAdapterRule()); } 
	 EOF 
;

// Rule AsynchronousAdapter
ruleAsynchronousAdapter 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getAsynchronousAdapterAccess().getGroup()); }
		(rule__AsynchronousAdapter__Group__0)
		{ after(grammarAccess.getAsynchronousAdapterAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleClockDeclaration
entryRuleClockDeclaration
:
{ before(grammarAccess.getClockDeclarationRule()); }
	 ruleClockDeclaration
{ after(grammarAccess.getClockDeclarationRule()); } 
	 EOF 
;

// Rule ClockDeclaration
ruleClockDeclaration 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getClockDeclarationAccess().getGroup()); }
		(rule__ClockDeclaration__Group__0)
		{ after(grammarAccess.getClockDeclarationAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleTimeSpecification
entryRuleTimeSpecification
:
{ before(grammarAccess.getTimeSpecificationRule()); }
	 ruleTimeSpecification
{ after(grammarAccess.getTimeSpecificationRule()); } 
	 EOF 
;

// Rule TimeSpecification
ruleTimeSpecification 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getTimeSpecificationAccess().getGroup()); }
		(rule__TimeSpecification__Group__0)
		{ after(grammarAccess.getTimeSpecificationAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleControlSpecificaiton
entryRuleControlSpecificaiton
:
{ before(grammarAccess.getControlSpecificaitonRule()); }
	 ruleControlSpecificaiton
{ after(grammarAccess.getControlSpecificaitonRule()); } 
	 EOF 
;

// Rule ControlSpecificaiton
ruleControlSpecificaiton 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getControlSpecificaitonAccess().getGroup()); }
		(rule__ControlSpecificaiton__Group__0)
		{ after(grammarAccess.getControlSpecificaitonAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleMessageQueue
entryRuleMessageQueue
:
{ before(grammarAccess.getMessageQueueRule()); }
	 ruleMessageQueue
{ after(grammarAccess.getMessageQueueRule()); } 
	 EOF 
;

// Rule MessageQueue
ruleMessageQueue 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getMessageQueueAccess().getGroup()); }
		(rule__MessageQueue__Group__0)
		{ after(grammarAccess.getMessageQueueAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleEventPassing
entryRuleEventPassing
:
{ before(grammarAccess.getEventPassingRule()); }
	 ruleEventPassing
{ after(grammarAccess.getEventPassingRule()); } 
	 EOF 
;

// Rule EventPassing
ruleEventPassing 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getEventPassingAccess().getGroup()); }
		(rule__EventPassing__Group__0)
		{ after(grammarAccess.getEventPassingAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRulePortBinding
entryRulePortBinding
:
{ before(grammarAccess.getPortBindingRule()); }
	 rulePortBinding
{ after(grammarAccess.getPortBindingRule()); } 
	 EOF 
;

// Rule PortBinding
rulePortBinding 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getPortBindingAccess().getGroup()); }
		(rule__PortBinding__Group__0)
		{ after(grammarAccess.getPortBindingAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleInstancePortReference
entryRuleInstancePortReference
:
{ before(grammarAccess.getInstancePortReferenceRule()); }
	 ruleInstancePortReference
{ after(grammarAccess.getInstancePortReferenceRule()); } 
	 EOF 
;

// Rule InstancePortReference
ruleInstancePortReference 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getInstancePortReferenceAccess().getGroup()); }
		(rule__InstancePortReference__Group__0)
		{ after(grammarAccess.getInstancePortReferenceAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleComponentInstance
entryRuleComponentInstance
:
{ before(grammarAccess.getComponentInstanceRule()); }
	 ruleComponentInstance
{ after(grammarAccess.getComponentInstanceRule()); } 
	 EOF 
;

// Rule ComponentInstance
ruleComponentInstance 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getComponentInstanceAccess().getAlternatives()); }
		(rule__ComponentInstance__Alternatives)
		{ after(grammarAccess.getComponentInstanceAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleAtomicComponentInstanceReferenceExpression
entryRuleAtomicComponentInstanceReferenceExpression
:
{ before(grammarAccess.getAtomicComponentInstanceReferenceExpressionRule()); }
	 ruleAtomicComponentInstanceReferenceExpression
{ after(grammarAccess.getAtomicComponentInstanceReferenceExpressionRule()); } 
	 EOF 
;

// Rule AtomicComponentInstanceReferenceExpression
ruleAtomicComponentInstanceReferenceExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getAtomicComponentInstanceReferenceExpressionAccess().getComponentInstanceAssignment()); }
		(rule__AtomicComponentInstanceReferenceExpression__ComponentInstanceAssignment)
		{ after(grammarAccess.getAtomicComponentInstanceReferenceExpressionAccess().getComponentInstanceAssignment()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleComponentInstanceReferenceExpression
entryRuleComponentInstanceReferenceExpression
:
{ before(grammarAccess.getComponentInstanceReferenceExpressionRule()); }
	 ruleComponentInstanceReferenceExpression
{ after(grammarAccess.getComponentInstanceReferenceExpressionRule()); } 
	 EOF 
;

// Rule ComponentInstanceReferenceExpression
ruleComponentInstanceReferenceExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getComponentInstanceReferenceExpressionAccess().getGroup()); }
		(rule__ComponentInstanceReferenceExpression__Group__0)
		{ after(grammarAccess.getComponentInstanceReferenceExpressionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleSynchronousComponentInstance
entryRuleSynchronousComponentInstance
:
{ before(grammarAccess.getSynchronousComponentInstanceRule()); }
	 ruleSynchronousComponentInstance
{ after(grammarAccess.getSynchronousComponentInstanceRule()); } 
	 EOF 
;

// Rule SynchronousComponentInstance
ruleSynchronousComponentInstance 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getSynchronousComponentInstanceAccess().getGroup()); }
		(rule__SynchronousComponentInstance__Group__0)
		{ after(grammarAccess.getSynchronousComponentInstanceAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleAsynchronousComponentInstance
entryRuleAsynchronousComponentInstance
:
{ before(grammarAccess.getAsynchronousComponentInstanceRule()); }
	 ruleAsynchronousComponentInstance
{ after(grammarAccess.getAsynchronousComponentInstanceRule()); } 
	 EOF 
;

// Rule AsynchronousComponentInstance
ruleAsynchronousComponentInstance 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getAsynchronousComponentInstanceAccess().getGroup()); }
		(rule__AsynchronousComponentInstance__Group__0)
		{ after(grammarAccess.getAsynchronousComponentInstanceAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleChannel
entryRuleChannel
:
{ before(grammarAccess.getChannelRule()); }
	 ruleChannel
{ after(grammarAccess.getChannelRule()); } 
	 EOF 
;

// Rule Channel
ruleChannel 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getChannelAccess().getAlternatives()); }
		(rule__Channel__Alternatives)
		{ after(grammarAccess.getChannelAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleSimpleChannel
entryRuleSimpleChannel
:
{ before(grammarAccess.getSimpleChannelRule()); }
	 ruleSimpleChannel
{ after(grammarAccess.getSimpleChannelRule()); } 
	 EOF 
;

// Rule SimpleChannel
ruleSimpleChannel 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getSimpleChannelAccess().getGroup()); }
		(rule__SimpleChannel__Group__0)
		{ after(grammarAccess.getSimpleChannelAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleBroadcastChannel
entryRuleBroadcastChannel
:
{ before(grammarAccess.getBroadcastChannelRule()); }
	 ruleBroadcastChannel
{ after(grammarAccess.getBroadcastChannelRule()); } 
	 EOF 
;

// Rule BroadcastChannel
ruleBroadcastChannel 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getBroadcastChannelAccess().getGroup()); }
		(rule__BroadcastChannel__Group__0)
		{ after(grammarAccess.getBroadcastChannelAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleInterface
entryRuleInterface
:
{ before(grammarAccess.getInterfaceRule()); }
	 ruleInterface
{ after(grammarAccess.getInterfaceRule()); } 
	 EOF 
;

// Rule Interface
ruleInterface 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getInterfaceAccess().getGroup()); }
		(rule__Interface__Group__0)
		{ after(grammarAccess.getInterfaceAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleEventDeclaration
entryRuleEventDeclaration
:
{ before(grammarAccess.getEventDeclarationRule()); }
	 ruleEventDeclaration
{ after(grammarAccess.getEventDeclarationRule()); } 
	 EOF 
;

// Rule EventDeclaration
ruleEventDeclaration 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getEventDeclarationAccess().getGroup()); }
		(rule__EventDeclaration__Group__0)
		{ after(grammarAccess.getEventDeclarationAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleEvent
entryRuleEvent
:
{ before(grammarAccess.getEventRule()); }
	 ruleEvent
{ after(grammarAccess.getEventRule()); } 
	 EOF 
;

// Rule Event
ruleEvent 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getEventAccess().getGroup()); }
		(rule__Event__Group__0)
		{ after(grammarAccess.getEventAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleInlineStatement
entryRuleInlineStatement
:
{ before(grammarAccess.getInlineStatementRule()); }
	 ruleInlineStatement
{ after(grammarAccess.getInlineStatementRule()); } 
	 EOF 
;

// Rule InlineStatement
ruleInlineStatement 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getInlineStatementAccess().getAlternatives()); }
		(rule__InlineStatement__Alternatives)
		{ after(grammarAccess.getInlineStatementAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleRaiseEventAction
entryRuleRaiseEventAction
:
{ before(grammarAccess.getRaiseEventActionRule()); }
	 ruleRaiseEventAction
{ after(grammarAccess.getRaiseEventActionRule()); } 
	 EOF 
;

// Rule RaiseEventAction
ruleRaiseEventAction 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getRaiseEventActionAccess().getGroup()); }
		(rule__RaiseEventAction__Group__0)
		{ after(grammarAccess.getRaiseEventActionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleSetTimeoutAction
entryRuleSetTimeoutAction
:
{ before(grammarAccess.getSetTimeoutActionRule()); }
	 ruleSetTimeoutAction
{ after(grammarAccess.getSetTimeoutActionRule()); } 
	 EOF 
;

// Rule SetTimeoutAction
ruleSetTimeoutAction 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getSetTimeoutActionAccess().getGroup()); }
		(rule__SetTimeoutAction__Group__0)
		{ after(grammarAccess.getSetTimeoutActionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleComponentInstanceStateExpression
entryRuleComponentInstanceStateExpression
:
{ before(grammarAccess.getComponentInstanceStateExpressionRule()); }
	 ruleComponentInstanceStateExpression
{ after(grammarAccess.getComponentInstanceStateExpressionRule()); } 
	 EOF 
;

// Rule ComponentInstanceStateExpression
ruleComponentInstanceStateExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getComponentInstanceStateExpressionAccess().getAlternatives()); }
		(rule__ComponentInstanceStateExpression__Alternatives)
		{ after(grammarAccess.getComponentInstanceStateExpressionAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleComponentInstanceStateReferenceExpression
entryRuleComponentInstanceStateReferenceExpression
:
{ before(grammarAccess.getComponentInstanceStateReferenceExpressionRule()); }
	 ruleComponentInstanceStateReferenceExpression
{ after(grammarAccess.getComponentInstanceStateReferenceExpressionRule()); } 
	 EOF 
;

// Rule ComponentInstanceStateReferenceExpression
ruleComponentInstanceStateReferenceExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getComponentInstanceStateReferenceExpressionAccess().getGroup()); }
		(rule__ComponentInstanceStateReferenceExpression__Group__0)
		{ after(grammarAccess.getComponentInstanceStateReferenceExpressionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleComponentInstanceVariableReferenceExpression
entryRuleComponentInstanceVariableReferenceExpression
:
{ before(grammarAccess.getComponentInstanceVariableReferenceExpressionRule()); }
	 ruleComponentInstanceVariableReferenceExpression
{ after(grammarAccess.getComponentInstanceVariableReferenceExpressionRule()); } 
	 EOF 
;

// Rule ComponentInstanceVariableReferenceExpression
ruleComponentInstanceVariableReferenceExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getComponentInstanceVariableReferenceExpressionAccess().getGroup()); }
		(rule__ComponentInstanceVariableReferenceExpression__Group__0)
		{ after(grammarAccess.getComponentInstanceVariableReferenceExpressionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleComponentInstanceEventReferenceExpression
entryRuleComponentInstanceEventReferenceExpression
:
{ before(grammarAccess.getComponentInstanceEventReferenceExpressionRule()); }
	 ruleComponentInstanceEventReferenceExpression
{ after(grammarAccess.getComponentInstanceEventReferenceExpressionRule()); } 
	 EOF 
;

// Rule ComponentInstanceEventReferenceExpression
ruleComponentInstanceEventReferenceExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getComponentInstanceEventReferenceExpressionAccess().getGroup()); }
		(rule__ComponentInstanceEventReferenceExpression__Group__0)
		{ after(grammarAccess.getComponentInstanceEventReferenceExpressionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleComponentInstanceEventParameterReferenceExpression
entryRuleComponentInstanceEventParameterReferenceExpression
:
{ before(grammarAccess.getComponentInstanceEventParameterReferenceExpressionRule()); }
	 ruleComponentInstanceEventParameterReferenceExpression
{ after(grammarAccess.getComponentInstanceEventParameterReferenceExpressionRule()); } 
	 EOF 
;

// Rule ComponentInstanceEventParameterReferenceExpression
ruleComponentInstanceEventParameterReferenceExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getComponentInstanceEventParameterReferenceExpressionAccess().getGroup()); }
		(rule__ComponentInstanceEventParameterReferenceExpression__Group__0)
		{ after(grammarAccess.getComponentInstanceEventParameterReferenceExpressionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleAction
entryRuleAction
:
{ before(grammarAccess.getActionRule()); }
	 ruleAction
{ after(grammarAccess.getActionRule()); } 
	 EOF 
;

// Rule Action
ruleAction 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getActionAccess().getAlternatives()); }
		(rule__Action__Alternatives)
		{ after(grammarAccess.getActionAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleFunctionDeclaration
entryRuleFunctionDeclaration
:
{ before(grammarAccess.getFunctionDeclarationRule()); }
	 ruleFunctionDeclaration
{ after(grammarAccess.getFunctionDeclarationRule()); } 
	 EOF 
;

// Rule FunctionDeclaration
ruleFunctionDeclaration 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getFunctionDeclarationAccess().getAlternatives()); }
		(rule__FunctionDeclaration__Alternatives)
		{ after(grammarAccess.getFunctionDeclarationAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleProcedureDeclaration
entryRuleProcedureDeclaration
:
{ before(grammarAccess.getProcedureDeclarationRule()); }
	 ruleProcedureDeclaration
{ after(grammarAccess.getProcedureDeclarationRule()); } 
	 EOF 
;

// Rule ProcedureDeclaration
ruleProcedureDeclaration 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getProcedureDeclarationAccess().getGroup()); }
		(rule__ProcedureDeclaration__Group__0)
		{ after(grammarAccess.getProcedureDeclarationAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleBlock
entryRuleBlock
:
{ before(grammarAccess.getBlockRule()); }
	 ruleBlock
{ after(grammarAccess.getBlockRule()); } 
	 EOF 
;

// Rule Block
ruleBlock 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getBlockAccess().getGroup()); }
		(rule__Block__Group__0)
		{ after(grammarAccess.getBlockAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleStatement
entryRuleStatement
:
{ before(grammarAccess.getStatementRule()); }
	 ruleStatement
{ after(grammarAccess.getStatementRule()); } 
	 EOF 
;

// Rule Statement
ruleStatement 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getStatementAccess().getAlternatives()); }
		(rule__Statement__Alternatives)
		{ after(grammarAccess.getStatementAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entrySuperInlineStatement
entrySuperInlineStatement
:
{ before(grammarAccess.getActionLanguageInlineStatementRule()); }
	 superInlineStatement
{ after(grammarAccess.getActionLanguageInlineStatementRule()); } 
	 EOF 
;

// Rule InlineStatement
superInlineStatement 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getActionLanguageInlineStatementAccess().getGroup()); }
		(superInlineStatement__Group__0)
		{ after(grammarAccess.getActionLanguageInlineStatementAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleMultilineStatement
entryRuleMultilineStatement
:
{ before(grammarAccess.getMultilineStatementRule()); }
	 ruleMultilineStatement
{ after(grammarAccess.getMultilineStatementRule()); } 
	 EOF 
;

// Rule MultilineStatement
ruleMultilineStatement 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getMultilineStatementAccess().getAlternatives()); }
		(rule__MultilineStatement__Alternatives)
		{ after(grammarAccess.getMultilineStatementAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleVariableDeclarationStatement
entryRuleVariableDeclarationStatement
:
{ before(grammarAccess.getVariableDeclarationStatementRule()); }
	 ruleVariableDeclarationStatement
{ after(grammarAccess.getVariableDeclarationStatementRule()); } 
	 EOF 
;

// Rule VariableDeclarationStatement
ruleVariableDeclarationStatement 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getVariableDeclarationStatementAccess().getVariableDeclarationAssignment()); }
		(rule__VariableDeclarationStatement__VariableDeclarationAssignment)
		{ after(grammarAccess.getVariableDeclarationStatementAccess().getVariableDeclarationAssignment()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleConstantDeclarationStatement
entryRuleConstantDeclarationStatement
:
{ before(grammarAccess.getConstantDeclarationStatementRule()); }
	 ruleConstantDeclarationStatement
{ after(grammarAccess.getConstantDeclarationStatementRule()); } 
	 EOF 
;

// Rule ConstantDeclarationStatement
ruleConstantDeclarationStatement 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getConstantDeclarationStatementAccess().getConstantDeclarationAssignment()); }
		(rule__ConstantDeclarationStatement__ConstantDeclarationAssignment)
		{ after(grammarAccess.getConstantDeclarationStatementAccess().getConstantDeclarationAssignment()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleExpressionStatement
entryRuleExpressionStatement
:
{ before(grammarAccess.getExpressionStatementRule()); }
	 ruleExpressionStatement
{ after(grammarAccess.getExpressionStatementRule()); } 
	 EOF 
;

// Rule ExpressionStatement
ruleExpressionStatement 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getExpressionStatementAccess().getExpressionAssignment()); }
		(rule__ExpressionStatement__ExpressionAssignment)
		{ after(grammarAccess.getExpressionStatementAccess().getExpressionAssignment()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleEmptyStatement
entryRuleEmptyStatement
:
{ before(grammarAccess.getEmptyStatementRule()); }
	 ruleEmptyStatement
{ after(grammarAccess.getEmptyStatementRule()); } 
	 EOF 
;

// Rule EmptyStatement
ruleEmptyStatement 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getEmptyStatementAccess().getGroup()); }
		(rule__EmptyStatement__Group__0)
		{ after(grammarAccess.getEmptyStatementAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleIfBranch
entryRuleIfBranch
:
{ before(grammarAccess.getIfBranchRule()); }
	 ruleIfBranch
{ after(grammarAccess.getIfBranchRule()); } 
	 EOF 
;

// Rule IfBranch
ruleIfBranch 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getIfBranchAccess().getGroup()); }
		(rule__IfBranch__Group__0)
		{ after(grammarAccess.getIfBranchAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleElseBranch
entryRuleElseBranch
:
{ before(grammarAccess.getElseBranchRule()); }
	 ruleElseBranch
{ after(grammarAccess.getElseBranchRule()); } 
	 EOF 
;

// Rule ElseBranch
ruleElseBranch 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getElseBranchAccess().getGroup()); }
		(rule__ElseBranch__Group__0)
		{ after(grammarAccess.getElseBranchAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleSwitchBranch
entryRuleSwitchBranch
:
{ before(grammarAccess.getSwitchBranchRule()); }
	 ruleSwitchBranch
{ after(grammarAccess.getSwitchBranchRule()); } 
	 EOF 
;

// Rule SwitchBranch
ruleSwitchBranch 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getSwitchBranchAccess().getGroup()); }
		(rule__SwitchBranch__Group__0)
		{ after(grammarAccess.getSwitchBranchAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleDefaultBranch
entryRuleDefaultBranch
:
{ before(grammarAccess.getDefaultBranchRule()); }
	 ruleDefaultBranch
{ after(grammarAccess.getDefaultBranchRule()); } 
	 EOF 
;

// Rule DefaultBranch
ruleDefaultBranch 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getDefaultBranchAccess().getGroup()); }
		(rule__DefaultBranch__Group__0)
		{ after(grammarAccess.getDefaultBranchAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleChoiceBranch
entryRuleChoiceBranch
:
{ before(grammarAccess.getChoiceBranchRule()); }
	 ruleChoiceBranch
{ after(grammarAccess.getChoiceBranchRule()); } 
	 EOF 
;

// Rule ChoiceBranch
ruleChoiceBranch 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getChoiceBranchAccess().getGroup()); }
		(rule__ChoiceBranch__Group__0)
		{ after(grammarAccess.getChoiceBranchAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleDefaultExpression
entryRuleDefaultExpression
:
{ before(grammarAccess.getDefaultExpressionRule()); }
	 ruleDefaultExpression
{ after(grammarAccess.getDefaultExpressionRule()); } 
	 EOF 
;

// Rule DefaultExpression
ruleDefaultExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getDefaultExpressionAccess().getGroup()); }
		(rule__DefaultExpression__Group__0)
		{ after(grammarAccess.getDefaultExpressionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleIfStatement
entryRuleIfStatement
:
{ before(grammarAccess.getIfStatementRule()); }
	 ruleIfStatement
{ after(grammarAccess.getIfStatementRule()); } 
	 EOF 
;

// Rule IfStatement
ruleIfStatement 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getIfStatementAccess().getGroup()); }
		(rule__IfStatement__Group__0)
		{ after(grammarAccess.getIfStatementAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleSwitchStatement
entryRuleSwitchStatement
:
{ before(grammarAccess.getSwitchStatementRule()); }
	 ruleSwitchStatement
{ after(grammarAccess.getSwitchStatementRule()); } 
	 EOF 
;

// Rule SwitchStatement
ruleSwitchStatement 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getSwitchStatementAccess().getGroup()); }
		(rule__SwitchStatement__Group__0)
		{ after(grammarAccess.getSwitchStatementAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleForStatement
entryRuleForStatement
:
{ before(grammarAccess.getForStatementRule()); }
	 ruleForStatement
{ after(grammarAccess.getForStatementRule()); } 
	 EOF 
;

// Rule ForStatement
ruleForStatement 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getForStatementAccess().getGroup()); }
		(rule__ForStatement__Group__0)
		{ after(grammarAccess.getForStatementAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleReturnStatement
entryRuleReturnStatement
:
{ before(grammarAccess.getReturnStatementRule()); }
	 ruleReturnStatement
{ after(grammarAccess.getReturnStatementRule()); } 
	 EOF 
;

// Rule ReturnStatement
ruleReturnStatement 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getReturnStatementAccess().getGroup()); }
		(rule__ReturnStatement__Group__0)
		{ after(grammarAccess.getReturnStatementAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleBreakStatement
entryRuleBreakStatement
:
{ before(grammarAccess.getBreakStatementRule()); }
	 ruleBreakStatement
{ after(grammarAccess.getBreakStatementRule()); } 
	 EOF 
;

// Rule BreakStatement
ruleBreakStatement 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getBreakStatementAccess().getGroup()); }
		(rule__BreakStatement__Group__0)
		{ after(grammarAccess.getBreakStatementAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleChoiceStatement
entryRuleChoiceStatement
:
{ before(grammarAccess.getChoiceStatementRule()); }
	 ruleChoiceStatement
{ after(grammarAccess.getChoiceStatementRule()); } 
	 EOF 
;

// Rule ChoiceStatement
ruleChoiceStatement 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getChoiceStatementAccess().getGroup()); }
		(rule__ChoiceStatement__Group__0)
		{ after(grammarAccess.getChoiceStatementAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleAssignmentStatement
entryRuleAssignmentStatement
:
{ before(grammarAccess.getAssignmentStatementRule()); }
	 ruleAssignmentStatement
{ after(grammarAccess.getAssignmentStatementRule()); } 
	 EOF 
;

// Rule AssignmentStatement
ruleAssignmentStatement 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getAssignmentStatementAccess().getGroup()); }
		(rule__AssignmentStatement__Group__0)
		{ after(grammarAccess.getAssignmentStatementAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleAssertionStatement
entryRuleAssertionStatement
:
{ before(grammarAccess.getAssertionStatementRule()); }
	 ruleAssertionStatement
{ after(grammarAccess.getAssertionStatementRule()); } 
	 EOF 
;

// Rule AssertionStatement
ruleAssertionStatement 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getAssertionStatementAccess().getGroup()); }
		(rule__AssertionStatement__Group__0)
		{ after(grammarAccess.getAssertionStatementAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleAssignableAccessExpression
entryRuleAssignableAccessExpression
:
{ before(grammarAccess.getAssignableAccessExpressionRule()); }
	 ruleAssignableAccessExpression
{ after(grammarAccess.getAssignableAccessExpressionRule()); } 
	 EOF 
;

// Rule AssignableAccessExpression
ruleAssignableAccessExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getAssignableAccessExpressionAccess().getGroup()); }
		(rule__AssignableAccessExpression__Group__0)
		{ after(grammarAccess.getAssignableAccessExpressionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleAssignablePrimaryExpression
entryRuleAssignablePrimaryExpression
:
{ before(grammarAccess.getAssignablePrimaryExpressionRule()); }
	 ruleAssignablePrimaryExpression
{ after(grammarAccess.getAssignablePrimaryExpressionRule()); } 
	 EOF 
;

// Rule AssignablePrimaryExpression
ruleAssignablePrimaryExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getAssignablePrimaryExpressionAccess().getAlternatives()); }
		(rule__AssignablePrimaryExpression__Alternatives)
		{ after(grammarAccess.getAssignablePrimaryExpressionAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleAssignableDirectReferenceExpression
entryRuleAssignableDirectReferenceExpression
:
{ before(grammarAccess.getAssignableDirectReferenceExpressionRule()); }
	 ruleAssignableDirectReferenceExpression
{ after(grammarAccess.getAssignableDirectReferenceExpressionRule()); } 
	 EOF 
;

// Rule AssignableDirectReferenceExpression
ruleAssignableDirectReferenceExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getAssignableDirectReferenceExpressionAccess().getGroup()); }
		(rule__AssignableDirectReferenceExpression__Group__0)
		{ after(grammarAccess.getAssignableDirectReferenceExpressionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleParameterDeclaration
entryRuleParameterDeclaration
:
{ before(grammarAccess.getParameterDeclarationRule()); }
	 ruleParameterDeclaration
{ after(grammarAccess.getParameterDeclarationRule()); } 
	 EOF 
;

// Rule ParameterDeclaration
ruleParameterDeclaration 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getParameterDeclarationAccess().getGroup()); }
		(rule__ParameterDeclaration__Group__0)
		{ after(grammarAccess.getParameterDeclarationAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleFieldDeclaration
entryRuleFieldDeclaration
:
{ before(grammarAccess.getFieldDeclarationRule()); }
	 ruleFieldDeclaration
{ after(grammarAccess.getFieldDeclarationRule()); } 
	 EOF 
;

// Rule FieldDeclaration
ruleFieldDeclaration 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getFieldDeclarationAccess().getGroup()); }
		(rule__FieldDeclaration__Group__0)
		{ after(grammarAccess.getFieldDeclarationAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleConstantDeclaration
entryRuleConstantDeclaration
:
{ before(grammarAccess.getConstantDeclarationRule()); }
	 ruleConstantDeclaration
{ after(grammarAccess.getConstantDeclarationRule()); } 
	 EOF 
;

// Rule ConstantDeclaration
ruleConstantDeclaration 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getConstantDeclarationAccess().getGroup()); }
		(rule__ConstantDeclaration__Group__0)
		{ after(grammarAccess.getConstantDeclarationAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleVariableDeclaration
entryRuleVariableDeclaration
:
{ before(grammarAccess.getVariableDeclarationRule()); }
	 ruleVariableDeclaration
{ after(grammarAccess.getVariableDeclarationRule()); } 
	 EOF 
;

// Rule VariableDeclaration
ruleVariableDeclaration 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getVariableDeclarationAccess().getGroup()); }
		(rule__VariableDeclaration__Group__0)
		{ after(grammarAccess.getVariableDeclarationAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleVariableDeclarationAnnotation
entryRuleVariableDeclarationAnnotation
:
{ before(grammarAccess.getVariableDeclarationAnnotationRule()); }
	 ruleVariableDeclarationAnnotation
{ after(grammarAccess.getVariableDeclarationAnnotationRule()); } 
	 EOF 
;

// Rule VariableDeclarationAnnotation
ruleVariableDeclarationAnnotation 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getVariableDeclarationAnnotationAccess().getAlternatives()); }
		(rule__VariableDeclarationAnnotation__Alternatives)
		{ after(grammarAccess.getVariableDeclarationAnnotationAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleTransientVariableDeclarationAnnotation
entryRuleTransientVariableDeclarationAnnotation
:
{ before(grammarAccess.getTransientVariableDeclarationAnnotationRule()); }
	 ruleTransientVariableDeclarationAnnotation
{ after(grammarAccess.getTransientVariableDeclarationAnnotationRule()); } 
	 EOF 
;

// Rule TransientVariableDeclarationAnnotation
ruleTransientVariableDeclarationAnnotation 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getTransientVariableDeclarationAnnotationAccess().getGroup()); }
		(rule__TransientVariableDeclarationAnnotation__Group__0)
		{ after(grammarAccess.getTransientVariableDeclarationAnnotationAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleResettableVariableDeclarationAnnotation
entryRuleResettableVariableDeclarationAnnotation
:
{ before(grammarAccess.getResettableVariableDeclarationAnnotationRule()); }
	 ruleResettableVariableDeclarationAnnotation
{ after(grammarAccess.getResettableVariableDeclarationAnnotationRule()); } 
	 EOF 
;

// Rule ResettableVariableDeclarationAnnotation
ruleResettableVariableDeclarationAnnotation 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getResettableVariableDeclarationAnnotationAccess().getGroup()); }
		(rule__ResettableVariableDeclarationAnnotation__Group__0)
		{ after(grammarAccess.getResettableVariableDeclarationAnnotationAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleUnremovableVariableDeclarationAnnotation
entryRuleUnremovableVariableDeclarationAnnotation
:
{ before(grammarAccess.getUnremovableVariableDeclarationAnnotationRule()); }
	 ruleUnremovableVariableDeclarationAnnotation
{ after(grammarAccess.getUnremovableVariableDeclarationAnnotationRule()); } 
	 EOF 
;

// Rule UnremovableVariableDeclarationAnnotation
ruleUnremovableVariableDeclarationAnnotation 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getUnremovableVariableDeclarationAnnotationAccess().getGroup()); }
		(rule__UnremovableVariableDeclarationAnnotation__Group__0)
		{ after(grammarAccess.getUnremovableVariableDeclarationAnnotationAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleInjectedVariableDeclarationAnnotation
entryRuleInjectedVariableDeclarationAnnotation
:
{ before(grammarAccess.getInjectedVariableDeclarationAnnotationRule()); }
	 ruleInjectedVariableDeclarationAnnotation
{ after(grammarAccess.getInjectedVariableDeclarationAnnotationRule()); } 
	 EOF 
;

// Rule InjectedVariableDeclarationAnnotation
ruleInjectedVariableDeclarationAnnotation 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getInjectedVariableDeclarationAnnotationAccess().getGroup()); }
		(rule__InjectedVariableDeclarationAnnotation__Group__0)
		{ after(grammarAccess.getInjectedVariableDeclarationAnnotationAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleLambdaDeclaration
entryRuleLambdaDeclaration
:
{ before(grammarAccess.getLambdaDeclarationRule()); }
	 ruleLambdaDeclaration
{ after(grammarAccess.getLambdaDeclarationRule()); } 
	 EOF 
;

// Rule LambdaDeclaration
ruleLambdaDeclaration 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getLambdaDeclarationAccess().getGroup()); }
		(rule__LambdaDeclaration__Group__0)
		{ after(grammarAccess.getLambdaDeclarationAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleType
entryRuleType
:
{ before(grammarAccess.getTypeRule()); }
	 ruleType
{ after(grammarAccess.getTypeRule()); } 
	 EOF 
;

// Rule Type
ruleType 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getTypeAccess().getAlternatives()); }
		(rule__Type__Alternatives)
		{ after(grammarAccess.getTypeAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleTypeDeclaration
entryRuleTypeDeclaration
:
{ before(grammarAccess.getTypeDeclarationRule()); }
	 ruleTypeDeclaration
{ after(grammarAccess.getTypeDeclarationRule()); } 
	 EOF 
;

// Rule TypeDeclaration
ruleTypeDeclaration 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getTypeDeclarationAccess().getGroup()); }
		(rule__TypeDeclaration__Group__0)
		{ after(grammarAccess.getTypeDeclarationAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleTypeReference
entryRuleTypeReference
:
{ before(grammarAccess.getTypeReferenceRule()); }
	 ruleTypeReference
{ after(grammarAccess.getTypeReferenceRule()); } 
	 EOF 
;

// Rule TypeReference
ruleTypeReference 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getTypeReferenceAccess().getReferenceAssignment()); }
		(rule__TypeReference__ReferenceAssignment)
		{ after(grammarAccess.getTypeReferenceAccess().getReferenceAssignment()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleTypeDefinition
entryRuleTypeDefinition
:
{ before(grammarAccess.getTypeDefinitionRule()); }
	 ruleTypeDefinition
{ after(grammarAccess.getTypeDefinitionRule()); } 
	 EOF 
;

// Rule TypeDefinition
ruleTypeDefinition 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getTypeDefinitionAccess().getAlternatives()); }
		(rule__TypeDefinition__Alternatives)
		{ after(grammarAccess.getTypeDefinitionAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleVoidTypeDefinition
entryRuleVoidTypeDefinition
:
{ before(grammarAccess.getVoidTypeDefinitionRule()); }
	 ruleVoidTypeDefinition
{ after(grammarAccess.getVoidTypeDefinitionRule()); } 
	 EOF 
;

// Rule VoidTypeDefinition
ruleVoidTypeDefinition 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getVoidTypeDefinitionAccess().getGroup()); }
		(rule__VoidTypeDefinition__Group__0)
		{ after(grammarAccess.getVoidTypeDefinitionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleIntegerTypeDefinition
entryRuleIntegerTypeDefinition
:
{ before(grammarAccess.getIntegerTypeDefinitionRule()); }
	 ruleIntegerTypeDefinition
{ after(grammarAccess.getIntegerTypeDefinitionRule()); } 
	 EOF 
;

// Rule IntegerTypeDefinition
ruleIntegerTypeDefinition 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getIntegerTypeDefinitionAccess().getGroup()); }
		(rule__IntegerTypeDefinition__Group__0)
		{ after(grammarAccess.getIntegerTypeDefinitionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleBooleanTypeDefinition
entryRuleBooleanTypeDefinition
:
{ before(grammarAccess.getBooleanTypeDefinitionRule()); }
	 ruleBooleanTypeDefinition
{ after(grammarAccess.getBooleanTypeDefinitionRule()); } 
	 EOF 
;

// Rule BooleanTypeDefinition
ruleBooleanTypeDefinition 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getBooleanTypeDefinitionAccess().getGroup()); }
		(rule__BooleanTypeDefinition__Group__0)
		{ after(grammarAccess.getBooleanTypeDefinitionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleDecimalTypeDefinition
entryRuleDecimalTypeDefinition
:
{ before(grammarAccess.getDecimalTypeDefinitionRule()); }
	 ruleDecimalTypeDefinition
{ after(grammarAccess.getDecimalTypeDefinitionRule()); } 
	 EOF 
;

// Rule DecimalTypeDefinition
ruleDecimalTypeDefinition 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getDecimalTypeDefinitionAccess().getGroup()); }
		(rule__DecimalTypeDefinition__Group__0)
		{ after(grammarAccess.getDecimalTypeDefinitionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleRationalTypeDefinition
entryRuleRationalTypeDefinition
:
{ before(grammarAccess.getRationalTypeDefinitionRule()); }
	 ruleRationalTypeDefinition
{ after(grammarAccess.getRationalTypeDefinitionRule()); } 
	 EOF 
;

// Rule RationalTypeDefinition
ruleRationalTypeDefinition 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getRationalTypeDefinitionAccess().getGroup()); }
		(rule__RationalTypeDefinition__Group__0)
		{ after(grammarAccess.getRationalTypeDefinitionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleArrayTypeDefinition
entryRuleArrayTypeDefinition
:
{ before(grammarAccess.getArrayTypeDefinitionRule()); }
	 ruleArrayTypeDefinition
{ after(grammarAccess.getArrayTypeDefinitionRule()); } 
	 EOF 
;

// Rule ArrayTypeDefinition
ruleArrayTypeDefinition 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getArrayTypeDefinitionAccess().getGroup()); }
		(rule__ArrayTypeDefinition__Group__0)
		{ after(grammarAccess.getArrayTypeDefinitionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleIntegerRangeTypeDefinition
entryRuleIntegerRangeTypeDefinition
:
{ before(grammarAccess.getIntegerRangeTypeDefinitionRule()); }
	 ruleIntegerRangeTypeDefinition
{ after(grammarAccess.getIntegerRangeTypeDefinitionRule()); } 
	 EOF 
;

// Rule IntegerRangeTypeDefinition
ruleIntegerRangeTypeDefinition 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getIntegerRangeTypeDefinitionAccess().getGroup()); }
		(rule__IntegerRangeTypeDefinition__Group__0)
		{ after(grammarAccess.getIntegerRangeTypeDefinitionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleEnumerationTypeDefinition
entryRuleEnumerationTypeDefinition
:
{ before(grammarAccess.getEnumerationTypeDefinitionRule()); }
	 ruleEnumerationTypeDefinition
{ after(grammarAccess.getEnumerationTypeDefinitionRule()); } 
	 EOF 
;

// Rule EnumerationTypeDefinition
ruleEnumerationTypeDefinition 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getEnumerationTypeDefinitionAccess().getGroup()); }
		(rule__EnumerationTypeDefinition__Group__0)
		{ after(grammarAccess.getEnumerationTypeDefinitionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleEnumerationLiteralDefinition
entryRuleEnumerationLiteralDefinition
:
{ before(grammarAccess.getEnumerationLiteralDefinitionRule()); }
	 ruleEnumerationLiteralDefinition
{ after(grammarAccess.getEnumerationLiteralDefinitionRule()); } 
	 EOF 
;

// Rule EnumerationLiteralDefinition
ruleEnumerationLiteralDefinition 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getEnumerationLiteralDefinitionAccess().getNameAssignment()); }
		(rule__EnumerationLiteralDefinition__NameAssignment)
		{ after(grammarAccess.getEnumerationLiteralDefinitionAccess().getNameAssignment()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleRecordTypeDefinition
entryRuleRecordTypeDefinition
:
{ before(grammarAccess.getRecordTypeDefinitionRule()); }
	 ruleRecordTypeDefinition
{ after(grammarAccess.getRecordTypeDefinitionRule()); } 
	 EOF 
;

// Rule RecordTypeDefinition
ruleRecordTypeDefinition 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getRecordTypeDefinitionAccess().getGroup()); }
		(rule__RecordTypeDefinition__Group__0)
		{ after(grammarAccess.getRecordTypeDefinitionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleSubrangeTypeDefinition
entryRuleSubrangeTypeDefinition
:
{ before(grammarAccess.getSubrangeTypeDefinitionRule()); }
	 ruleSubrangeTypeDefinition
{ after(grammarAccess.getSubrangeTypeDefinitionRule()); } 
	 EOF 
;

// Rule SubrangeTypeDefinition
ruleSubrangeTypeDefinition 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getSubrangeTypeDefinitionAccess().getGroup()); }
		(rule__SubrangeTypeDefinition__Group__0)
		{ after(grammarAccess.getSubrangeTypeDefinitionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleFunctionTypeDefinition
entryRuleFunctionTypeDefinition
:
{ before(grammarAccess.getFunctionTypeDefinitionRule()); }
	 ruleFunctionTypeDefinition
{ after(grammarAccess.getFunctionTypeDefinitionRule()); } 
	 EOF 
;

// Rule FunctionTypeDefinition
ruleFunctionTypeDefinition 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getFunctionTypeDefinitionAccess().getGroup()); }
		(rule__FunctionTypeDefinition__Group__0)
		{ after(grammarAccess.getFunctionTypeDefinitionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleExpression
entryRuleExpression
:
{ before(grammarAccess.getExpressionRule()); }
	 ruleExpression
{ after(grammarAccess.getExpressionRule()); } 
	 EOF 
;

// Rule Expression
ruleExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getExpressionAccess().getAlternatives()); }
		(rule__Expression__Alternatives)
		{ after(grammarAccess.getExpressionAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleElseExpression
entryRuleElseExpression
:
{ before(grammarAccess.getElseExpressionRule()); }
	 ruleElseExpression
{ after(grammarAccess.getElseExpressionRule()); } 
	 EOF 
;

// Rule ElseExpression
ruleElseExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getElseExpressionAccess().getGroup()); }
		(rule__ElseExpression__Group__0)
		{ after(grammarAccess.getElseExpressionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleIfThenElseExpression
entryRuleIfThenElseExpression
:
{ before(grammarAccess.getIfThenElseExpressionRule()); }
	 ruleIfThenElseExpression
{ after(grammarAccess.getIfThenElseExpressionRule()); } 
	 EOF 
;

// Rule IfThenElseExpression
ruleIfThenElseExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getIfThenElseExpressionAccess().getGroup()); }
		(rule__IfThenElseExpression__Group__0)
		{ after(grammarAccess.getIfThenElseExpressionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleImplyExpression
entryRuleImplyExpression
:
{ before(grammarAccess.getImplyExpressionRule()); }
	 ruleImplyExpression
{ after(grammarAccess.getImplyExpressionRule()); } 
	 EOF 
;

// Rule ImplyExpression
ruleImplyExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getImplyExpressionAccess().getGroup()); }
		(rule__ImplyExpression__Group__0)
		{ after(grammarAccess.getImplyExpressionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleOrExpression
entryRuleOrExpression
:
{ before(grammarAccess.getOrExpressionRule()); }
	 ruleOrExpression
{ after(grammarAccess.getOrExpressionRule()); } 
	 EOF 
;

// Rule OrExpression
ruleOrExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getOrExpressionAccess().getGroup()); }
		(rule__OrExpression__Group__0)
		{ after(grammarAccess.getOrExpressionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleXorExpression
entryRuleXorExpression
:
{ before(grammarAccess.getXorExpressionRule()); }
	 ruleXorExpression
{ after(grammarAccess.getXorExpressionRule()); } 
	 EOF 
;

// Rule XorExpression
ruleXorExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getXorExpressionAccess().getGroup()); }
		(rule__XorExpression__Group__0)
		{ after(grammarAccess.getXorExpressionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleAndExpression
entryRuleAndExpression
:
{ before(grammarAccess.getAndExpressionRule()); }
	 ruleAndExpression
{ after(grammarAccess.getAndExpressionRule()); } 
	 EOF 
;

// Rule AndExpression
ruleAndExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getAndExpressionAccess().getGroup()); }
		(rule__AndExpression__Group__0)
		{ after(grammarAccess.getAndExpressionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleUnaryLogicExpression
entryRuleUnaryLogicExpression
:
{ before(grammarAccess.getUnaryLogicExpressionRule()); }
	 ruleUnaryLogicExpression
{ after(grammarAccess.getUnaryLogicExpressionRule()); } 
	 EOF 
;

// Rule UnaryLogicExpression
ruleUnaryLogicExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getUnaryLogicExpressionAccess().getAlternatives()); }
		(rule__UnaryLogicExpression__Alternatives)
		{ after(grammarAccess.getUnaryLogicExpressionAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleNotExpression
entryRuleNotExpression
:
{ before(grammarAccess.getNotExpressionRule()); }
	 ruleNotExpression
{ after(grammarAccess.getNotExpressionRule()); } 
	 EOF 
;

// Rule NotExpression
ruleNotExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getNotExpressionAccess().getGroup()); }
		(rule__NotExpression__Group__0)
		{ after(grammarAccess.getNotExpressionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleForallExpression
entryRuleForallExpression
:
{ before(grammarAccess.getForallExpressionRule()); }
	 ruleForallExpression
{ after(grammarAccess.getForallExpressionRule()); } 
	 EOF 
;

// Rule ForallExpression
ruleForallExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getForallExpressionAccess().getGroup()); }
		(rule__ForallExpression__Group__0)
		{ after(grammarAccess.getForallExpressionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleExistsExpression
entryRuleExistsExpression
:
{ before(grammarAccess.getExistsExpressionRule()); }
	 ruleExistsExpression
{ after(grammarAccess.getExistsExpressionRule()); } 
	 EOF 
;

// Rule ExistsExpression
ruleExistsExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getExistsExpressionAccess().getGroup()); }
		(rule__ExistsExpression__Group__0)
		{ after(grammarAccess.getExistsExpressionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleEquivalenceExpression
entryRuleEquivalenceExpression
:
{ before(grammarAccess.getEquivalenceExpressionRule()); }
	 ruleEquivalenceExpression
{ after(grammarAccess.getEquivalenceExpressionRule()); } 
	 EOF 
;

// Rule EquivalenceExpression
ruleEquivalenceExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getEquivalenceExpressionAccess().getGroup()); }
		(rule__EquivalenceExpression__Group__0)
		{ after(grammarAccess.getEquivalenceExpressionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleComparisionExpression
entryRuleComparisionExpression
:
{ before(grammarAccess.getComparisionExpressionRule()); }
	 ruleComparisionExpression
{ after(grammarAccess.getComparisionExpressionRule()); } 
	 EOF 
;

// Rule ComparisionExpression
ruleComparisionExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getComparisionExpressionAccess().getGroup()); }
		(rule__ComparisionExpression__Group__0)
		{ after(grammarAccess.getComparisionExpressionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleAdditiveExpression
entryRuleAdditiveExpression
:
{ before(grammarAccess.getAdditiveExpressionRule()); }
	 ruleAdditiveExpression
{ after(grammarAccess.getAdditiveExpressionRule()); } 
	 EOF 
;

// Rule AdditiveExpression
ruleAdditiveExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getAdditiveExpressionAccess().getGroup()); }
		(rule__AdditiveExpression__Group__0)
		{ after(grammarAccess.getAdditiveExpressionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleMultiplicativeExpression
entryRuleMultiplicativeExpression
:
{ before(grammarAccess.getMultiplicativeExpressionRule()); }
	 ruleMultiplicativeExpression
{ after(grammarAccess.getMultiplicativeExpressionRule()); } 
	 EOF 
;

// Rule MultiplicativeExpression
ruleMultiplicativeExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getMultiplicativeExpressionAccess().getGroup()); }
		(rule__MultiplicativeExpression__Group__0)
		{ after(grammarAccess.getMultiplicativeExpressionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleSignumExpression
entryRuleSignumExpression
:
{ before(grammarAccess.getSignumExpressionRule()); }
	 ruleSignumExpression
{ after(grammarAccess.getSignumExpressionRule()); } 
	 EOF 
;

// Rule SignumExpression
ruleSignumExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getSignumExpressionAccess().getAlternatives()); }
		(rule__SignumExpression__Alternatives)
		{ after(grammarAccess.getSignumExpressionAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleAccessExpression
entryRuleAccessExpression
:
{ before(grammarAccess.getAccessExpressionRule()); }
	 ruleAccessExpression
{ after(grammarAccess.getAccessExpressionRule()); } 
	 EOF 
;

// Rule AccessExpression
ruleAccessExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getAccessExpressionAccess().getGroup()); }
		(rule__AccessExpression__Group__0)
		{ after(grammarAccess.getAccessExpressionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleFieldReferenceExpression
entryRuleFieldReferenceExpression
:
{ before(grammarAccess.getFieldReferenceExpressionRule()); }
	 ruleFieldReferenceExpression
{ after(grammarAccess.getFieldReferenceExpressionRule()); } 
	 EOF 
;

// Rule FieldReferenceExpression
ruleFieldReferenceExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getFieldReferenceExpressionAccess().getGroup()); }
		(rule__FieldReferenceExpression__Group__0)
		{ after(grammarAccess.getFieldReferenceExpressionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleDirectReferenceExpression
entryRuleDirectReferenceExpression
:
{ before(grammarAccess.getDirectReferenceExpressionRule()); }
	 ruleDirectReferenceExpression
{ after(grammarAccess.getDirectReferenceExpressionRule()); } 
	 EOF 
;

// Rule DirectReferenceExpression
ruleDirectReferenceExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getDirectReferenceExpressionAccess().getDeclarationAssignment()); }
		(rule__DirectReferenceExpression__DeclarationAssignment)
		{ after(grammarAccess.getDirectReferenceExpressionAccess().getDeclarationAssignment()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleLiteralExpression
entryRuleLiteralExpression
:
{ before(grammarAccess.getLiteralExpressionRule()); }
	 ruleLiteralExpression
{ after(grammarAccess.getLiteralExpressionRule()); } 
	 EOF 
;

// Rule LiteralExpression
ruleLiteralExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getLiteralExpressionAccess().getAlternatives()); }
		(rule__LiteralExpression__Alternatives)
		{ after(grammarAccess.getLiteralExpressionAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleInfinityExpression
entryRuleInfinityExpression
:
{ before(grammarAccess.getInfinityExpressionRule()); }
	 ruleInfinityExpression
{ after(grammarAccess.getInfinityExpressionRule()); } 
	 EOF 
;

// Rule InfinityExpression
ruleInfinityExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getInfinityExpressionAccess().getGroup()); }
		(rule__InfinityExpression__Group__0)
		{ after(grammarAccess.getInfinityExpressionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleIntegerLiteralExpression
entryRuleIntegerLiteralExpression
:
{ before(grammarAccess.getIntegerLiteralExpressionRule()); }
	 ruleIntegerLiteralExpression
{ after(grammarAccess.getIntegerLiteralExpressionRule()); } 
	 EOF 
;

// Rule IntegerLiteralExpression
ruleIntegerLiteralExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getIntegerLiteralExpressionAccess().getValueAssignment()); }
		(rule__IntegerLiteralExpression__ValueAssignment)
		{ after(grammarAccess.getIntegerLiteralExpressionAccess().getValueAssignment()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleDecimalLiteralExpression
entryRuleDecimalLiteralExpression
:
{ before(grammarAccess.getDecimalLiteralExpressionRule()); }
	 ruleDecimalLiteralExpression
{ after(grammarAccess.getDecimalLiteralExpressionRule()); } 
	 EOF 
;

// Rule DecimalLiteralExpression
ruleDecimalLiteralExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getDecimalLiteralExpressionAccess().getValueAssignment()); }
		(rule__DecimalLiteralExpression__ValueAssignment)
		{ after(grammarAccess.getDecimalLiteralExpressionAccess().getValueAssignment()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleRationalLiteralExpression
entryRuleRationalLiteralExpression
:
{ before(grammarAccess.getRationalLiteralExpressionRule()); }
	 ruleRationalLiteralExpression
{ after(grammarAccess.getRationalLiteralExpressionRule()); } 
	 EOF 
;

// Rule RationalLiteralExpression
ruleRationalLiteralExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getRationalLiteralExpressionAccess().getGroup()); }
		(rule__RationalLiteralExpression__Group__0)
		{ after(grammarAccess.getRationalLiteralExpressionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleBooleanLiteralExpression
entryRuleBooleanLiteralExpression
:
{ before(grammarAccess.getBooleanLiteralExpressionRule()); }
	 ruleBooleanLiteralExpression
{ after(grammarAccess.getBooleanLiteralExpressionRule()); } 
	 EOF 
;

// Rule BooleanLiteralExpression
ruleBooleanLiteralExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getBooleanLiteralExpressionAccess().getAlternatives()); }
		(rule__BooleanLiteralExpression__Alternatives)
		{ after(grammarAccess.getBooleanLiteralExpressionAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleTrueExpression
entryRuleTrueExpression
:
{ before(grammarAccess.getTrueExpressionRule()); }
	 ruleTrueExpression
{ after(grammarAccess.getTrueExpressionRule()); } 
	 EOF 
;

// Rule TrueExpression
ruleTrueExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getTrueExpressionAccess().getGroup()); }
		(rule__TrueExpression__Group__0)
		{ after(grammarAccess.getTrueExpressionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleFalseExpression
entryRuleFalseExpression
:
{ before(grammarAccess.getFalseExpressionRule()); }
	 ruleFalseExpression
{ after(grammarAccess.getFalseExpressionRule()); } 
	 EOF 
;

// Rule FalseExpression
ruleFalseExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getFalseExpressionAccess().getGroup()); }
		(rule__FalseExpression__Group__0)
		{ after(grammarAccess.getFalseExpressionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleRecordLiteralExpression
entryRuleRecordLiteralExpression
:
{ before(grammarAccess.getRecordLiteralExpressionRule()); }
	 ruleRecordLiteralExpression
{ after(grammarAccess.getRecordLiteralExpressionRule()); } 
	 EOF 
;

// Rule RecordLiteralExpression
ruleRecordLiteralExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getRecordLiteralExpressionAccess().getGroup()); }
		(rule__RecordLiteralExpression__Group__0)
		{ after(grammarAccess.getRecordLiteralExpressionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleFieldAssignment
entryRuleFieldAssignment
:
{ before(grammarAccess.getFieldAssignmentRule()); }
	 ruleFieldAssignment
{ after(grammarAccess.getFieldAssignmentRule()); } 
	 EOF 
;

// Rule FieldAssignment
ruleFieldAssignment 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getFieldAssignmentAccess().getGroup()); }
		(rule__FieldAssignment__Group__0)
		{ after(grammarAccess.getFieldAssignmentAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleEnumerationLiteralExpression
entryRuleEnumerationLiteralExpression
:
{ before(grammarAccess.getEnumerationLiteralExpressionRule()); }
	 ruleEnumerationLiteralExpression
{ after(grammarAccess.getEnumerationLiteralExpressionRule()); } 
	 EOF 
;

// Rule EnumerationLiteralExpression
ruleEnumerationLiteralExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getEnumerationLiteralExpressionAccess().getGroup()); }
		(rule__EnumerationLiteralExpression__Group__0)
		{ after(grammarAccess.getEnumerationLiteralExpressionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleOpaqueExpression
entryRuleOpaqueExpression
:
{ before(grammarAccess.getOpaqueExpressionRule()); }
	 ruleOpaqueExpression
{ after(grammarAccess.getOpaqueExpressionRule()); } 
	 EOF 
;

// Rule OpaqueExpression
ruleOpaqueExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getOpaqueExpressionAccess().getExpressionAssignment()); }
		(rule__OpaqueExpression__ExpressionAssignment)
		{ after(grammarAccess.getOpaqueExpressionAccess().getExpressionAssignment()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleArrayLiteralExpression
entryRuleArrayLiteralExpression
:
{ before(grammarAccess.getArrayLiteralExpressionRule()); }
	 ruleArrayLiteralExpression
{ after(grammarAccess.getArrayLiteralExpressionRule()); } 
	 EOF 
;

// Rule ArrayLiteralExpression
ruleArrayLiteralExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getArrayLiteralExpressionAccess().getGroup()); }
		(rule__ArrayLiteralExpression__Group__0)
		{ after(grammarAccess.getArrayLiteralExpressionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleIntegerRangeLiteralExpression
entryRuleIntegerRangeLiteralExpression
:
{ before(grammarAccess.getIntegerRangeLiteralExpressionRule()); }
	 ruleIntegerRangeLiteralExpression
{ after(grammarAccess.getIntegerRangeLiteralExpressionRule()); } 
	 EOF 
;

// Rule IntegerRangeLiteralExpression
ruleIntegerRangeLiteralExpression 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getIntegerRangeLiteralExpressionAccess().getGroup()); }
		(rule__IntegerRangeLiteralExpression__Group__0)
		{ after(grammarAccess.getIntegerRangeLiteralExpressionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Entry rule entryRuleBasicConstraintDefinition
entryRuleBasicConstraintDefinition
:
{ before(grammarAccess.getBasicConstraintDefinitionRule()); }
	 ruleBasicConstraintDefinition
{ after(grammarAccess.getBasicConstraintDefinitionRule()); } 
	 EOF 
;

// Rule BasicConstraintDefinition
ruleBasicConstraintDefinition 
	@init {
		int stackSize = keepStackSize();
	}
	:
	(
		{ before(grammarAccess.getBasicConstraintDefinitionAccess().getGroup()); }
		(rule__BasicConstraintDefinition__Group__0)
		{ after(grammarAccess.getBasicConstraintDefinitionAccess().getGroup()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Rule LocalDynamics
ruleLocalDynamics
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getLocalDynamicsAccess().getAlternatives()); }
		(rule__LocalDynamics__Alternatives)
		{ after(grammarAccess.getLocalDynamicsAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Rule Occurrence
ruleOccurrence
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getOccurrenceAccess().getAlternatives()); }
		(rule__Occurrence__Alternatives)
		{ after(grammarAccess.getOccurrenceAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Rule TermReferenceMode
ruleTermReferenceMode
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getTermReferenceModeAccess().getAlternatives()); }
		(rule__TermReferenceMode__Alternatives)
		{ after(grammarAccess.getTermReferenceModeAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Rule FaultModeState
ruleFaultModeState
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getFaultModeStateAccess().getAlternatives()); }
		(rule__FaultModeState__Alternatives)
		{ after(grammarAccess.getFaultModeStateAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Rule FaultEvent
ruleFaultEvent
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getFaultEventAccess().getAlternatives()); }
		(rule__FaultEvent__Alternatives)
		{ after(grammarAccess.getFaultEventAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Rule And
ruleAnd
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAndAccess().getANDEnumLiteralDeclaration()); }
		('&&')
		{ after(grammarAccess.getAndAccess().getANDEnumLiteralDeclaration()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Rule Or
ruleOr
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getOrAccess().getOREnumLiteralDeclaration()); }
		('||')
		{ after(grammarAccess.getOrAccess().getOREnumLiteralDeclaration()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Rule Equal
ruleEqual
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getEqualAccess().getEQUALEnumLiteralDeclaration()); }
		('==')
		{ after(grammarAccess.getEqualAccess().getEQUALEnumLiteralDeclaration()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Rule Xor
ruleXor
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getXorAccess().getXOREnumLiteralDeclaration()); }
		('^')
		{ after(grammarAccess.getXorAccess().getXOREnumLiteralDeclaration()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Rule Imply
ruleImply
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getImplyAccess().getIMPLYEnumLiteralDeclaration()); }
		('->')
		{ after(grammarAccess.getImplyAccess().getIMPLYEnumLiteralDeclaration()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Rule Not
ruleNot
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getNotAccess().getNOTEnumLiteralDeclaration()); }
		('!')
		{ after(grammarAccess.getNotAccess().getNOTEnumLiteralDeclaration()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Rule RealizationMode
ruleRealizationMode
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getRealizationModeAccess().getAlternatives()); }
		(rule__RealizationMode__Alternatives)
		{ after(grammarAccess.getRealizationModeAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Rule SchedulingOrder
ruleSchedulingOrder
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSchedulingOrderAccess().getAlternatives()); }
		(rule__SchedulingOrder__Alternatives)
		{ after(grammarAccess.getSchedulingOrderAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Rule OrthogonalRegionSchedulingOrder
ruleOrthogonalRegionSchedulingOrder
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getOrthogonalRegionSchedulingOrderAccess().getAlternatives()); }
		(rule__OrthogonalRegionSchedulingOrder__Alternatives)
		{ after(grammarAccess.getOrthogonalRegionSchedulingOrderAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Rule TransitionPriority
ruleTransitionPriority
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getTransitionPriorityAccess().getAlternatives()); }
		(rule__TransitionPriority__Alternatives)
		{ after(grammarAccess.getTransitionPriorityAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Rule GuardEvaluation
ruleGuardEvaluation
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getGuardEvaluationAccess().getAlternatives()); }
		(rule__GuardEvaluation__Alternatives)
		{ after(grammarAccess.getGuardEvaluationAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Rule SpecialStateKind
ruleSpecialStateKind
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSpecialStateKindAccess().getACCEPTINGEnumLiteralDeclaration()); }
		('accepting')
		{ after(grammarAccess.getSpecialStateKindAccess().getACCEPTINGEnumLiteralDeclaration()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Rule NotDefinedEventMode
ruleNotDefinedEventMode
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getNotDefinedEventModeAccess().getAlternatives()); }
		(rule__NotDefinedEventMode__Alternatives)
		{ after(grammarAccess.getNotDefinedEventModeAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Rule LinkType
ruleLinkType
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getLinkTypeAccess().getAlternatives()); }
		(rule__LinkType__Alternatives)
		{ after(grammarAccess.getLinkTypeAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Rule History
ruleHistory
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getHistoryAccess().getAlternatives()); }
		(rule__History__Alternatives)
		{ after(grammarAccess.getHistoryAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Rule TimeUnit
ruleTimeUnit
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getTimeUnitAccess().getAlternatives()); }
		(rule__TimeUnit__Alternatives)
		{ after(grammarAccess.getTimeUnitAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Rule ControlFunction
ruleControlFunction
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getControlFunctionAccess().getAlternatives()); }
		(rule__ControlFunction__Alternatives)
		{ after(grammarAccess.getControlFunctionAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Rule DiscardStrategy
ruleDiscardStrategy
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getDiscardStrategyAccess().getAlternatives()); }
		(rule__DiscardStrategy__Alternatives)
		{ after(grammarAccess.getDiscardStrategyAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Rule EventDirection
ruleEventDirection
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getEventDirectionAccess().getAlternatives()); }
		(rule__EventDirection__Alternatives)
		{ after(grammarAccess.getEventDirectionAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

// Rule Persistency
rulePersistency
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getPersistencyAccess().getAlternatives()); }
		(rule__Persistency__Alternatives)
		{ after(grammarAccess.getPersistencyAccess().getAlternatives()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__Effect__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getEffectAccess().getStuckAtEffectParserRuleCall_0()); }
		ruleStuckAtEffect
		{ after(grammarAccess.getEffectAccess().getStuckAtEffectParserRuleCall_0()); }
	)
	|
	(
		{ before(grammarAccess.getEffectAccess().getFrozenEffectParserRuleCall_1()); }
		ruleFrozenEffect
		{ after(grammarAccess.getEffectAccess().getFrozenEffectParserRuleCall_1()); }
	)
	|
	(
		{ before(grammarAccess.getEffectAccess().getNonDeterminismEffectParserRuleCall_2()); }
		ruleNonDeterminismEffect
		{ after(grammarAccess.getEffectAccess().getNonDeterminismEffectParserRuleCall_2()); }
	)
	|
	(
		{ before(grammarAccess.getEffectAccess().getNonDeterminismBooleanEffectParserRuleCall_3()); }
		ruleNonDeterminismBooleanEffect
		{ after(grammarAccess.getEffectAccess().getNonDeterminismBooleanEffectParserRuleCall_3()); }
	)
	|
	(
		{ before(grammarAccess.getEffectAccess().getConditionalEffectParserRuleCall_4()); }
		ruleConditionalEffect
		{ after(grammarAccess.getEffectAccess().getConditionalEffectParserRuleCall_4()); }
	)
	|
	(
		{ before(grammarAccess.getEffectAccess().getRampDownEffectParserRuleCall_5()); }
		ruleRampDownEffect
		{ after(grammarAccess.getEffectAccess().getRampDownEffectParserRuleCall_5()); }
	)
	|
	(
		{ before(grammarAccess.getEffectAccess().getInvertedEffectParserRuleCall_6()); }
		ruleInvertedEffect
		{ after(grammarAccess.getEffectAccess().getInvertedEffectParserRuleCall_6()); }
	)
	|
	(
		{ before(grammarAccess.getEffectAccess().getStuckAtFixedEffectParserRuleCall_7()); }
		ruleStuckAtFixedEffect
		{ after(grammarAccess.getEffectAccess().getStuckAtFixedEffectParserRuleCall_7()); }
	)
	|
	(
		{ before(grammarAccess.getEffectAccess().getRandomEffectParserRuleCall_8()); }
		ruleRandomEffect
		{ after(grammarAccess.getEffectAccess().getRandomEffectParserRuleCall_8()); }
	)
	|
	(
		{ before(grammarAccess.getEffectAccess().getErroneousEffectParserRuleCall_9()); }
		ruleErroneousEffect
		{ after(grammarAccess.getEffectAccess().getErroneousEffectParserRuleCall_9()); }
	)
	|
	(
		{ before(grammarAccess.getEffectAccess().getDeltaOutEffectParserRuleCall_10()); }
		ruleDeltaOutEffect
		{ after(grammarAccess.getEffectAccess().getDeltaOutEffectParserRuleCall_10()); }
	)
	|
	(
		{ before(grammarAccess.getEffectAccess().getDeltaInErroneousEffectParserRuleCall_11()); }
		ruleDeltaInErroneousEffect
		{ after(grammarAccess.getEffectAccess().getDeltaInErroneousEffectParserRuleCall_11()); }
	)
	|
	(
		{ before(grammarAccess.getEffectAccess().getDeltaInRandomEffectParserRuleCall_12()); }
		ruleDeltaInRandomEffect
		{ after(grammarAccess.getEffectAccess().getDeltaInRandomEffectParserRuleCall_12()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauseRange__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getCommonCauseRangeAccess().getGroup_0()); }
		(rule__CommonCauseRange__Group_0__0)
		{ after(grammarAccess.getCommonCauseRangeAccess().getGroup_0()); }
	)
	|
	(
		{ before(grammarAccess.getCommonCauseRangeAccess().getGroup_1()); }
		(rule__CommonCauseRange__Group_1__0)
		{ after(grammarAccess.getCommonCauseRangeAccess().getGroup_1()); }
	)
	|
	(
		{ before(grammarAccess.getCommonCauseRangeAccess().getGroup_2()); }
		(rule__CommonCauseRange__Group_2__0)
		{ after(grammarAccess.getCommonCauseRangeAccess().getGroup_2()); }
	)
	|
	(
		{ before(grammarAccess.getCommonCauseRangeAccess().getGroup_3()); }
		(rule__CommonCauseRange__Group_3__0)
		{ after(grammarAccess.getCommonCauseRangeAccess().getGroup_3()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__PrimaryExpression__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getPrimaryExpressionAccess().getLiteralExpressionParserRuleCall_0()); }
		ruleLiteralExpression
		{ after(grammarAccess.getPrimaryExpressionAccess().getLiteralExpressionParserRuleCall_0()); }
	)
	|
	(
		{ before(grammarAccess.getPrimaryExpressionAccess().getComponentInstanceStateExpressionParserRuleCall_1()); }
		ruleComponentInstanceStateExpression
		{ after(grammarAccess.getPrimaryExpressionAccess().getComponentInstanceStateExpressionParserRuleCall_1()); }
	)
	|
	(
		{ before(grammarAccess.getPrimaryExpressionAccess().getGroup_2()); }
		(rule__PrimaryExpression__Group_2__0)
		{ after(grammarAccess.getPrimaryExpressionAccess().getGroup_2()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__Trigger__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getTriggerAccess().getOnCycleTriggerParserRuleCall_0()); }
		ruleOnCycleTrigger
		{ after(grammarAccess.getTriggerAccess().getOnCycleTriggerParserRuleCall_0()); }
	)
	|
	(
		{ before(grammarAccess.getTriggerAccess().getAnyTriggerParserRuleCall_1()); }
		ruleAnyTrigger
		{ after(grammarAccess.getTriggerAccess().getAnyTriggerParserRuleCall_1()); }
	)
	|
	(
		{ before(grammarAccess.getTriggerAccess().getComplexTriggerParserRuleCall_2()); }
		ruleComplexTrigger
		{ after(grammarAccess.getTriggerAccess().getComplexTriggerParserRuleCall_2()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SimpleTrigger__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSimpleTriggerAccess().getOnCycleTriggerParserRuleCall_0()); }
		ruleOnCycleTrigger
		{ after(grammarAccess.getSimpleTriggerAccess().getOnCycleTriggerParserRuleCall_0()); }
	)
	|
	(
		{ before(grammarAccess.getSimpleTriggerAccess().getAnyTriggerParserRuleCall_1()); }
		ruleAnyTrigger
		{ after(grammarAccess.getSimpleTriggerAccess().getAnyTriggerParserRuleCall_1()); }
	)
	|
	(
		{ before(grammarAccess.getSimpleTriggerAccess().getOpaqueTriggerParserRuleCall_2()); }
		ruleOpaqueTrigger
		{ after(grammarAccess.getSimpleTriggerAccess().getOpaqueTriggerParserRuleCall_2()); }
	)
	|
	(
		{ before(grammarAccess.getSimpleTriggerAccess().getEventTriggerParserRuleCall_3()); }
		ruleEventTrigger
		{ after(grammarAccess.getSimpleTriggerAccess().getEventTriggerParserRuleCall_3()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__BinaryOperand__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getBinaryOperandAccess().getUnaryTriggerParserRuleCall_0()); }
		ruleUnaryTrigger
		{ after(grammarAccess.getBinaryOperandAccess().getUnaryTriggerParserRuleCall_0()); }
	)
	|
	(
		{ before(grammarAccess.getBinaryOperandAccess().getParenthesesTriggerParserRuleCall_1()); }
		ruleParenthesesTrigger
		{ after(grammarAccess.getBinaryOperandAccess().getParenthesesTriggerParserRuleCall_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__UnaryTrigger__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getUnaryTriggerAccess().getNotTriggerParserRuleCall_0()); }
		ruleNotTrigger
		{ after(grammarAccess.getUnaryTriggerAccess().getNotTriggerParserRuleCall_0()); }
	)
	|
	(
		{ before(grammarAccess.getUnaryTriggerAccess().getEventTriggerParserRuleCall_1()); }
		ruleEventTrigger
		{ after(grammarAccess.getUnaryTriggerAccess().getEventTriggerParserRuleCall_1()); }
	)
	|
	(
		{ before(grammarAccess.getUnaryTriggerAccess().getOpaqueTriggerParserRuleCall_2()); }
		ruleOpaqueTrigger
		{ after(grammarAccess.getUnaryTriggerAccess().getOpaqueTriggerParserRuleCall_2()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__EventReference__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getEventReferenceAccess().getClockTickReferenceParserRuleCall_0()); }
		ruleClockTickReference
		{ after(grammarAccess.getEventReferenceAccess().getClockTickReferenceParserRuleCall_0()); }
	)
	|
	(
		{ before(grammarAccess.getEventReferenceAccess().getTimeoutEventReferenceParserRuleCall_1()); }
		ruleTimeoutEventReference
		{ after(grammarAccess.getEventReferenceAccess().getTimeoutEventReferenceParserRuleCall_1()); }
	)
	|
	(
		{ before(grammarAccess.getEventReferenceAccess().getAnyPortEventReferenceParserRuleCall_2()); }
		ruleAnyPortEventReference
		{ after(grammarAccess.getEventReferenceAccess().getAnyPortEventReferenceParserRuleCall_2()); }
	)
	|
	(
		{ before(grammarAccess.getEventReferenceAccess().getPortEventReferenceParserRuleCall_3()); }
		rulePortEventReference
		{ after(grammarAccess.getEventReferenceAccess().getPortEventReferenceParserRuleCall_3()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Alternatives_0_4
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getAnnotationsAssignment_0_4_0()); }
		(rule__SynchronousStatechartDefinition__AnnotationsAssignment_0_4_0)
		{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getAnnotationsAssignment_0_4_0()); }
	)
	|
	(
		{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getAnnotationsAssignment_0_4_1()); }
		(rule__SynchronousStatechartDefinition__AnnotationsAssignment_0_4_1)
		{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getAnnotationsAssignment_0_4_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Alternatives_6
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getVariableDeclarationsAssignment_6_0()); }
		(rule__SynchronousStatechartDefinition__VariableDeclarationsAssignment_6_0)
		{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getVariableDeclarationsAssignment_6_0()); }
	)
	|
	(
		{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getTimeoutDeclarationsAssignment_6_1()); }
		(rule__SynchronousStatechartDefinition__TimeoutDeclarationsAssignment_6_1)
		{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getTimeoutDeclarationsAssignment_6_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Alternatives_7
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getRegionsAssignment_7_0()); }
		(rule__SynchronousStatechartDefinition__RegionsAssignment_7_0)
		{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getRegionsAssignment_7_0()); }
	)
	|
	(
		{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getTransitionsAssignment_7_1()); }
		(rule__SynchronousStatechartDefinition__TransitionsAssignment_7_1)
		{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getTransitionsAssignment_7_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Alternatives_1_4
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getAnnotationsAssignment_1_4_0()); }
		(rule__AsynchronousStatechartDefinition__AnnotationsAssignment_1_4_0)
		{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getAnnotationsAssignment_1_4_0()); }
	)
	|
	(
		{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getAnnotationsAssignment_1_4_1()); }
		(rule__AsynchronousStatechartDefinition__AnnotationsAssignment_1_4_1)
		{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getAnnotationsAssignment_1_4_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Alternatives_8
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getVariableDeclarationsAssignment_8_0()); }
		(rule__AsynchronousStatechartDefinition__VariableDeclarationsAssignment_8_0)
		{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getVariableDeclarationsAssignment_8_0()); }
	)
	|
	(
		{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getTimeoutDeclarationsAssignment_8_1()); }
		(rule__AsynchronousStatechartDefinition__TimeoutDeclarationsAssignment_8_1)
		{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getTimeoutDeclarationsAssignment_8_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Alternatives_9
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getRegionsAssignment_9_0()); }
		(rule__AsynchronousStatechartDefinition__RegionsAssignment_9_0)
		{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getRegionsAssignment_9_0()); }
	)
	|
	(
		{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getTransitionsAssignment_9_1()); }
		(rule__AsynchronousStatechartDefinition__TransitionsAssignment_9_1)
		{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getTransitionsAssignment_9_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentAnnotation__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getComponentAnnotationAccess().getWrapperComponentAnnotationParserRuleCall_0()); }
		ruleWrapperComponentAnnotation
		{ after(grammarAccess.getComponentAnnotationAccess().getWrapperComponentAnnotationParserRuleCall_0()); }
	)
	|
	(
		{ before(grammarAccess.getComponentAnnotationAccess().getRunUponExternalEventAnnotationParserRuleCall_1()); }
		ruleRunUponExternalEventAnnotation
		{ after(grammarAccess.getComponentAnnotationAccess().getRunUponExternalEventAnnotationParserRuleCall_1()); }
	)
	|
	(
		{ before(grammarAccess.getComponentAnnotationAccess().getRunUponExternalEventOrInternalTimeoutAnnotationParserRuleCall_2()); }
		ruleRunUponExternalEventOrInternalTimeoutAnnotation
		{ after(grammarAccess.getComponentAnnotationAccess().getRunUponExternalEventOrInternalTimeoutAnnotationParserRuleCall_2()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__StatechartContractAnnotation__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getStatechartContractAnnotationAccess().getScenarioAllowedWaitingAnnotationParserRuleCall_0()); }
		ruleScenarioAllowedWaitingAnnotation
		{ after(grammarAccess.getStatechartContractAnnotationAccess().getScenarioAllowedWaitingAnnotationParserRuleCall_0()); }
	)
	|
	(
		{ before(grammarAccess.getStatechartContractAnnotationAccess().getScenarioContractAnnotationParserRuleCall_1()); }
		ruleScenarioContractAnnotation
		{ after(grammarAccess.getStatechartContractAnnotationAccess().getScenarioContractAnnotationParserRuleCall_1()); }
	)
	|
	(
		{ before(grammarAccess.getStatechartContractAnnotationAccess().getAdaptiveContractAnnotationParserRuleCall_2()); }
		ruleAdaptiveContractAnnotation
		{ after(grammarAccess.getStatechartContractAnnotationAccess().getAdaptiveContractAnnotationParserRuleCall_2()); }
	)
	|
	(
		{ before(grammarAccess.getStatechartContractAnnotationAccess().getMissionPhaseAnnotationParserRuleCall_3()); }
		ruleMissionPhaseAnnotation
		{ after(grammarAccess.getStatechartContractAnnotationAccess().getMissionPhaseAnnotationParserRuleCall_3()); }
	)
	|
	(
		{ before(grammarAccess.getStatechartContractAnnotationAccess().getHasInitialoutputsBlockAnnotationParserRuleCall_4()); }
		ruleHasInitialoutputsBlockAnnotation
		{ after(grammarAccess.getStatechartContractAnnotationAccess().getHasInitialoutputsBlockAnnotationParserRuleCall_4()); }
	)
	|
	(
		{ before(grammarAccess.getStatechartContractAnnotationAccess().getNegativeContractStatechartAnnotationParserRuleCall_5()); }
		ruleNegativeContractStatechartAnnotation
		{ after(grammarAccess.getStatechartContractAnnotationAccess().getNegativeContractStatechartAnnotationParserRuleCall_5()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScenarioAllowedWaitingAnnotation__Alternatives_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getScenarioAllowedWaitingAnnotationAccess().getScenarioAllowedWaitAnnotationKeyword_1_0()); }
		'@ScenarioAllowedWaitAnnotation'
		{ after(grammarAccess.getScenarioAllowedWaitingAnnotationAccess().getScenarioAllowedWaitAnnotationKeyword_1_0()); }
	)
	|
	(
		{ before(grammarAccess.getScenarioAllowedWaitingAnnotationAccess().getScenarioAllowedWaitKeyword_1_1()); }
		'@ScenarioAllowedWait'
		{ after(grammarAccess.getScenarioAllowedWaitingAnnotationAccess().getScenarioAllowedWaitKeyword_1_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__RunUponExternalEventAnnotation__Alternatives_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getRunUponExternalEventAnnotationAccess().getRunUponExternalEventAnnotationKeyword_1_0()); }
		'@RunUponExternalEventAnnotation'
		{ after(grammarAccess.getRunUponExternalEventAnnotationAccess().getRunUponExternalEventAnnotationKeyword_1_0()); }
	)
	|
	(
		{ before(grammarAccess.getRunUponExternalEventAnnotationAccess().getRunUponExternalEventKeyword_1_1()); }
		'@RunUponExternalEvent'
		{ after(grammarAccess.getRunUponExternalEventAnnotationAccess().getRunUponExternalEventKeyword_1_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__RunUponExternalEventOrInternalTimeoutAnnotation__Alternatives_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getRunUponExternalEventOrInternalTimeoutAnnotationAccess().getRunUponExternalEventOrInternalTimeoutAnnotationKeyword_1_0()); }
		'@RunUponExternalEventOrInternalTimeoutAnnotation'
		{ after(grammarAccess.getRunUponExternalEventOrInternalTimeoutAnnotationAccess().getRunUponExternalEventOrInternalTimeoutAnnotationKeyword_1_0()); }
	)
	|
	(
		{ before(grammarAccess.getRunUponExternalEventOrInternalTimeoutAnnotationAccess().getRunUponExternalEventOrInternalTimeoutKeyword_1_1()); }
		'@RunUponExternalEventOrInternalTimeout'
		{ after(grammarAccess.getRunUponExternalEventOrInternalTimeoutAnnotationAccess().getRunUponExternalEventOrInternalTimeoutKeyword_1_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__StateNode__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getStateNodeAccess().getPseudoStateParserRuleCall_0()); }
		rulePseudoState
		{ after(grammarAccess.getStateNodeAccess().getPseudoStateParserRuleCall_0()); }
	)
	|
	(
		{ before(grammarAccess.getStateNodeAccess().getStateParserRuleCall_1()); }
		ruleState
		{ after(grammarAccess.getStateNodeAccess().getStateParserRuleCall_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__StateAnnotation__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getStateAnnotationAccess().getStateContractAnnotationParserRuleCall_0()); }
		ruleStateContractAnnotation
		{ after(grammarAccess.getStateAnnotationAccess().getStateContractAnnotationParserRuleCall_0()); }
	)
	|
	(
		{ before(grammarAccess.getStateAnnotationAccess().getMissionPhaseStateAnnotationParserRuleCall_1()); }
		ruleMissionPhaseStateAnnotation
		{ after(grammarAccess.getStateAnnotationAccess().getMissionPhaseStateAnnotationParserRuleCall_1()); }
	)
	|
	(
		{ before(grammarAccess.getStateAnnotationAccess().getSpecialStateAnnotationParserRuleCall_2()); }
		ruleSpecialStateAnnotation
		{ after(grammarAccess.getStateAnnotationAccess().getSpecialStateAnnotationParserRuleCall_2()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SpecialStateAnnotation__Alternatives_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSpecialStateAnnotationAccess().getSpecialStateAnnotationKeyword_1_0()); }
		'@SpecialStateAnnotation'
		{ after(grammarAccess.getSpecialStateAnnotationAccess().getSpecialStateAnnotationKeyword_1_0()); }
	)
	|
	(
		{ before(grammarAccess.getSpecialStateAnnotationAccess().getSpecialStateKeyword_1_1()); }
		'@SpecialState'
		{ after(grammarAccess.getSpecialStateAnnotationAccess().getSpecialStateKeyword_1_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__VariableBinding__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getVariableBindingAccess().getGroup_0()); }
		(rule__VariableBinding__Group_0__0)
		{ after(grammarAccess.getVariableBindingAccess().getGroup_0()); }
	)
	|
	(
		{ before(grammarAccess.getVariableBindingAccess().getGroup_1()); }
		(rule__VariableBinding__Group_1__0)
		{ after(grammarAccess.getVariableBindingAccess().getGroup_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__PseudoState__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getPseudoStateAccess().getEntryStateParserRuleCall_0()); }
		ruleEntryState
		{ after(grammarAccess.getPseudoStateAccess().getEntryStateParserRuleCall_0()); }
	)
	|
	(
		{ before(grammarAccess.getPseudoStateAccess().getForkStateParserRuleCall_1()); }
		ruleForkState
		{ after(grammarAccess.getPseudoStateAccess().getForkStateParserRuleCall_1()); }
	)
	|
	(
		{ before(grammarAccess.getPseudoStateAccess().getJoinStateParserRuleCall_2()); }
		ruleJoinState
		{ after(grammarAccess.getPseudoStateAccess().getJoinStateParserRuleCall_2()); }
	)
	|
	(
		{ before(grammarAccess.getPseudoStateAccess().getChoiceStateParserRuleCall_3()); }
		ruleChoiceState
		{ after(grammarAccess.getPseudoStateAccess().getChoiceStateParserRuleCall_3()); }
	)
	|
	(
		{ before(grammarAccess.getPseudoStateAccess().getMergeStateParserRuleCall_4()); }
		ruleMergeState
		{ after(grammarAccess.getPseudoStateAccess().getMergeStateParserRuleCall_4()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__EntryState__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getEntryStateAccess().getInitialStateParserRuleCall_0()); }
		ruleInitialState
		{ after(grammarAccess.getEntryStateAccess().getInitialStateParserRuleCall_0()); }
	)
	|
	(
		{ before(grammarAccess.getEntryStateAccess().getShallowHistoryStateParserRuleCall_1()); }
		ruleShallowHistoryState
		{ after(grammarAccess.getEntryStateAccess().getShallowHistoryStateParserRuleCall_1()); }
	)
	|
	(
		{ before(grammarAccess.getEntryStateAccess().getDeepHistoryStateParserRuleCall_2()); }
		ruleDeepHistoryState
		{ after(grammarAccess.getEntryStateAccess().getDeepHistoryStateParserRuleCall_2()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__Component__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getComponentAccess().getSynchronousComponentParserRuleCall_0()); }
		ruleSynchronousComponent
		{ after(grammarAccess.getComponentAccess().getSynchronousComponentParserRuleCall_0()); }
	)
	|
	(
		{ before(grammarAccess.getComponentAccess().getAsynchronousComponentParserRuleCall_1()); }
		ruleAsynchronousComponent
		{ after(grammarAccess.getComponentAccess().getAsynchronousComponentParserRuleCall_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousComponent__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSynchronousComponentAccess().getSynchronousStatechartDefinitionParserRuleCall_0()); }
		ruleSynchronousStatechartDefinition
		{ after(grammarAccess.getSynchronousComponentAccess().getSynchronousStatechartDefinitionParserRuleCall_0()); }
	)
	|
	(
		{ before(grammarAccess.getSynchronousComponentAccess().getAbstractSynchronousComponentParserRuleCall_1()); }
		ruleAbstractSynchronousComponent
		{ after(grammarAccess.getSynchronousComponentAccess().getAbstractSynchronousComponentParserRuleCall_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AbstractSynchronousComponent__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAbstractSynchronousComponentAccess().getCascadeCompositeComponentParserRuleCall_0()); }
		ruleCascadeCompositeComponent
		{ after(grammarAccess.getAbstractSynchronousComponentAccess().getCascadeCompositeComponentParserRuleCall_0()); }
	)
	|
	(
		{ before(grammarAccess.getAbstractSynchronousComponentAccess().getSynchronousCompositeComponentParserRuleCall_1()); }
		ruleSynchronousCompositeComponent
		{ after(grammarAccess.getAbstractSynchronousComponentAccess().getSynchronousCompositeComponentParserRuleCall_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Alternatives_6
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getCascadeCompositeComponentAccess().getComponentsAssignment_6_0()); }
		(rule__CascadeCompositeComponent__ComponentsAssignment_6_0)
		{ after(grammarAccess.getCascadeCompositeComponentAccess().getComponentsAssignment_6_0()); }
	)
	|
	(
		{ before(grammarAccess.getCascadeCompositeComponentAccess().getPortBindingsAssignment_6_1()); }
		(rule__CascadeCompositeComponent__PortBindingsAssignment_6_1)
		{ after(grammarAccess.getCascadeCompositeComponentAccess().getPortBindingsAssignment_6_1()); }
	)
	|
	(
		{ before(grammarAccess.getCascadeCompositeComponentAccess().getChannelsAssignment_6_2()); }
		(rule__CascadeCompositeComponent__ChannelsAssignment_6_2)
		{ after(grammarAccess.getCascadeCompositeComponentAccess().getChannelsAssignment_6_2()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__Alternatives_6
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSynchronousCompositeComponentAccess().getComponentsAssignment_6_0()); }
		(rule__SynchronousCompositeComponent__ComponentsAssignment_6_0)
		{ after(grammarAccess.getSynchronousCompositeComponentAccess().getComponentsAssignment_6_0()); }
	)
	|
	(
		{ before(grammarAccess.getSynchronousCompositeComponentAccess().getPortBindingsAssignment_6_1()); }
		(rule__SynchronousCompositeComponent__PortBindingsAssignment_6_1)
		{ after(grammarAccess.getSynchronousCompositeComponentAccess().getPortBindingsAssignment_6_1()); }
	)
	|
	(
		{ before(grammarAccess.getSynchronousCompositeComponentAccess().getChannelsAssignment_6_2()); }
		(rule__SynchronousCompositeComponent__ChannelsAssignment_6_2)
		{ after(grammarAccess.getSynchronousCompositeComponentAccess().getChannelsAssignment_6_2()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousComponent__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousComponentAccess().getAbstractAsynchronousCompositeComponentParserRuleCall_0()); }
		ruleAbstractAsynchronousCompositeComponent
		{ after(grammarAccess.getAsynchronousComponentAccess().getAbstractAsynchronousCompositeComponentParserRuleCall_0()); }
	)
	|
	(
		{ before(grammarAccess.getAsynchronousComponentAccess().getAsynchronousAdapterParserRuleCall_1()); }
		ruleAsynchronousAdapter
		{ after(grammarAccess.getAsynchronousComponentAccess().getAsynchronousAdapterParserRuleCall_1()); }
	)
	|
	(
		{ before(grammarAccess.getAsynchronousComponentAccess().getAsynchronousStatechartDefinitionParserRuleCall_2()); }
		ruleAsynchronousStatechartDefinition
		{ after(grammarAccess.getAsynchronousComponentAccess().getAsynchronousStatechartDefinitionParserRuleCall_2()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AbstractAsynchronousCompositeComponent__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAbstractAsynchronousCompositeComponentAccess().getAsynchronousCompositeComponentParserRuleCall_0()); }
		ruleAsynchronousCompositeComponent
		{ after(grammarAccess.getAbstractAsynchronousCompositeComponentAccess().getAsynchronousCompositeComponentParserRuleCall_0()); }
	)
	|
	(
		{ before(grammarAccess.getAbstractAsynchronousCompositeComponentAccess().getScheduledAsynchronousCompositeComponentParserRuleCall_1()); }
		ruleScheduledAsynchronousCompositeComponent
		{ after(grammarAccess.getAbstractAsynchronousCompositeComponentAccess().getScheduledAsynchronousCompositeComponentParserRuleCall_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__Alternatives_6
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousCompositeComponentAccess().getComponentsAssignment_6_0()); }
		(rule__AsynchronousCompositeComponent__ComponentsAssignment_6_0)
		{ after(grammarAccess.getAsynchronousCompositeComponentAccess().getComponentsAssignment_6_0()); }
	)
	|
	(
		{ before(grammarAccess.getAsynchronousCompositeComponentAccess().getPortBindingsAssignment_6_1()); }
		(rule__AsynchronousCompositeComponent__PortBindingsAssignment_6_1)
		{ after(grammarAccess.getAsynchronousCompositeComponentAccess().getPortBindingsAssignment_6_1()); }
	)
	|
	(
		{ before(grammarAccess.getAsynchronousCompositeComponentAccess().getChannelsAssignment_6_2()); }
		(rule__AsynchronousCompositeComponent__ChannelsAssignment_6_2)
		{ after(grammarAccess.getAsynchronousCompositeComponentAccess().getChannelsAssignment_6_2()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Alternatives_6
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getComponentsAssignment_6_0()); }
		(rule__ScheduledAsynchronousCompositeComponent__ComponentsAssignment_6_0)
		{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getComponentsAssignment_6_0()); }
	)
	|
	(
		{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getPortBindingsAssignment_6_1()); }
		(rule__ScheduledAsynchronousCompositeComponent__PortBindingsAssignment_6_1)
		{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getPortBindingsAssignment_6_1()); }
	)
	|
	(
		{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getChannelsAssignment_6_2()); }
		(rule__ScheduledAsynchronousCompositeComponent__ChannelsAssignment_6_2)
		{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getChannelsAssignment_6_2()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__Alternatives_8
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousAdapterAccess().getClocksAssignment_8_0()); }
		(rule__AsynchronousAdapter__ClocksAssignment_8_0)
		{ after(grammarAccess.getAsynchronousAdapterAccess().getClocksAssignment_8_0()); }
	)
	|
	(
		{ before(grammarAccess.getAsynchronousAdapterAccess().getControlSpecificationsAssignment_8_1()); }
		(rule__AsynchronousAdapter__ControlSpecificationsAssignment_8_1)
		{ after(grammarAccess.getAsynchronousAdapterAccess().getControlSpecificationsAssignment_8_1()); }
	)
	|
	(
		{ before(grammarAccess.getAsynchronousAdapterAccess().getMessageQueuesAssignment_8_2()); }
		(rule__AsynchronousAdapter__MessageQueuesAssignment_8_2)
		{ after(grammarAccess.getAsynchronousAdapterAccess().getMessageQueuesAssignment_8_2()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstance__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getComponentInstanceAccess().getSynchronousComponentInstanceParserRuleCall_0()); }
		ruleSynchronousComponentInstance
		{ after(grammarAccess.getComponentInstanceAccess().getSynchronousComponentInstanceParserRuleCall_0()); }
	)
	|
	(
		{ before(grammarAccess.getComponentInstanceAccess().getGroup_1()); }
		(rule__ComponentInstance__Group_1__0)
		{ after(grammarAccess.getComponentInstanceAccess().getGroup_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__Channel__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getChannelAccess().getSimpleChannelParserRuleCall_0()); }
		ruleSimpleChannel
		{ after(grammarAccess.getChannelAccess().getSimpleChannelParserRuleCall_0()); }
	)
	|
	(
		{ before(grammarAccess.getChannelAccess().getBroadcastChannelParserRuleCall_1()); }
		ruleBroadcastChannel
		{ after(grammarAccess.getChannelAccess().getBroadcastChannelParserRuleCall_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__InlineStatement__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getInlineStatementAccess().getInlineStatementParserRuleCall_0()); }
		superInlineStatement
		{ after(grammarAccess.getInlineStatementAccess().getInlineStatementParserRuleCall_0()); }
	)
	|
	(
		{ before(grammarAccess.getInlineStatementAccess().getGroup_1()); }
		(rule__InlineStatement__Group_1__0)
		{ after(grammarAccess.getInlineStatementAccess().getGroup_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__InlineStatement__Alternatives_1_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getInlineStatementAccess().getRaiseEventActionParserRuleCall_1_0_0()); }
		ruleRaiseEventAction
		{ after(grammarAccess.getInlineStatementAccess().getRaiseEventActionParserRuleCall_1_0_0()); }
	)
	|
	(
		{ before(grammarAccess.getInlineStatementAccess().getSetTimeoutActionParserRuleCall_1_0_1()); }
		ruleSetTimeoutAction
		{ after(grammarAccess.getInlineStatementAccess().getSetTimeoutActionParserRuleCall_1_0_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceStateExpression__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getComponentInstanceStateExpressionAccess().getComponentInstanceStateReferenceExpressionParserRuleCall_0()); }
		ruleComponentInstanceStateReferenceExpression
		{ after(grammarAccess.getComponentInstanceStateExpressionAccess().getComponentInstanceStateReferenceExpressionParserRuleCall_0()); }
	)
	|
	(
		{ before(grammarAccess.getComponentInstanceStateExpressionAccess().getComponentInstanceVariableReferenceExpressionParserRuleCall_1()); }
		ruleComponentInstanceVariableReferenceExpression
		{ after(grammarAccess.getComponentInstanceStateExpressionAccess().getComponentInstanceVariableReferenceExpressionParserRuleCall_1()); }
	)
	|
	(
		{ before(grammarAccess.getComponentInstanceStateExpressionAccess().getComponentInstanceEventReferenceExpressionParserRuleCall_2()); }
		ruleComponentInstanceEventReferenceExpression
		{ after(grammarAccess.getComponentInstanceStateExpressionAccess().getComponentInstanceEventReferenceExpressionParserRuleCall_2()); }
	)
	|
	(
		{ before(grammarAccess.getComponentInstanceStateExpressionAccess().getComponentInstanceEventParameterReferenceExpressionParserRuleCall_3()); }
		ruleComponentInstanceEventParameterReferenceExpression
		{ after(grammarAccess.getComponentInstanceStateExpressionAccess().getComponentInstanceEventParameterReferenceExpressionParserRuleCall_3()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__Action__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getActionAccess().getBlockParserRuleCall_0()); }
		ruleBlock
		{ after(grammarAccess.getActionAccess().getBlockParserRuleCall_0()); }
	)
	|
	(
		{ before(grammarAccess.getActionAccess().getStatementParserRuleCall_1()); }
		ruleStatement
		{ after(grammarAccess.getActionAccess().getStatementParserRuleCall_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__FunctionDeclaration__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getFunctionDeclarationAccess().getLambdaDeclarationParserRuleCall_0()); }
		ruleLambdaDeclaration
		{ after(grammarAccess.getFunctionDeclarationAccess().getLambdaDeclarationParserRuleCall_0()); }
	)
	|
	(
		{ before(grammarAccess.getFunctionDeclarationAccess().getProcedureDeclarationParserRuleCall_1()); }
		ruleProcedureDeclaration
		{ after(grammarAccess.getFunctionDeclarationAccess().getProcedureDeclarationParserRuleCall_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__Statement__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getStatementAccess().getInlineStatementParserRuleCall_0()); }
		ruleInlineStatement
		{ after(grammarAccess.getStatementAccess().getInlineStatementParserRuleCall_0()); }
	)
	|
	(
		{ before(grammarAccess.getStatementAccess().getMultilineStatementParserRuleCall_1()); }
		ruleMultilineStatement
		{ after(grammarAccess.getStatementAccess().getMultilineStatementParserRuleCall_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

superInlineStatement__Alternatives_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getActionLanguageInlineStatementAccess().getVariableDeclarationStatementParserRuleCall_0_0()); }
		ruleVariableDeclarationStatement
		{ after(grammarAccess.getActionLanguageInlineStatementAccess().getVariableDeclarationStatementParserRuleCall_0_0()); }
	)
	|
	(
		{ before(grammarAccess.getActionLanguageInlineStatementAccess().getConstantDeclarationStatementParserRuleCall_0_1()); }
		ruleConstantDeclarationStatement
		{ after(grammarAccess.getActionLanguageInlineStatementAccess().getConstantDeclarationStatementParserRuleCall_0_1()); }
	)
	|
	(
		{ before(grammarAccess.getActionLanguageInlineStatementAccess().getReturnStatementParserRuleCall_0_2()); }
		ruleReturnStatement
		{ after(grammarAccess.getActionLanguageInlineStatementAccess().getReturnStatementParserRuleCall_0_2()); }
	)
	|
	(
		{ before(grammarAccess.getActionLanguageInlineStatementAccess().getBreakStatementParserRuleCall_0_3()); }
		ruleBreakStatement
		{ after(grammarAccess.getActionLanguageInlineStatementAccess().getBreakStatementParserRuleCall_0_3()); }
	)
	|
	(
		{ before(grammarAccess.getActionLanguageInlineStatementAccess().getAssignmentStatementParserRuleCall_0_4()); }
		(ruleAssignmentStatement)
		{ after(grammarAccess.getActionLanguageInlineStatementAccess().getAssignmentStatementParserRuleCall_0_4()); }
	)
	|
	(
		{ before(grammarAccess.getActionLanguageInlineStatementAccess().getAssertionStatementParserRuleCall_0_5()); }
		ruleAssertionStatement
		{ after(grammarAccess.getActionLanguageInlineStatementAccess().getAssertionStatementParserRuleCall_0_5()); }
	)
	|
	(
		{ before(grammarAccess.getActionLanguageInlineStatementAccess().getExpressionStatementParserRuleCall_0_6()); }
		ruleExpressionStatement
		{ after(grammarAccess.getActionLanguageInlineStatementAccess().getExpressionStatementParserRuleCall_0_6()); }
	)
	|
	(
		{ before(grammarAccess.getActionLanguageInlineStatementAccess().getEmptyStatementParserRuleCall_0_7()); }
		ruleEmptyStatement
		{ after(grammarAccess.getActionLanguageInlineStatementAccess().getEmptyStatementParserRuleCall_0_7()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__MultilineStatement__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getMultilineStatementAccess().getIfStatementParserRuleCall_0()); }
		ruleIfStatement
		{ after(grammarAccess.getMultilineStatementAccess().getIfStatementParserRuleCall_0()); }
	)
	|
	(
		{ before(grammarAccess.getMultilineStatementAccess().getSwitchStatementParserRuleCall_1()); }
		ruleSwitchStatement
		{ after(grammarAccess.getMultilineStatementAccess().getSwitchStatementParserRuleCall_1()); }
	)
	|
	(
		{ before(grammarAccess.getMultilineStatementAccess().getForStatementParserRuleCall_2()); }
		ruleForStatement
		{ after(grammarAccess.getMultilineStatementAccess().getForStatementParserRuleCall_2()); }
	)
	|
	(
		{ before(grammarAccess.getMultilineStatementAccess().getChoiceStatementParserRuleCall_3()); }
		ruleChoiceStatement
		{ after(grammarAccess.getMultilineStatementAccess().getChoiceStatementParserRuleCall_3()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AssignableAccessExpression__Alternatives_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAssignableAccessExpressionAccess().getGroup_1_0()); }
		(rule__AssignableAccessExpression__Group_1_0__0)
		{ after(grammarAccess.getAssignableAccessExpressionAccess().getGroup_1_0()); }
	)
	|
	(
		{ before(grammarAccess.getAssignableAccessExpressionAccess().getGroup_1_1()); }
		(rule__AssignableAccessExpression__Group_1_1__0)
		{ after(grammarAccess.getAssignableAccessExpressionAccess().getGroup_1_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AssignablePrimaryExpression__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAssignablePrimaryExpressionAccess().getAssignableDirectReferenceExpressionParserRuleCall_0()); }
		ruleAssignableDirectReferenceExpression
		{ after(grammarAccess.getAssignablePrimaryExpressionAccess().getAssignableDirectReferenceExpressionParserRuleCall_0()); }
	)
	|
	(
		{ before(grammarAccess.getAssignablePrimaryExpressionAccess().getGroup_1()); }
		(rule__AssignablePrimaryExpression__Group_1__0)
		{ after(grammarAccess.getAssignablePrimaryExpressionAccess().getGroup_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__VariableDeclarationAnnotation__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getVariableDeclarationAnnotationAccess().getTransientVariableDeclarationAnnotationParserRuleCall_0()); }
		ruleTransientVariableDeclarationAnnotation
		{ after(grammarAccess.getVariableDeclarationAnnotationAccess().getTransientVariableDeclarationAnnotationParserRuleCall_0()); }
	)
	|
	(
		{ before(grammarAccess.getVariableDeclarationAnnotationAccess().getResettableVariableDeclarationAnnotationParserRuleCall_1()); }
		ruleResettableVariableDeclarationAnnotation
		{ after(grammarAccess.getVariableDeclarationAnnotationAccess().getResettableVariableDeclarationAnnotationParserRuleCall_1()); }
	)
	|
	(
		{ before(grammarAccess.getVariableDeclarationAnnotationAccess().getUnremovableVariableDeclarationAnnotationParserRuleCall_2()); }
		ruleUnremovableVariableDeclarationAnnotation
		{ after(grammarAccess.getVariableDeclarationAnnotationAccess().getUnremovableVariableDeclarationAnnotationParserRuleCall_2()); }
	)
	|
	(
		{ before(grammarAccess.getVariableDeclarationAnnotationAccess().getInjectedVariableDeclarationAnnotationParserRuleCall_3()); }
		ruleInjectedVariableDeclarationAnnotation
		{ after(grammarAccess.getVariableDeclarationAnnotationAccess().getInjectedVariableDeclarationAnnotationParserRuleCall_3()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__Type__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getTypeAccess().getTypeReferenceParserRuleCall_0()); }
		ruleTypeReference
		{ after(grammarAccess.getTypeAccess().getTypeReferenceParserRuleCall_0()); }
	)
	|
	(
		{ before(grammarAccess.getTypeAccess().getTypeDefinitionParserRuleCall_1()); }
		ruleTypeDefinition
		{ after(grammarAccess.getTypeAccess().getTypeDefinitionParserRuleCall_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__TypeDefinition__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getTypeDefinitionAccess().getVoidTypeDefinitionParserRuleCall_0()); }
		ruleVoidTypeDefinition
		{ after(grammarAccess.getTypeDefinitionAccess().getVoidTypeDefinitionParserRuleCall_0()); }
	)
	|
	(
		{ before(grammarAccess.getTypeDefinitionAccess().getIntegerTypeDefinitionParserRuleCall_1()); }
		ruleIntegerTypeDefinition
		{ after(grammarAccess.getTypeDefinitionAccess().getIntegerTypeDefinitionParserRuleCall_1()); }
	)
	|
	(
		{ before(grammarAccess.getTypeDefinitionAccess().getBooleanTypeDefinitionParserRuleCall_2()); }
		ruleBooleanTypeDefinition
		{ after(grammarAccess.getTypeDefinitionAccess().getBooleanTypeDefinitionParserRuleCall_2()); }
	)
	|
	(
		{ before(grammarAccess.getTypeDefinitionAccess().getDecimalTypeDefinitionParserRuleCall_3()); }
		ruleDecimalTypeDefinition
		{ after(grammarAccess.getTypeDefinitionAccess().getDecimalTypeDefinitionParserRuleCall_3()); }
	)
	|
	(
		{ before(grammarAccess.getTypeDefinitionAccess().getRationalTypeDefinitionParserRuleCall_4()); }
		ruleRationalTypeDefinition
		{ after(grammarAccess.getTypeDefinitionAccess().getRationalTypeDefinitionParserRuleCall_4()); }
	)
	|
	(
		{ before(grammarAccess.getTypeDefinitionAccess().getArrayTypeDefinitionParserRuleCall_5()); }
		ruleArrayTypeDefinition
		{ after(grammarAccess.getTypeDefinitionAccess().getArrayTypeDefinitionParserRuleCall_5()); }
	)
	|
	(
		{ before(grammarAccess.getTypeDefinitionAccess().getIntegerRangeTypeDefinitionParserRuleCall_6()); }
		ruleIntegerRangeTypeDefinition
		{ after(grammarAccess.getTypeDefinitionAccess().getIntegerRangeTypeDefinitionParserRuleCall_6()); }
	)
	|
	(
		{ before(grammarAccess.getTypeDefinitionAccess().getEnumerationTypeDefinitionParserRuleCall_7()); }
		ruleEnumerationTypeDefinition
		{ after(grammarAccess.getTypeDefinitionAccess().getEnumerationTypeDefinitionParserRuleCall_7()); }
	)
	|
	(
		{ before(grammarAccess.getTypeDefinitionAccess().getRecordTypeDefinitionParserRuleCall_8()); }
		ruleRecordTypeDefinition
		{ after(grammarAccess.getTypeDefinitionAccess().getRecordTypeDefinitionParserRuleCall_8()); }
	)
	|
	(
		{ before(grammarAccess.getTypeDefinitionAccess().getSubrangeTypeDefinitionParserRuleCall_9()); }
		ruleSubrangeTypeDefinition
		{ after(grammarAccess.getTypeDefinitionAccess().getSubrangeTypeDefinitionParserRuleCall_9()); }
	)
	|
	(
		{ before(grammarAccess.getTypeDefinitionAccess().getFunctionTypeDefinitionParserRuleCall_10()); }
		ruleFunctionTypeDefinition
		{ after(grammarAccess.getTypeDefinitionAccess().getFunctionTypeDefinitionParserRuleCall_10()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SubrangeTypeDefinition__Alternatives_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSubrangeTypeDefinitionAccess().getGroup_2_0()); }
		(rule__SubrangeTypeDefinition__Group_2_0__0)
		{ after(grammarAccess.getSubrangeTypeDefinitionAccess().getGroup_2_0()); }
	)
	|
	(
		{ before(grammarAccess.getSubrangeTypeDefinitionAccess().getLowerBoundAssignment_2_1()); }
		(rule__SubrangeTypeDefinition__LowerBoundAssignment_2_1)
		{ after(grammarAccess.getSubrangeTypeDefinitionAccess().getLowerBoundAssignment_2_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SubrangeTypeDefinition__Alternatives_4
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSubrangeTypeDefinitionAccess().getInfKeyword_4_0()); }
		'inf'
		{ after(grammarAccess.getSubrangeTypeDefinitionAccess().getInfKeyword_4_0()); }
	)
	|
	(
		{ before(grammarAccess.getSubrangeTypeDefinitionAccess().getUpperBoundAssignment_4_1()); }
		(rule__SubrangeTypeDefinition__UpperBoundAssignment_4_1)
		{ after(grammarAccess.getSubrangeTypeDefinitionAccess().getUpperBoundAssignment_4_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__Expression__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getExpressionAccess().getIfThenElseExpressionParserRuleCall_0()); }
		ruleIfThenElseExpression
		{ after(grammarAccess.getExpressionAccess().getIfThenElseExpressionParserRuleCall_0()); }
	)
	|
	(
		{ before(grammarAccess.getExpressionAccess().getElseExpressionParserRuleCall_1()); }
		ruleElseExpression
		{ after(grammarAccess.getExpressionAccess().getElseExpressionParserRuleCall_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__UnaryLogicExpression__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getUnaryLogicExpressionAccess().getNotExpressionParserRuleCall_0()); }
		ruleNotExpression
		{ after(grammarAccess.getUnaryLogicExpressionAccess().getNotExpressionParserRuleCall_0()); }
	)
	|
	(
		{ before(grammarAccess.getUnaryLogicExpressionAccess().getForallExpressionParserRuleCall_1()); }
		ruleForallExpression
		{ after(grammarAccess.getUnaryLogicExpressionAccess().getForallExpressionParserRuleCall_1()); }
	)
	|
	(
		{ before(grammarAccess.getUnaryLogicExpressionAccess().getExistsExpressionParserRuleCall_2()); }
		ruleExistsExpression
		{ after(grammarAccess.getUnaryLogicExpressionAccess().getExistsExpressionParserRuleCall_2()); }
	)
	|
	(
		{ before(grammarAccess.getUnaryLogicExpressionAccess().getEquivalenceExpressionParserRuleCall_3()); }
		ruleEquivalenceExpression
		{ after(grammarAccess.getUnaryLogicExpressionAccess().getEquivalenceExpressionParserRuleCall_3()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__NotExpression__Alternatives_1_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getNotExpressionAccess().getNotKeyword_1_0_0()); }
		'not'
		{ after(grammarAccess.getNotExpressionAccess().getNotKeyword_1_0_0()); }
	)
	|
	(
		{ before(grammarAccess.getNotExpressionAccess().getExclamationMarkKeyword_1_0_1()); }
		'!'
		{ after(grammarAccess.getNotExpressionAccess().getExclamationMarkKeyword_1_0_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__EquivalenceExpression__Alternatives_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getEquivalenceExpressionAccess().getGroup_1_0()); }
		(rule__EquivalenceExpression__Group_1_0__0)
		{ after(grammarAccess.getEquivalenceExpressionAccess().getGroup_1_0()); }
	)
	|
	(
		{ before(grammarAccess.getEquivalenceExpressionAccess().getGroup_1_1()); }
		(rule__EquivalenceExpression__Group_1_1__0)
		{ after(grammarAccess.getEquivalenceExpressionAccess().getGroup_1_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__EquivalenceExpression__Alternatives_1_0_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getEquivalenceExpressionAccess().getEqualsSignKeyword_1_0_1_0()); }
		'='
		{ after(grammarAccess.getEquivalenceExpressionAccess().getEqualsSignKeyword_1_0_1_0()); }
	)
	|
	(
		{ before(grammarAccess.getEquivalenceExpressionAccess().getEqualsSignEqualsSignKeyword_1_0_1_1()); }
		'=='
		{ after(grammarAccess.getEquivalenceExpressionAccess().getEqualsSignEqualsSignKeyword_1_0_1_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__EquivalenceExpression__Alternatives_1_1_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getEquivalenceExpressionAccess().getSolidusEqualsSignKeyword_1_1_1_0()); }
		'/='
		{ after(grammarAccess.getEquivalenceExpressionAccess().getSolidusEqualsSignKeyword_1_1_1_0()); }
	)
	|
	(
		{ before(grammarAccess.getEquivalenceExpressionAccess().getExclamationMarkEqualsSignKeyword_1_1_1_1()); }
		'!='
		{ after(grammarAccess.getEquivalenceExpressionAccess().getExclamationMarkEqualsSignKeyword_1_1_1_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComparisionExpression__Alternatives_1_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getComparisionExpressionAccess().getGroup_1_0_0()); }
		(rule__ComparisionExpression__Group_1_0_0__0)
		{ after(grammarAccess.getComparisionExpressionAccess().getGroup_1_0_0()); }
	)
	|
	(
		{ before(grammarAccess.getComparisionExpressionAccess().getGroup_1_0_1()); }
		(rule__ComparisionExpression__Group_1_0_1__0)
		{ after(grammarAccess.getComparisionExpressionAccess().getGroup_1_0_1()); }
	)
	|
	(
		{ before(grammarAccess.getComparisionExpressionAccess().getGroup_1_0_2()); }
		(rule__ComparisionExpression__Group_1_0_2__0)
		{ after(grammarAccess.getComparisionExpressionAccess().getGroup_1_0_2()); }
	)
	|
	(
		{ before(grammarAccess.getComparisionExpressionAccess().getGroup_1_0_3()); }
		(rule__ComparisionExpression__Group_1_0_3__0)
		{ after(grammarAccess.getComparisionExpressionAccess().getGroup_1_0_3()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AdditiveExpression__Alternatives_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAdditiveExpressionAccess().getGroup_1_0()); }
		(rule__AdditiveExpression__Group_1_0__0)
		{ after(grammarAccess.getAdditiveExpressionAccess().getGroup_1_0()); }
	)
	|
	(
		{ before(grammarAccess.getAdditiveExpressionAccess().getGroup_1_1()); }
		(rule__AdditiveExpression__Group_1_1__0)
		{ after(grammarAccess.getAdditiveExpressionAccess().getGroup_1_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__MultiplicativeExpression__Alternatives_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getMultiplicativeExpressionAccess().getGroup_1_0()); }
		(rule__MultiplicativeExpression__Group_1_0__0)
		{ after(grammarAccess.getMultiplicativeExpressionAccess().getGroup_1_0()); }
	)
	|
	(
		{ before(grammarAccess.getMultiplicativeExpressionAccess().getGroup_1_1()); }
		(rule__MultiplicativeExpression__Group_1_1__0)
		{ after(grammarAccess.getMultiplicativeExpressionAccess().getGroup_1_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__MultiplicativeExpression__Alternatives_1_1_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getMultiplicativeExpressionAccess().getGroup_1_1_0_0()); }
		(rule__MultiplicativeExpression__Group_1_1_0_0__0)
		{ after(grammarAccess.getMultiplicativeExpressionAccess().getGroup_1_1_0_0()); }
	)
	|
	(
		{ before(grammarAccess.getMultiplicativeExpressionAccess().getGroup_1_1_0_1()); }
		(rule__MultiplicativeExpression__Group_1_1_0_1__0)
		{ after(grammarAccess.getMultiplicativeExpressionAccess().getGroup_1_1_0_1()); }
	)
	|
	(
		{ before(grammarAccess.getMultiplicativeExpressionAccess().getGroup_1_1_0_2()); }
		(rule__MultiplicativeExpression__Group_1_1_0_2__0)
		{ after(grammarAccess.getMultiplicativeExpressionAccess().getGroup_1_1_0_2()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SignumExpression__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSignumExpressionAccess().getGroup_0()); }
		(rule__SignumExpression__Group_0__0)
		{ after(grammarAccess.getSignumExpressionAccess().getGroup_0()); }
	)
	|
	(
		{ before(grammarAccess.getSignumExpressionAccess().getGroup_1()); }
		(rule__SignumExpression__Group_1__0)
		{ after(grammarAccess.getSignumExpressionAccess().getGroup_1()); }
	)
	|
	(
		{ before(grammarAccess.getSignumExpressionAccess().getAccessExpressionParserRuleCall_2()); }
		ruleAccessExpression
		{ after(grammarAccess.getSignumExpressionAccess().getAccessExpressionParserRuleCall_2()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AccessExpression__Alternatives_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAccessExpressionAccess().getGroup_1_0()); }
		(rule__AccessExpression__Group_1_0__0)
		{ after(grammarAccess.getAccessExpressionAccess().getGroup_1_0()); }
	)
	|
	(
		{ before(grammarAccess.getAccessExpressionAccess().getGroup_1_1()); }
		(rule__AccessExpression__Group_1_1__0)
		{ after(grammarAccess.getAccessExpressionAccess().getGroup_1_1()); }
	)
	|
	(
		{ before(grammarAccess.getAccessExpressionAccess().getGroup_1_2()); }
		(rule__AccessExpression__Group_1_2__0)
		{ after(grammarAccess.getAccessExpressionAccess().getGroup_1_2()); }
	)
	|
	(
		{ before(grammarAccess.getAccessExpressionAccess().getGroup_1_3()); }
		(rule__AccessExpression__Group_1_3__0)
		{ after(grammarAccess.getAccessExpressionAccess().getGroup_1_3()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__LiteralExpression__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getLiteralExpressionAccess().getIntegerLiteralExpressionParserRuleCall_0()); }
		ruleIntegerLiteralExpression
		{ after(grammarAccess.getLiteralExpressionAccess().getIntegerLiteralExpressionParserRuleCall_0()); }
	)
	|
	(
		{ before(grammarAccess.getLiteralExpressionAccess().getDecimalLiteralExpressionParserRuleCall_1()); }
		ruleDecimalLiteralExpression
		{ after(grammarAccess.getLiteralExpressionAccess().getDecimalLiteralExpressionParserRuleCall_1()); }
	)
	|
	(
		{ before(grammarAccess.getLiteralExpressionAccess().getRationalLiteralExpressionParserRuleCall_2()); }
		ruleRationalLiteralExpression
		{ after(grammarAccess.getLiteralExpressionAccess().getRationalLiteralExpressionParserRuleCall_2()); }
	)
	|
	(
		{ before(grammarAccess.getLiteralExpressionAccess().getBooleanLiteralExpressionParserRuleCall_3()); }
		ruleBooleanLiteralExpression
		{ after(grammarAccess.getLiteralExpressionAccess().getBooleanLiteralExpressionParserRuleCall_3()); }
	)
	|
	(
		{ before(grammarAccess.getLiteralExpressionAccess().getArrayLiteralExpressionParserRuleCall_4()); }
		ruleArrayLiteralExpression
		{ after(grammarAccess.getLiteralExpressionAccess().getArrayLiteralExpressionParserRuleCall_4()); }
	)
	|
	(
		{ before(grammarAccess.getLiteralExpressionAccess().getRecordLiteralExpressionParserRuleCall_5()); }
		ruleRecordLiteralExpression
		{ after(grammarAccess.getLiteralExpressionAccess().getRecordLiteralExpressionParserRuleCall_5()); }
	)
	|
	(
		{ before(grammarAccess.getLiteralExpressionAccess().getEnumerationLiteralExpressionParserRuleCall_6()); }
		ruleEnumerationLiteralExpression
		{ after(grammarAccess.getLiteralExpressionAccess().getEnumerationLiteralExpressionParserRuleCall_6()); }
	)
	|
	(
		{ before(grammarAccess.getLiteralExpressionAccess().getIntegerRangeLiteralExpressionParserRuleCall_7()); }
		ruleIntegerRangeLiteralExpression
		{ after(grammarAccess.getLiteralExpressionAccess().getIntegerRangeLiteralExpressionParserRuleCall_7()); }
	)
	|
	(
		{ before(grammarAccess.getLiteralExpressionAccess().getOpaqueExpressionParserRuleCall_8()); }
		ruleOpaqueExpression
		{ after(grammarAccess.getLiteralExpressionAccess().getOpaqueExpressionParserRuleCall_8()); }
	)
	|
	(
		{ before(grammarAccess.getLiteralExpressionAccess().getInfinityExpressionParserRuleCall_9()); }
		ruleInfinityExpression
		{ after(grammarAccess.getLiteralExpressionAccess().getInfinityExpressionParserRuleCall_9()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__BooleanLiteralExpression__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getBooleanLiteralExpressionAccess().getTrueExpressionParserRuleCall_0()); }
		ruleTrueExpression
		{ after(grammarAccess.getBooleanLiteralExpressionAccess().getTrueExpressionParserRuleCall_0()); }
	)
	|
	(
		{ before(grammarAccess.getBooleanLiteralExpressionAccess().getFalseExpressionParserRuleCall_1()); }
		ruleFalseExpression
		{ after(grammarAccess.getBooleanLiteralExpressionAccess().getFalseExpressionParserRuleCall_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__IntegerRangeLiteralExpression__Alternatives_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getIntegerRangeLiteralExpressionAccess().getLeftInclusiveAssignment_1_0()); }
		(rule__IntegerRangeLiteralExpression__LeftInclusiveAssignment_1_0)
		{ after(grammarAccess.getIntegerRangeLiteralExpressionAccess().getLeftInclusiveAssignment_1_0()); }
	)
	|
	(
		{ before(grammarAccess.getIntegerRangeLiteralExpressionAccess().getLessThanSignHyphenMinusKeyword_1_1()); }
		'<-'
		{ after(grammarAccess.getIntegerRangeLiteralExpressionAccess().getLessThanSignHyphenMinusKeyword_1_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__IntegerRangeLiteralExpression__Alternatives_5
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getIntegerRangeLiteralExpressionAccess().getRightInclusiveAssignment_5_0()); }
		(rule__IntegerRangeLiteralExpression__RightInclusiveAssignment_5_0)
		{ after(grammarAccess.getIntegerRangeLiteralExpressionAccess().getRightInclusiveAssignment_5_0()); }
	)
	|
	(
		{ before(grammarAccess.getIntegerRangeLiteralExpressionAccess().getHyphenMinusGreaterThanSignKeyword_5_1()); }
		'->'
		{ after(grammarAccess.getIntegerRangeLiteralExpressionAccess().getHyphenMinusGreaterThanSignKeyword_5_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__LocalDynamics__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getLocalDynamicsAccess().getPERMANENTEnumLiteralDeclaration_0()); }
		('permanent')
		{ after(grammarAccess.getLocalDynamicsAccess().getPERMANENTEnumLiteralDeclaration_0()); }
	)
	|
	(
		{ before(grammarAccess.getLocalDynamicsAccess().getTRANSIENTEnumLiteralDeclaration_1()); }
		('transient')
		{ after(grammarAccess.getLocalDynamicsAccess().getTRANSIENTEnumLiteralDeclaration_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__Occurrence__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getOccurrenceAccess().getINSTANTANEOUSEnumLiteralDeclaration_0()); }
		('instantaneous')
		{ after(grammarAccess.getOccurrenceAccess().getINSTANTANEOUSEnumLiteralDeclaration_0()); }
	)
	|
	(
		{ before(grammarAccess.getOccurrenceAccess().getDELAYEDEnumLiteralDeclaration_1()); }
		('delayed')
		{ after(grammarAccess.getOccurrenceAccess().getDELAYEDEnumLiteralDeclaration_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__TermReferenceMode__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getTermReferenceModeAccess().getREFERENCEEnumLiteralDeclaration_0()); }
		('reference')
		{ after(grammarAccess.getTermReferenceModeAccess().getREFERENCEEnumLiteralDeclaration_0()); }
	)
	|
	(
		{ before(grammarAccess.getTermReferenceModeAccess().getVALUEEnumLiteralDeclaration_1()); }
		('value')
		{ after(grammarAccess.getTermReferenceModeAccess().getVALUEEnumLiteralDeclaration_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultModeState__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getFaultModeStateAccess().getFAULTYEnumLiteralDeclaration_0()); }
		('fault')
		{ after(grammarAccess.getFaultModeStateAccess().getFAULTYEnumLiteralDeclaration_0()); }
	)
	|
	(
		{ before(grammarAccess.getFaultModeStateAccess().getNOMINALEnumLiteralDeclaration_1()); }
		('nominal')
		{ after(grammarAccess.getFaultModeStateAccess().getNOMINALEnumLiteralDeclaration_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultEvent__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getFaultEventAccess().getFAILUREEnumLiteralDeclaration_0()); }
		('failure')
		{ after(grammarAccess.getFaultEventAccess().getFAILUREEnumLiteralDeclaration_0()); }
	)
	|
	(
		{ before(grammarAccess.getFaultEventAccess().getSELF_FIXEnumLiteralDeclaration_1()); }
		('self-fix')
		{ after(grammarAccess.getFaultEventAccess().getSELF_FIXEnumLiteralDeclaration_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__RealizationMode__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getRealizationModeAccess().getPROVIDEDEnumLiteralDeclaration_0()); }
		('provides')
		{ after(grammarAccess.getRealizationModeAccess().getPROVIDEDEnumLiteralDeclaration_0()); }
	)
	|
	(
		{ before(grammarAccess.getRealizationModeAccess().getREQUIREDEnumLiteralDeclaration_1()); }
		('requires')
		{ after(grammarAccess.getRealizationModeAccess().getREQUIREDEnumLiteralDeclaration_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SchedulingOrder__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSchedulingOrderAccess().getTOP_DOWNEnumLiteralDeclaration_0()); }
		('top-down')
		{ after(grammarAccess.getSchedulingOrderAccess().getTOP_DOWNEnumLiteralDeclaration_0()); }
	)
	|
	(
		{ before(grammarAccess.getSchedulingOrderAccess().getBOTTOM_UPEnumLiteralDeclaration_1()); }
		('bottom-up')
		{ after(grammarAccess.getSchedulingOrderAccess().getBOTTOM_UPEnumLiteralDeclaration_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__OrthogonalRegionSchedulingOrder__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getOrthogonalRegionSchedulingOrderAccess().getSEQUENTIALEnumLiteralDeclaration_0()); }
		('sequential')
		{ after(grammarAccess.getOrthogonalRegionSchedulingOrderAccess().getSEQUENTIALEnumLiteralDeclaration_0()); }
	)
	|
	(
		{ before(grammarAccess.getOrthogonalRegionSchedulingOrderAccess().getUNORDEREDEnumLiteralDeclaration_1()); }
		('unordered')
		{ after(grammarAccess.getOrthogonalRegionSchedulingOrderAccess().getUNORDEREDEnumLiteralDeclaration_1()); }
	)
	|
	(
		{ before(grammarAccess.getOrthogonalRegionSchedulingOrderAccess().getPARALLELEnumLiteralDeclaration_2()); }
		('parallel')
		{ after(grammarAccess.getOrthogonalRegionSchedulingOrderAccess().getPARALLELEnumLiteralDeclaration_2()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__TransitionPriority__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getTransitionPriorityAccess().getOFFEnumLiteralDeclaration_0()); }
		('off')
		{ after(grammarAccess.getTransitionPriorityAccess().getOFFEnumLiteralDeclaration_0()); }
	)
	|
	(
		{ before(grammarAccess.getTransitionPriorityAccess().getORDER_BASEDEnumLiteralDeclaration_1()); }
		('order-based')
		{ after(grammarAccess.getTransitionPriorityAccess().getORDER_BASEDEnumLiteralDeclaration_1()); }
	)
	|
	(
		{ before(grammarAccess.getTransitionPriorityAccess().getVALUE_BASEDEnumLiteralDeclaration_2()); }
		('value-based')
		{ after(grammarAccess.getTransitionPriorityAccess().getVALUE_BASEDEnumLiteralDeclaration_2()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__GuardEvaluation__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getGuardEvaluationAccess().getON_THE_FLYEnumLiteralDeclaration_0()); }
		('on-the-fly')
		{ after(grammarAccess.getGuardEvaluationAccess().getON_THE_FLYEnumLiteralDeclaration_0()); }
	)
	|
	(
		{ before(grammarAccess.getGuardEvaluationAccess().getBEGINNING_OF_STEPEnumLiteralDeclaration_1()); }
		('beginning-of-step')
		{ after(grammarAccess.getGuardEvaluationAccess().getBEGINNING_OF_STEPEnumLiteralDeclaration_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__NotDefinedEventMode__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getNotDefinedEventModeAccess().getPERMISSIVEEnumLiteralDeclaration_0()); }
		('permissive')
		{ after(grammarAccess.getNotDefinedEventModeAccess().getPERMISSIVEEnumLiteralDeclaration_0()); }
	)
	|
	(
		{ before(grammarAccess.getNotDefinedEventModeAccess().getSTRICTEnumLiteralDeclaration_1()); }
		('strict')
		{ after(grammarAccess.getNotDefinedEventModeAccess().getSTRICTEnumLiteralDeclaration_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__LinkType__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getLinkTypeAccess().getTO_CONTROLLEREnumLiteralDeclaration_0()); }
		('self')
		{ after(grammarAccess.getLinkTypeAccess().getTO_CONTROLLEREnumLiteralDeclaration_0()); }
	)
	|
	(
		{ before(grammarAccess.getLinkTypeAccess().getTO_COMPONENTEnumLiteralDeclaration_1()); }
		('component')
		{ after(grammarAccess.getLinkTypeAccess().getTO_COMPONENTEnumLiteralDeclaration_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__History__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getHistoryAccess().getNO_HISTORYEnumLiteralDeclaration_0()); }
		('no-history')
		{ after(grammarAccess.getHistoryAccess().getNO_HISTORYEnumLiteralDeclaration_0()); }
	)
	|
	(
		{ before(grammarAccess.getHistoryAccess().getSHALLOW_HISTORYEnumLiteralDeclaration_1()); }
		('shallow')
		{ after(grammarAccess.getHistoryAccess().getSHALLOW_HISTORYEnumLiteralDeclaration_1()); }
	)
	|
	(
		{ before(grammarAccess.getHistoryAccess().getDEEP_HISTORYEnumLiteralDeclaration_2()); }
		('deep')
		{ after(grammarAccess.getHistoryAccess().getDEEP_HISTORYEnumLiteralDeclaration_2()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__TimeUnit__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getTimeUnitAccess().getMILLISECONDEnumLiteralDeclaration_0()); }
		('ms')
		{ after(grammarAccess.getTimeUnitAccess().getMILLISECONDEnumLiteralDeclaration_0()); }
	)
	|
	(
		{ before(grammarAccess.getTimeUnitAccess().getSECONDEnumLiteralDeclaration_1()); }
		('s')
		{ after(grammarAccess.getTimeUnitAccess().getSECONDEnumLiteralDeclaration_1()); }
	)
	|
	(
		{ before(grammarAccess.getTimeUnitAccess().getHOUREnumLiteralDeclaration_2()); }
		('h')
		{ after(grammarAccess.getTimeUnitAccess().getHOUREnumLiteralDeclaration_2()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ControlFunction__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getControlFunctionAccess().getRUN_ONCEEnumLiteralDeclaration_0()); }
		('run')
		{ after(grammarAccess.getControlFunctionAccess().getRUN_ONCEEnumLiteralDeclaration_0()); }
	)
	|
	(
		{ before(grammarAccess.getControlFunctionAccess().getRUN_TO_COMPLETIONEnumLiteralDeclaration_1()); }
		('full-step')
		{ after(grammarAccess.getControlFunctionAccess().getRUN_TO_COMPLETIONEnumLiteralDeclaration_1()); }
	)
	|
	(
		{ before(grammarAccess.getControlFunctionAccess().getRESETEnumLiteralDeclaration_2()); }
		('reset')
		{ after(grammarAccess.getControlFunctionAccess().getRESETEnumLiteralDeclaration_2()); }
	)
	|
	(
		{ before(grammarAccess.getControlFunctionAccess().getRESET_MESSAGE_QUEUEEnumLiteralDeclaration_3()); }
		('reset-queue')
		{ after(grammarAccess.getControlFunctionAccess().getRESET_MESSAGE_QUEUEEnumLiteralDeclaration_3()); }
	)
	|
	(
		{ before(grammarAccess.getControlFunctionAccess().getRESET_MESSAGE_QUEUESEnumLiteralDeclaration_4()); }
		('reset-queues')
		{ after(grammarAccess.getControlFunctionAccess().getRESET_MESSAGE_QUEUESEnumLiteralDeclaration_4()); }
	)
	|
	(
		{ before(grammarAccess.getControlFunctionAccess().getRESET_OTHER_MESSAGE_QUEUESEnumLiteralDeclaration_5()); }
		('reset-other-queues')
		{ after(grammarAccess.getControlFunctionAccess().getRESET_OTHER_MESSAGE_QUEUESEnumLiteralDeclaration_5()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__DiscardStrategy__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getDiscardStrategyAccess().getINCOMINGEnumLiteralDeclaration_0()); }
		('incoming')
		{ after(grammarAccess.getDiscardStrategyAccess().getINCOMINGEnumLiteralDeclaration_0()); }
	)
	|
	(
		{ before(grammarAccess.getDiscardStrategyAccess().getOLDESTEnumLiteralDeclaration_1()); }
		('oldest')
		{ after(grammarAccess.getDiscardStrategyAccess().getOLDESTEnumLiteralDeclaration_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__EventDirection__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getEventDirectionAccess().getINEnumLiteralDeclaration_0()); }
		('in')
		{ after(grammarAccess.getEventDirectionAccess().getINEnumLiteralDeclaration_0()); }
	)
	|
	(
		{ before(grammarAccess.getEventDirectionAccess().getOUTEnumLiteralDeclaration_1()); }
		('out')
		{ after(grammarAccess.getEventDirectionAccess().getOUTEnumLiteralDeclaration_1()); }
	)
	|
	(
		{ before(grammarAccess.getEventDirectionAccess().getINOUTEnumLiteralDeclaration_2()); }
		('inout')
		{ after(grammarAccess.getEventDirectionAccess().getINOUTEnumLiteralDeclaration_2()); }
	)
	|
	(
		{ before(grammarAccess.getEventDirectionAccess().getINTERNALEnumLiteralDeclaration_3()); }
		('internal')
		{ after(grammarAccess.getEventDirectionAccess().getINTERNALEnumLiteralDeclaration_3()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__Persistency__Alternatives
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getPersistencyAccess().getPERSISTENTEnumLiteralDeclaration_0()); }
		('persistent')
		{ after(grammarAccess.getPersistencyAccess().getPERSISTENTEnumLiteralDeclaration_0()); }
	)
	|
	(
		{ before(grammarAccess.getPersistencyAccess().getTRANSIENTEnumLiteralDeclaration_1()); }
		('transient')
		{ after(grammarAccess.getPersistencyAccess().getTRANSIENTEnumLiteralDeclaration_1()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultExtensionInstructions__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultExtensionInstructions__Group__0__Impl
	rule__FaultExtensionInstructions__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultExtensionInstructions__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultExtensionInstructionsAccess().getImportKeyword_0()); }
	'import'
	{ after(grammarAccess.getFaultExtensionInstructionsAccess().getImportKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultExtensionInstructions__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultExtensionInstructions__Group__1__Impl
	rule__FaultExtensionInstructions__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultExtensionInstructions__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultExtensionInstructionsAccess().getImportsAssignment_1()); }
	(rule__FaultExtensionInstructions__ImportsAssignment_1)
	{ after(grammarAccess.getFaultExtensionInstructionsAccess().getImportsAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultExtensionInstructions__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultExtensionInstructions__Group__2__Impl
	rule__FaultExtensionInstructions__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultExtensionInstructions__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultExtensionInstructionsAccess().getFaultExtensionKeyword_2()); }
	'fault-extension'
	{ after(grammarAccess.getFaultExtensionInstructionsAccess().getFaultExtensionKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultExtensionInstructions__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultExtensionInstructions__Group__3__Impl
	rule__FaultExtensionInstructions__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultExtensionInstructions__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultExtensionInstructionsAccess().getNameAssignment_3()); }
	(rule__FaultExtensionInstructions__NameAssignment_3)
	{ after(grammarAccess.getFaultExtensionInstructionsAccess().getNameAssignment_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultExtensionInstructions__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultExtensionInstructions__Group__4__Impl
	rule__FaultExtensionInstructions__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultExtensionInstructions__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultExtensionInstructionsAccess().getForKeyword_4()); }
	'for'
	{ after(grammarAccess.getFaultExtensionInstructionsAccess().getForKeyword_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultExtensionInstructions__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultExtensionInstructions__Group__5__Impl
	rule__FaultExtensionInstructions__Group__6
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultExtensionInstructions__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultExtensionInstructionsAccess().getComponentAssignment_5()); }
	(rule__FaultExtensionInstructions__ComponentAssignment_5)
	{ after(grammarAccess.getFaultExtensionInstructionsAccess().getComponentAssignment_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultExtensionInstructions__Group__6
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultExtensionInstructions__Group__6__Impl
	rule__FaultExtensionInstructions__Group__7
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultExtensionInstructions__Group__6__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultExtensionInstructionsAccess().getFaultSlicesAssignment_6()); }
	(rule__FaultExtensionInstructions__FaultSlicesAssignment_6)*
	{ after(grammarAccess.getFaultExtensionInstructionsAccess().getFaultSlicesAssignment_6()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultExtensionInstructions__Group__7
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultExtensionInstructions__Group__7__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultExtensionInstructions__Group__7__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultExtensionInstructionsAccess().getCommonCausesAssignment_7()); }
	(rule__FaultExtensionInstructions__CommonCausesAssignment_7)?
	{ after(grammarAccess.getFaultExtensionInstructionsAccess().getCommonCausesAssignment_7()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__FaultSlice__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultSlice__Group__0__Impl
	rule__FaultSlice__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultSlice__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultSliceAccess().getSliceKeyword_0()); }
	'slice'
	{ after(grammarAccess.getFaultSliceAccess().getSliceKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultSlice__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultSlice__Group__1__Impl
	rule__FaultSlice__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultSlice__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultSliceAccess().getNameAssignment_1()); }
	(rule__FaultSlice__NameAssignment_1)
	{ after(grammarAccess.getFaultSliceAccess().getNameAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultSlice__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultSlice__Group__2__Impl
	rule__FaultSlice__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultSlice__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultSliceAccess().getAffectsKeyword_2()); }
	'affects'
	{ after(grammarAccess.getFaultSliceAccess().getAffectsKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultSlice__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultSlice__Group__3__Impl
	rule__FaultSlice__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultSlice__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultSliceAccess().getAffectedElementsAssignment_3()); }
	(rule__FaultSlice__AffectedElementsAssignment_3)
	{ after(grammarAccess.getFaultSliceAccess().getAffectedElementsAssignment_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultSlice__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultSlice__Group__4__Impl
	rule__FaultSlice__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultSlice__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultSliceAccess().getGroup_4()); }
	(rule__FaultSlice__Group_4__0)*
	{ after(grammarAccess.getFaultSliceAccess().getGroup_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultSlice__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultSlice__Group__5__Impl
	rule__FaultSlice__Group__6
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultSlice__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultSliceAccess().getWithKeyword_5()); }
	'with'
	{ after(grammarAccess.getFaultSliceAccess().getWithKeyword_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultSlice__Group__6
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultSlice__Group__6__Impl
	rule__FaultSlice__Group__7
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultSlice__Group__6__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	(
		{ before(grammarAccess.getFaultSliceAccess().getFaultModesAssignment_6()); }
		(rule__FaultSlice__FaultModesAssignment_6)
		{ after(grammarAccess.getFaultSliceAccess().getFaultModesAssignment_6()); }
	)
	(
		{ before(grammarAccess.getFaultSliceAccess().getFaultModesAssignment_6()); }
		(rule__FaultSlice__FaultModesAssignment_6)*
		{ after(grammarAccess.getFaultSliceAccess().getFaultModesAssignment_6()); }
	)
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultSlice__Group__7
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultSlice__Group__7__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultSlice__Group__7__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultSliceAccess().getGlobalDynamicsAssignment_7()); }
	(rule__FaultSlice__GlobalDynamicsAssignment_7)?
	{ after(grammarAccess.getFaultSliceAccess().getGlobalDynamicsAssignment_7()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__FaultSlice__Group_4__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultSlice__Group_4__0__Impl
	rule__FaultSlice__Group_4__1
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultSlice__Group_4__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultSliceAccess().getCommaKeyword_4_0()); }
	','
	{ after(grammarAccess.getFaultSliceAccess().getCommaKeyword_4_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultSlice__Group_4__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultSlice__Group_4__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultSlice__Group_4__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultSliceAccess().getAffectedElementsAssignment_4_1()); }
	(rule__FaultSlice__AffectedElementsAssignment_4_1)
	{ after(grammarAccess.getFaultSliceAccess().getAffectedElementsAssignment_4_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__FaultMode__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultMode__Group__0__Impl
	rule__FaultMode__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultMode__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultModeAccess().getModeKeyword_0()); }
	'mode'
	{ after(grammarAccess.getFaultModeAccess().getModeKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultMode__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultMode__Group__1__Impl
	rule__FaultMode__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultMode__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultModeAccess().getNameAssignment_1()); }
	(rule__FaultMode__NameAssignment_1)
	{ after(grammarAccess.getFaultModeAccess().getNameAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultMode__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultMode__Group__2__Impl
	rule__FaultMode__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultMode__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultModeAccess().getGroup_2()); }
	(rule__FaultMode__Group_2__0)?
	{ after(grammarAccess.getFaultModeAccess().getGroup_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultMode__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultMode__Group__3__Impl
	rule__FaultMode__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultMode__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultModeAccess().getLocalDynamicsAssignment_3()); }
	(rule__FaultMode__LocalDynamicsAssignment_3)?
	{ after(grammarAccess.getFaultModeAccess().getLocalDynamicsAssignment_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultMode__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultMode__Group__4__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultMode__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultModeAccess().getEffectAssignment_4()); }
	(rule__FaultMode__EffectAssignment_4)
	{ after(grammarAccess.getFaultModeAccess().getEffectAssignment_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__FaultMode__Group_2__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultMode__Group_2__0__Impl
	rule__FaultMode__Group_2__1
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultMode__Group_2__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultModeAccess().getLeftParenthesisKeyword_2_0()); }
	'('
	{ after(grammarAccess.getFaultModeAccess().getLeftParenthesisKeyword_2_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultMode__Group_2__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultMode__Group_2__1__Impl
	rule__FaultMode__Group_2__2
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultMode__Group_2__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultModeAccess().getProbabilityAssignment_2_1()); }
	(rule__FaultMode__ProbabilityAssignment_2_1)
	{ after(grammarAccess.getFaultModeAccess().getProbabilityAssignment_2_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultMode__Group_2__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultMode__Group_2__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultMode__Group_2__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultModeAccess().getRightParenthesisKeyword_2_2()); }
	')'
	{ after(grammarAccess.getFaultModeAccess().getRightParenthesisKeyword_2_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__StuckAtEffect__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtEffect__Group__0__Impl
	rule__StuckAtEffect__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtEffectAccess().getStuckAtKeyword_0()); }
	'stuck-at'
	{ after(grammarAccess.getStuckAtEffectAccess().getStuckAtKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtEffect__Group__1__Impl
	rule__StuckAtEffect__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtEffectAccess().getLeftCurlyBracketKeyword_1()); }
	'{'
	{ after(grammarAccess.getStuckAtEffectAccess().getLeftCurlyBracketKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtEffect__Group__2__Impl
	rule__StuckAtEffect__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtEffectAccess().getGroup_2()); }
	(rule__StuckAtEffect__Group_2__0)?
	{ after(grammarAccess.getStuckAtEffectAccess().getGroup_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtEffect__Group__3__Impl
	rule__StuckAtEffect__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtEffectAccess().getGroup_3()); }
	(rule__StuckAtEffect__Group_3__0)?
	{ after(grammarAccess.getStuckAtEffectAccess().getGroup_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtEffect__Group__4__Impl
	rule__StuckAtEffect__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtEffectAccess().getGroup_4()); }
	(rule__StuckAtEffect__Group_4__0)
	{ after(grammarAccess.getStuckAtEffectAccess().getGroup_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtEffect__Group__5__Impl
	rule__StuckAtEffect__Group__6
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtEffectAccess().getGroup_5()); }
	(rule__StuckAtEffect__Group_5__0)?
	{ after(grammarAccess.getStuckAtEffectAccess().getGroup_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group__6
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtEffect__Group__6__Impl
	rule__StuckAtEffect__Group__7
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group__6__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtEffectAccess().getGroup_6()); }
	(rule__StuckAtEffect__Group_6__0)?
	{ after(grammarAccess.getStuckAtEffectAccess().getGroup_6()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group__7
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtEffect__Group__7__Impl
	rule__StuckAtEffect__Group__8
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group__7__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtEffectAccess().getTemplateAssignment_7()); }
	(rule__StuckAtEffect__TemplateAssignment_7)?
	{ after(grammarAccess.getStuckAtEffectAccess().getTemplateAssignment_7()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group__8
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtEffect__Group__8__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group__8__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtEffectAccess().getRightCurlyBracketKeyword_8()); }
	'}'
	{ after(grammarAccess.getStuckAtEffectAccess().getRightCurlyBracketKeyword_8()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__StuckAtEffect__Group_2__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtEffect__Group_2__0__Impl
	rule__StuckAtEffect__Group_2__1
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group_2__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtEffectAccess().getReferenceModeKeyword_2_0()); }
	'reference-mode'
	{ after(grammarAccess.getStuckAtEffectAccess().getReferenceModeKeyword_2_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group_2__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtEffect__Group_2__1__Impl
	rule__StuckAtEffect__Group_2__2
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group_2__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtEffectAccess().getColonKeyword_2_1()); }
	':'
	{ after(grammarAccess.getStuckAtEffectAccess().getColonKeyword_2_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group_2__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtEffect__Group_2__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group_2__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtEffectAccess().getTermReferenceModeAssignment_2_2()); }
	(rule__StuckAtEffect__TermReferenceModeAssignment_2_2)
	{ after(grammarAccess.getStuckAtEffectAccess().getTermReferenceModeAssignment_2_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__StuckAtEffect__Group_3__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtEffect__Group_3__0__Impl
	rule__StuckAtEffect__Group_3__1
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group_3__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtEffectAccess().getOccurrenceKeyword_3_0()); }
	'occurrence'
	{ after(grammarAccess.getStuckAtEffectAccess().getOccurrenceKeyword_3_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group_3__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtEffect__Group_3__1__Impl
	rule__StuckAtEffect__Group_3__2
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group_3__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtEffectAccess().getColonKeyword_3_1()); }
	':'
	{ after(grammarAccess.getStuckAtEffectAccess().getColonKeyword_3_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group_3__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtEffect__Group_3__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group_3__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtEffectAccess().getOccurrenceAssignment_3_2()); }
	(rule__StuckAtEffect__OccurrenceAssignment_3_2)
	{ after(grammarAccess.getStuckAtEffectAccess().getOccurrenceAssignment_3_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__StuckAtEffect__Group_4__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtEffect__Group_4__0__Impl
	rule__StuckAtEffect__Group_4__1
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group_4__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtEffectAccess().getTermKeyword_4_0()); }
	'term'
	{ after(grammarAccess.getStuckAtEffectAccess().getTermKeyword_4_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group_4__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtEffect__Group_4__1__Impl
	rule__StuckAtEffect__Group_4__2
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group_4__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtEffectAccess().getColonKeyword_4_1()); }
	':'
	{ after(grammarAccess.getStuckAtEffectAccess().getColonKeyword_4_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group_4__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtEffect__Group_4__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group_4__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtEffectAccess().getTermAssignment_4_2()); }
	(rule__StuckAtEffect__TermAssignment_4_2)
	{ after(grammarAccess.getStuckAtEffectAccess().getTermAssignment_4_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__StuckAtEffect__Group_5__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtEffect__Group_5__0__Impl
	rule__StuckAtEffect__Group_5__1
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group_5__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtEffectAccess().getGroup_5_0()); }
	(rule__StuckAtEffect__Group_5_0__0)
	{ after(grammarAccess.getStuckAtEffectAccess().getGroup_5_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group_5__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtEffect__Group_5__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group_5__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtEffectAccess().getGroup_5_1()); }
	(rule__StuckAtEffect__Group_5_1__0)
	{ after(grammarAccess.getStuckAtEffectAccess().getGroup_5_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__StuckAtEffect__Group_5_0__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtEffect__Group_5_0__0__Impl
	rule__StuckAtEffect__Group_5_0__1
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group_5_0__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtEffectAccess().getInputKeyword_5_0_0()); }
	'input'
	{ after(grammarAccess.getStuckAtEffectAccess().getInputKeyword_5_0_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group_5_0__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtEffect__Group_5_0__1__Impl
	rule__StuckAtEffect__Group_5_0__2
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group_5_0__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtEffectAccess().getColonKeyword_5_0_1()); }
	':'
	{ after(grammarAccess.getStuckAtEffectAccess().getColonKeyword_5_0_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group_5_0__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtEffect__Group_5_0__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group_5_0__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtEffectAccess().getInputAssignment_5_0_2()); }
	(rule__StuckAtEffect__InputAssignment_5_0_2)
	{ after(grammarAccess.getStuckAtEffectAccess().getInputAssignment_5_0_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__StuckAtEffect__Group_5_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtEffect__Group_5_1__0__Impl
	rule__StuckAtEffect__Group_5_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group_5_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtEffectAccess().getVaroutKeyword_5_1_0()); }
	'varout'
	{ after(grammarAccess.getStuckAtEffectAccess().getVaroutKeyword_5_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group_5_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtEffect__Group_5_1__1__Impl
	rule__StuckAtEffect__Group_5_1__2
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group_5_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtEffectAccess().getColonKeyword_5_1_1()); }
	':'
	{ after(grammarAccess.getStuckAtEffectAccess().getColonKeyword_5_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group_5_1__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtEffect__Group_5_1__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group_5_1__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtEffectAccess().getVaroutAssignment_5_1_2()); }
	(rule__StuckAtEffect__VaroutAssignment_5_1_2)
	{ after(grammarAccess.getStuckAtEffectAccess().getVaroutAssignment_5_1_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__StuckAtEffect__Group_6__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtEffect__Group_6__0__Impl
	rule__StuckAtEffect__Group_6__1
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group_6__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtEffectAccess().getFailureEventKeyword_6_0()); }
	'failure-event'
	{ after(grammarAccess.getStuckAtEffectAccess().getFailureEventKeyword_6_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group_6__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtEffect__Group_6__1__Impl
	rule__StuckAtEffect__Group_6__2
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group_6__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtEffectAccess().getColonKeyword_6_1()); }
	':'
	{ after(grammarAccess.getStuckAtEffectAccess().getColonKeyword_6_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group_6__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtEffect__Group_6__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__Group_6__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtEffectAccess().getFailureEventAssignment_6_2()); }
	(rule__StuckAtEffect__FailureEventAssignment_6_2)
	{ after(grammarAccess.getStuckAtEffectAccess().getFailureEventAssignment_6_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__FrozenEffect__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FrozenEffect__Group__0__Impl
	rule__FrozenEffect__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__FrozenEffect__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFrozenEffectAccess().getFrozenEffectAction_0()); }
	()
	{ after(grammarAccess.getFrozenEffectAccess().getFrozenEffectAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FrozenEffect__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FrozenEffect__Group__1__Impl
	rule__FrozenEffect__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__FrozenEffect__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFrozenEffectAccess().getFrozenKeyword_1()); }
	'frozen'
	{ after(grammarAccess.getFrozenEffectAccess().getFrozenKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FrozenEffect__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FrozenEffect__Group__2__Impl
	rule__FrozenEffect__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__FrozenEffect__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFrozenEffectAccess().getLeftCurlyBracketKeyword_2()); }
	'{'
	{ after(grammarAccess.getFrozenEffectAccess().getLeftCurlyBracketKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FrozenEffect__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FrozenEffect__Group__3__Impl
	rule__FrozenEffect__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__FrozenEffect__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFrozenEffectAccess().getGroup_3()); }
	(rule__FrozenEffect__Group_3__0)?
	{ after(grammarAccess.getFrozenEffectAccess().getGroup_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FrozenEffect__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FrozenEffect__Group__4__Impl
	rule__FrozenEffect__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__FrozenEffect__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFrozenEffectAccess().getGroup_4()); }
	(rule__FrozenEffect__Group_4__0)?
	{ after(grammarAccess.getFrozenEffectAccess().getGroup_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FrozenEffect__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FrozenEffect__Group__5__Impl
	rule__FrozenEffect__Group__6
;
finally {
	restoreStackSize(stackSize);
}

rule__FrozenEffect__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFrozenEffectAccess().getTemplateAssignment_5()); }
	(rule__FrozenEffect__TemplateAssignment_5)?
	{ after(grammarAccess.getFrozenEffectAccess().getTemplateAssignment_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FrozenEffect__Group__6
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FrozenEffect__Group__6__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__FrozenEffect__Group__6__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFrozenEffectAccess().getRightCurlyBracketKeyword_6()); }
	'}'
	{ after(grammarAccess.getFrozenEffectAccess().getRightCurlyBracketKeyword_6()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__FrozenEffect__Group_3__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FrozenEffect__Group_3__0__Impl
	rule__FrozenEffect__Group_3__1
;
finally {
	restoreStackSize(stackSize);
}

rule__FrozenEffect__Group_3__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFrozenEffectAccess().getGroup_3_0()); }
	(rule__FrozenEffect__Group_3_0__0)
	{ after(grammarAccess.getFrozenEffectAccess().getGroup_3_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FrozenEffect__Group_3__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FrozenEffect__Group_3__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__FrozenEffect__Group_3__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFrozenEffectAccess().getGroup_3_1()); }
	(rule__FrozenEffect__Group_3_1__0)
	{ after(grammarAccess.getFrozenEffectAccess().getGroup_3_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__FrozenEffect__Group_3_0__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FrozenEffect__Group_3_0__0__Impl
	rule__FrozenEffect__Group_3_0__1
;
finally {
	restoreStackSize(stackSize);
}

rule__FrozenEffect__Group_3_0__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFrozenEffectAccess().getInputKeyword_3_0_0()); }
	'input'
	{ after(grammarAccess.getFrozenEffectAccess().getInputKeyword_3_0_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FrozenEffect__Group_3_0__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FrozenEffect__Group_3_0__1__Impl
	rule__FrozenEffect__Group_3_0__2
;
finally {
	restoreStackSize(stackSize);
}

rule__FrozenEffect__Group_3_0__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFrozenEffectAccess().getColonKeyword_3_0_1()); }
	':'
	{ after(grammarAccess.getFrozenEffectAccess().getColonKeyword_3_0_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FrozenEffect__Group_3_0__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FrozenEffect__Group_3_0__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__FrozenEffect__Group_3_0__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFrozenEffectAccess().getInputAssignment_3_0_2()); }
	(rule__FrozenEffect__InputAssignment_3_0_2)
	{ after(grammarAccess.getFrozenEffectAccess().getInputAssignment_3_0_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__FrozenEffect__Group_3_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FrozenEffect__Group_3_1__0__Impl
	rule__FrozenEffect__Group_3_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__FrozenEffect__Group_3_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFrozenEffectAccess().getVaroutKeyword_3_1_0()); }
	'varout'
	{ after(grammarAccess.getFrozenEffectAccess().getVaroutKeyword_3_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FrozenEffect__Group_3_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FrozenEffect__Group_3_1__1__Impl
	rule__FrozenEffect__Group_3_1__2
;
finally {
	restoreStackSize(stackSize);
}

rule__FrozenEffect__Group_3_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFrozenEffectAccess().getColonKeyword_3_1_1()); }
	':'
	{ after(grammarAccess.getFrozenEffectAccess().getColonKeyword_3_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FrozenEffect__Group_3_1__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FrozenEffect__Group_3_1__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__FrozenEffect__Group_3_1__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFrozenEffectAccess().getVaroutAssignment_3_1_2()); }
	(rule__FrozenEffect__VaroutAssignment_3_1_2)
	{ after(grammarAccess.getFrozenEffectAccess().getVaroutAssignment_3_1_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__FrozenEffect__Group_4__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FrozenEffect__Group_4__0__Impl
	rule__FrozenEffect__Group_4__1
;
finally {
	restoreStackSize(stackSize);
}

rule__FrozenEffect__Group_4__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFrozenEffectAccess().getFailureEventKeyword_4_0()); }
	'failure-event'
	{ after(grammarAccess.getFrozenEffectAccess().getFailureEventKeyword_4_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FrozenEffect__Group_4__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FrozenEffect__Group_4__1__Impl
	rule__FrozenEffect__Group_4__2
;
finally {
	restoreStackSize(stackSize);
}

rule__FrozenEffect__Group_4__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFrozenEffectAccess().getColonKeyword_4_1()); }
	':'
	{ after(grammarAccess.getFrozenEffectAccess().getColonKeyword_4_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FrozenEffect__Group_4__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FrozenEffect__Group_4__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__FrozenEffect__Group_4__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFrozenEffectAccess().getFailureEventAssignment_4_2()); }
	(rule__FrozenEffect__FailureEventAssignment_4_2)
	{ after(grammarAccess.getFrozenEffectAccess().getFailureEventAssignment_4_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__NonDeterminismEffect__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismEffect__Group__0__Impl
	rule__NonDeterminismEffect__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismEffectAccess().getNondeterminismKeyword_0()); }
	'nondeterminism'
	{ after(grammarAccess.getNonDeterminismEffectAccess().getNondeterminismKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismEffect__Group__1__Impl
	rule__NonDeterminismEffect__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismEffectAccess().getLeftCurlyBracketKeyword_1()); }
	'{'
	{ after(grammarAccess.getNonDeterminismEffectAccess().getLeftCurlyBracketKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismEffect__Group__2__Impl
	rule__NonDeterminismEffect__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismEffectAccess().getGroup_2()); }
	(rule__NonDeterminismEffect__Group_2__0)?
	{ after(grammarAccess.getNonDeterminismEffectAccess().getGroup_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismEffect__Group__3__Impl
	rule__NonDeterminismEffect__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismEffectAccess().getGroup_3()); }
	(rule__NonDeterminismEffect__Group_3__0)?
	{ after(grammarAccess.getNonDeterminismEffectAccess().getGroup_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismEffect__Group__4__Impl
	rule__NonDeterminismEffect__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismEffectAccess().getGroup_4()); }
	(rule__NonDeterminismEffect__Group_4__0)
	{ after(grammarAccess.getNonDeterminismEffectAccess().getGroup_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismEffect__Group__5__Impl
	rule__NonDeterminismEffect__Group__6
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismEffectAccess().getGroup_5()); }
	(rule__NonDeterminismEffect__Group_5__0)
	{ after(grammarAccess.getNonDeterminismEffectAccess().getGroup_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group__6
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismEffect__Group__6__Impl
	rule__NonDeterminismEffect__Group__7
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group__6__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismEffectAccess().getGroup_6()); }
	(rule__NonDeterminismEffect__Group_6__0)?
	{ after(grammarAccess.getNonDeterminismEffectAccess().getGroup_6()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group__7
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismEffect__Group__7__Impl
	rule__NonDeterminismEffect__Group__8
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group__7__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismEffectAccess().getGroup_7()); }
	(rule__NonDeterminismEffect__Group_7__0)?
	{ after(grammarAccess.getNonDeterminismEffectAccess().getGroup_7()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group__8
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismEffect__Group__8__Impl
	rule__NonDeterminismEffect__Group__9
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group__8__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismEffectAccess().getTemplateAssignment_8()); }
	(rule__NonDeterminismEffect__TemplateAssignment_8)?
	{ after(grammarAccess.getNonDeterminismEffectAccess().getTemplateAssignment_8()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group__9
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismEffect__Group__9__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group__9__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismEffectAccess().getRightCurlyBracketKeyword_9()); }
	'}'
	{ after(grammarAccess.getNonDeterminismEffectAccess().getRightCurlyBracketKeyword_9()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__NonDeterminismEffect__Group_2__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismEffect__Group_2__0__Impl
	rule__NonDeterminismEffect__Group_2__1
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group_2__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismEffectAccess().getReferenceModeKeyword_2_0()); }
	'reference-mode'
	{ after(grammarAccess.getNonDeterminismEffectAccess().getReferenceModeKeyword_2_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group_2__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismEffect__Group_2__1__Impl
	rule__NonDeterminismEffect__Group_2__2
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group_2__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismEffectAccess().getColonKeyword_2_1()); }
	':'
	{ after(grammarAccess.getNonDeterminismEffectAccess().getColonKeyword_2_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group_2__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismEffect__Group_2__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group_2__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismEffectAccess().getTermReferenceModeAssignment_2_2()); }
	(rule__NonDeterminismEffect__TermReferenceModeAssignment_2_2)
	{ after(grammarAccess.getNonDeterminismEffectAccess().getTermReferenceModeAssignment_2_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__NonDeterminismEffect__Group_3__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismEffect__Group_3__0__Impl
	rule__NonDeterminismEffect__Group_3__1
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group_3__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismEffectAccess().getOccurrenceKeyword_3_0()); }
	'occurrence'
	{ after(grammarAccess.getNonDeterminismEffectAccess().getOccurrenceKeyword_3_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group_3__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismEffect__Group_3__1__Impl
	rule__NonDeterminismEffect__Group_3__2
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group_3__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismEffectAccess().getColonKeyword_3_1()); }
	':'
	{ after(grammarAccess.getNonDeterminismEffectAccess().getColonKeyword_3_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group_3__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismEffect__Group_3__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group_3__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismEffectAccess().getOccurrenceAssignment_3_2()); }
	(rule__NonDeterminismEffect__OccurrenceAssignment_3_2)
	{ after(grammarAccess.getNonDeterminismEffectAccess().getOccurrenceAssignment_3_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__NonDeterminismEffect__Group_4__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismEffect__Group_4__0__Impl
	rule__NonDeterminismEffect__Group_4__1
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group_4__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismEffectAccess().getMinimumKeyword_4_0()); }
	'minimum'
	{ after(grammarAccess.getNonDeterminismEffectAccess().getMinimumKeyword_4_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group_4__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismEffect__Group_4__1__Impl
	rule__NonDeterminismEffect__Group_4__2
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group_4__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismEffectAccess().getColonKeyword_4_1()); }
	':'
	{ after(grammarAccess.getNonDeterminismEffectAccess().getColonKeyword_4_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group_4__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismEffect__Group_4__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group_4__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismEffectAccess().getMinimumAssignment_4_2()); }
	(rule__NonDeterminismEffect__MinimumAssignment_4_2)
	{ after(grammarAccess.getNonDeterminismEffectAccess().getMinimumAssignment_4_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__NonDeterminismEffect__Group_5__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismEffect__Group_5__0__Impl
	rule__NonDeterminismEffect__Group_5__1
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group_5__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismEffectAccess().getMaximumKeyword_5_0()); }
	'maximum'
	{ after(grammarAccess.getNonDeterminismEffectAccess().getMaximumKeyword_5_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group_5__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismEffect__Group_5__1__Impl
	rule__NonDeterminismEffect__Group_5__2
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group_5__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismEffectAccess().getColonKeyword_5_1()); }
	':'
	{ after(grammarAccess.getNonDeterminismEffectAccess().getColonKeyword_5_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group_5__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismEffect__Group_5__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group_5__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismEffectAccess().getMaximumAssignment_5_2()); }
	(rule__NonDeterminismEffect__MaximumAssignment_5_2)
	{ after(grammarAccess.getNonDeterminismEffectAccess().getMaximumAssignment_5_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__NonDeterminismEffect__Group_6__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismEffect__Group_6__0__Impl
	rule__NonDeterminismEffect__Group_6__1
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group_6__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismEffectAccess().getGroup_6_0()); }
	(rule__NonDeterminismEffect__Group_6_0__0)
	{ after(grammarAccess.getNonDeterminismEffectAccess().getGroup_6_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group_6__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismEffect__Group_6__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group_6__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismEffectAccess().getGroup_6_1()); }
	(rule__NonDeterminismEffect__Group_6_1__0)
	{ after(grammarAccess.getNonDeterminismEffectAccess().getGroup_6_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__NonDeterminismEffect__Group_6_0__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismEffect__Group_6_0__0__Impl
	rule__NonDeterminismEffect__Group_6_0__1
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group_6_0__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismEffectAccess().getInputKeyword_6_0_0()); }
	'input'
	{ after(grammarAccess.getNonDeterminismEffectAccess().getInputKeyword_6_0_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group_6_0__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismEffect__Group_6_0__1__Impl
	rule__NonDeterminismEffect__Group_6_0__2
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group_6_0__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismEffectAccess().getColonKeyword_6_0_1()); }
	':'
	{ after(grammarAccess.getNonDeterminismEffectAccess().getColonKeyword_6_0_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group_6_0__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismEffect__Group_6_0__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group_6_0__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismEffectAccess().getInputAssignment_6_0_2()); }
	(rule__NonDeterminismEffect__InputAssignment_6_0_2)
	{ after(grammarAccess.getNonDeterminismEffectAccess().getInputAssignment_6_0_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__NonDeterminismEffect__Group_6_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismEffect__Group_6_1__0__Impl
	rule__NonDeterminismEffect__Group_6_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group_6_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismEffectAccess().getVaroutKeyword_6_1_0()); }
	'varout'
	{ after(grammarAccess.getNonDeterminismEffectAccess().getVaroutKeyword_6_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group_6_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismEffect__Group_6_1__1__Impl
	rule__NonDeterminismEffect__Group_6_1__2
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group_6_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismEffectAccess().getColonKeyword_6_1_1()); }
	':'
	{ after(grammarAccess.getNonDeterminismEffectAccess().getColonKeyword_6_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group_6_1__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismEffect__Group_6_1__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group_6_1__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismEffectAccess().getVaroutAssignment_6_1_2()); }
	(rule__NonDeterminismEffect__VaroutAssignment_6_1_2)
	{ after(grammarAccess.getNonDeterminismEffectAccess().getVaroutAssignment_6_1_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__NonDeterminismEffect__Group_7__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismEffect__Group_7__0__Impl
	rule__NonDeterminismEffect__Group_7__1
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group_7__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismEffectAccess().getFailureEventKeyword_7_0()); }
	'failure-event'
	{ after(grammarAccess.getNonDeterminismEffectAccess().getFailureEventKeyword_7_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group_7__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismEffect__Group_7__1__Impl
	rule__NonDeterminismEffect__Group_7__2
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group_7__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismEffectAccess().getColonKeyword_7_1()); }
	':'
	{ after(grammarAccess.getNonDeterminismEffectAccess().getColonKeyword_7_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group_7__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismEffect__Group_7__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__Group_7__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismEffectAccess().getFailureEventAssignment_7_2()); }
	(rule__NonDeterminismEffect__FailureEventAssignment_7_2)
	{ after(grammarAccess.getNonDeterminismEffectAccess().getFailureEventAssignment_7_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__NonDeterminismBooleanEffect__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismBooleanEffect__Group__0__Impl
	rule__NonDeterminismBooleanEffect__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismBooleanEffectAccess().getNonDeterminismBooleanEffectAction_0()); }
	()
	{ after(grammarAccess.getNonDeterminismBooleanEffectAccess().getNonDeterminismBooleanEffectAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismBooleanEffect__Group__1__Impl
	rule__NonDeterminismBooleanEffect__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismBooleanEffectAccess().getNondeterminismBooleanKeyword_1()); }
	'nondeterminism-boolean'
	{ after(grammarAccess.getNonDeterminismBooleanEffectAccess().getNondeterminismBooleanKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismBooleanEffect__Group__2__Impl
	rule__NonDeterminismBooleanEffect__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismBooleanEffectAccess().getLeftCurlyBracketKeyword_2()); }
	'{'
	{ after(grammarAccess.getNonDeterminismBooleanEffectAccess().getLeftCurlyBracketKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismBooleanEffect__Group__3__Impl
	rule__NonDeterminismBooleanEffect__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismBooleanEffectAccess().getGroup_3()); }
	(rule__NonDeterminismBooleanEffect__Group_3__0)?
	{ after(grammarAccess.getNonDeterminismBooleanEffectAccess().getGroup_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismBooleanEffect__Group__4__Impl
	rule__NonDeterminismBooleanEffect__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismBooleanEffectAccess().getGroup_4()); }
	(rule__NonDeterminismBooleanEffect__Group_4__0)?
	{ after(grammarAccess.getNonDeterminismBooleanEffectAccess().getGroup_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismBooleanEffect__Group__5__Impl
	rule__NonDeterminismBooleanEffect__Group__6
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismBooleanEffectAccess().getGroup_5()); }
	(rule__NonDeterminismBooleanEffect__Group_5__0)?
	{ after(grammarAccess.getNonDeterminismBooleanEffectAccess().getGroup_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__Group__6
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismBooleanEffect__Group__6__Impl
	rule__NonDeterminismBooleanEffect__Group__7
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__Group__6__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismBooleanEffectAccess().getTemplateAssignment_6()); }
	(rule__NonDeterminismBooleanEffect__TemplateAssignment_6)?
	{ after(grammarAccess.getNonDeterminismBooleanEffectAccess().getTemplateAssignment_6()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__Group__7
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismBooleanEffect__Group__7__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__Group__7__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismBooleanEffectAccess().getRightCurlyBracketKeyword_7()); }
	'}'
	{ after(grammarAccess.getNonDeterminismBooleanEffectAccess().getRightCurlyBracketKeyword_7()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__NonDeterminismBooleanEffect__Group_3__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismBooleanEffect__Group_3__0__Impl
	rule__NonDeterminismBooleanEffect__Group_3__1
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__Group_3__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismBooleanEffectAccess().getReferenceModeKeyword_3_0()); }
	'reference-mode'
	{ after(grammarAccess.getNonDeterminismBooleanEffectAccess().getReferenceModeKeyword_3_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__Group_3__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismBooleanEffect__Group_3__1__Impl
	rule__NonDeterminismBooleanEffect__Group_3__2
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__Group_3__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismBooleanEffectAccess().getColonKeyword_3_1()); }
	':'
	{ after(grammarAccess.getNonDeterminismBooleanEffectAccess().getColonKeyword_3_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__Group_3__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismBooleanEffect__Group_3__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__Group_3__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismBooleanEffectAccess().getTermReferenceModeAssignment_3_2()); }
	(rule__NonDeterminismBooleanEffect__TermReferenceModeAssignment_3_2)
	{ after(grammarAccess.getNonDeterminismBooleanEffectAccess().getTermReferenceModeAssignment_3_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__NonDeterminismBooleanEffect__Group_4__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismBooleanEffect__Group_4__0__Impl
	rule__NonDeterminismBooleanEffect__Group_4__1
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__Group_4__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismBooleanEffectAccess().getGroup_4_0()); }
	(rule__NonDeterminismBooleanEffect__Group_4_0__0)
	{ after(grammarAccess.getNonDeterminismBooleanEffectAccess().getGroup_4_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__Group_4__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismBooleanEffect__Group_4__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__Group_4__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismBooleanEffectAccess().getGroup_4_1()); }
	(rule__NonDeterminismBooleanEffect__Group_4_1__0)
	{ after(grammarAccess.getNonDeterminismBooleanEffectAccess().getGroup_4_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__NonDeterminismBooleanEffect__Group_4_0__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismBooleanEffect__Group_4_0__0__Impl
	rule__NonDeterminismBooleanEffect__Group_4_0__1
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__Group_4_0__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismBooleanEffectAccess().getInputKeyword_4_0_0()); }
	'input'
	{ after(grammarAccess.getNonDeterminismBooleanEffectAccess().getInputKeyword_4_0_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__Group_4_0__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismBooleanEffect__Group_4_0__1__Impl
	rule__NonDeterminismBooleanEffect__Group_4_0__2
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__Group_4_0__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismBooleanEffectAccess().getColonKeyword_4_0_1()); }
	':'
	{ after(grammarAccess.getNonDeterminismBooleanEffectAccess().getColonKeyword_4_0_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__Group_4_0__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismBooleanEffect__Group_4_0__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__Group_4_0__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismBooleanEffectAccess().getInputAssignment_4_0_2()); }
	(rule__NonDeterminismBooleanEffect__InputAssignment_4_0_2)
	{ after(grammarAccess.getNonDeterminismBooleanEffectAccess().getInputAssignment_4_0_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__NonDeterminismBooleanEffect__Group_4_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismBooleanEffect__Group_4_1__0__Impl
	rule__NonDeterminismBooleanEffect__Group_4_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__Group_4_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismBooleanEffectAccess().getVaroutKeyword_4_1_0()); }
	'varout'
	{ after(grammarAccess.getNonDeterminismBooleanEffectAccess().getVaroutKeyword_4_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__Group_4_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismBooleanEffect__Group_4_1__1__Impl
	rule__NonDeterminismBooleanEffect__Group_4_1__2
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__Group_4_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismBooleanEffectAccess().getColonKeyword_4_1_1()); }
	':'
	{ after(grammarAccess.getNonDeterminismBooleanEffectAccess().getColonKeyword_4_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__Group_4_1__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismBooleanEffect__Group_4_1__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__Group_4_1__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismBooleanEffectAccess().getVaroutAssignment_4_1_2()); }
	(rule__NonDeterminismBooleanEffect__VaroutAssignment_4_1_2)
	{ after(grammarAccess.getNonDeterminismBooleanEffectAccess().getVaroutAssignment_4_1_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__NonDeterminismBooleanEffect__Group_5__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismBooleanEffect__Group_5__0__Impl
	rule__NonDeterminismBooleanEffect__Group_5__1
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__Group_5__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismBooleanEffectAccess().getFailureEventKeyword_5_0()); }
	'failure-event'
	{ after(grammarAccess.getNonDeterminismBooleanEffectAccess().getFailureEventKeyword_5_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__Group_5__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismBooleanEffect__Group_5__1__Impl
	rule__NonDeterminismBooleanEffect__Group_5__2
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__Group_5__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismBooleanEffectAccess().getColonKeyword_5_1()); }
	':'
	{ after(grammarAccess.getNonDeterminismBooleanEffectAccess().getColonKeyword_5_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__Group_5__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NonDeterminismBooleanEffect__Group_5__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__Group_5__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNonDeterminismBooleanEffectAccess().getFailureEventAssignment_5_2()); }
	(rule__NonDeterminismBooleanEffect__FailureEventAssignment_5_2)
	{ after(grammarAccess.getNonDeterminismBooleanEffectAccess().getFailureEventAssignment_5_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ConditionalEffect__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ConditionalEffect__Group__0__Impl
	rule__ConditionalEffect__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getConditionalEffectAccess().getConditionalKeyword_0()); }
	'conditional'
	{ after(grammarAccess.getConditionalEffectAccess().getConditionalKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ConditionalEffect__Group__1__Impl
	rule__ConditionalEffect__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getConditionalEffectAccess().getLeftCurlyBracketKeyword_1()); }
	'{'
	{ after(grammarAccess.getConditionalEffectAccess().getLeftCurlyBracketKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ConditionalEffect__Group__2__Impl
	rule__ConditionalEffect__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getConditionalEffectAccess().getGroup_2()); }
	(rule__ConditionalEffect__Group_2__0)?
	{ after(grammarAccess.getConditionalEffectAccess().getGroup_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ConditionalEffect__Group__3__Impl
	rule__ConditionalEffect__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getConditionalEffectAccess().getGroup_3()); }
	(rule__ConditionalEffect__Group_3__0)
	{ after(grammarAccess.getConditionalEffectAccess().getGroup_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ConditionalEffect__Group__4__Impl
	rule__ConditionalEffect__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getConditionalEffectAccess().getGroup_4()); }
	(rule__ConditionalEffect__Group_4__0)
	{ after(grammarAccess.getConditionalEffectAccess().getGroup_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ConditionalEffect__Group__5__Impl
	rule__ConditionalEffect__Group__6
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getConditionalEffectAccess().getGroup_5()); }
	(rule__ConditionalEffect__Group_5__0)
	{ after(grammarAccess.getConditionalEffectAccess().getGroup_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group__6
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ConditionalEffect__Group__6__Impl
	rule__ConditionalEffect__Group__7
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group__6__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getConditionalEffectAccess().getGroup_6()); }
	(rule__ConditionalEffect__Group_6__0)?
	{ after(grammarAccess.getConditionalEffectAccess().getGroup_6()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group__7
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ConditionalEffect__Group__7__Impl
	rule__ConditionalEffect__Group__8
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group__7__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getConditionalEffectAccess().getGroup_7()); }
	(rule__ConditionalEffect__Group_7__0)?
	{ after(grammarAccess.getConditionalEffectAccess().getGroup_7()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group__8
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ConditionalEffect__Group__8__Impl
	rule__ConditionalEffect__Group__9
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group__8__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getConditionalEffectAccess().getTemplateAssignment_8()); }
	(rule__ConditionalEffect__TemplateAssignment_8)?
	{ after(grammarAccess.getConditionalEffectAccess().getTemplateAssignment_8()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group__9
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ConditionalEffect__Group__9__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group__9__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getConditionalEffectAccess().getRightCurlyBracketKeyword_9()); }
	'}'
	{ after(grammarAccess.getConditionalEffectAccess().getRightCurlyBracketKeyword_9()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ConditionalEffect__Group_2__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ConditionalEffect__Group_2__0__Impl
	rule__ConditionalEffect__Group_2__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group_2__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getConditionalEffectAccess().getOccurrenceKeyword_2_0()); }
	'occurrence'
	{ after(grammarAccess.getConditionalEffectAccess().getOccurrenceKeyword_2_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group_2__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ConditionalEffect__Group_2__1__Impl
	rule__ConditionalEffect__Group_2__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group_2__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getConditionalEffectAccess().getColonKeyword_2_1()); }
	':'
	{ after(grammarAccess.getConditionalEffectAccess().getColonKeyword_2_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group_2__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ConditionalEffect__Group_2__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group_2__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getConditionalEffectAccess().getOccurrenceAssignment_2_2()); }
	(rule__ConditionalEffect__OccurrenceAssignment_2_2)
	{ after(grammarAccess.getConditionalEffectAccess().getOccurrenceAssignment_2_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ConditionalEffect__Group_3__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ConditionalEffect__Group_3__0__Impl
	rule__ConditionalEffect__Group_3__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group_3__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getConditionalEffectAccess().getConditionKeyword_3_0()); }
	'condition'
	{ after(grammarAccess.getConditionalEffectAccess().getConditionKeyword_3_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group_3__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ConditionalEffect__Group_3__1__Impl
	rule__ConditionalEffect__Group_3__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group_3__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getConditionalEffectAccess().getColonKeyword_3_1()); }
	':'
	{ after(grammarAccess.getConditionalEffectAccess().getColonKeyword_3_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group_3__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ConditionalEffect__Group_3__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group_3__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getConditionalEffectAccess().getConditionAssignment_3_2()); }
	(rule__ConditionalEffect__ConditionAssignment_3_2)
	{ after(grammarAccess.getConditionalEffectAccess().getConditionAssignment_3_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ConditionalEffect__Group_4__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ConditionalEffect__Group_4__0__Impl
	rule__ConditionalEffect__Group_4__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group_4__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getConditionalEffectAccess().getThenKeyword_4_0()); }
	'then'
	{ after(grammarAccess.getConditionalEffectAccess().getThenKeyword_4_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group_4__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ConditionalEffect__Group_4__1__Impl
	rule__ConditionalEffect__Group_4__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group_4__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getConditionalEffectAccess().getColonKeyword_4_1()); }
	':'
	{ after(grammarAccess.getConditionalEffectAccess().getColonKeyword_4_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group_4__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ConditionalEffect__Group_4__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group_4__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getConditionalEffectAccess().getThenAssignment_4_2()); }
	(rule__ConditionalEffect__ThenAssignment_4_2)
	{ after(grammarAccess.getConditionalEffectAccess().getThenAssignment_4_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ConditionalEffect__Group_5__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ConditionalEffect__Group_5__0__Impl
	rule__ConditionalEffect__Group_5__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group_5__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getConditionalEffectAccess().getElseKeyword_5_0()); }
	'else'
	{ after(grammarAccess.getConditionalEffectAccess().getElseKeyword_5_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group_5__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ConditionalEffect__Group_5__1__Impl
	rule__ConditionalEffect__Group_5__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group_5__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getConditionalEffectAccess().getColonKeyword_5_1()); }
	':'
	{ after(grammarAccess.getConditionalEffectAccess().getColonKeyword_5_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group_5__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ConditionalEffect__Group_5__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group_5__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getConditionalEffectAccess().getElseAssignment_5_2()); }
	(rule__ConditionalEffect__ElseAssignment_5_2)
	{ after(grammarAccess.getConditionalEffectAccess().getElseAssignment_5_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ConditionalEffect__Group_6__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ConditionalEffect__Group_6__0__Impl
	rule__ConditionalEffect__Group_6__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group_6__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getConditionalEffectAccess().getGroup_6_0()); }
	(rule__ConditionalEffect__Group_6_0__0)
	{ after(grammarAccess.getConditionalEffectAccess().getGroup_6_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group_6__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ConditionalEffect__Group_6__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group_6__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getConditionalEffectAccess().getGroup_6_1()); }
	(rule__ConditionalEffect__Group_6_1__0)
	{ after(grammarAccess.getConditionalEffectAccess().getGroup_6_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ConditionalEffect__Group_6_0__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ConditionalEffect__Group_6_0__0__Impl
	rule__ConditionalEffect__Group_6_0__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group_6_0__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getConditionalEffectAccess().getInputKeyword_6_0_0()); }
	'input'
	{ after(grammarAccess.getConditionalEffectAccess().getInputKeyword_6_0_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group_6_0__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ConditionalEffect__Group_6_0__1__Impl
	rule__ConditionalEffect__Group_6_0__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group_6_0__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getConditionalEffectAccess().getColonKeyword_6_0_1()); }
	':'
	{ after(grammarAccess.getConditionalEffectAccess().getColonKeyword_6_0_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group_6_0__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ConditionalEffect__Group_6_0__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group_6_0__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getConditionalEffectAccess().getInputAssignment_6_0_2()); }
	(rule__ConditionalEffect__InputAssignment_6_0_2)
	{ after(grammarAccess.getConditionalEffectAccess().getInputAssignment_6_0_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ConditionalEffect__Group_6_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ConditionalEffect__Group_6_1__0__Impl
	rule__ConditionalEffect__Group_6_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group_6_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getConditionalEffectAccess().getVaroutKeyword_6_1_0()); }
	'varout'
	{ after(grammarAccess.getConditionalEffectAccess().getVaroutKeyword_6_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group_6_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ConditionalEffect__Group_6_1__1__Impl
	rule__ConditionalEffect__Group_6_1__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group_6_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getConditionalEffectAccess().getColonKeyword_6_1_1()); }
	':'
	{ after(grammarAccess.getConditionalEffectAccess().getColonKeyword_6_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group_6_1__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ConditionalEffect__Group_6_1__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group_6_1__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getConditionalEffectAccess().getVaroutAssignment_6_1_2()); }
	(rule__ConditionalEffect__VaroutAssignment_6_1_2)
	{ after(grammarAccess.getConditionalEffectAccess().getVaroutAssignment_6_1_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ConditionalEffect__Group_7__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ConditionalEffect__Group_7__0__Impl
	rule__ConditionalEffect__Group_7__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group_7__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getConditionalEffectAccess().getFailureEventKeyword_7_0()); }
	'failure-event'
	{ after(grammarAccess.getConditionalEffectAccess().getFailureEventKeyword_7_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group_7__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ConditionalEffect__Group_7__1__Impl
	rule__ConditionalEffect__Group_7__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group_7__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getConditionalEffectAccess().getColonKeyword_7_1()); }
	':'
	{ after(grammarAccess.getConditionalEffectAccess().getColonKeyword_7_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group_7__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ConditionalEffect__Group_7__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__Group_7__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getConditionalEffectAccess().getFailureEventAssignment_7_2()); }
	(rule__ConditionalEffect__FailureEventAssignment_7_2)
	{ after(grammarAccess.getConditionalEffectAccess().getFailureEventAssignment_7_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__RampDownEffect__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RampDownEffect__Group__0__Impl
	rule__RampDownEffect__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRampDownEffectAccess().getRampDownKeyword_0()); }
	'ramp-down'
	{ after(grammarAccess.getRampDownEffectAccess().getRampDownKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RampDownEffect__Group__1__Impl
	rule__RampDownEffect__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRampDownEffectAccess().getLeftCurlyBracketKeyword_1()); }
	'{'
	{ after(grammarAccess.getRampDownEffectAccess().getLeftCurlyBracketKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RampDownEffect__Group__2__Impl
	rule__RampDownEffect__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRampDownEffectAccess().getGroup_2()); }
	(rule__RampDownEffect__Group_2__0)
	{ after(grammarAccess.getRampDownEffectAccess().getGroup_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RampDownEffect__Group__3__Impl
	rule__RampDownEffect__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRampDownEffectAccess().getGroup_3()); }
	(rule__RampDownEffect__Group_3__0)
	{ after(grammarAccess.getRampDownEffectAccess().getGroup_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RampDownEffect__Group__4__Impl
	rule__RampDownEffect__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRampDownEffectAccess().getGroup_4()); }
	(rule__RampDownEffect__Group_4__0)?
	{ after(grammarAccess.getRampDownEffectAccess().getGroup_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RampDownEffect__Group__5__Impl
	rule__RampDownEffect__Group__6
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRampDownEffectAccess().getGroup_5()); }
	(rule__RampDownEffect__Group_5__0)?
	{ after(grammarAccess.getRampDownEffectAccess().getGroup_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group__6
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RampDownEffect__Group__6__Impl
	rule__RampDownEffect__Group__7
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group__6__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRampDownEffectAccess().getTemplateAssignment_6()); }
	(rule__RampDownEffect__TemplateAssignment_6)?
	{ after(grammarAccess.getRampDownEffectAccess().getTemplateAssignment_6()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group__7
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RampDownEffect__Group__7__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group__7__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRampDownEffectAccess().getRightCurlyBracketKeyword_7()); }
	'}'
	{ after(grammarAccess.getRampDownEffectAccess().getRightCurlyBracketKeyword_7()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__RampDownEffect__Group_2__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RampDownEffect__Group_2__0__Impl
	rule__RampDownEffect__Group_2__1
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group_2__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRampDownEffectAccess().getDeltaKeyword_2_0()); }
	'delta'
	{ after(grammarAccess.getRampDownEffectAccess().getDeltaKeyword_2_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group_2__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RampDownEffect__Group_2__1__Impl
	rule__RampDownEffect__Group_2__2
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group_2__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRampDownEffectAccess().getColonKeyword_2_1()); }
	':'
	{ after(grammarAccess.getRampDownEffectAccess().getColonKeyword_2_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group_2__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RampDownEffect__Group_2__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group_2__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRampDownEffectAccess().getDeltaAssignment_2_2()); }
	(rule__RampDownEffect__DeltaAssignment_2_2)
	{ after(grammarAccess.getRampDownEffectAccess().getDeltaAssignment_2_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__RampDownEffect__Group_3__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RampDownEffect__Group_3__0__Impl
	rule__RampDownEffect__Group_3__1
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group_3__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRampDownEffectAccess().getBoundKeyword_3_0()); }
	'bound'
	{ after(grammarAccess.getRampDownEffectAccess().getBoundKeyword_3_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group_3__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RampDownEffect__Group_3__1__Impl
	rule__RampDownEffect__Group_3__2
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group_3__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRampDownEffectAccess().getColonKeyword_3_1()); }
	':'
	{ after(grammarAccess.getRampDownEffectAccess().getColonKeyword_3_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group_3__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RampDownEffect__Group_3__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group_3__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRampDownEffectAccess().getBoundAssignment_3_2()); }
	(rule__RampDownEffect__BoundAssignment_3_2)
	{ after(grammarAccess.getRampDownEffectAccess().getBoundAssignment_3_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__RampDownEffect__Group_4__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RampDownEffect__Group_4__0__Impl
	rule__RampDownEffect__Group_4__1
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group_4__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRampDownEffectAccess().getGroup_4_0()); }
	(rule__RampDownEffect__Group_4_0__0)
	{ after(grammarAccess.getRampDownEffectAccess().getGroup_4_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group_4__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RampDownEffect__Group_4__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group_4__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRampDownEffectAccess().getGroup_4_1()); }
	(rule__RampDownEffect__Group_4_1__0)
	{ after(grammarAccess.getRampDownEffectAccess().getGroup_4_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__RampDownEffect__Group_4_0__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RampDownEffect__Group_4_0__0__Impl
	rule__RampDownEffect__Group_4_0__1
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group_4_0__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRampDownEffectAccess().getInputKeyword_4_0_0()); }
	'input'
	{ after(grammarAccess.getRampDownEffectAccess().getInputKeyword_4_0_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group_4_0__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RampDownEffect__Group_4_0__1__Impl
	rule__RampDownEffect__Group_4_0__2
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group_4_0__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRampDownEffectAccess().getColonKeyword_4_0_1()); }
	':'
	{ after(grammarAccess.getRampDownEffectAccess().getColonKeyword_4_0_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group_4_0__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RampDownEffect__Group_4_0__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group_4_0__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRampDownEffectAccess().getInputAssignment_4_0_2()); }
	(rule__RampDownEffect__InputAssignment_4_0_2)
	{ after(grammarAccess.getRampDownEffectAccess().getInputAssignment_4_0_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__RampDownEffect__Group_4_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RampDownEffect__Group_4_1__0__Impl
	rule__RampDownEffect__Group_4_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group_4_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRampDownEffectAccess().getVaroutKeyword_4_1_0()); }
	'varout'
	{ after(grammarAccess.getRampDownEffectAccess().getVaroutKeyword_4_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group_4_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RampDownEffect__Group_4_1__1__Impl
	rule__RampDownEffect__Group_4_1__2
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group_4_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRampDownEffectAccess().getColonKeyword_4_1_1()); }
	':'
	{ after(grammarAccess.getRampDownEffectAccess().getColonKeyword_4_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group_4_1__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RampDownEffect__Group_4_1__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group_4_1__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRampDownEffectAccess().getVaroutAssignment_4_1_2()); }
	(rule__RampDownEffect__VaroutAssignment_4_1_2)
	{ after(grammarAccess.getRampDownEffectAccess().getVaroutAssignment_4_1_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__RampDownEffect__Group_5__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RampDownEffect__Group_5__0__Impl
	rule__RampDownEffect__Group_5__1
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group_5__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRampDownEffectAccess().getFailureEventKeyword_5_0()); }
	'failure-event'
	{ after(grammarAccess.getRampDownEffectAccess().getFailureEventKeyword_5_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group_5__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RampDownEffect__Group_5__1__Impl
	rule__RampDownEffect__Group_5__2
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group_5__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRampDownEffectAccess().getColonKeyword_5_1()); }
	':'
	{ after(grammarAccess.getRampDownEffectAccess().getColonKeyword_5_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group_5__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RampDownEffect__Group_5__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__Group_5__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRampDownEffectAccess().getFailureEventAssignment_5_2()); }
	(rule__RampDownEffect__FailureEventAssignment_5_2)
	{ after(grammarAccess.getRampDownEffectAccess().getFailureEventAssignment_5_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__InvertedEffect__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__InvertedEffect__Group__0__Impl
	rule__InvertedEffect__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__InvertedEffect__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInvertedEffectAccess().getInvertedEffectAction_0()); }
	()
	{ after(grammarAccess.getInvertedEffectAccess().getInvertedEffectAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__InvertedEffect__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__InvertedEffect__Group__1__Impl
	rule__InvertedEffect__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__InvertedEffect__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInvertedEffectAccess().getInvertedKeyword_1()); }
	'inverted'
	{ after(grammarAccess.getInvertedEffectAccess().getInvertedKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__InvertedEffect__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__InvertedEffect__Group__2__Impl
	rule__InvertedEffect__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__InvertedEffect__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInvertedEffectAccess().getLeftCurlyBracketKeyword_2()); }
	'{'
	{ after(grammarAccess.getInvertedEffectAccess().getLeftCurlyBracketKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__InvertedEffect__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__InvertedEffect__Group__3__Impl
	rule__InvertedEffect__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__InvertedEffect__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInvertedEffectAccess().getGroup_3()); }
	(rule__InvertedEffect__Group_3__0)?
	{ after(grammarAccess.getInvertedEffectAccess().getGroup_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__InvertedEffect__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__InvertedEffect__Group__4__Impl
	rule__InvertedEffect__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__InvertedEffect__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInvertedEffectAccess().getGroup_4()); }
	(rule__InvertedEffect__Group_4__0)?
	{ after(grammarAccess.getInvertedEffectAccess().getGroup_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__InvertedEffect__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__InvertedEffect__Group__5__Impl
	rule__InvertedEffect__Group__6
;
finally {
	restoreStackSize(stackSize);
}

rule__InvertedEffect__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInvertedEffectAccess().getTemplateAssignment_5()); }
	(rule__InvertedEffect__TemplateAssignment_5)?
	{ after(grammarAccess.getInvertedEffectAccess().getTemplateAssignment_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__InvertedEffect__Group__6
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__InvertedEffect__Group__6__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__InvertedEffect__Group__6__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInvertedEffectAccess().getRightCurlyBracketKeyword_6()); }
	'}'
	{ after(grammarAccess.getInvertedEffectAccess().getRightCurlyBracketKeyword_6()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__InvertedEffect__Group_3__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__InvertedEffect__Group_3__0__Impl
	rule__InvertedEffect__Group_3__1
;
finally {
	restoreStackSize(stackSize);
}

rule__InvertedEffect__Group_3__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInvertedEffectAccess().getGroup_3_0()); }
	(rule__InvertedEffect__Group_3_0__0)
	{ after(grammarAccess.getInvertedEffectAccess().getGroup_3_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__InvertedEffect__Group_3__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__InvertedEffect__Group_3__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__InvertedEffect__Group_3__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInvertedEffectAccess().getGroup_3_1()); }
	(rule__InvertedEffect__Group_3_1__0)
	{ after(grammarAccess.getInvertedEffectAccess().getGroup_3_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__InvertedEffect__Group_3_0__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__InvertedEffect__Group_3_0__0__Impl
	rule__InvertedEffect__Group_3_0__1
;
finally {
	restoreStackSize(stackSize);
}

rule__InvertedEffect__Group_3_0__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInvertedEffectAccess().getInputKeyword_3_0_0()); }
	'input'
	{ after(grammarAccess.getInvertedEffectAccess().getInputKeyword_3_0_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__InvertedEffect__Group_3_0__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__InvertedEffect__Group_3_0__1__Impl
	rule__InvertedEffect__Group_3_0__2
;
finally {
	restoreStackSize(stackSize);
}

rule__InvertedEffect__Group_3_0__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInvertedEffectAccess().getColonKeyword_3_0_1()); }
	':'
	{ after(grammarAccess.getInvertedEffectAccess().getColonKeyword_3_0_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__InvertedEffect__Group_3_0__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__InvertedEffect__Group_3_0__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__InvertedEffect__Group_3_0__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInvertedEffectAccess().getInputAssignment_3_0_2()); }
	(rule__InvertedEffect__InputAssignment_3_0_2)
	{ after(grammarAccess.getInvertedEffectAccess().getInputAssignment_3_0_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__InvertedEffect__Group_3_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__InvertedEffect__Group_3_1__0__Impl
	rule__InvertedEffect__Group_3_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__InvertedEffect__Group_3_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInvertedEffectAccess().getVaroutKeyword_3_1_0()); }
	'varout'
	{ after(grammarAccess.getInvertedEffectAccess().getVaroutKeyword_3_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__InvertedEffect__Group_3_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__InvertedEffect__Group_3_1__1__Impl
	rule__InvertedEffect__Group_3_1__2
;
finally {
	restoreStackSize(stackSize);
}

rule__InvertedEffect__Group_3_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInvertedEffectAccess().getColonKeyword_3_1_1()); }
	':'
	{ after(grammarAccess.getInvertedEffectAccess().getColonKeyword_3_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__InvertedEffect__Group_3_1__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__InvertedEffect__Group_3_1__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__InvertedEffect__Group_3_1__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInvertedEffectAccess().getVaroutAssignment_3_1_2()); }
	(rule__InvertedEffect__VaroutAssignment_3_1_2)
	{ after(grammarAccess.getInvertedEffectAccess().getVaroutAssignment_3_1_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__InvertedEffect__Group_4__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__InvertedEffect__Group_4__0__Impl
	rule__InvertedEffect__Group_4__1
;
finally {
	restoreStackSize(stackSize);
}

rule__InvertedEffect__Group_4__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInvertedEffectAccess().getFailureEventKeyword_4_0()); }
	'failure-event'
	{ after(grammarAccess.getInvertedEffectAccess().getFailureEventKeyword_4_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__InvertedEffect__Group_4__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__InvertedEffect__Group_4__1__Impl
	rule__InvertedEffect__Group_4__2
;
finally {
	restoreStackSize(stackSize);
}

rule__InvertedEffect__Group_4__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInvertedEffectAccess().getColonKeyword_4_1()); }
	':'
	{ after(grammarAccess.getInvertedEffectAccess().getColonKeyword_4_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__InvertedEffect__Group_4__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__InvertedEffect__Group_4__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__InvertedEffect__Group_4__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInvertedEffectAccess().getFailureEventAssignment_4_2()); }
	(rule__InvertedEffect__FailureEventAssignment_4_2)
	{ after(grammarAccess.getInvertedEffectAccess().getFailureEventAssignment_4_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__StuckAtFixedEffect__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtFixedEffect__Group__0__Impl
	rule__StuckAtFixedEffect__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtFixedEffect__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtFixedEffectAccess().getStuckAtFixedEffectAction_0()); }
	()
	{ after(grammarAccess.getStuckAtFixedEffectAccess().getStuckAtFixedEffectAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtFixedEffect__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtFixedEffect__Group__1__Impl
	rule__StuckAtFixedEffect__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtFixedEffect__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtFixedEffectAccess().getStuckAtFixedKeyword_1()); }
	'stuck-at-fixed'
	{ after(grammarAccess.getStuckAtFixedEffectAccess().getStuckAtFixedKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtFixedEffect__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtFixedEffect__Group__2__Impl
	rule__StuckAtFixedEffect__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtFixedEffect__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtFixedEffectAccess().getLeftCurlyBracketKeyword_2()); }
	'{'
	{ after(grammarAccess.getStuckAtFixedEffectAccess().getLeftCurlyBracketKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtFixedEffect__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtFixedEffect__Group__3__Impl
	rule__StuckAtFixedEffect__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtFixedEffect__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtFixedEffectAccess().getGroup_3()); }
	(rule__StuckAtFixedEffect__Group_3__0)?
	{ after(grammarAccess.getStuckAtFixedEffectAccess().getGroup_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtFixedEffect__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtFixedEffect__Group__4__Impl
	rule__StuckAtFixedEffect__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtFixedEffect__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtFixedEffectAccess().getGroup_4()); }
	(rule__StuckAtFixedEffect__Group_4__0)?
	{ after(grammarAccess.getStuckAtFixedEffectAccess().getGroup_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtFixedEffect__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtFixedEffect__Group__5__Impl
	rule__StuckAtFixedEffect__Group__6
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtFixedEffect__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtFixedEffectAccess().getTemplateAssignment_5()); }
	(rule__StuckAtFixedEffect__TemplateAssignment_5)?
	{ after(grammarAccess.getStuckAtFixedEffectAccess().getTemplateAssignment_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtFixedEffect__Group__6
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtFixedEffect__Group__6__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtFixedEffect__Group__6__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtFixedEffectAccess().getRightCurlyBracketKeyword_6()); }
	'}'
	{ after(grammarAccess.getStuckAtFixedEffectAccess().getRightCurlyBracketKeyword_6()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__StuckAtFixedEffect__Group_3__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtFixedEffect__Group_3__0__Impl
	rule__StuckAtFixedEffect__Group_3__1
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtFixedEffect__Group_3__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtFixedEffectAccess().getGroup_3_0()); }
	(rule__StuckAtFixedEffect__Group_3_0__0)
	{ after(grammarAccess.getStuckAtFixedEffectAccess().getGroup_3_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtFixedEffect__Group_3__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtFixedEffect__Group_3__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtFixedEffect__Group_3__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtFixedEffectAccess().getGroup_3_1()); }
	(rule__StuckAtFixedEffect__Group_3_1__0)
	{ after(grammarAccess.getStuckAtFixedEffectAccess().getGroup_3_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__StuckAtFixedEffect__Group_3_0__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtFixedEffect__Group_3_0__0__Impl
	rule__StuckAtFixedEffect__Group_3_0__1
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtFixedEffect__Group_3_0__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtFixedEffectAccess().getInputKeyword_3_0_0()); }
	'input'
	{ after(grammarAccess.getStuckAtFixedEffectAccess().getInputKeyword_3_0_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtFixedEffect__Group_3_0__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtFixedEffect__Group_3_0__1__Impl
	rule__StuckAtFixedEffect__Group_3_0__2
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtFixedEffect__Group_3_0__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtFixedEffectAccess().getColonKeyword_3_0_1()); }
	':'
	{ after(grammarAccess.getStuckAtFixedEffectAccess().getColonKeyword_3_0_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtFixedEffect__Group_3_0__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtFixedEffect__Group_3_0__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtFixedEffect__Group_3_0__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtFixedEffectAccess().getInputAssignment_3_0_2()); }
	(rule__StuckAtFixedEffect__InputAssignment_3_0_2)
	{ after(grammarAccess.getStuckAtFixedEffectAccess().getInputAssignment_3_0_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__StuckAtFixedEffect__Group_3_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtFixedEffect__Group_3_1__0__Impl
	rule__StuckAtFixedEffect__Group_3_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtFixedEffect__Group_3_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtFixedEffectAccess().getVaroutKeyword_3_1_0()); }
	'varout'
	{ after(grammarAccess.getStuckAtFixedEffectAccess().getVaroutKeyword_3_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtFixedEffect__Group_3_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtFixedEffect__Group_3_1__1__Impl
	rule__StuckAtFixedEffect__Group_3_1__2
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtFixedEffect__Group_3_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtFixedEffectAccess().getColonKeyword_3_1_1()); }
	':'
	{ after(grammarAccess.getStuckAtFixedEffectAccess().getColonKeyword_3_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtFixedEffect__Group_3_1__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtFixedEffect__Group_3_1__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtFixedEffect__Group_3_1__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtFixedEffectAccess().getVaroutAssignment_3_1_2()); }
	(rule__StuckAtFixedEffect__VaroutAssignment_3_1_2)
	{ after(grammarAccess.getStuckAtFixedEffectAccess().getVaroutAssignment_3_1_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__StuckAtFixedEffect__Group_4__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtFixedEffect__Group_4__0__Impl
	rule__StuckAtFixedEffect__Group_4__1
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtFixedEffect__Group_4__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtFixedEffectAccess().getFailureEventKeyword_4_0()); }
	'failure-event'
	{ after(grammarAccess.getStuckAtFixedEffectAccess().getFailureEventKeyword_4_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtFixedEffect__Group_4__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtFixedEffect__Group_4__1__Impl
	rule__StuckAtFixedEffect__Group_4__2
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtFixedEffect__Group_4__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtFixedEffectAccess().getColonKeyword_4_1()); }
	':'
	{ after(grammarAccess.getStuckAtFixedEffectAccess().getColonKeyword_4_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtFixedEffect__Group_4__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StuckAtFixedEffect__Group_4__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtFixedEffect__Group_4__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStuckAtFixedEffectAccess().getFailureEventAssignment_4_2()); }
	(rule__StuckAtFixedEffect__FailureEventAssignment_4_2)
	{ after(grammarAccess.getStuckAtFixedEffectAccess().getFailureEventAssignment_4_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__RandomEffect__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RandomEffect__Group__0__Impl
	rule__RandomEffect__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRandomEffectAccess().getRandomEffectAction_0()); }
	()
	{ after(grammarAccess.getRandomEffectAccess().getRandomEffectAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RandomEffect__Group__1__Impl
	rule__RandomEffect__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRandomEffectAccess().getRandomKeyword_1()); }
	'random'
	{ after(grammarAccess.getRandomEffectAccess().getRandomKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RandomEffect__Group__2__Impl
	rule__RandomEffect__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRandomEffectAccess().getLeftCurlyBracketKeyword_2()); }
	'{'
	{ after(grammarAccess.getRandomEffectAccess().getLeftCurlyBracketKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RandomEffect__Group__3__Impl
	rule__RandomEffect__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRandomEffectAccess().getGroup_3()); }
	(rule__RandomEffect__Group_3__0)?
	{ after(grammarAccess.getRandomEffectAccess().getGroup_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RandomEffect__Group__4__Impl
	rule__RandomEffect__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRandomEffectAccess().getGroup_4()); }
	(rule__RandomEffect__Group_4__0)?
	{ after(grammarAccess.getRandomEffectAccess().getGroup_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RandomEffect__Group__5__Impl
	rule__RandomEffect__Group__6
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRandomEffectAccess().getGroup_5()); }
	(rule__RandomEffect__Group_5__0)?
	{ after(grammarAccess.getRandomEffectAccess().getGroup_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__Group__6
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RandomEffect__Group__6__Impl
	rule__RandomEffect__Group__7
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__Group__6__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRandomEffectAccess().getTemplateAssignment_6()); }
	(rule__RandomEffect__TemplateAssignment_6)?
	{ after(grammarAccess.getRandomEffectAccess().getTemplateAssignment_6()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__Group__7
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RandomEffect__Group__7__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__Group__7__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRandomEffectAccess().getRightCurlyBracketKeyword_7()); }
	'}'
	{ after(grammarAccess.getRandomEffectAccess().getRightCurlyBracketKeyword_7()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__RandomEffect__Group_3__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RandomEffect__Group_3__0__Impl
	rule__RandomEffect__Group_3__1
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__Group_3__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRandomEffectAccess().getReferenceModeKeyword_3_0()); }
	'reference-mode'
	{ after(grammarAccess.getRandomEffectAccess().getReferenceModeKeyword_3_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__Group_3__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RandomEffect__Group_3__1__Impl
	rule__RandomEffect__Group_3__2
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__Group_3__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRandomEffectAccess().getColonKeyword_3_1()); }
	':'
	{ after(grammarAccess.getRandomEffectAccess().getColonKeyword_3_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__Group_3__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RandomEffect__Group_3__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__Group_3__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRandomEffectAccess().getTermReferenceModeAssignment_3_2()); }
	(rule__RandomEffect__TermReferenceModeAssignment_3_2)
	{ after(grammarAccess.getRandomEffectAccess().getTermReferenceModeAssignment_3_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__RandomEffect__Group_4__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RandomEffect__Group_4__0__Impl
	rule__RandomEffect__Group_4__1
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__Group_4__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRandomEffectAccess().getGroup_4_0()); }
	(rule__RandomEffect__Group_4_0__0)
	{ after(grammarAccess.getRandomEffectAccess().getGroup_4_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__Group_4__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RandomEffect__Group_4__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__Group_4__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRandomEffectAccess().getGroup_4_1()); }
	(rule__RandomEffect__Group_4_1__0)
	{ after(grammarAccess.getRandomEffectAccess().getGroup_4_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__RandomEffect__Group_4_0__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RandomEffect__Group_4_0__0__Impl
	rule__RandomEffect__Group_4_0__1
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__Group_4_0__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRandomEffectAccess().getInputKeyword_4_0_0()); }
	'input'
	{ after(grammarAccess.getRandomEffectAccess().getInputKeyword_4_0_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__Group_4_0__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RandomEffect__Group_4_0__1__Impl
	rule__RandomEffect__Group_4_0__2
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__Group_4_0__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRandomEffectAccess().getColonKeyword_4_0_1()); }
	':'
	{ after(grammarAccess.getRandomEffectAccess().getColonKeyword_4_0_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__Group_4_0__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RandomEffect__Group_4_0__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__Group_4_0__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRandomEffectAccess().getInputAssignment_4_0_2()); }
	(rule__RandomEffect__InputAssignment_4_0_2)
	{ after(grammarAccess.getRandomEffectAccess().getInputAssignment_4_0_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__RandomEffect__Group_4_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RandomEffect__Group_4_1__0__Impl
	rule__RandomEffect__Group_4_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__Group_4_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRandomEffectAccess().getVaroutKeyword_4_1_0()); }
	'varout'
	{ after(grammarAccess.getRandomEffectAccess().getVaroutKeyword_4_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__Group_4_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RandomEffect__Group_4_1__1__Impl
	rule__RandomEffect__Group_4_1__2
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__Group_4_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRandomEffectAccess().getColonKeyword_4_1_1()); }
	':'
	{ after(grammarAccess.getRandomEffectAccess().getColonKeyword_4_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__Group_4_1__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RandomEffect__Group_4_1__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__Group_4_1__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRandomEffectAccess().getVaroutAssignment_4_1_2()); }
	(rule__RandomEffect__VaroutAssignment_4_1_2)
	{ after(grammarAccess.getRandomEffectAccess().getVaroutAssignment_4_1_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__RandomEffect__Group_5__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RandomEffect__Group_5__0__Impl
	rule__RandomEffect__Group_5__1
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__Group_5__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRandomEffectAccess().getFailureEventKeyword_5_0()); }
	'failure-event'
	{ after(grammarAccess.getRandomEffectAccess().getFailureEventKeyword_5_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__Group_5__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RandomEffect__Group_5__1__Impl
	rule__RandomEffect__Group_5__2
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__Group_5__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRandomEffectAccess().getColonKeyword_5_1()); }
	':'
	{ after(grammarAccess.getRandomEffectAccess().getColonKeyword_5_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__Group_5__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RandomEffect__Group_5__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__Group_5__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRandomEffectAccess().getFailureEventAssignment_5_2()); }
	(rule__RandomEffect__FailureEventAssignment_5_2)
	{ after(grammarAccess.getRandomEffectAccess().getFailureEventAssignment_5_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ErroneousEffect__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ErroneousEffect__Group__0__Impl
	rule__ErroneousEffect__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getErroneousEffectAccess().getErroneousEffectAction_0()); }
	()
	{ after(grammarAccess.getErroneousEffectAccess().getErroneousEffectAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ErroneousEffect__Group__1__Impl
	rule__ErroneousEffect__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getErroneousEffectAccess().getErroneousKeyword_1()); }
	'erroneous'
	{ after(grammarAccess.getErroneousEffectAccess().getErroneousKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ErroneousEffect__Group__2__Impl
	rule__ErroneousEffect__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getErroneousEffectAccess().getLeftCurlyBracketKeyword_2()); }
	'{'
	{ after(grammarAccess.getErroneousEffectAccess().getLeftCurlyBracketKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ErroneousEffect__Group__3__Impl
	rule__ErroneousEffect__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getErroneousEffectAccess().getGroup_3()); }
	(rule__ErroneousEffect__Group_3__0)?
	{ after(grammarAccess.getErroneousEffectAccess().getGroup_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ErroneousEffect__Group__4__Impl
	rule__ErroneousEffect__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getErroneousEffectAccess().getGroup_4()); }
	(rule__ErroneousEffect__Group_4__0)?
	{ after(grammarAccess.getErroneousEffectAccess().getGroup_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ErroneousEffect__Group__5__Impl
	rule__ErroneousEffect__Group__6
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getErroneousEffectAccess().getGroup_5()); }
	(rule__ErroneousEffect__Group_5__0)?
	{ after(grammarAccess.getErroneousEffectAccess().getGroup_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__Group__6
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ErroneousEffect__Group__6__Impl
	rule__ErroneousEffect__Group__7
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__Group__6__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getErroneousEffectAccess().getTemplateAssignment_6()); }
	(rule__ErroneousEffect__TemplateAssignment_6)?
	{ after(grammarAccess.getErroneousEffectAccess().getTemplateAssignment_6()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__Group__7
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ErroneousEffect__Group__7__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__Group__7__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getErroneousEffectAccess().getRightCurlyBracketKeyword_7()); }
	'}'
	{ after(grammarAccess.getErroneousEffectAccess().getRightCurlyBracketKeyword_7()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ErroneousEffect__Group_3__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ErroneousEffect__Group_3__0__Impl
	rule__ErroneousEffect__Group_3__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__Group_3__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getErroneousEffectAccess().getReferenceModeKeyword_3_0()); }
	'reference-mode'
	{ after(grammarAccess.getErroneousEffectAccess().getReferenceModeKeyword_3_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__Group_3__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ErroneousEffect__Group_3__1__Impl
	rule__ErroneousEffect__Group_3__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__Group_3__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getErroneousEffectAccess().getColonKeyword_3_1()); }
	':'
	{ after(grammarAccess.getErroneousEffectAccess().getColonKeyword_3_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__Group_3__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ErroneousEffect__Group_3__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__Group_3__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getErroneousEffectAccess().getTermReferenceModeAssignment_3_2()); }
	(rule__ErroneousEffect__TermReferenceModeAssignment_3_2)
	{ after(grammarAccess.getErroneousEffectAccess().getTermReferenceModeAssignment_3_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ErroneousEffect__Group_4__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ErroneousEffect__Group_4__0__Impl
	rule__ErroneousEffect__Group_4__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__Group_4__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getErroneousEffectAccess().getGroup_4_0()); }
	(rule__ErroneousEffect__Group_4_0__0)
	{ after(grammarAccess.getErroneousEffectAccess().getGroup_4_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__Group_4__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ErroneousEffect__Group_4__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__Group_4__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getErroneousEffectAccess().getGroup_4_1()); }
	(rule__ErroneousEffect__Group_4_1__0)
	{ after(grammarAccess.getErroneousEffectAccess().getGroup_4_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ErroneousEffect__Group_4_0__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ErroneousEffect__Group_4_0__0__Impl
	rule__ErroneousEffect__Group_4_0__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__Group_4_0__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getErroneousEffectAccess().getInputKeyword_4_0_0()); }
	'input'
	{ after(grammarAccess.getErroneousEffectAccess().getInputKeyword_4_0_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__Group_4_0__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ErroneousEffect__Group_4_0__1__Impl
	rule__ErroneousEffect__Group_4_0__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__Group_4_0__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getErroneousEffectAccess().getColonKeyword_4_0_1()); }
	':'
	{ after(grammarAccess.getErroneousEffectAccess().getColonKeyword_4_0_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__Group_4_0__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ErroneousEffect__Group_4_0__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__Group_4_0__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getErroneousEffectAccess().getInputAssignment_4_0_2()); }
	(rule__ErroneousEffect__InputAssignment_4_0_2)
	{ after(grammarAccess.getErroneousEffectAccess().getInputAssignment_4_0_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ErroneousEffect__Group_4_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ErroneousEffect__Group_4_1__0__Impl
	rule__ErroneousEffect__Group_4_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__Group_4_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getErroneousEffectAccess().getVaroutKeyword_4_1_0()); }
	'varout'
	{ after(grammarAccess.getErroneousEffectAccess().getVaroutKeyword_4_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__Group_4_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ErroneousEffect__Group_4_1__1__Impl
	rule__ErroneousEffect__Group_4_1__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__Group_4_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getErroneousEffectAccess().getColonKeyword_4_1_1()); }
	':'
	{ after(grammarAccess.getErroneousEffectAccess().getColonKeyword_4_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__Group_4_1__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ErroneousEffect__Group_4_1__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__Group_4_1__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getErroneousEffectAccess().getVaroutAssignment_4_1_2()); }
	(rule__ErroneousEffect__VaroutAssignment_4_1_2)
	{ after(grammarAccess.getErroneousEffectAccess().getVaroutAssignment_4_1_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ErroneousEffect__Group_5__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ErroneousEffect__Group_5__0__Impl
	rule__ErroneousEffect__Group_5__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__Group_5__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getErroneousEffectAccess().getFailureEventKeyword_5_0()); }
	'failure-event'
	{ after(grammarAccess.getErroneousEffectAccess().getFailureEventKeyword_5_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__Group_5__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ErroneousEffect__Group_5__1__Impl
	rule__ErroneousEffect__Group_5__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__Group_5__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getErroneousEffectAccess().getColonKeyword_5_1()); }
	':'
	{ after(grammarAccess.getErroneousEffectAccess().getColonKeyword_5_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__Group_5__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ErroneousEffect__Group_5__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__Group_5__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getErroneousEffectAccess().getFailureEventAssignment_5_2()); }
	(rule__ErroneousEffect__FailureEventAssignment_5_2)
	{ after(grammarAccess.getErroneousEffectAccess().getFailureEventAssignment_5_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__DeltaOutEffect__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaOutEffect__Group__0__Impl
	rule__DeltaOutEffect__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaOutEffectAccess().getDeltaOutKeyword_0()); }
	'delta-out'
	{ after(grammarAccess.getDeltaOutEffectAccess().getDeltaOutKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaOutEffect__Group__1__Impl
	rule__DeltaOutEffect__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaOutEffectAccess().getLeftCurlyBracketKeyword_1()); }
	'{'
	{ after(grammarAccess.getDeltaOutEffectAccess().getLeftCurlyBracketKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaOutEffect__Group__2__Impl
	rule__DeltaOutEffect__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaOutEffectAccess().getGroup_2()); }
	(rule__DeltaOutEffect__Group_2__0)?
	{ after(grammarAccess.getDeltaOutEffectAccess().getGroup_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaOutEffect__Group__3__Impl
	rule__DeltaOutEffect__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaOutEffectAccess().getGroup_3()); }
	(rule__DeltaOutEffect__Group_3__0)
	{ after(grammarAccess.getDeltaOutEffectAccess().getGroup_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaOutEffect__Group__4__Impl
	rule__DeltaOutEffect__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaOutEffectAccess().getGroup_4()); }
	(rule__DeltaOutEffect__Group_4__0)?
	{ after(grammarAccess.getDeltaOutEffectAccess().getGroup_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaOutEffect__Group__5__Impl
	rule__DeltaOutEffect__Group__6
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaOutEffectAccess().getGroup_5()); }
	(rule__DeltaOutEffect__Group_5__0)?
	{ after(grammarAccess.getDeltaOutEffectAccess().getGroup_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group__6
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaOutEffect__Group__6__Impl
	rule__DeltaOutEffect__Group__7
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group__6__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaOutEffectAccess().getTemplateAssignment_6()); }
	(rule__DeltaOutEffect__TemplateAssignment_6)?
	{ after(grammarAccess.getDeltaOutEffectAccess().getTemplateAssignment_6()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group__7
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaOutEffect__Group__7__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group__7__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaOutEffectAccess().getRightCurlyBracketKeyword_7()); }
	'}'
	{ after(grammarAccess.getDeltaOutEffectAccess().getRightCurlyBracketKeyword_7()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__DeltaOutEffect__Group_2__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaOutEffect__Group_2__0__Impl
	rule__DeltaOutEffect__Group_2__1
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group_2__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaOutEffectAccess().getReferenceModeKeyword_2_0()); }
	'reference-mode'
	{ after(grammarAccess.getDeltaOutEffectAccess().getReferenceModeKeyword_2_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group_2__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaOutEffect__Group_2__1__Impl
	rule__DeltaOutEffect__Group_2__2
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group_2__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaOutEffectAccess().getColonKeyword_2_1()); }
	':'
	{ after(grammarAccess.getDeltaOutEffectAccess().getColonKeyword_2_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group_2__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaOutEffect__Group_2__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group_2__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaOutEffectAccess().getTermReferenceModeAssignment_2_2()); }
	(rule__DeltaOutEffect__TermReferenceModeAssignment_2_2)
	{ after(grammarAccess.getDeltaOutEffectAccess().getTermReferenceModeAssignment_2_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__DeltaOutEffect__Group_3__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaOutEffect__Group_3__0__Impl
	rule__DeltaOutEffect__Group_3__1
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group_3__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaOutEffectAccess().getDeltaKeyword_3_0()); }
	'delta'
	{ after(grammarAccess.getDeltaOutEffectAccess().getDeltaKeyword_3_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group_3__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaOutEffect__Group_3__1__Impl
	rule__DeltaOutEffect__Group_3__2
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group_3__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaOutEffectAccess().getColonKeyword_3_1()); }
	':'
	{ after(grammarAccess.getDeltaOutEffectAccess().getColonKeyword_3_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group_3__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaOutEffect__Group_3__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group_3__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaOutEffectAccess().getDeltaAssignment_3_2()); }
	(rule__DeltaOutEffect__DeltaAssignment_3_2)
	{ after(grammarAccess.getDeltaOutEffectAccess().getDeltaAssignment_3_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__DeltaOutEffect__Group_4__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaOutEffect__Group_4__0__Impl
	rule__DeltaOutEffect__Group_4__1
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group_4__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaOutEffectAccess().getGroup_4_0()); }
	(rule__DeltaOutEffect__Group_4_0__0)
	{ after(grammarAccess.getDeltaOutEffectAccess().getGroup_4_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group_4__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaOutEffect__Group_4__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group_4__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaOutEffectAccess().getGroup_4_1()); }
	(rule__DeltaOutEffect__Group_4_1__0)
	{ after(grammarAccess.getDeltaOutEffectAccess().getGroup_4_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__DeltaOutEffect__Group_4_0__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaOutEffect__Group_4_0__0__Impl
	rule__DeltaOutEffect__Group_4_0__1
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group_4_0__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaOutEffectAccess().getInputKeyword_4_0_0()); }
	'input'
	{ after(grammarAccess.getDeltaOutEffectAccess().getInputKeyword_4_0_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group_4_0__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaOutEffect__Group_4_0__1__Impl
	rule__DeltaOutEffect__Group_4_0__2
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group_4_0__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaOutEffectAccess().getColonKeyword_4_0_1()); }
	':'
	{ after(grammarAccess.getDeltaOutEffectAccess().getColonKeyword_4_0_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group_4_0__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaOutEffect__Group_4_0__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group_4_0__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaOutEffectAccess().getInputAssignment_4_0_2()); }
	(rule__DeltaOutEffect__InputAssignment_4_0_2)
	{ after(grammarAccess.getDeltaOutEffectAccess().getInputAssignment_4_0_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__DeltaOutEffect__Group_4_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaOutEffect__Group_4_1__0__Impl
	rule__DeltaOutEffect__Group_4_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group_4_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaOutEffectAccess().getVaroutKeyword_4_1_0()); }
	'varout'
	{ after(grammarAccess.getDeltaOutEffectAccess().getVaroutKeyword_4_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group_4_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaOutEffect__Group_4_1__1__Impl
	rule__DeltaOutEffect__Group_4_1__2
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group_4_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaOutEffectAccess().getColonKeyword_4_1_1()); }
	':'
	{ after(grammarAccess.getDeltaOutEffectAccess().getColonKeyword_4_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group_4_1__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaOutEffect__Group_4_1__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group_4_1__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaOutEffectAccess().getVaroutAssignment_4_1_2()); }
	(rule__DeltaOutEffect__VaroutAssignment_4_1_2)
	{ after(grammarAccess.getDeltaOutEffectAccess().getVaroutAssignment_4_1_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__DeltaOutEffect__Group_5__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaOutEffect__Group_5__0__Impl
	rule__DeltaOutEffect__Group_5__1
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group_5__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaOutEffectAccess().getFailureEventKeyword_5_0()); }
	'failure-event'
	{ after(grammarAccess.getDeltaOutEffectAccess().getFailureEventKeyword_5_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group_5__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaOutEffect__Group_5__1__Impl
	rule__DeltaOutEffect__Group_5__2
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group_5__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaOutEffectAccess().getColonKeyword_5_1()); }
	':'
	{ after(grammarAccess.getDeltaOutEffectAccess().getColonKeyword_5_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group_5__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaOutEffect__Group_5__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__Group_5__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaOutEffectAccess().getFailureEventAssignment_5_2()); }
	(rule__DeltaOutEffect__FailureEventAssignment_5_2)
	{ after(grammarAccess.getDeltaOutEffectAccess().getFailureEventAssignment_5_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__DeltaInRandomEffect__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInRandomEffect__Group__0__Impl
	rule__DeltaInRandomEffect__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInRandomEffectAccess().getDeltaInRandomKeyword_0()); }
	'delta-in-random'
	{ after(grammarAccess.getDeltaInRandomEffectAccess().getDeltaInRandomKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInRandomEffect__Group__1__Impl
	rule__DeltaInRandomEffect__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInRandomEffectAccess().getLeftCurlyBracketKeyword_1()); }
	'{'
	{ after(grammarAccess.getDeltaInRandomEffectAccess().getLeftCurlyBracketKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInRandomEffect__Group__2__Impl
	rule__DeltaInRandomEffect__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInRandomEffectAccess().getGroup_2()); }
	(rule__DeltaInRandomEffect__Group_2__0)?
	{ after(grammarAccess.getDeltaInRandomEffectAccess().getGroup_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInRandomEffect__Group__3__Impl
	rule__DeltaInRandomEffect__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInRandomEffectAccess().getGroup_3()); }
	(rule__DeltaInRandomEffect__Group_3__0)
	{ after(grammarAccess.getDeltaInRandomEffectAccess().getGroup_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInRandomEffect__Group__4__Impl
	rule__DeltaInRandomEffect__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInRandomEffectAccess().getGroup_4()); }
	(rule__DeltaInRandomEffect__Group_4__0)?
	{ after(grammarAccess.getDeltaInRandomEffectAccess().getGroup_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInRandomEffect__Group__5__Impl
	rule__DeltaInRandomEffect__Group__6
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInRandomEffectAccess().getGroup_5()); }
	(rule__DeltaInRandomEffect__Group_5__0)?
	{ after(grammarAccess.getDeltaInRandomEffectAccess().getGroup_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group__6
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInRandomEffect__Group__6__Impl
	rule__DeltaInRandomEffect__Group__7
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group__6__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInRandomEffectAccess().getTemplateAssignment_6()); }
	(rule__DeltaInRandomEffect__TemplateAssignment_6)?
	{ after(grammarAccess.getDeltaInRandomEffectAccess().getTemplateAssignment_6()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group__7
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInRandomEffect__Group__7__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group__7__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInRandomEffectAccess().getRightCurlyBracketKeyword_7()); }
	'}'
	{ after(grammarAccess.getDeltaInRandomEffectAccess().getRightCurlyBracketKeyword_7()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__DeltaInRandomEffect__Group_2__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInRandomEffect__Group_2__0__Impl
	rule__DeltaInRandomEffect__Group_2__1
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group_2__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInRandomEffectAccess().getReferenceModeKeyword_2_0()); }
	'reference-mode'
	{ after(grammarAccess.getDeltaInRandomEffectAccess().getReferenceModeKeyword_2_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group_2__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInRandomEffect__Group_2__1__Impl
	rule__DeltaInRandomEffect__Group_2__2
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group_2__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInRandomEffectAccess().getColonKeyword_2_1()); }
	':'
	{ after(grammarAccess.getDeltaInRandomEffectAccess().getColonKeyword_2_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group_2__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInRandomEffect__Group_2__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group_2__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInRandomEffectAccess().getTermReferenceModeAssignment_2_2()); }
	(rule__DeltaInRandomEffect__TermReferenceModeAssignment_2_2)
	{ after(grammarAccess.getDeltaInRandomEffectAccess().getTermReferenceModeAssignment_2_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__DeltaInRandomEffect__Group_3__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInRandomEffect__Group_3__0__Impl
	rule__DeltaInRandomEffect__Group_3__1
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group_3__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInRandomEffectAccess().getDeltaKeyword_3_0()); }
	'delta'
	{ after(grammarAccess.getDeltaInRandomEffectAccess().getDeltaKeyword_3_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group_3__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInRandomEffect__Group_3__1__Impl
	rule__DeltaInRandomEffect__Group_3__2
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group_3__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInRandomEffectAccess().getColonKeyword_3_1()); }
	':'
	{ after(grammarAccess.getDeltaInRandomEffectAccess().getColonKeyword_3_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group_3__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInRandomEffect__Group_3__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group_3__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInRandomEffectAccess().getDeltaAssignment_3_2()); }
	(rule__DeltaInRandomEffect__DeltaAssignment_3_2)
	{ after(grammarAccess.getDeltaInRandomEffectAccess().getDeltaAssignment_3_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__DeltaInRandomEffect__Group_4__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInRandomEffect__Group_4__0__Impl
	rule__DeltaInRandomEffect__Group_4__1
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group_4__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInRandomEffectAccess().getGroup_4_0()); }
	(rule__DeltaInRandomEffect__Group_4_0__0)
	{ after(grammarAccess.getDeltaInRandomEffectAccess().getGroup_4_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group_4__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInRandomEffect__Group_4__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group_4__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInRandomEffectAccess().getGroup_4_1()); }
	(rule__DeltaInRandomEffect__Group_4_1__0)
	{ after(grammarAccess.getDeltaInRandomEffectAccess().getGroup_4_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__DeltaInRandomEffect__Group_4_0__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInRandomEffect__Group_4_0__0__Impl
	rule__DeltaInRandomEffect__Group_4_0__1
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group_4_0__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInRandomEffectAccess().getInputKeyword_4_0_0()); }
	'input'
	{ after(grammarAccess.getDeltaInRandomEffectAccess().getInputKeyword_4_0_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group_4_0__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInRandomEffect__Group_4_0__1__Impl
	rule__DeltaInRandomEffect__Group_4_0__2
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group_4_0__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInRandomEffectAccess().getColonKeyword_4_0_1()); }
	':'
	{ after(grammarAccess.getDeltaInRandomEffectAccess().getColonKeyword_4_0_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group_4_0__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInRandomEffect__Group_4_0__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group_4_0__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInRandomEffectAccess().getInputAssignment_4_0_2()); }
	(rule__DeltaInRandomEffect__InputAssignment_4_0_2)
	{ after(grammarAccess.getDeltaInRandomEffectAccess().getInputAssignment_4_0_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__DeltaInRandomEffect__Group_4_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInRandomEffect__Group_4_1__0__Impl
	rule__DeltaInRandomEffect__Group_4_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group_4_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInRandomEffectAccess().getVaroutKeyword_4_1_0()); }
	'varout'
	{ after(grammarAccess.getDeltaInRandomEffectAccess().getVaroutKeyword_4_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group_4_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInRandomEffect__Group_4_1__1__Impl
	rule__DeltaInRandomEffect__Group_4_1__2
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group_4_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInRandomEffectAccess().getColonKeyword_4_1_1()); }
	':'
	{ after(grammarAccess.getDeltaInRandomEffectAccess().getColonKeyword_4_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group_4_1__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInRandomEffect__Group_4_1__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group_4_1__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInRandomEffectAccess().getVaroutAssignment_4_1_2()); }
	(rule__DeltaInRandomEffect__VaroutAssignment_4_1_2)
	{ after(grammarAccess.getDeltaInRandomEffectAccess().getVaroutAssignment_4_1_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__DeltaInRandomEffect__Group_5__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInRandomEffect__Group_5__0__Impl
	rule__DeltaInRandomEffect__Group_5__1
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group_5__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInRandomEffectAccess().getFailureEventKeyword_5_0()); }
	'failure-event'
	{ after(grammarAccess.getDeltaInRandomEffectAccess().getFailureEventKeyword_5_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group_5__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInRandomEffect__Group_5__1__Impl
	rule__DeltaInRandomEffect__Group_5__2
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group_5__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInRandomEffectAccess().getColonKeyword_5_1()); }
	':'
	{ after(grammarAccess.getDeltaInRandomEffectAccess().getColonKeyword_5_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group_5__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInRandomEffect__Group_5__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__Group_5__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInRandomEffectAccess().getFailureEventAssignment_5_2()); }
	(rule__DeltaInRandomEffect__FailureEventAssignment_5_2)
	{ after(grammarAccess.getDeltaInRandomEffectAccess().getFailureEventAssignment_5_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__DeltaInErroneousEffect__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInErroneousEffect__Group__0__Impl
	rule__DeltaInErroneousEffect__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInErroneousEffectAccess().getDeltaInErroneousKeyword_0()); }
	'delta-in-erroneous'
	{ after(grammarAccess.getDeltaInErroneousEffectAccess().getDeltaInErroneousKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInErroneousEffect__Group__1__Impl
	rule__DeltaInErroneousEffect__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInErroneousEffectAccess().getLeftCurlyBracketKeyword_1()); }
	'{'
	{ after(grammarAccess.getDeltaInErroneousEffectAccess().getLeftCurlyBracketKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInErroneousEffect__Group__2__Impl
	rule__DeltaInErroneousEffect__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInErroneousEffectAccess().getGroup_2()); }
	(rule__DeltaInErroneousEffect__Group_2__0)?
	{ after(grammarAccess.getDeltaInErroneousEffectAccess().getGroup_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInErroneousEffect__Group__3__Impl
	rule__DeltaInErroneousEffect__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInErroneousEffectAccess().getGroup_3()); }
	(rule__DeltaInErroneousEffect__Group_3__0)
	{ after(grammarAccess.getDeltaInErroneousEffectAccess().getGroup_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInErroneousEffect__Group__4__Impl
	rule__DeltaInErroneousEffect__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInErroneousEffectAccess().getGroup_4()); }
	(rule__DeltaInErroneousEffect__Group_4__0)?
	{ after(grammarAccess.getDeltaInErroneousEffectAccess().getGroup_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInErroneousEffect__Group__5__Impl
	rule__DeltaInErroneousEffect__Group__6
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInErroneousEffectAccess().getGroup_5()); }
	(rule__DeltaInErroneousEffect__Group_5__0)?
	{ after(grammarAccess.getDeltaInErroneousEffectAccess().getGroup_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group__6
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInErroneousEffect__Group__6__Impl
	rule__DeltaInErroneousEffect__Group__7
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group__6__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInErroneousEffectAccess().getTemplateAssignment_6()); }
	(rule__DeltaInErroneousEffect__TemplateAssignment_6)?
	{ after(grammarAccess.getDeltaInErroneousEffectAccess().getTemplateAssignment_6()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group__7
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInErroneousEffect__Group__7__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group__7__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInErroneousEffectAccess().getRightCurlyBracketKeyword_7()); }
	'}'
	{ after(grammarAccess.getDeltaInErroneousEffectAccess().getRightCurlyBracketKeyword_7()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__DeltaInErroneousEffect__Group_2__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInErroneousEffect__Group_2__0__Impl
	rule__DeltaInErroneousEffect__Group_2__1
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group_2__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInErroneousEffectAccess().getReferenceModeKeyword_2_0()); }
	'reference-mode'
	{ after(grammarAccess.getDeltaInErroneousEffectAccess().getReferenceModeKeyword_2_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group_2__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInErroneousEffect__Group_2__1__Impl
	rule__DeltaInErroneousEffect__Group_2__2
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group_2__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInErroneousEffectAccess().getColonKeyword_2_1()); }
	':'
	{ after(grammarAccess.getDeltaInErroneousEffectAccess().getColonKeyword_2_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group_2__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInErroneousEffect__Group_2__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group_2__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInErroneousEffectAccess().getTermReferenceModeAssignment_2_2()); }
	(rule__DeltaInErroneousEffect__TermReferenceModeAssignment_2_2)
	{ after(grammarAccess.getDeltaInErroneousEffectAccess().getTermReferenceModeAssignment_2_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__DeltaInErroneousEffect__Group_3__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInErroneousEffect__Group_3__0__Impl
	rule__DeltaInErroneousEffect__Group_3__1
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group_3__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInErroneousEffectAccess().getDeltaKeyword_3_0()); }
	'delta'
	{ after(grammarAccess.getDeltaInErroneousEffectAccess().getDeltaKeyword_3_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group_3__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInErroneousEffect__Group_3__1__Impl
	rule__DeltaInErroneousEffect__Group_3__2
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group_3__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInErroneousEffectAccess().getColonKeyword_3_1()); }
	':'
	{ after(grammarAccess.getDeltaInErroneousEffectAccess().getColonKeyword_3_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group_3__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInErroneousEffect__Group_3__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group_3__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInErroneousEffectAccess().getDeltaAssignment_3_2()); }
	(rule__DeltaInErroneousEffect__DeltaAssignment_3_2)
	{ after(grammarAccess.getDeltaInErroneousEffectAccess().getDeltaAssignment_3_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__DeltaInErroneousEffect__Group_4__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInErroneousEffect__Group_4__0__Impl
	rule__DeltaInErroneousEffect__Group_4__1
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group_4__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInErroneousEffectAccess().getGroup_4_0()); }
	(rule__DeltaInErroneousEffect__Group_4_0__0)
	{ after(grammarAccess.getDeltaInErroneousEffectAccess().getGroup_4_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group_4__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInErroneousEffect__Group_4__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group_4__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInErroneousEffectAccess().getGroup_4_1()); }
	(rule__DeltaInErroneousEffect__Group_4_1__0)
	{ after(grammarAccess.getDeltaInErroneousEffectAccess().getGroup_4_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__DeltaInErroneousEffect__Group_4_0__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInErroneousEffect__Group_4_0__0__Impl
	rule__DeltaInErroneousEffect__Group_4_0__1
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group_4_0__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInErroneousEffectAccess().getInputKeyword_4_0_0()); }
	'input'
	{ after(grammarAccess.getDeltaInErroneousEffectAccess().getInputKeyword_4_0_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group_4_0__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInErroneousEffect__Group_4_0__1__Impl
	rule__DeltaInErroneousEffect__Group_4_0__2
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group_4_0__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInErroneousEffectAccess().getColonKeyword_4_0_1()); }
	':'
	{ after(grammarAccess.getDeltaInErroneousEffectAccess().getColonKeyword_4_0_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group_4_0__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInErroneousEffect__Group_4_0__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group_4_0__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInErroneousEffectAccess().getInputAssignment_4_0_2()); }
	(rule__DeltaInErroneousEffect__InputAssignment_4_0_2)
	{ after(grammarAccess.getDeltaInErroneousEffectAccess().getInputAssignment_4_0_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__DeltaInErroneousEffect__Group_4_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInErroneousEffect__Group_4_1__0__Impl
	rule__DeltaInErroneousEffect__Group_4_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group_4_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInErroneousEffectAccess().getVaroutKeyword_4_1_0()); }
	'varout'
	{ after(grammarAccess.getDeltaInErroneousEffectAccess().getVaroutKeyword_4_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group_4_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInErroneousEffect__Group_4_1__1__Impl
	rule__DeltaInErroneousEffect__Group_4_1__2
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group_4_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInErroneousEffectAccess().getColonKeyword_4_1_1()); }
	':'
	{ after(grammarAccess.getDeltaInErroneousEffectAccess().getColonKeyword_4_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group_4_1__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInErroneousEffect__Group_4_1__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group_4_1__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInErroneousEffectAccess().getVaroutAssignment_4_1_2()); }
	(rule__DeltaInErroneousEffect__VaroutAssignment_4_1_2)
	{ after(grammarAccess.getDeltaInErroneousEffectAccess().getVaroutAssignment_4_1_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__DeltaInErroneousEffect__Group_5__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInErroneousEffect__Group_5__0__Impl
	rule__DeltaInErroneousEffect__Group_5__1
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group_5__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInErroneousEffectAccess().getFailureEventKeyword_5_0()); }
	'failure-event'
	{ after(grammarAccess.getDeltaInErroneousEffectAccess().getFailureEventKeyword_5_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group_5__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInErroneousEffect__Group_5__1__Impl
	rule__DeltaInErroneousEffect__Group_5__2
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group_5__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInErroneousEffectAccess().getColonKeyword_5_1()); }
	':'
	{ after(grammarAccess.getDeltaInErroneousEffectAccess().getColonKeyword_5_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group_5__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeltaInErroneousEffect__Group_5__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__Group_5__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeltaInErroneousEffectAccess().getFailureEventAssignment_5_2()); }
	(rule__DeltaInErroneousEffect__FailureEventAssignment_5_2)
	{ after(grammarAccess.getDeltaInErroneousEffectAccess().getFailureEventAssignment_5_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__SelfFixTemplate__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SelfFixTemplate__Group__0__Impl
	rule__SelfFixTemplate__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__SelfFixTemplate__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSelfFixTemplateAccess().getSelfFixTemplateAction_0()); }
	()
	{ after(grammarAccess.getSelfFixTemplateAccess().getSelfFixTemplateAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SelfFixTemplate__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SelfFixTemplate__Group__1__Impl
	rule__SelfFixTemplate__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__SelfFixTemplate__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSelfFixTemplateAccess().getTemplateKeyword_1()); }
	'template'
	{ after(grammarAccess.getSelfFixTemplateAccess().getTemplateKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SelfFixTemplate__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SelfFixTemplate__Group__2__Impl
	rule__SelfFixTemplate__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__SelfFixTemplate__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSelfFixTemplateAccess().getColonKeyword_2()); }
	':'
	{ after(grammarAccess.getSelfFixTemplateAccess().getColonKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SelfFixTemplate__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SelfFixTemplate__Group__3__Impl
	rule__SelfFixTemplate__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__SelfFixTemplate__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSelfFixTemplateAccess().getSelfFixKeyword_3()); }
	'self-fix'
	{ after(grammarAccess.getSelfFixTemplateAccess().getSelfFixKeyword_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SelfFixTemplate__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SelfFixTemplate__Group__4__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__SelfFixTemplate__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSelfFixTemplateAccess().getGroup_4()); }
	(rule__SelfFixTemplate__Group_4__0)?
	{ after(grammarAccess.getSelfFixTemplateAccess().getGroup_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__SelfFixTemplate__Group_4__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SelfFixTemplate__Group_4__0__Impl
	rule__SelfFixTemplate__Group_4__1
;
finally {
	restoreStackSize(stackSize);
}

rule__SelfFixTemplate__Group_4__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSelfFixTemplateAccess().getLeftCurlyBracketKeyword_4_0()); }
	'{'
	{ after(grammarAccess.getSelfFixTemplateAccess().getLeftCurlyBracketKeyword_4_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SelfFixTemplate__Group_4__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SelfFixTemplate__Group_4__1__Impl
	rule__SelfFixTemplate__Group_4__2
;
finally {
	restoreStackSize(stackSize);
}

rule__SelfFixTemplate__Group_4__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSelfFixTemplateAccess().getGroup_4_1()); }
	(rule__SelfFixTemplate__Group_4_1__0)?
	{ after(grammarAccess.getSelfFixTemplateAccess().getGroup_4_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SelfFixTemplate__Group_4__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SelfFixTemplate__Group_4__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__SelfFixTemplate__Group_4__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSelfFixTemplateAccess().getRightCurlyBracketKeyword_4_2()); }
	'}'
	{ after(grammarAccess.getSelfFixTemplateAccess().getRightCurlyBracketKeyword_4_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__SelfFixTemplate__Group_4_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SelfFixTemplate__Group_4_1__0__Impl
	rule__SelfFixTemplate__Group_4_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__SelfFixTemplate__Group_4_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSelfFixTemplateAccess().getEventKeyword_4_1_0()); }
	'event'
	{ after(grammarAccess.getSelfFixTemplateAccess().getEventKeyword_4_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SelfFixTemplate__Group_4_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SelfFixTemplate__Group_4_1__1__Impl
	rule__SelfFixTemplate__Group_4_1__2
;
finally {
	restoreStackSize(stackSize);
}

rule__SelfFixTemplate__Group_4_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSelfFixTemplateAccess().getColonKeyword_4_1_1()); }
	':'
	{ after(grammarAccess.getSelfFixTemplateAccess().getColonKeyword_4_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SelfFixTemplate__Group_4_1__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SelfFixTemplate__Group_4_1__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__SelfFixTemplate__Group_4_1__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSelfFixTemplateAccess().getSelfFixEventAssignment_4_1_2()); }
	(rule__SelfFixTemplate__SelfFixEventAssignment_4_1_2)
	{ after(grammarAccess.getSelfFixTemplateAccess().getSelfFixEventAssignment_4_1_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__GlobalDynamics__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__GlobalDynamics__Group__0__Impl
	rule__GlobalDynamics__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__GlobalDynamics__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getGlobalDynamicsAccess().getGlobalDynamicsAction_0()); }
	()
	{ after(grammarAccess.getGlobalDynamicsAccess().getGlobalDynamicsAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__GlobalDynamics__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__GlobalDynamics__Group__1__Impl
	rule__GlobalDynamics__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__GlobalDynamics__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getGlobalDynamicsAccess().getGlobalDynamicsKeyword_1()); }
	'global-dynamics'
	{ after(grammarAccess.getGlobalDynamicsAccess().getGlobalDynamicsKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__GlobalDynamics__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__GlobalDynamics__Group__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__GlobalDynamics__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	(
		{ before(grammarAccess.getGlobalDynamicsAccess().getTransitionsAssignment_2()); }
		(rule__GlobalDynamics__TransitionsAssignment_2)
		{ after(grammarAccess.getGlobalDynamicsAccess().getTransitionsAssignment_2()); }
	)
	(
		{ before(grammarAccess.getGlobalDynamicsAccess().getTransitionsAssignment_2()); }
		(rule__GlobalDynamics__TransitionsAssignment_2)*
		{ after(grammarAccess.getGlobalDynamicsAccess().getTransitionsAssignment_2()); }
	)
)
;
finally {
	restoreStackSize(stackSize);
}


rule__FaultTransition__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultTransition__Group__0__Impl
	rule__FaultTransition__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultTransition__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultTransitionAccess().getTransitionKeyword_0()); }
	'transition'
	{ after(grammarAccess.getFaultTransitionAccess().getTransitionKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultTransition__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultTransition__Group__1__Impl
	rule__FaultTransition__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultTransition__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultTransitionAccess().getFromKeyword_1()); }
	'from'
	{ after(grammarAccess.getFaultTransitionAccess().getFromKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultTransition__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultTransition__Group__2__Impl
	rule__FaultTransition__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultTransition__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultTransitionAccess().getSourceAssignment_2()); }
	(rule__FaultTransition__SourceAssignment_2)
	{ after(grammarAccess.getFaultTransitionAccess().getSourceAssignment_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultTransition__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultTransition__Group__3__Impl
	rule__FaultTransition__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultTransition__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultTransitionAccess().getToKeyword_3()); }
	'to'
	{ after(grammarAccess.getFaultTransitionAccess().getToKeyword_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultTransition__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultTransition__Group__4__Impl
	rule__FaultTransition__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultTransition__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultTransitionAccess().getTargetAssignment_4()); }
	(rule__FaultTransition__TargetAssignment_4)
	{ after(grammarAccess.getFaultTransitionAccess().getTargetAssignment_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultTransition__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultTransition__Group__5__Impl
	rule__FaultTransition__Group__6
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultTransition__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultTransitionAccess().getGroup_5()); }
	(rule__FaultTransition__Group_5__0)?
	{ after(grammarAccess.getFaultTransitionAccess().getGroup_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultTransition__Group__6
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultTransition__Group__6__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultTransition__Group__6__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultTransitionAccess().getGroup_6()); }
	(rule__FaultTransition__Group_6__0)?
	{ after(grammarAccess.getFaultTransitionAccess().getGroup_6()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__FaultTransition__Group_5__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultTransition__Group_5__0__Impl
	rule__FaultTransition__Group_5__1
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultTransition__Group_5__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultTransitionAccess().getWhenKeyword_5_0()); }
	'when'
	{ after(grammarAccess.getFaultTransitionAccess().getWhenKeyword_5_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultTransition__Group_5__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultTransition__Group_5__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultTransition__Group_5__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultTransitionAccess().getTriggerAssignment_5_1()); }
	(rule__FaultTransition__TriggerAssignment_5_1)
	{ after(grammarAccess.getFaultTransitionAccess().getTriggerAssignment_5_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__FaultTransition__Group_6__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultTransition__Group_6__0__Impl
	rule__FaultTransition__Group_6__1
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultTransition__Group_6__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultTransitionAccess().getLeftSquareBracketKeyword_6_0()); }
	'['
	{ after(grammarAccess.getFaultTransitionAccess().getLeftSquareBracketKeyword_6_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultTransition__Group_6__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultTransition__Group_6__1__Impl
	rule__FaultTransition__Group_6__2
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultTransition__Group_6__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultTransitionAccess().getGuardAssignment_6_1()); }
	(rule__FaultTransition__GuardAssignment_6_1)
	{ after(grammarAccess.getFaultTransitionAccess().getGuardAssignment_6_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultTransition__Group_6__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultTransition__Group_6__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultTransition__Group_6__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultTransitionAccess().getRightSquareBracketKeyword_6_2()); }
	']'
	{ after(grammarAccess.getFaultTransitionAccess().getRightSquareBracketKeyword_6_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__FaultModeStateReference__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultModeStateReference__Group__0__Impl
	rule__FaultModeStateReference__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultModeStateReference__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultModeStateReferenceAccess().getFaultModeAssignment_0()); }
	(rule__FaultModeStateReference__FaultModeAssignment_0)
	{ after(grammarAccess.getFaultModeStateReferenceAccess().getFaultModeAssignment_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultModeStateReference__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultModeStateReference__Group__1__Impl
	rule__FaultModeStateReference__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultModeStateReference__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultModeStateReferenceAccess().getFullStopKeyword_1()); }
	'.'
	{ after(grammarAccess.getFaultModeStateReferenceAccess().getFullStopKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultModeStateReference__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultModeStateReference__Group__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultModeStateReference__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultModeStateReferenceAccess().getStateAssignment_2()); }
	(rule__FaultModeStateReference__StateAssignment_2)
	{ after(grammarAccess.getFaultModeStateReferenceAccess().getStateAssignment_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__FaultTransitionTrigger__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultTransitionTrigger__Group__0__Impl
	rule__FaultTransitionTrigger__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultTransitionTrigger__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultTransitionTriggerAccess().getFaultModeAssignment_0()); }
	(rule__FaultTransitionTrigger__FaultModeAssignment_0)
	{ after(grammarAccess.getFaultTransitionTriggerAccess().getFaultModeAssignment_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultTransitionTrigger__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultTransitionTrigger__Group__1__Impl
	rule__FaultTransitionTrigger__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultTransitionTrigger__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultTransitionTriggerAccess().getFullStopKeyword_1()); }
	'.'
	{ after(grammarAccess.getFaultTransitionTriggerAccess().getFullStopKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultTransitionTrigger__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FaultTransitionTrigger__Group__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultTransitionTrigger__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFaultTransitionTriggerAccess().getEventAssignment_2()); }
	(rule__FaultTransitionTrigger__EventAssignment_2)
	{ after(grammarAccess.getFaultTransitionTriggerAccess().getEventAssignment_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__CommonCauses__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CommonCauses__Group__0__Impl
	rule__CommonCauses__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauses__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCommonCausesAccess().getCommonCausesKeyword_0()); }
	'common-causes'
	{ after(grammarAccess.getCommonCausesAccess().getCommonCausesKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauses__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CommonCauses__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauses__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	(
		{ before(grammarAccess.getCommonCausesAccess().getCommonCausesAssignment_1()); }
		(rule__CommonCauses__CommonCausesAssignment_1)
		{ after(grammarAccess.getCommonCausesAccess().getCommonCausesAssignment_1()); }
	)
	(
		{ before(grammarAccess.getCommonCausesAccess().getCommonCausesAssignment_1()); }
		(rule__CommonCauses__CommonCausesAssignment_1)*
		{ after(grammarAccess.getCommonCausesAccess().getCommonCausesAssignment_1()); }
	)
)
;
finally {
	restoreStackSize(stackSize);
}


rule__CommonCause__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CommonCause__Group__0__Impl
	rule__CommonCause__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCause__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCommonCauseAccess().getCauseKeyword_0()); }
	'cause'
	{ after(grammarAccess.getCommonCauseAccess().getCauseKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCause__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CommonCause__Group__1__Impl
	rule__CommonCause__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCause__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCommonCauseAccess().getNameAssignment_1()); }
	(rule__CommonCause__NameAssignment_1)
	{ after(grammarAccess.getCommonCauseAccess().getNameAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCause__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CommonCause__Group__2__Impl
	rule__CommonCause__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCause__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCommonCauseAccess().getProbabilityAssignment_2()); }
	(rule__CommonCause__ProbabilityAssignment_2)?
	{ after(grammarAccess.getCommonCauseAccess().getProbabilityAssignment_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCause__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CommonCause__Group__3__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCause__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	(
		{ before(grammarAccess.getCommonCauseAccess().getModesAssignment_3()); }
		(rule__CommonCause__ModesAssignment_3)
		{ after(grammarAccess.getCommonCauseAccess().getModesAssignment_3()); }
	)
	(
		{ before(grammarAccess.getCommonCauseAccess().getModesAssignment_3()); }
		(rule__CommonCause__ModesAssignment_3)*
		{ after(grammarAccess.getCommonCauseAccess().getModesAssignment_3()); }
	)
)
;
finally {
	restoreStackSize(stackSize);
}


rule__CommonCauseProbability__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CommonCauseProbability__Group__0__Impl
	rule__CommonCauseProbability__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauseProbability__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCommonCauseProbabilityAccess().getLeftParenthesisKeyword_0()); }
	'('
	{ after(grammarAccess.getCommonCauseProbabilityAccess().getLeftParenthesisKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauseProbability__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CommonCauseProbability__Group__1__Impl
	rule__CommonCauseProbability__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauseProbability__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCommonCauseProbabilityAccess().getValueAssignment_1()); }
	(rule__CommonCauseProbability__ValueAssignment_1)
	{ after(grammarAccess.getCommonCauseProbabilityAccess().getValueAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauseProbability__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CommonCauseProbability__Group__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauseProbability__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCommonCauseProbabilityAccess().getRightParenthesisKeyword_2()); }
	')'
	{ after(grammarAccess.getCommonCauseProbabilityAccess().getRightParenthesisKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__CommonCauseMode__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CommonCauseMode__Group__0__Impl
	rule__CommonCauseMode__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauseMode__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCommonCauseModeAccess().getModeKeyword_0()); }
	'mode'
	{ after(grammarAccess.getCommonCauseModeAccess().getModeKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauseMode__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CommonCauseMode__Group__1__Impl
	rule__CommonCauseMode__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauseMode__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCommonCauseModeAccess().getFaultSliceAssignment_1()); }
	(rule__CommonCauseMode__FaultSliceAssignment_1)
	{ after(grammarAccess.getCommonCauseModeAccess().getFaultSliceAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauseMode__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CommonCauseMode__Group__2__Impl
	rule__CommonCauseMode__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauseMode__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCommonCauseModeAccess().getFullStopKeyword_2()); }
	'.'
	{ after(grammarAccess.getCommonCauseModeAccess().getFullStopKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauseMode__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CommonCauseMode__Group__3__Impl
	rule__CommonCauseMode__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauseMode__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCommonCauseModeAccess().getFaultModeAssignment_3()); }
	(rule__CommonCauseMode__FaultModeAssignment_3)
	{ after(grammarAccess.getCommonCauseModeAccess().getFaultModeAssignment_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauseMode__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CommonCauseMode__Group__4__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauseMode__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCommonCauseModeAccess().getRangeAssignment_4()); }
	(rule__CommonCauseMode__RangeAssignment_4)
	{ after(grammarAccess.getCommonCauseModeAccess().getRangeAssignment_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__CommonCauseRange__Group_0__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CommonCauseRange__Group_0__0__Impl
	rule__CommonCauseRange__Group_0__1
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauseRange__Group_0__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCommonCauseRangeAccess().getCommonCauseRangeAction_0_0()); }
	()
	{ after(grammarAccess.getCommonCauseRangeAccess().getCommonCauseRangeAction_0_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauseRange__Group_0__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CommonCauseRange__Group_0__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauseRange__Group_0__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCommonCauseRangeAccess().getSimultaneouslyKeyword_0_1()); }
	'simultaneously'
	{ after(grammarAccess.getCommonCauseRangeAccess().getSimultaneouslyKeyword_0_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__CommonCauseRange__Group_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CommonCauseRange__Group_1__0__Impl
	rule__CommonCauseRange__Group_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauseRange__Group_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCommonCauseRangeAccess().getAfterKeyword_1_0()); }
	'after'
	{ after(grammarAccess.getCommonCauseRangeAccess().getAfterKeyword_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauseRange__Group_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CommonCauseRange__Group_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauseRange__Group_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCommonCauseRangeAccess().getLowerBoundAssignment_1_1()); }
	(rule__CommonCauseRange__LowerBoundAssignment_1_1)
	{ after(grammarAccess.getCommonCauseRangeAccess().getLowerBoundAssignment_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__CommonCauseRange__Group_2__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CommonCauseRange__Group_2__0__Impl
	rule__CommonCauseRange__Group_2__1
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauseRange__Group_2__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCommonCauseRangeAccess().getWithinKeyword_2_0()); }
	'within'
	{ after(grammarAccess.getCommonCauseRangeAccess().getWithinKeyword_2_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauseRange__Group_2__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CommonCauseRange__Group_2__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauseRange__Group_2__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCommonCauseRangeAccess().getHigherBoundAssignment_2_1()); }
	(rule__CommonCauseRange__HigherBoundAssignment_2_1)
	{ after(grammarAccess.getCommonCauseRangeAccess().getHigherBoundAssignment_2_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__CommonCauseRange__Group_3__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CommonCauseRange__Group_3__0__Impl
	rule__CommonCauseRange__Group_3__1
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauseRange__Group_3__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCommonCauseRangeAccess().getBetweenKeyword_3_0()); }
	'between'
	{ after(grammarAccess.getCommonCauseRangeAccess().getBetweenKeyword_3_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauseRange__Group_3__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CommonCauseRange__Group_3__1__Impl
	rule__CommonCauseRange__Group_3__2
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauseRange__Group_3__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCommonCauseRangeAccess().getLowerBoundAssignment_3_1()); }
	(rule__CommonCauseRange__LowerBoundAssignment_3_1)
	{ after(grammarAccess.getCommonCauseRangeAccess().getLowerBoundAssignment_3_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauseRange__Group_3__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CommonCauseRange__Group_3__2__Impl
	rule__CommonCauseRange__Group_3__3
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauseRange__Group_3__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCommonCauseRangeAccess().getFullStopFullStopKeyword_3_2()); }
	'..'
	{ after(grammarAccess.getCommonCauseRangeAccess().getFullStopFullStopKeyword_3_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauseRange__Group_3__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CommonCauseRange__Group_3__3__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauseRange__Group_3__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCommonCauseRangeAccess().getHigherBoundAssignment_3_3()); }
	(rule__CommonCauseRange__HigherBoundAssignment_3_3)
	{ after(grammarAccess.getCommonCauseRangeAccess().getHigherBoundAssignment_3_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__PrimaryExpression__Group_2__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__PrimaryExpression__Group_2__0__Impl
	rule__PrimaryExpression__Group_2__1
;
finally {
	restoreStackSize(stackSize);
}

rule__PrimaryExpression__Group_2__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getPrimaryExpressionAccess().getLeftParenthesisKeyword_2_0()); }
	'('
	{ after(grammarAccess.getPrimaryExpressionAccess().getLeftParenthesisKeyword_2_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__PrimaryExpression__Group_2__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__PrimaryExpression__Group_2__1__Impl
	rule__PrimaryExpression__Group_2__2
;
finally {
	restoreStackSize(stackSize);
}

rule__PrimaryExpression__Group_2__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getPrimaryExpressionAccess().getExpressionParserRuleCall_2_1()); }
	ruleExpression
	{ after(grammarAccess.getPrimaryExpressionAccess().getExpressionParserRuleCall_2_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__PrimaryExpression__Group_2__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__PrimaryExpression__Group_2__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__PrimaryExpression__Group_2__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getPrimaryExpressionAccess().getRightParenthesisKeyword_2_2()); }
	')'
	{ after(grammarAccess.getPrimaryExpressionAccess().getRightParenthesisKeyword_2_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__TimeoutDeclaration__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__TimeoutDeclaration__Group__0__Impl
	rule__TimeoutDeclaration__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__TimeoutDeclaration__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getTimeoutDeclarationAccess().getTimeoutKeyword_0()); }
	'timeout'
	{ after(grammarAccess.getTimeoutDeclarationAccess().getTimeoutKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__TimeoutDeclaration__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__TimeoutDeclaration__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__TimeoutDeclaration__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getTimeoutDeclarationAccess().getNameAssignment_1()); }
	(rule__TimeoutDeclaration__NameAssignment_1)
	{ after(grammarAccess.getTimeoutDeclarationAccess().getNameAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__EqualTrigger__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__EqualTrigger__Group__0__Impl
	rule__EqualTrigger__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__EqualTrigger__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEqualTriggerAccess().getImplyTriggerParserRuleCall_0()); }
	ruleImplyTrigger
	{ after(grammarAccess.getEqualTriggerAccess().getImplyTriggerParserRuleCall_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__EqualTrigger__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__EqualTrigger__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__EqualTrigger__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEqualTriggerAccess().getGroup_1()); }
	(rule__EqualTrigger__Group_1__0)?
	{ after(grammarAccess.getEqualTriggerAccess().getGroup_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__EqualTrigger__Group_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__EqualTrigger__Group_1__0__Impl
	rule__EqualTrigger__Group_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__EqualTrigger__Group_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEqualTriggerAccess().getBinaryTriggerLeftOperandAction_1_0()); }
	()
	{ after(grammarAccess.getEqualTriggerAccess().getBinaryTriggerLeftOperandAction_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__EqualTrigger__Group_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__EqualTrigger__Group_1__1__Impl
	rule__EqualTrigger__Group_1__2
;
finally {
	restoreStackSize(stackSize);
}

rule__EqualTrigger__Group_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEqualTriggerAccess().getTypeAssignment_1_1()); }
	(rule__EqualTrigger__TypeAssignment_1_1)
	{ after(grammarAccess.getEqualTriggerAccess().getTypeAssignment_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__EqualTrigger__Group_1__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__EqualTrigger__Group_1__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__EqualTrigger__Group_1__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEqualTriggerAccess().getRightOperandAssignment_1_2()); }
	(rule__EqualTrigger__RightOperandAssignment_1_2)
	{ after(grammarAccess.getEqualTriggerAccess().getRightOperandAssignment_1_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ImplyTrigger__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ImplyTrigger__Group__0__Impl
	rule__ImplyTrigger__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ImplyTrigger__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getImplyTriggerAccess().getOrTriggerParserRuleCall_0()); }
	ruleOrTrigger
	{ after(grammarAccess.getImplyTriggerAccess().getOrTriggerParserRuleCall_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ImplyTrigger__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ImplyTrigger__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ImplyTrigger__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getImplyTriggerAccess().getGroup_1()); }
	(rule__ImplyTrigger__Group_1__0)?
	{ after(grammarAccess.getImplyTriggerAccess().getGroup_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ImplyTrigger__Group_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ImplyTrigger__Group_1__0__Impl
	rule__ImplyTrigger__Group_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ImplyTrigger__Group_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getImplyTriggerAccess().getBinaryTriggerLeftOperandAction_1_0()); }
	()
	{ after(grammarAccess.getImplyTriggerAccess().getBinaryTriggerLeftOperandAction_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ImplyTrigger__Group_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ImplyTrigger__Group_1__1__Impl
	rule__ImplyTrigger__Group_1__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ImplyTrigger__Group_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getImplyTriggerAccess().getTypeAssignment_1_1()); }
	(rule__ImplyTrigger__TypeAssignment_1_1)
	{ after(grammarAccess.getImplyTriggerAccess().getTypeAssignment_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ImplyTrigger__Group_1__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ImplyTrigger__Group_1__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ImplyTrigger__Group_1__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getImplyTriggerAccess().getRightOperandAssignment_1_2()); }
	(rule__ImplyTrigger__RightOperandAssignment_1_2)
	{ after(grammarAccess.getImplyTriggerAccess().getRightOperandAssignment_1_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__OrTrigger__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__OrTrigger__Group__0__Impl
	rule__OrTrigger__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__OrTrigger__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getOrTriggerAccess().getXorTriggerParserRuleCall_0()); }
	ruleXorTrigger
	{ after(grammarAccess.getOrTriggerAccess().getXorTriggerParserRuleCall_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__OrTrigger__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__OrTrigger__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__OrTrigger__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getOrTriggerAccess().getGroup_1()); }
	(rule__OrTrigger__Group_1__0)?
	{ after(grammarAccess.getOrTriggerAccess().getGroup_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__OrTrigger__Group_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__OrTrigger__Group_1__0__Impl
	rule__OrTrigger__Group_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__OrTrigger__Group_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getOrTriggerAccess().getBinaryTriggerLeftOperandAction_1_0()); }
	()
	{ after(grammarAccess.getOrTriggerAccess().getBinaryTriggerLeftOperandAction_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__OrTrigger__Group_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__OrTrigger__Group_1__1__Impl
	rule__OrTrigger__Group_1__2
;
finally {
	restoreStackSize(stackSize);
}

rule__OrTrigger__Group_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getOrTriggerAccess().getTypeAssignment_1_1()); }
	(rule__OrTrigger__TypeAssignment_1_1)
	{ after(grammarAccess.getOrTriggerAccess().getTypeAssignment_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__OrTrigger__Group_1__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__OrTrigger__Group_1__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__OrTrigger__Group_1__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getOrTriggerAccess().getRightOperandAssignment_1_2()); }
	(rule__OrTrigger__RightOperandAssignment_1_2)
	{ after(grammarAccess.getOrTriggerAccess().getRightOperandAssignment_1_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__XorTrigger__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__XorTrigger__Group__0__Impl
	rule__XorTrigger__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__XorTrigger__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getXorTriggerAccess().getAndTriggerParserRuleCall_0()); }
	ruleAndTrigger
	{ after(grammarAccess.getXorTriggerAccess().getAndTriggerParserRuleCall_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__XorTrigger__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__XorTrigger__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__XorTrigger__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getXorTriggerAccess().getGroup_1()); }
	(rule__XorTrigger__Group_1__0)?
	{ after(grammarAccess.getXorTriggerAccess().getGroup_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__XorTrigger__Group_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__XorTrigger__Group_1__0__Impl
	rule__XorTrigger__Group_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__XorTrigger__Group_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getXorTriggerAccess().getBinaryTriggerLeftOperandAction_1_0()); }
	()
	{ after(grammarAccess.getXorTriggerAccess().getBinaryTriggerLeftOperandAction_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__XorTrigger__Group_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__XorTrigger__Group_1__1__Impl
	rule__XorTrigger__Group_1__2
;
finally {
	restoreStackSize(stackSize);
}

rule__XorTrigger__Group_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getXorTriggerAccess().getTypeAssignment_1_1()); }
	(rule__XorTrigger__TypeAssignment_1_1)
	{ after(grammarAccess.getXorTriggerAccess().getTypeAssignment_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__XorTrigger__Group_1__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__XorTrigger__Group_1__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__XorTrigger__Group_1__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getXorTriggerAccess().getRightOperandAssignment_1_2()); }
	(rule__XorTrigger__RightOperandAssignment_1_2)
	{ after(grammarAccess.getXorTriggerAccess().getRightOperandAssignment_1_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AndTrigger__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AndTrigger__Group__0__Impl
	rule__AndTrigger__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AndTrigger__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAndTriggerAccess().getBinaryOperandParserRuleCall_0()); }
	ruleBinaryOperand
	{ after(grammarAccess.getAndTriggerAccess().getBinaryOperandParserRuleCall_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AndTrigger__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AndTrigger__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AndTrigger__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAndTriggerAccess().getGroup_1()); }
	(rule__AndTrigger__Group_1__0)?
	{ after(grammarAccess.getAndTriggerAccess().getGroup_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AndTrigger__Group_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AndTrigger__Group_1__0__Impl
	rule__AndTrigger__Group_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AndTrigger__Group_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAndTriggerAccess().getBinaryTriggerLeftOperandAction_1_0()); }
	()
	{ after(grammarAccess.getAndTriggerAccess().getBinaryTriggerLeftOperandAction_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AndTrigger__Group_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AndTrigger__Group_1__1__Impl
	rule__AndTrigger__Group_1__2
;
finally {
	restoreStackSize(stackSize);
}

rule__AndTrigger__Group_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAndTriggerAccess().getTypeAssignment_1_1()); }
	(rule__AndTrigger__TypeAssignment_1_1)
	{ after(grammarAccess.getAndTriggerAccess().getTypeAssignment_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AndTrigger__Group_1__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AndTrigger__Group_1__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AndTrigger__Group_1__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAndTriggerAccess().getRightOperandAssignment_1_2()); }
	(rule__AndTrigger__RightOperandAssignment_1_2)
	{ after(grammarAccess.getAndTriggerAccess().getRightOperandAssignment_1_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__NotTrigger__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NotTrigger__Group__0__Impl
	rule__NotTrigger__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__NotTrigger__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNotTriggerAccess().getTypeAssignment_0()); }
	(rule__NotTrigger__TypeAssignment_0)
	{ after(grammarAccess.getNotTriggerAccess().getTypeAssignment_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NotTrigger__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NotTrigger__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__NotTrigger__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNotTriggerAccess().getOperandAssignment_1()); }
	(rule__NotTrigger__OperandAssignment_1)
	{ after(grammarAccess.getNotTriggerAccess().getOperandAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ParenthesesTrigger__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ParenthesesTrigger__Group__0__Impl
	rule__ParenthesesTrigger__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ParenthesesTrigger__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getParenthesesTriggerAccess().getLeftParenthesisKeyword_0()); }
	'('
	{ after(grammarAccess.getParenthesesTriggerAccess().getLeftParenthesisKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ParenthesesTrigger__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ParenthesesTrigger__Group__1__Impl
	rule__ParenthesesTrigger__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ParenthesesTrigger__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getParenthesesTriggerAccess().getEqualTriggerParserRuleCall_1()); }
	ruleEqualTrigger
	{ after(grammarAccess.getParenthesesTriggerAccess().getEqualTriggerParserRuleCall_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ParenthesesTrigger__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ParenthesesTrigger__Group__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ParenthesesTrigger__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getParenthesesTriggerAccess().getRightParenthesisKeyword_2()); }
	')'
	{ after(grammarAccess.getParenthesesTriggerAccess().getRightParenthesisKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__OnCycleTrigger__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__OnCycleTrigger__Group__0__Impl
	rule__OnCycleTrigger__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__OnCycleTrigger__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getOnCycleTriggerAccess().getOnCycleTriggerAction_0()); }
	()
	{ after(grammarAccess.getOnCycleTriggerAccess().getOnCycleTriggerAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__OnCycleTrigger__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__OnCycleTrigger__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__OnCycleTrigger__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getOnCycleTriggerAccess().getCycleKeyword_1()); }
	'cycle'
	{ after(grammarAccess.getOnCycleTriggerAccess().getCycleKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AnyTrigger__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AnyTrigger__Group__0__Impl
	rule__AnyTrigger__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AnyTrigger__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAnyTriggerAccess().getAnyTriggerAction_0()); }
	()
	{ after(grammarAccess.getAnyTriggerAccess().getAnyTriggerAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AnyTrigger__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AnyTrigger__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AnyTrigger__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAnyTriggerAccess().getAnyKeyword_1()); }
	'any'
	{ after(grammarAccess.getAnyTriggerAccess().getAnyKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__TimeoutEventReference__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__TimeoutEventReference__Group__0__Impl
	rule__TimeoutEventReference__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__TimeoutEventReference__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getTimeoutEventReferenceAccess().getTimeoutKeyword_0()); }
	'timeout'
	{ after(grammarAccess.getTimeoutEventReferenceAccess().getTimeoutKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__TimeoutEventReference__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__TimeoutEventReference__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__TimeoutEventReference__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getTimeoutEventReferenceAccess().getTimeoutAssignment_1()); }
	(rule__TimeoutEventReference__TimeoutAssignment_1)
	{ after(grammarAccess.getTimeoutEventReferenceAccess().getTimeoutAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AnyPortEventReference__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AnyPortEventReference__Group__0__Impl
	rule__AnyPortEventReference__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AnyPortEventReference__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAnyPortEventReferenceAccess().getPortAssignment_0()); }
	(rule__AnyPortEventReference__PortAssignment_0)
	{ after(grammarAccess.getAnyPortEventReferenceAccess().getPortAssignment_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AnyPortEventReference__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AnyPortEventReference__Group__1__Impl
	rule__AnyPortEventReference__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__AnyPortEventReference__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAnyPortEventReferenceAccess().getFullStopKeyword_1()); }
	'.'
	{ after(grammarAccess.getAnyPortEventReferenceAccess().getFullStopKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AnyPortEventReference__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AnyPortEventReference__Group__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AnyPortEventReference__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAnyPortEventReferenceAccess().getAnyKeyword_2()); }
	'any'
	{ after(grammarAccess.getAnyPortEventReferenceAccess().getAnyKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__PortEventReference__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__PortEventReference__Group__0__Impl
	rule__PortEventReference__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__PortEventReference__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getPortEventReferenceAccess().getPortAssignment_0()); }
	(rule__PortEventReference__PortAssignment_0)
	{ after(grammarAccess.getPortEventReferenceAccess().getPortAssignment_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__PortEventReference__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__PortEventReference__Group__1__Impl
	rule__PortEventReference__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__PortEventReference__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getPortEventReferenceAccess().getFullStopKeyword_1()); }
	'.'
	{ after(grammarAccess.getPortEventReferenceAccess().getFullStopKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__PortEventReference__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__PortEventReference__Group__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__PortEventReference__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getPortEventReferenceAccess().getEventAssignment_2()); }
	(rule__PortEventReference__EventAssignment_2)
	{ after(grammarAccess.getPortEventReferenceAccess().getEventAssignment_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__Port__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Port__Group__0__Impl
	rule__Port__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__Port__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getPortAccess().getPortKeyword_0()); }
	'port'
	{ after(grammarAccess.getPortAccess().getPortKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Port__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Port__Group__1__Impl
	rule__Port__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__Port__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getPortAccess().getNameAssignment_1()); }
	(rule__Port__NameAssignment_1)
	{ after(grammarAccess.getPortAccess().getNameAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Port__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Port__Group__2__Impl
	rule__Port__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__Port__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getPortAccess().getColonKeyword_2()); }
	':'
	{ after(grammarAccess.getPortAccess().getColonKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Port__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Port__Group__3__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__Port__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getPortAccess().getInterfaceRealizationAssignment_3()); }
	(rule__Port__InterfaceRealizationAssignment_3)
	{ after(grammarAccess.getPortAccess().getInterfaceRealizationAssignment_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__InterfaceRealization__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__InterfaceRealization__Group__0__Impl
	rule__InterfaceRealization__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__InterfaceRealization__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInterfaceRealizationAccess().getRealizationModeAssignment_0()); }
	(rule__InterfaceRealization__RealizationModeAssignment_0)
	{ after(grammarAccess.getInterfaceRealizationAccess().getRealizationModeAssignment_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__InterfaceRealization__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__InterfaceRealization__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__InterfaceRealization__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInterfaceRealizationAccess().getInterfaceAssignment_1()); }
	(rule__InterfaceRealization__InterfaceAssignment_1)
	{ after(grammarAccess.getInterfaceRealizationAccess().getInterfaceAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__SynchronousStatechartDefinition__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousStatechartDefinition__Group__0__Impl
	rule__SynchronousStatechartDefinition__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getUnorderedGroup_0()); }
	(rule__SynchronousStatechartDefinition__UnorderedGroup_0)
	{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getUnorderedGroup_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousStatechartDefinition__Group__1__Impl
	rule__SynchronousStatechartDefinition__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getStatechartKeyword_1()); }
	'statechart'
	{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getStatechartKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousStatechartDefinition__Group__2__Impl
	rule__SynchronousStatechartDefinition__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getNameAssignment_2()); }
	(rule__SynchronousStatechartDefinition__NameAssignment_2)
	{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getNameAssignment_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousStatechartDefinition__Group__3__Impl
	rule__SynchronousStatechartDefinition__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getGroup_3()); }
	(rule__SynchronousStatechartDefinition__Group_3__0)?
	{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getGroup_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousStatechartDefinition__Group__4__Impl
	rule__SynchronousStatechartDefinition__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getGroup_4()); }
	(rule__SynchronousStatechartDefinition__Group_4__0)?
	{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getGroup_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousStatechartDefinition__Group__5__Impl
	rule__SynchronousStatechartDefinition__Group__6
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getLeftCurlyBracketKeyword_5()); }
	'{'
	{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getLeftCurlyBracketKeyword_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group__6
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousStatechartDefinition__Group__6__Impl
	rule__SynchronousStatechartDefinition__Group__7
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group__6__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getAlternatives_6()); }
	(rule__SynchronousStatechartDefinition__Alternatives_6)*
	{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getAlternatives_6()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group__7
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousStatechartDefinition__Group__7__Impl
	rule__SynchronousStatechartDefinition__Group__8
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group__7__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getAlternatives_7()); }
	(rule__SynchronousStatechartDefinition__Alternatives_7)*
	{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getAlternatives_7()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group__8
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousStatechartDefinition__Group__8__Impl
	rule__SynchronousStatechartDefinition__Group__9
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group__8__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getFunctionDeclarationsAssignment_8()); }
	(rule__SynchronousStatechartDefinition__FunctionDeclarationsAssignment_8)*
	{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getFunctionDeclarationsAssignment_8()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group__9
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousStatechartDefinition__Group__9__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group__9__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getRightCurlyBracketKeyword_9()); }
	'}'
	{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getRightCurlyBracketKeyword_9()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__SynchronousStatechartDefinition__Group_0_0__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousStatechartDefinition__Group_0_0__0__Impl
	rule__SynchronousStatechartDefinition__Group_0_0__1
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group_0_0__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getRegionScheduleKeyword_0_0_0()); }
	'@RegionSchedule'
	{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getRegionScheduleKeyword_0_0_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group_0_0__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousStatechartDefinition__Group_0_0__1__Impl
	rule__SynchronousStatechartDefinition__Group_0_0__2
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group_0_0__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getEqualsSignKeyword_0_0_1()); }
	'='
	{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getEqualsSignKeyword_0_0_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group_0_0__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousStatechartDefinition__Group_0_0__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group_0_0__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getSchedulingOrderAssignment_0_0_2()); }
	(rule__SynchronousStatechartDefinition__SchedulingOrderAssignment_0_0_2)
	{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getSchedulingOrderAssignment_0_0_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__SynchronousStatechartDefinition__Group_0_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousStatechartDefinition__Group_0_1__0__Impl
	rule__SynchronousStatechartDefinition__Group_0_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group_0_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getOrthogonalRegionScheduleKeyword_0_1_0()); }
	'@OrthogonalRegionSchedule'
	{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getOrthogonalRegionScheduleKeyword_0_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group_0_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousStatechartDefinition__Group_0_1__1__Impl
	rule__SynchronousStatechartDefinition__Group_0_1__2
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group_0_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getEqualsSignKeyword_0_1_1()); }
	'='
	{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getEqualsSignKeyword_0_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group_0_1__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousStatechartDefinition__Group_0_1__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group_0_1__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getOrthogonalRegionSchedulingOrderAssignment_0_1_2()); }
	(rule__SynchronousStatechartDefinition__OrthogonalRegionSchedulingOrderAssignment_0_1_2)
	{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getOrthogonalRegionSchedulingOrderAssignment_0_1_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__SynchronousStatechartDefinition__Group_0_2__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousStatechartDefinition__Group_0_2__0__Impl
	rule__SynchronousStatechartDefinition__Group_0_2__1
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group_0_2__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getTransitionPriorityKeyword_0_2_0()); }
	'@TransitionPriority'
	{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getTransitionPriorityKeyword_0_2_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group_0_2__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousStatechartDefinition__Group_0_2__1__Impl
	rule__SynchronousStatechartDefinition__Group_0_2__2
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group_0_2__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getEqualsSignKeyword_0_2_1()); }
	'='
	{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getEqualsSignKeyword_0_2_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group_0_2__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousStatechartDefinition__Group_0_2__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group_0_2__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getTransitionPriorityAssignment_0_2_2()); }
	(rule__SynchronousStatechartDefinition__TransitionPriorityAssignment_0_2_2)
	{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getTransitionPriorityAssignment_0_2_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__SynchronousStatechartDefinition__Group_0_3__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousStatechartDefinition__Group_0_3__0__Impl
	rule__SynchronousStatechartDefinition__Group_0_3__1
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group_0_3__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getGuardEvaluationKeyword_0_3_0()); }
	'@GuardEvaluation'
	{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getGuardEvaluationKeyword_0_3_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group_0_3__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousStatechartDefinition__Group_0_3__1__Impl
	rule__SynchronousStatechartDefinition__Group_0_3__2
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group_0_3__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getEqualsSignKeyword_0_3_1()); }
	'='
	{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getEqualsSignKeyword_0_3_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group_0_3__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousStatechartDefinition__Group_0_3__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group_0_3__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getGuardEvaluationAssignment_0_3_2()); }
	(rule__SynchronousStatechartDefinition__GuardEvaluationAssignment_0_3_2)
	{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getGuardEvaluationAssignment_0_3_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__SynchronousStatechartDefinition__Group_3__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousStatechartDefinition__Group_3__0__Impl
	rule__SynchronousStatechartDefinition__Group_3__1
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group_3__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getLeftParenthesisKeyword_3_0()); }
	'('
	{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getLeftParenthesisKeyword_3_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group_3__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousStatechartDefinition__Group_3__1__Impl
	rule__SynchronousStatechartDefinition__Group_3__2
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group_3__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getGroup_3_1()); }
	(rule__SynchronousStatechartDefinition__Group_3_1__0)?
	{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getGroup_3_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group_3__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousStatechartDefinition__Group_3__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group_3__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getRightParenthesisKeyword_3_2()); }
	')'
	{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getRightParenthesisKeyword_3_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__SynchronousStatechartDefinition__Group_3_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousStatechartDefinition__Group_3_1__0__Impl
	rule__SynchronousStatechartDefinition__Group_3_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group_3_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getParameterDeclarationsAssignment_3_1_0()); }
	(rule__SynchronousStatechartDefinition__ParameterDeclarationsAssignment_3_1_0)
	{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getParameterDeclarationsAssignment_3_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group_3_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousStatechartDefinition__Group_3_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group_3_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getGroup_3_1_1()); }
	(rule__SynchronousStatechartDefinition__Group_3_1_1__0)*
	{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getGroup_3_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__SynchronousStatechartDefinition__Group_3_1_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousStatechartDefinition__Group_3_1_1__0__Impl
	rule__SynchronousStatechartDefinition__Group_3_1_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group_3_1_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getCommaKeyword_3_1_1_0()); }
	','
	{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getCommaKeyword_3_1_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group_3_1_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousStatechartDefinition__Group_3_1_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group_3_1_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getParameterDeclarationsAssignment_3_1_1_1()); }
	(rule__SynchronousStatechartDefinition__ParameterDeclarationsAssignment_3_1_1_1)
	{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getParameterDeclarationsAssignment_3_1_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__SynchronousStatechartDefinition__Group_4__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousStatechartDefinition__Group_4__0__Impl
	rule__SynchronousStatechartDefinition__Group_4__1
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group_4__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getLeftSquareBracketKeyword_4_0()); }
	'['
	{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getLeftSquareBracketKeyword_4_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group_4__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousStatechartDefinition__Group_4__1__Impl
	rule__SynchronousStatechartDefinition__Group_4__2
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group_4__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getGroup_4_1()); }
	(rule__SynchronousStatechartDefinition__Group_4_1__0)?
	{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getGroup_4_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group_4__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousStatechartDefinition__Group_4__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group_4__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getRightSquareBracketKeyword_4_2()); }
	']'
	{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getRightSquareBracketKeyword_4_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__SynchronousStatechartDefinition__Group_4_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousStatechartDefinition__Group_4_1__0__Impl
	rule__SynchronousStatechartDefinition__Group_4_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group_4_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getPortsAssignment_4_1_0()); }
	(rule__SynchronousStatechartDefinition__PortsAssignment_4_1_0)
	{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getPortsAssignment_4_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group_4_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousStatechartDefinition__Group_4_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group_4_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getGroup_4_1_1()); }
	(rule__SynchronousStatechartDefinition__Group_4_1_1__0)*
	{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getGroup_4_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__SynchronousStatechartDefinition__Group_4_1_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousStatechartDefinition__Group_4_1_1__0__Impl
	rule__SynchronousStatechartDefinition__Group_4_1_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group_4_1_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getCommaKeyword_4_1_1_0()); }
	(',')?
	{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getCommaKeyword_4_1_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group_4_1_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousStatechartDefinition__Group_4_1_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__Group_4_1_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getPortsAssignment_4_1_1_1()); }
	(rule__SynchronousStatechartDefinition__PortsAssignment_4_1_1_1)
	{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getPortsAssignment_4_1_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AsynchronousStatechartDefinition__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__Group__0__Impl
	rule__AsynchronousStatechartDefinition__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getAsynchronousKeyword_0()); }
	'@Asynchronous'
	{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getAsynchronousKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__Group__1__Impl
	rule__AsynchronousStatechartDefinition__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getUnorderedGroup_1()); }
	(rule__AsynchronousStatechartDefinition__UnorderedGroup_1)
	{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getUnorderedGroup_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__Group__2__Impl
	rule__AsynchronousStatechartDefinition__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getStatechartKeyword_2()); }
	'statechart'
	{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getStatechartKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__Group__3__Impl
	rule__AsynchronousStatechartDefinition__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getNameAssignment_3()); }
	(rule__AsynchronousStatechartDefinition__NameAssignment_3)
	{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getNameAssignment_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__Group__4__Impl
	rule__AsynchronousStatechartDefinition__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getGroup_4()); }
	(rule__AsynchronousStatechartDefinition__Group_4__0)?
	{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getGroup_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__Group__5__Impl
	rule__AsynchronousStatechartDefinition__Group__6
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getGroup_5()); }
	(rule__AsynchronousStatechartDefinition__Group_5__0)?
	{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getGroup_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group__6
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__Group__6__Impl
	rule__AsynchronousStatechartDefinition__Group__7
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group__6__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getGroup_6()); }
	(rule__AsynchronousStatechartDefinition__Group_6__0)?
	{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getGroup_6()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group__7
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__Group__7__Impl
	rule__AsynchronousStatechartDefinition__Group__8
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group__7__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getLeftCurlyBracketKeyword_7()); }
	'{'
	{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getLeftCurlyBracketKeyword_7()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group__8
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__Group__8__Impl
	rule__AsynchronousStatechartDefinition__Group__9
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group__8__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getAlternatives_8()); }
	(rule__AsynchronousStatechartDefinition__Alternatives_8)*
	{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getAlternatives_8()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group__9
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__Group__9__Impl
	rule__AsynchronousStatechartDefinition__Group__10
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group__9__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getAlternatives_9()); }
	(rule__AsynchronousStatechartDefinition__Alternatives_9)*
	{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getAlternatives_9()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group__10
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__Group__10__Impl
	rule__AsynchronousStatechartDefinition__Group__11
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group__10__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getFunctionDeclarationsAssignment_10()); }
	(rule__AsynchronousStatechartDefinition__FunctionDeclarationsAssignment_10)*
	{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getFunctionDeclarationsAssignment_10()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group__11
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__Group__11__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group__11__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getRightCurlyBracketKeyword_11()); }
	'}'
	{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getRightCurlyBracketKeyword_11()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AsynchronousStatechartDefinition__Group_1_0__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__Group_1_0__0__Impl
	rule__AsynchronousStatechartDefinition__Group_1_0__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_1_0__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getRegionScheduleKeyword_1_0_0()); }
	'@RegionSchedule'
	{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getRegionScheduleKeyword_1_0_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_1_0__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__Group_1_0__1__Impl
	rule__AsynchronousStatechartDefinition__Group_1_0__2
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_1_0__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getEqualsSignKeyword_1_0_1()); }
	'='
	{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getEqualsSignKeyword_1_0_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_1_0__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__Group_1_0__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_1_0__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getSchedulingOrderAssignment_1_0_2()); }
	(rule__AsynchronousStatechartDefinition__SchedulingOrderAssignment_1_0_2)
	{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getSchedulingOrderAssignment_1_0_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AsynchronousStatechartDefinition__Group_1_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__Group_1_1__0__Impl
	rule__AsynchronousStatechartDefinition__Group_1_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_1_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getOrthogonalRegionScheduleKeyword_1_1_0()); }
	'@OrthogonalRegionSchedule'
	{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getOrthogonalRegionScheduleKeyword_1_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_1_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__Group_1_1__1__Impl
	rule__AsynchronousStatechartDefinition__Group_1_1__2
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_1_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getEqualsSignKeyword_1_1_1()); }
	'='
	{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getEqualsSignKeyword_1_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_1_1__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__Group_1_1__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_1_1__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getOrthogonalRegionSchedulingOrderAssignment_1_1_2()); }
	(rule__AsynchronousStatechartDefinition__OrthogonalRegionSchedulingOrderAssignment_1_1_2)
	{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getOrthogonalRegionSchedulingOrderAssignment_1_1_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AsynchronousStatechartDefinition__Group_1_2__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__Group_1_2__0__Impl
	rule__AsynchronousStatechartDefinition__Group_1_2__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_1_2__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getTransitionPriorityKeyword_1_2_0()); }
	'@TransitionPriority'
	{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getTransitionPriorityKeyword_1_2_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_1_2__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__Group_1_2__1__Impl
	rule__AsynchronousStatechartDefinition__Group_1_2__2
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_1_2__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getEqualsSignKeyword_1_2_1()); }
	'='
	{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getEqualsSignKeyword_1_2_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_1_2__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__Group_1_2__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_1_2__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getTransitionPriorityAssignment_1_2_2()); }
	(rule__AsynchronousStatechartDefinition__TransitionPriorityAssignment_1_2_2)
	{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getTransitionPriorityAssignment_1_2_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AsynchronousStatechartDefinition__Group_1_3__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__Group_1_3__0__Impl
	rule__AsynchronousStatechartDefinition__Group_1_3__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_1_3__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getGuardEvaluationKeyword_1_3_0()); }
	'@GuardEvaluation'
	{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getGuardEvaluationKeyword_1_3_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_1_3__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__Group_1_3__1__Impl
	rule__AsynchronousStatechartDefinition__Group_1_3__2
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_1_3__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getEqualsSignKeyword_1_3_1()); }
	'='
	{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getEqualsSignKeyword_1_3_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_1_3__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__Group_1_3__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_1_3__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getGuardEvaluationAssignment_1_3_2()); }
	(rule__AsynchronousStatechartDefinition__GuardEvaluationAssignment_1_3_2)
	{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getGuardEvaluationAssignment_1_3_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AsynchronousStatechartDefinition__Group_4__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__Group_4__0__Impl
	rule__AsynchronousStatechartDefinition__Group_4__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_4__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getCapacityKeyword_4_0()); }
	'capacity'
	{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getCapacityKeyword_4_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_4__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__Group_4__1__Impl
	rule__AsynchronousStatechartDefinition__Group_4__2
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_4__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getLeftParenthesisKeyword_4_1()); }
	'('
	{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getLeftParenthesisKeyword_4_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_4__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__Group_4__2__Impl
	rule__AsynchronousStatechartDefinition__Group_4__3
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_4__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getCapacityAssignment_4_2()); }
	(rule__AsynchronousStatechartDefinition__CapacityAssignment_4_2)
	{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getCapacityAssignment_4_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_4__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__Group_4__3__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_4__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getRightParenthesisKeyword_4_3()); }
	')'
	{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getRightParenthesisKeyword_4_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AsynchronousStatechartDefinition__Group_5__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__Group_5__0__Impl
	rule__AsynchronousStatechartDefinition__Group_5__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_5__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getLeftParenthesisKeyword_5_0()); }
	'('
	{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getLeftParenthesisKeyword_5_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_5__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__Group_5__1__Impl
	rule__AsynchronousStatechartDefinition__Group_5__2
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_5__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getGroup_5_1()); }
	(rule__AsynchronousStatechartDefinition__Group_5_1__0)?
	{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getGroup_5_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_5__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__Group_5__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_5__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getRightParenthesisKeyword_5_2()); }
	')'
	{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getRightParenthesisKeyword_5_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AsynchronousStatechartDefinition__Group_5_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__Group_5_1__0__Impl
	rule__AsynchronousStatechartDefinition__Group_5_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_5_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getParameterDeclarationsAssignment_5_1_0()); }
	(rule__AsynchronousStatechartDefinition__ParameterDeclarationsAssignment_5_1_0)
	{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getParameterDeclarationsAssignment_5_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_5_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__Group_5_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_5_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getGroup_5_1_1()); }
	(rule__AsynchronousStatechartDefinition__Group_5_1_1__0)*
	{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getGroup_5_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AsynchronousStatechartDefinition__Group_5_1_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__Group_5_1_1__0__Impl
	rule__AsynchronousStatechartDefinition__Group_5_1_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_5_1_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getCommaKeyword_5_1_1_0()); }
	','
	{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getCommaKeyword_5_1_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_5_1_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__Group_5_1_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_5_1_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getParameterDeclarationsAssignment_5_1_1_1()); }
	(rule__AsynchronousStatechartDefinition__ParameterDeclarationsAssignment_5_1_1_1)
	{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getParameterDeclarationsAssignment_5_1_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AsynchronousStatechartDefinition__Group_6__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__Group_6__0__Impl
	rule__AsynchronousStatechartDefinition__Group_6__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_6__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getLeftSquareBracketKeyword_6_0()); }
	'['
	{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getLeftSquareBracketKeyword_6_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_6__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__Group_6__1__Impl
	rule__AsynchronousStatechartDefinition__Group_6__2
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_6__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getGroup_6_1()); }
	(rule__AsynchronousStatechartDefinition__Group_6_1__0)?
	{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getGroup_6_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_6__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__Group_6__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_6__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getRightSquareBracketKeyword_6_2()); }
	']'
	{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getRightSquareBracketKeyword_6_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AsynchronousStatechartDefinition__Group_6_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__Group_6_1__0__Impl
	rule__AsynchronousStatechartDefinition__Group_6_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_6_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getPortsAssignment_6_1_0()); }
	(rule__AsynchronousStatechartDefinition__PortsAssignment_6_1_0)
	{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getPortsAssignment_6_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_6_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__Group_6_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_6_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getGroup_6_1_1()); }
	(rule__AsynchronousStatechartDefinition__Group_6_1_1__0)*
	{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getGroup_6_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AsynchronousStatechartDefinition__Group_6_1_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__Group_6_1_1__0__Impl
	rule__AsynchronousStatechartDefinition__Group_6_1_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_6_1_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getCommaKeyword_6_1_1_0()); }
	(',')?
	{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getCommaKeyword_6_1_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_6_1_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__Group_6_1_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__Group_6_1_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getPortsAssignment_6_1_1_1()); }
	(rule__AsynchronousStatechartDefinition__PortsAssignment_6_1_1_1)
	{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getPortsAssignment_6_1_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__NegativeContractStatechartAnnotation__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NegativeContractStatechartAnnotation__Group__0__Impl
	rule__NegativeContractStatechartAnnotation__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__NegativeContractStatechartAnnotation__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNegativeContractStatechartAnnotationAccess().getNegativeContractStatechartAnnotationAction_0()); }
	()
	{ after(grammarAccess.getNegativeContractStatechartAnnotationAccess().getNegativeContractStatechartAnnotationAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NegativeContractStatechartAnnotation__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NegativeContractStatechartAnnotation__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__NegativeContractStatechartAnnotation__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNegativeContractStatechartAnnotationAccess().getNegativeContractStatechartKeyword_1()); }
	'@NegativeContractStatechart'
	{ after(grammarAccess.getNegativeContractStatechartAnnotationAccess().getNegativeContractStatechartKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__HasInitialoutputsBlockAnnotation__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__HasInitialoutputsBlockAnnotation__Group__0__Impl
	rule__HasInitialoutputsBlockAnnotation__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__HasInitialoutputsBlockAnnotation__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getHasInitialoutputsBlockAnnotationAccess().getHasInitialOutputsBlockAnnotationAction_0()); }
	()
	{ after(grammarAccess.getHasInitialoutputsBlockAnnotationAccess().getHasInitialOutputsBlockAnnotationAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__HasInitialoutputsBlockAnnotation__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__HasInitialoutputsBlockAnnotation__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__HasInitialoutputsBlockAnnotation__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getHasInitialoutputsBlockAnnotationAccess().getHasInitialOutputsBlockKeyword_1()); }
	'@HasInitialOutputsBlock'
	{ after(grammarAccess.getHasInitialoutputsBlockAnnotationAccess().getHasInitialOutputsBlockKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ScenarioAllowedWaitingAnnotation__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScenarioAllowedWaitingAnnotation__Group__0__Impl
	rule__ScenarioAllowedWaitingAnnotation__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ScenarioAllowedWaitingAnnotation__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScenarioAllowedWaitingAnnotationAccess().getScenarioAllowedWaitAnnotationAction_0()); }
	()
	{ after(grammarAccess.getScenarioAllowedWaitingAnnotationAccess().getScenarioAllowedWaitAnnotationAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScenarioAllowedWaitingAnnotation__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScenarioAllowedWaitingAnnotation__Group__1__Impl
	rule__ScenarioAllowedWaitingAnnotation__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ScenarioAllowedWaitingAnnotation__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScenarioAllowedWaitingAnnotationAccess().getAlternatives_1()); }
	(rule__ScenarioAllowedWaitingAnnotation__Alternatives_1)
	{ after(grammarAccess.getScenarioAllowedWaitingAnnotationAccess().getAlternatives_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScenarioAllowedWaitingAnnotation__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScenarioAllowedWaitingAnnotation__Group__2__Impl
	rule__ScenarioAllowedWaitingAnnotation__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__ScenarioAllowedWaitingAnnotation__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScenarioAllowedWaitingAnnotationAccess().getEqualsSignKeyword_2()); }
	'='
	{ after(grammarAccess.getScenarioAllowedWaitingAnnotationAccess().getEqualsSignKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScenarioAllowedWaitingAnnotation__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScenarioAllowedWaitingAnnotation__Group__3__Impl
	rule__ScenarioAllowedWaitingAnnotation__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__ScenarioAllowedWaitingAnnotation__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScenarioAllowedWaitingAnnotationAccess().getLowerLimitAssignment_3()); }
	(rule__ScenarioAllowedWaitingAnnotation__LowerLimitAssignment_3)
	{ after(grammarAccess.getScenarioAllowedWaitingAnnotationAccess().getLowerLimitAssignment_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScenarioAllowedWaitingAnnotation__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScenarioAllowedWaitingAnnotation__Group__4__Impl
	rule__ScenarioAllowedWaitingAnnotation__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__ScenarioAllowedWaitingAnnotation__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScenarioAllowedWaitingAnnotationAccess().getFullStopFullStopKeyword_4()); }
	'..'
	{ after(grammarAccess.getScenarioAllowedWaitingAnnotationAccess().getFullStopFullStopKeyword_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScenarioAllowedWaitingAnnotation__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScenarioAllowedWaitingAnnotation__Group__5__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ScenarioAllowedWaitingAnnotation__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScenarioAllowedWaitingAnnotationAccess().getUpperLimitAssignment_5()); }
	(rule__ScenarioAllowedWaitingAnnotation__UpperLimitAssignment_5)
	{ after(grammarAccess.getScenarioAllowedWaitingAnnotationAccess().getUpperLimitAssignment_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ScenarioContractAnnotation__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScenarioContractAnnotation__Group__0__Impl
	rule__ScenarioContractAnnotation__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ScenarioContractAnnotation__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScenarioContractAnnotationAccess().getScenarioContractStatechartKeyword_0()); }
	'@ScenarioContractStatechart'
	{ after(grammarAccess.getScenarioContractAnnotationAccess().getScenarioContractStatechartKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScenarioContractAnnotation__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScenarioContractAnnotation__Group__1__Impl
	rule__ScenarioContractAnnotation__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ScenarioContractAnnotation__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScenarioContractAnnotationAccess().getEqualsSignKeyword_1()); }
	'='
	{ after(grammarAccess.getScenarioContractAnnotationAccess().getEqualsSignKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScenarioContractAnnotation__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScenarioContractAnnotation__Group__2__Impl
	rule__ScenarioContractAnnotation__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__ScenarioContractAnnotation__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScenarioContractAnnotationAccess().getMonitoredComponentAssignment_2()); }
	(rule__ScenarioContractAnnotation__MonitoredComponentAssignment_2)
	{ after(grammarAccess.getScenarioContractAnnotationAccess().getMonitoredComponentAssignment_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScenarioContractAnnotation__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScenarioContractAnnotation__Group__3__Impl
	rule__ScenarioContractAnnotation__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__ScenarioContractAnnotation__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScenarioContractAnnotationAccess().getLeftParenthesisKeyword_3()); }
	'('
	{ after(grammarAccess.getScenarioContractAnnotationAccess().getLeftParenthesisKeyword_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScenarioContractAnnotation__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScenarioContractAnnotation__Group__4__Impl
	rule__ScenarioContractAnnotation__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__ScenarioContractAnnotation__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScenarioContractAnnotationAccess().getScenarioTypeAssignment_4()); }
	(rule__ScenarioContractAnnotation__ScenarioTypeAssignment_4)
	{ after(grammarAccess.getScenarioContractAnnotationAccess().getScenarioTypeAssignment_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScenarioContractAnnotation__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScenarioContractAnnotation__Group__5__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ScenarioContractAnnotation__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScenarioContractAnnotationAccess().getRightParenthesisKeyword_5()); }
	')'
	{ after(grammarAccess.getScenarioContractAnnotationAccess().getRightParenthesisKeyword_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AdaptiveContractAnnotation__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AdaptiveContractAnnotation__Group__0__Impl
	rule__AdaptiveContractAnnotation__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AdaptiveContractAnnotation__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAdaptiveContractAnnotationAccess().getAdaptiveContractStatechartKeyword_0()); }
	'@AdaptiveContractStatechart'
	{ after(grammarAccess.getAdaptiveContractAnnotationAccess().getAdaptiveContractStatechartKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AdaptiveContractAnnotation__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AdaptiveContractAnnotation__Group__1__Impl
	rule__AdaptiveContractAnnotation__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__AdaptiveContractAnnotation__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAdaptiveContractAnnotationAccess().getEqualsSignKeyword_1()); }
	'='
	{ after(grammarAccess.getAdaptiveContractAnnotationAccess().getEqualsSignKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AdaptiveContractAnnotation__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AdaptiveContractAnnotation__Group__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AdaptiveContractAnnotation__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAdaptiveContractAnnotationAccess().getMonitoredComponentAssignment_2()); }
	(rule__AdaptiveContractAnnotation__MonitoredComponentAssignment_2)
	{ after(grammarAccess.getAdaptiveContractAnnotationAccess().getMonitoredComponentAssignment_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__MissionPhaseAnnotation__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MissionPhaseAnnotation__Group__0__Impl
	rule__MissionPhaseAnnotation__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__MissionPhaseAnnotation__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMissionPhaseAnnotationAccess().getMissionPhaseAnnotationAction_0()); }
	()
	{ after(grammarAccess.getMissionPhaseAnnotationAccess().getMissionPhaseAnnotationAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__MissionPhaseAnnotation__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MissionPhaseAnnotation__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__MissionPhaseAnnotation__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMissionPhaseAnnotationAccess().getPhaseStatechartKeyword_1()); }
	'@PhaseStatechart'
	{ after(grammarAccess.getMissionPhaseAnnotationAccess().getPhaseStatechartKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__WrapperComponentAnnotation__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__WrapperComponentAnnotation__Group__0__Impl
	rule__WrapperComponentAnnotation__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__WrapperComponentAnnotation__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getWrapperComponentAnnotationAccess().getWrapperComponentAnnotationAction_0()); }
	()
	{ after(grammarAccess.getWrapperComponentAnnotationAccess().getWrapperComponentAnnotationAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__WrapperComponentAnnotation__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__WrapperComponentAnnotation__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__WrapperComponentAnnotation__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getWrapperComponentAnnotationAccess().getWrapperComponentKeyword_1()); }
	'@WrapperComponent'
	{ after(grammarAccess.getWrapperComponentAnnotationAccess().getWrapperComponentKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__RunUponExternalEventAnnotation__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RunUponExternalEventAnnotation__Group__0__Impl
	rule__RunUponExternalEventAnnotation__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__RunUponExternalEventAnnotation__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRunUponExternalEventAnnotationAccess().getRunUponExternalEventAnnotationAction_0()); }
	()
	{ after(grammarAccess.getRunUponExternalEventAnnotationAccess().getRunUponExternalEventAnnotationAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RunUponExternalEventAnnotation__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RunUponExternalEventAnnotation__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__RunUponExternalEventAnnotation__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRunUponExternalEventAnnotationAccess().getAlternatives_1()); }
	(rule__RunUponExternalEventAnnotation__Alternatives_1)
	{ after(grammarAccess.getRunUponExternalEventAnnotationAccess().getAlternatives_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__RunUponExternalEventOrInternalTimeoutAnnotation__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RunUponExternalEventOrInternalTimeoutAnnotation__Group__0__Impl
	rule__RunUponExternalEventOrInternalTimeoutAnnotation__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__RunUponExternalEventOrInternalTimeoutAnnotation__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRunUponExternalEventOrInternalTimeoutAnnotationAccess().getRunUponExternalEventOrInternalTimeoutAnnotationAction_0()); }
	()
	{ after(grammarAccess.getRunUponExternalEventOrInternalTimeoutAnnotationAccess().getRunUponExternalEventOrInternalTimeoutAnnotationAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RunUponExternalEventOrInternalTimeoutAnnotation__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RunUponExternalEventOrInternalTimeoutAnnotation__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__RunUponExternalEventOrInternalTimeoutAnnotation__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRunUponExternalEventOrInternalTimeoutAnnotationAccess().getAlternatives_1()); }
	(rule__RunUponExternalEventOrInternalTimeoutAnnotation__Alternatives_1)
	{ after(grammarAccess.getRunUponExternalEventOrInternalTimeoutAnnotationAccess().getAlternatives_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__Region__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Region__Group__0__Impl
	rule__Region__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__Region__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRegionAccess().getRegionKeyword_0()); }
	'region'
	{ after(grammarAccess.getRegionAccess().getRegionKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Region__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Region__Group__1__Impl
	rule__Region__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__Region__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRegionAccess().getNameAssignment_1()); }
	(rule__Region__NameAssignment_1)
	{ after(grammarAccess.getRegionAccess().getNameAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Region__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Region__Group__2__Impl
	rule__Region__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__Region__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRegionAccess().getLeftCurlyBracketKeyword_2()); }
	'{'
	{ after(grammarAccess.getRegionAccess().getLeftCurlyBracketKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Region__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Region__Group__3__Impl
	rule__Region__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__Region__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRegionAccess().getStateNodesAssignment_3()); }
	(rule__Region__StateNodesAssignment_3)*
	{ after(grammarAccess.getRegionAccess().getStateNodesAssignment_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Region__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Region__Group__4__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__Region__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRegionAccess().getRightCurlyBracketKeyword_4()); }
	'}'
	{ after(grammarAccess.getRegionAccess().getRightCurlyBracketKeyword_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__Transition__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Transition__Group__0__Impl
	rule__Transition__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getTransitionAccess().getAnnotationsAssignment_0()); }
	(rule__Transition__AnnotationsAssignment_0)*
	{ after(grammarAccess.getTransitionAccess().getAnnotationsAssignment_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Transition__Group__1__Impl
	rule__Transition__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getTransitionAccess().getTransitionKeyword_1()); }
	'transition'
	{ after(grammarAccess.getTransitionAccess().getTransitionKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Transition__Group__2__Impl
	rule__Transition__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getTransitionAccess().getGroup_2()); }
	(rule__Transition__Group_2__0)?
	{ after(grammarAccess.getTransitionAccess().getGroup_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Transition__Group__3__Impl
	rule__Transition__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getTransitionAccess().getFromKeyword_3()); }
	'from'
	{ after(grammarAccess.getTransitionAccess().getFromKeyword_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Transition__Group__4__Impl
	rule__Transition__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getTransitionAccess().getSourceStateAssignment_4()); }
	(rule__Transition__SourceStateAssignment_4)
	{ after(grammarAccess.getTransitionAccess().getSourceStateAssignment_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Transition__Group__5__Impl
	rule__Transition__Group__6
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getTransitionAccess().getToKeyword_5()); }
	'to'
	{ after(grammarAccess.getTransitionAccess().getToKeyword_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__Group__6
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Transition__Group__6__Impl
	rule__Transition__Group__7
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__Group__6__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getTransitionAccess().getTargetStateAssignment_6()); }
	(rule__Transition__TargetStateAssignment_6)
	{ after(grammarAccess.getTransitionAccess().getTargetStateAssignment_6()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__Group__7
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Transition__Group__7__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__Group__7__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getTransitionAccess().getGroup_7()); }
	(rule__Transition__Group_7__0)
	{ after(grammarAccess.getTransitionAccess().getGroup_7()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__Transition__Group_2__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Transition__Group_2__0__Impl
	rule__Transition__Group_2__1
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__Group_2__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getTransitionAccess().getLeftParenthesisKeyword_2_0()); }
	'('
	{ after(grammarAccess.getTransitionAccess().getLeftParenthesisKeyword_2_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__Group_2__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Transition__Group_2__1__Impl
	rule__Transition__Group_2__2
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__Group_2__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getTransitionAccess().getPriorityAssignment_2_1()); }
	(rule__Transition__PriorityAssignment_2_1)
	{ after(grammarAccess.getTransitionAccess().getPriorityAssignment_2_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__Group_2__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Transition__Group_2__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__Group_2__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getTransitionAccess().getRightParenthesisKeyword_2_2()); }
	')'
	{ after(grammarAccess.getTransitionAccess().getRightParenthesisKeyword_2_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__Transition__Group_7__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Transition__Group_7__0__Impl
	rule__Transition__Group_7__1
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__Group_7__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getTransitionAccess().getGroup_7_0()); }
	(rule__Transition__Group_7_0__0)?
	{ after(grammarAccess.getTransitionAccess().getGroup_7_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__Group_7__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Transition__Group_7__1__Impl
	rule__Transition__Group_7__2
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__Group_7__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getTransitionAccess().getGroup_7_1()); }
	(rule__Transition__Group_7_1__0)?
	{ after(grammarAccess.getTransitionAccess().getGroup_7_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__Group_7__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Transition__Group_7__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__Group_7__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getTransitionAccess().getGroup_7_2()); }
	(rule__Transition__Group_7_2__0)?
	{ after(grammarAccess.getTransitionAccess().getGroup_7_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__Transition__Group_7_0__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Transition__Group_7_0__0__Impl
	rule__Transition__Group_7_0__1
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__Group_7_0__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getTransitionAccess().getWhenKeyword_7_0_0()); }
	'when'
	{ after(grammarAccess.getTransitionAccess().getWhenKeyword_7_0_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__Group_7_0__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Transition__Group_7_0__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__Group_7_0__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getTransitionAccess().getTriggerAssignment_7_0_1()); }
	(rule__Transition__TriggerAssignment_7_0_1)
	{ after(grammarAccess.getTransitionAccess().getTriggerAssignment_7_0_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__Transition__Group_7_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Transition__Group_7_1__0__Impl
	rule__Transition__Group_7_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__Group_7_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getTransitionAccess().getLeftSquareBracketKeyword_7_1_0()); }
	'['
	{ after(grammarAccess.getTransitionAccess().getLeftSquareBracketKeyword_7_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__Group_7_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Transition__Group_7_1__1__Impl
	rule__Transition__Group_7_1__2
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__Group_7_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getTransitionAccess().getGuardAssignment_7_1_1()); }
	(rule__Transition__GuardAssignment_7_1_1)
	{ after(grammarAccess.getTransitionAccess().getGuardAssignment_7_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__Group_7_1__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Transition__Group_7_1__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__Group_7_1__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getTransitionAccess().getRightSquareBracketKeyword_7_1_2()); }
	']'
	{ after(grammarAccess.getTransitionAccess().getRightSquareBracketKeyword_7_1_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__Transition__Group_7_2__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Transition__Group_7_2__0__Impl
	rule__Transition__Group_7_2__1
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__Group_7_2__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getTransitionAccess().getSolidusKeyword_7_2_0()); }
	'/'
	{ after(grammarAccess.getTransitionAccess().getSolidusKeyword_7_2_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__Group_7_2__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Transition__Group_7_2__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__Group_7_2__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getTransitionAccess().getEffectsAssignment_7_2_1()); }
	(rule__Transition__EffectsAssignment_7_2_1)*
	{ after(grammarAccess.getTransitionAccess().getEffectsAssignment_7_2_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__TransitionIdAnnotation__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__TransitionIdAnnotation__Group__0__Impl
	rule__TransitionIdAnnotation__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__TransitionIdAnnotation__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getTransitionIdAnnotationAccess().getCommercialAtKeyword_0()); }
	'@'
	{ after(grammarAccess.getTransitionIdAnnotationAccess().getCommercialAtKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__TransitionIdAnnotation__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__TransitionIdAnnotation__Group__1__Impl
	rule__TransitionIdAnnotation__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__TransitionIdAnnotation__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getTransitionIdAnnotationAccess().getLeftParenthesisKeyword_1()); }
	'('
	{ after(grammarAccess.getTransitionIdAnnotationAccess().getLeftParenthesisKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__TransitionIdAnnotation__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__TransitionIdAnnotation__Group__2__Impl
	rule__TransitionIdAnnotation__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__TransitionIdAnnotation__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getTransitionIdAnnotationAccess().getNameAssignment_2()); }
	(rule__TransitionIdAnnotation__NameAssignment_2)
	{ after(grammarAccess.getTransitionIdAnnotationAccess().getNameAssignment_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__TransitionIdAnnotation__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__TransitionIdAnnotation__Group__3__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__TransitionIdAnnotation__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getTransitionIdAnnotationAccess().getRightParenthesisKeyword_3()); }
	')'
	{ after(grammarAccess.getTransitionIdAnnotationAccess().getRightParenthesisKeyword_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__State__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__State__Group__0__Impl
	rule__State__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__State__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStateAccess().getAnnotationsAssignment_0()); }
	(rule__State__AnnotationsAssignment_0)*
	{ after(grammarAccess.getStateAccess().getAnnotationsAssignment_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__State__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__State__Group__1__Impl
	rule__State__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__State__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStateAccess().getStateKeyword_1()); }
	'state'
	{ after(grammarAccess.getStateAccess().getStateKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__State__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__State__Group__2__Impl
	rule__State__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__State__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStateAccess().getNameAssignment_2()); }
	(rule__State__NameAssignment_2)
	{ after(grammarAccess.getStateAccess().getNameAssignment_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__State__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__State__Group__3__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__State__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStateAccess().getGroup_3()); }
	(rule__State__Group_3__0)?
	{ after(grammarAccess.getStateAccess().getGroup_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__State__Group_3__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__State__Group_3__0__Impl
	rule__State__Group_3__1
;
finally {
	restoreStackSize(stackSize);
}

rule__State__Group_3__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStateAccess().getLeftCurlyBracketKeyword_3_0()); }
	'{'
	{ after(grammarAccess.getStateAccess().getLeftCurlyBracketKeyword_3_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__State__Group_3__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__State__Group_3__1__Impl
	rule__State__Group_3__2
;
finally {
	restoreStackSize(stackSize);
}

rule__State__Group_3__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStateAccess().getGroup_3_1()); }
	(rule__State__Group_3_1__0)
	{ after(grammarAccess.getStateAccess().getGroup_3_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__State__Group_3__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__State__Group_3__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__State__Group_3__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStateAccess().getRightCurlyBracketKeyword_3_2()); }
	'}'
	{ after(grammarAccess.getStateAccess().getRightCurlyBracketKeyword_3_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__State__Group_3_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__State__Group_3_1__0__Impl
	rule__State__Group_3_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__State__Group_3_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStateAccess().getGroup_3_1_0()); }
	(rule__State__Group_3_1_0__0)
	{ after(grammarAccess.getStateAccess().getGroup_3_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__State__Group_3_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__State__Group_3_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__State__Group_3_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStateAccess().getRegionsAssignment_3_1_1()); }
	(rule__State__RegionsAssignment_3_1_1)*
	{ after(grammarAccess.getStateAccess().getRegionsAssignment_3_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__State__Group_3_1_0__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__State__Group_3_1_0__0__Impl
	rule__State__Group_3_1_0__1
;
finally {
	restoreStackSize(stackSize);
}

rule__State__Group_3_1_0__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStateAccess().getGroup_3_1_0_0()); }
	(rule__State__Group_3_1_0_0__0)*
	{ after(grammarAccess.getStateAccess().getGroup_3_1_0_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__State__Group_3_1_0__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__State__Group_3_1_0__1__Impl
	rule__State__Group_3_1_0__2
;
finally {
	restoreStackSize(stackSize);
}

rule__State__Group_3_1_0__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStateAccess().getGroup_3_1_0_1()); }
	(rule__State__Group_3_1_0_1__0)?
	{ after(grammarAccess.getStateAccess().getGroup_3_1_0_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__State__Group_3_1_0__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__State__Group_3_1_0__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__State__Group_3_1_0__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStateAccess().getGroup_3_1_0_2()); }
	(rule__State__Group_3_1_0_2__0)?
	{ after(grammarAccess.getStateAccess().getGroup_3_1_0_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__State__Group_3_1_0_0__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__State__Group_3_1_0_0__0__Impl
	rule__State__Group_3_1_0_0__1
;
finally {
	restoreStackSize(stackSize);
}

rule__State__Group_3_1_0_0__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStateAccess().getInvariantKeyword_3_1_0_0_0()); }
	'invariant'
	{ after(grammarAccess.getStateAccess().getInvariantKeyword_3_1_0_0_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__State__Group_3_1_0_0__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__State__Group_3_1_0_0__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__State__Group_3_1_0_0__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStateAccess().getInvariantsAssignment_3_1_0_0_1()); }
	(rule__State__InvariantsAssignment_3_1_0_0_1)
	{ after(grammarAccess.getStateAccess().getInvariantsAssignment_3_1_0_0_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__State__Group_3_1_0_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__State__Group_3_1_0_1__0__Impl
	rule__State__Group_3_1_0_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__State__Group_3_1_0_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStateAccess().getEntryKeyword_3_1_0_1_0()); }
	'entry'
	{ after(grammarAccess.getStateAccess().getEntryKeyword_3_1_0_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__State__Group_3_1_0_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__State__Group_3_1_0_1__1__Impl
	rule__State__Group_3_1_0_1__2
;
finally {
	restoreStackSize(stackSize);
}

rule__State__Group_3_1_0_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStateAccess().getSolidusKeyword_3_1_0_1_1()); }
	'/'
	{ after(grammarAccess.getStateAccess().getSolidusKeyword_3_1_0_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__State__Group_3_1_0_1__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__State__Group_3_1_0_1__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__State__Group_3_1_0_1__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStateAccess().getEntryActionsAssignment_3_1_0_1_2()); }
	(rule__State__EntryActionsAssignment_3_1_0_1_2)*
	{ after(grammarAccess.getStateAccess().getEntryActionsAssignment_3_1_0_1_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__State__Group_3_1_0_2__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__State__Group_3_1_0_2__0__Impl
	rule__State__Group_3_1_0_2__1
;
finally {
	restoreStackSize(stackSize);
}

rule__State__Group_3_1_0_2__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStateAccess().getExitKeyword_3_1_0_2_0()); }
	'exit'
	{ after(grammarAccess.getStateAccess().getExitKeyword_3_1_0_2_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__State__Group_3_1_0_2__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__State__Group_3_1_0_2__1__Impl
	rule__State__Group_3_1_0_2__2
;
finally {
	restoreStackSize(stackSize);
}

rule__State__Group_3_1_0_2__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStateAccess().getSolidusKeyword_3_1_0_2_1()); }
	'/'
	{ after(grammarAccess.getStateAccess().getSolidusKeyword_3_1_0_2_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__State__Group_3_1_0_2__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__State__Group_3_1_0_2__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__State__Group_3_1_0_2__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStateAccess().getExitActionsAssignment_3_1_0_2_2()); }
	(rule__State__ExitActionsAssignment_3_1_0_2_2)*
	{ after(grammarAccess.getStateAccess().getExitActionsAssignment_3_1_0_2_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__SpecialStateAnnotation__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SpecialStateAnnotation__Group__0__Impl
	rule__SpecialStateAnnotation__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__SpecialStateAnnotation__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSpecialStateAnnotationAccess().getSpecialStateAnnotationAction_0()); }
	()
	{ after(grammarAccess.getSpecialStateAnnotationAccess().getSpecialStateAnnotationAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SpecialStateAnnotation__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SpecialStateAnnotation__Group__1__Impl
	rule__SpecialStateAnnotation__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__SpecialStateAnnotation__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSpecialStateAnnotationAccess().getAlternatives_1()); }
	(rule__SpecialStateAnnotation__Alternatives_1)
	{ after(grammarAccess.getSpecialStateAnnotationAccess().getAlternatives_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SpecialStateAnnotation__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SpecialStateAnnotation__Group__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__SpecialStateAnnotation__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSpecialStateAnnotationAccess().getKindAssignment_2()); }
	(rule__SpecialStateAnnotation__KindAssignment_2)
	{ after(grammarAccess.getSpecialStateAnnotationAccess().getKindAssignment_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__StateContractAnnotation__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StateContractAnnotation__Group__0__Impl
	rule__StateContractAnnotation__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__StateContractAnnotation__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStateContractAnnotationAccess().getStatechartContractKeyword_0()); }
	'@StatechartContract'
	{ after(grammarAccess.getStateContractAnnotationAccess().getStatechartContractKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StateContractAnnotation__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StateContractAnnotation__Group__1__Impl
	rule__StateContractAnnotation__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__StateContractAnnotation__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStateContractAnnotationAccess().getLinkTypeAssignment_1()); }
	(rule__StateContractAnnotation__LinkTypeAssignment_1)?
	{ after(grammarAccess.getStateContractAnnotationAccess().getLinkTypeAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StateContractAnnotation__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StateContractAnnotation__Group__2__Impl
	rule__StateContractAnnotation__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__StateContractAnnotation__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStateContractAnnotationAccess().getHasHistoryAssignment_2()); }
	(rule__StateContractAnnotation__HasHistoryAssignment_2)?
	{ after(grammarAccess.getStateContractAnnotationAccess().getHasHistoryAssignment_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StateContractAnnotation__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StateContractAnnotation__Group__3__Impl
	rule__StateContractAnnotation__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__StateContractAnnotation__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStateContractAnnotationAccess().getEqualsSignKeyword_3()); }
	'='
	{ after(grammarAccess.getStateContractAnnotationAccess().getEqualsSignKeyword_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StateContractAnnotation__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StateContractAnnotation__Group__4__Impl
	rule__StateContractAnnotation__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__StateContractAnnotation__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStateContractAnnotationAccess().getContractStatechartAssignment_4()); }
	(rule__StateContractAnnotation__ContractStatechartAssignment_4)
	{ after(grammarAccess.getStateContractAnnotationAccess().getContractStatechartAssignment_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StateContractAnnotation__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StateContractAnnotation__Group__5__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__StateContractAnnotation__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStateContractAnnotationAccess().getGroup_5()); }
	(rule__StateContractAnnotation__Group_5__0)?
	{ after(grammarAccess.getStateContractAnnotationAccess().getGroup_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__StateContractAnnotation__Group_5__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StateContractAnnotation__Group_5__0__Impl
	rule__StateContractAnnotation__Group_5__1
;
finally {
	restoreStackSize(stackSize);
}

rule__StateContractAnnotation__Group_5__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStateContractAnnotationAccess().getLeftParenthesisKeyword_5_0()); }
	'('
	{ after(grammarAccess.getStateContractAnnotationAccess().getLeftParenthesisKeyword_5_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StateContractAnnotation__Group_5__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StateContractAnnotation__Group_5__1__Impl
	rule__StateContractAnnotation__Group_5__2
;
finally {
	restoreStackSize(stackSize);
}

rule__StateContractAnnotation__Group_5__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStateContractAnnotationAccess().getArgumentsAssignment_5_1()); }
	(rule__StateContractAnnotation__ArgumentsAssignment_5_1)
	{ after(grammarAccess.getStateContractAnnotationAccess().getArgumentsAssignment_5_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StateContractAnnotation__Group_5__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StateContractAnnotation__Group_5__2__Impl
	rule__StateContractAnnotation__Group_5__3
;
finally {
	restoreStackSize(stackSize);
}

rule__StateContractAnnotation__Group_5__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStateContractAnnotationAccess().getGroup_5_2()); }
	(rule__StateContractAnnotation__Group_5_2__0)*
	{ after(grammarAccess.getStateContractAnnotationAccess().getGroup_5_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StateContractAnnotation__Group_5__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StateContractAnnotation__Group_5__3__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__StateContractAnnotation__Group_5__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStateContractAnnotationAccess().getRightParenthesisKeyword_5_3()); }
	')'
	{ after(grammarAccess.getStateContractAnnotationAccess().getRightParenthesisKeyword_5_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__StateContractAnnotation__Group_5_2__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StateContractAnnotation__Group_5_2__0__Impl
	rule__StateContractAnnotation__Group_5_2__1
;
finally {
	restoreStackSize(stackSize);
}

rule__StateContractAnnotation__Group_5_2__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStateContractAnnotationAccess().getCommaKeyword_5_2_0()); }
	','
	{ after(grammarAccess.getStateContractAnnotationAccess().getCommaKeyword_5_2_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__StateContractAnnotation__Group_5_2__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__StateContractAnnotation__Group_5_2__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__StateContractAnnotation__Group_5_2__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getStateContractAnnotationAccess().getArgumentsAssignment_5_2_1()); }
	(rule__StateContractAnnotation__ArgumentsAssignment_5_2_1)
	{ after(grammarAccess.getStateContractAnnotationAccess().getArgumentsAssignment_5_2_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__MissionPhaseStateAnnotation__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MissionPhaseStateAnnotation__Group__0__Impl
	rule__MissionPhaseStateAnnotation__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__MissionPhaseStateAnnotation__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMissionPhaseStateAnnotationAccess().getMissionPhaseStateAnnotationAction_0()); }
	()
	{ after(grammarAccess.getMissionPhaseStateAnnotationAccess().getMissionPhaseStateAnnotationAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__MissionPhaseStateAnnotation__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MissionPhaseStateAnnotation__Group__1__Impl
	rule__MissionPhaseStateAnnotation__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__MissionPhaseStateAnnotation__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMissionPhaseStateAnnotationAccess().getPhaseKeyword_1()); }
	'@Phase'
	{ after(grammarAccess.getMissionPhaseStateAnnotationAccess().getPhaseKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__MissionPhaseStateAnnotation__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MissionPhaseStateAnnotation__Group__2__Impl
	rule__MissionPhaseStateAnnotation__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__MissionPhaseStateAnnotation__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMissionPhaseStateAnnotationAccess().getEqualsSignKeyword_2()); }
	'='
	{ after(grammarAccess.getMissionPhaseStateAnnotationAccess().getEqualsSignKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__MissionPhaseStateAnnotation__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MissionPhaseStateAnnotation__Group__3__Impl
	rule__MissionPhaseStateAnnotation__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__MissionPhaseStateAnnotation__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMissionPhaseStateAnnotationAccess().getLeftSquareBracketKeyword_3()); }
	'['
	{ after(grammarAccess.getMissionPhaseStateAnnotationAccess().getLeftSquareBracketKeyword_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__MissionPhaseStateAnnotation__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MissionPhaseStateAnnotation__Group__4__Impl
	rule__MissionPhaseStateAnnotation__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__MissionPhaseStateAnnotation__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMissionPhaseStateAnnotationAccess().getGroup_4()); }
	(rule__MissionPhaseStateAnnotation__Group_4__0)
	{ after(grammarAccess.getMissionPhaseStateAnnotationAccess().getGroup_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__MissionPhaseStateAnnotation__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MissionPhaseStateAnnotation__Group__5__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__MissionPhaseStateAnnotation__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMissionPhaseStateAnnotationAccess().getRightSquareBracketKeyword_5()); }
	']'
	{ after(grammarAccess.getMissionPhaseStateAnnotationAccess().getRightSquareBracketKeyword_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__MissionPhaseStateAnnotation__Group_4__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MissionPhaseStateAnnotation__Group_4__0__Impl
	rule__MissionPhaseStateAnnotation__Group_4__1
;
finally {
	restoreStackSize(stackSize);
}

rule__MissionPhaseStateAnnotation__Group_4__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMissionPhaseStateAnnotationAccess().getHistoryAssignment_4_0()); }
	(rule__MissionPhaseStateAnnotation__HistoryAssignment_4_0)
	{ after(grammarAccess.getMissionPhaseStateAnnotationAccess().getHistoryAssignment_4_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__MissionPhaseStateAnnotation__Group_4__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MissionPhaseStateAnnotation__Group_4__1__Impl
	rule__MissionPhaseStateAnnotation__Group_4__2
;
finally {
	restoreStackSize(stackSize);
}

rule__MissionPhaseStateAnnotation__Group_4__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMissionPhaseStateAnnotationAccess().getComponentAssignment_4_1()); }
	(rule__MissionPhaseStateAnnotation__ComponentAssignment_4_1)
	{ after(grammarAccess.getMissionPhaseStateAnnotationAccess().getComponentAssignment_4_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__MissionPhaseStateAnnotation__Group_4__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MissionPhaseStateAnnotation__Group_4__2__Impl
	rule__MissionPhaseStateAnnotation__Group_4__3
;
finally {
	restoreStackSize(stackSize);
}

rule__MissionPhaseStateAnnotation__Group_4__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMissionPhaseStateAnnotationAccess().getLeftCurlyBracketKeyword_4_2()); }
	'{'
	{ after(grammarAccess.getMissionPhaseStateAnnotationAccess().getLeftCurlyBracketKeyword_4_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__MissionPhaseStateAnnotation__Group_4__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MissionPhaseStateAnnotation__Group_4__3__Impl
	rule__MissionPhaseStateAnnotation__Group_4__4
;
finally {
	restoreStackSize(stackSize);
}

rule__MissionPhaseStateAnnotation__Group_4__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMissionPhaseStateAnnotationAccess().getPortBindingsAssignment_4_3()); }
	(rule__MissionPhaseStateAnnotation__PortBindingsAssignment_4_3)*
	{ after(grammarAccess.getMissionPhaseStateAnnotationAccess().getPortBindingsAssignment_4_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__MissionPhaseStateAnnotation__Group_4__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MissionPhaseStateAnnotation__Group_4__4__Impl
	rule__MissionPhaseStateAnnotation__Group_4__5
;
finally {
	restoreStackSize(stackSize);
}

rule__MissionPhaseStateAnnotation__Group_4__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMissionPhaseStateAnnotationAccess().getVariableBindingsAssignment_4_4()); }
	(rule__MissionPhaseStateAnnotation__VariableBindingsAssignment_4_4)*
	{ after(grammarAccess.getMissionPhaseStateAnnotationAccess().getVariableBindingsAssignment_4_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__MissionPhaseStateAnnotation__Group_4__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MissionPhaseStateAnnotation__Group_4__5__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__MissionPhaseStateAnnotation__Group_4__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMissionPhaseStateAnnotationAccess().getRightCurlyBracketKeyword_4_5()); }
	'}'
	{ after(grammarAccess.getMissionPhaseStateAnnotationAccess().getRightCurlyBracketKeyword_4_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__VariableBinding__Group_0__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__VariableBinding__Group_0__0__Impl
	rule__VariableBinding__Group_0__1
;
finally {
	restoreStackSize(stackSize);
}

rule__VariableBinding__Group_0__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getVariableBindingAccess().getTieKeyword_0_0()); }
	'tie'
	{ after(grammarAccess.getVariableBindingAccess().getTieKeyword_0_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__VariableBinding__Group_0__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__VariableBinding__Group_0__1__Impl
	rule__VariableBinding__Group_0__2
;
finally {
	restoreStackSize(stackSize);
}

rule__VariableBinding__Group_0__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getVariableBindingAccess().getStatechartVariableAssignment_0_1()); }
	(rule__VariableBinding__StatechartVariableAssignment_0_1)
	{ after(grammarAccess.getVariableBindingAccess().getStatechartVariableAssignment_0_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__VariableBinding__Group_0__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__VariableBinding__Group_0__2__Impl
	rule__VariableBinding__Group_0__3
;
finally {
	restoreStackSize(stackSize);
}

rule__VariableBinding__Group_0__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getVariableBindingAccess().getHyphenMinusGreaterThanSignKeyword_0_2()); }
	'->'
	{ after(grammarAccess.getVariableBindingAccess().getHyphenMinusGreaterThanSignKeyword_0_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__VariableBinding__Group_0__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__VariableBinding__Group_0__3__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__VariableBinding__Group_0__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getVariableBindingAccess().getInstanceVariableReferenceAssignment_0_3()); }
	(rule__VariableBinding__InstanceVariableReferenceAssignment_0_3)
	{ after(grammarAccess.getVariableBindingAccess().getInstanceVariableReferenceAssignment_0_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__VariableBinding__Group_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__VariableBinding__Group_1__0__Impl
	rule__VariableBinding__Group_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__VariableBinding__Group_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getVariableBindingAccess().getBindKeyword_1_0()); }
	'bind'
	{ after(grammarAccess.getVariableBindingAccess().getBindKeyword_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__VariableBinding__Group_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__VariableBinding__Group_1__1__Impl
	rule__VariableBinding__Group_1__2
;
finally {
	restoreStackSize(stackSize);
}

rule__VariableBinding__Group_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getVariableBindingAccess().getStatechartVariableAssignment_1_1()); }
	(rule__VariableBinding__StatechartVariableAssignment_1_1)
	{ after(grammarAccess.getVariableBindingAccess().getStatechartVariableAssignment_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__VariableBinding__Group_1__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__VariableBinding__Group_1__2__Impl
	rule__VariableBinding__Group_1__3
;
finally {
	restoreStackSize(stackSize);
}

rule__VariableBinding__Group_1__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getVariableBindingAccess().getEqualsSignKeyword_1_2()); }
	'='
	{ after(grammarAccess.getVariableBindingAccess().getEqualsSignKeyword_1_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__VariableBinding__Group_1__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__VariableBinding__Group_1__3__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__VariableBinding__Group_1__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getVariableBindingAccess().getInstanceVariableReferenceAssignment_1_3()); }
	(rule__VariableBinding__InstanceVariableReferenceAssignment_1_3)
	{ after(grammarAccess.getVariableBindingAccess().getInstanceVariableReferenceAssignment_1_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__InstanceVariableReference__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__InstanceVariableReference__Group__0__Impl
	rule__InstanceVariableReference__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__InstanceVariableReference__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInstanceVariableReferenceAccess().getInstanceAssignment_0()); }
	(rule__InstanceVariableReference__InstanceAssignment_0)
	{ after(grammarAccess.getInstanceVariableReferenceAccess().getInstanceAssignment_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__InstanceVariableReference__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__InstanceVariableReference__Group__1__Impl
	rule__InstanceVariableReference__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__InstanceVariableReference__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInstanceVariableReferenceAccess().getFullStopKeyword_1()); }
	'.'
	{ after(grammarAccess.getInstanceVariableReferenceAccess().getFullStopKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__InstanceVariableReference__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__InstanceVariableReference__Group__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__InstanceVariableReference__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInstanceVariableReferenceAccess().getVariableAssignment_2()); }
	(rule__InstanceVariableReference__VariableAssignment_2)
	{ after(grammarAccess.getInstanceVariableReferenceAccess().getVariableAssignment_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__MergeState__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MergeState__Group__0__Impl
	rule__MergeState__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__MergeState__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMergeStateAccess().getMergeKeyword_0()); }
	'merge'
	{ after(grammarAccess.getMergeStateAccess().getMergeKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__MergeState__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MergeState__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__MergeState__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMergeStateAccess().getNameAssignment_1()); }
	(rule__MergeState__NameAssignment_1)
	{ after(grammarAccess.getMergeStateAccess().getNameAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ChoiceState__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ChoiceState__Group__0__Impl
	rule__ChoiceState__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ChoiceState__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getChoiceStateAccess().getChoiceKeyword_0()); }
	'choice'
	{ after(grammarAccess.getChoiceStateAccess().getChoiceKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ChoiceState__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ChoiceState__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ChoiceState__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getChoiceStateAccess().getNameAssignment_1()); }
	(rule__ChoiceState__NameAssignment_1)
	{ after(grammarAccess.getChoiceStateAccess().getNameAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__JoinState__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__JoinState__Group__0__Impl
	rule__JoinState__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__JoinState__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getJoinStateAccess().getJoinKeyword_0()); }
	'join'
	{ after(grammarAccess.getJoinStateAccess().getJoinKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__JoinState__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__JoinState__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__JoinState__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getJoinStateAccess().getNameAssignment_1()); }
	(rule__JoinState__NameAssignment_1)
	{ after(grammarAccess.getJoinStateAccess().getNameAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ForkState__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ForkState__Group__0__Impl
	rule__ForkState__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ForkState__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getForkStateAccess().getForkKeyword_0()); }
	'fork'
	{ after(grammarAccess.getForkStateAccess().getForkKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ForkState__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ForkState__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ForkState__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getForkStateAccess().getNameAssignment_1()); }
	(rule__ForkState__NameAssignment_1)
	{ after(grammarAccess.getForkStateAccess().getNameAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__InitialState__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__InitialState__Group__0__Impl
	rule__InitialState__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__InitialState__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInitialStateAccess().getInitialKeyword_0()); }
	'initial'
	{ after(grammarAccess.getInitialStateAccess().getInitialKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__InitialState__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__InitialState__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__InitialState__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInitialStateAccess().getNameAssignment_1()); }
	(rule__InitialState__NameAssignment_1)
	{ after(grammarAccess.getInitialStateAccess().getNameAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ShallowHistoryState__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ShallowHistoryState__Group__0__Impl
	rule__ShallowHistoryState__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ShallowHistoryState__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getShallowHistoryStateAccess().getShallowKeyword_0()); }
	'shallow'
	{ after(grammarAccess.getShallowHistoryStateAccess().getShallowKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ShallowHistoryState__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ShallowHistoryState__Group__1__Impl
	rule__ShallowHistoryState__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ShallowHistoryState__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getShallowHistoryStateAccess().getHistoryKeyword_1()); }
	'history'
	{ after(grammarAccess.getShallowHistoryStateAccess().getHistoryKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ShallowHistoryState__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ShallowHistoryState__Group__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ShallowHistoryState__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getShallowHistoryStateAccess().getNameAssignment_2()); }
	(rule__ShallowHistoryState__NameAssignment_2)
	{ after(grammarAccess.getShallowHistoryStateAccess().getNameAssignment_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__DeepHistoryState__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeepHistoryState__Group__0__Impl
	rule__DeepHistoryState__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__DeepHistoryState__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeepHistoryStateAccess().getDeepKeyword_0()); }
	'deep'
	{ after(grammarAccess.getDeepHistoryStateAccess().getDeepKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeepHistoryState__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeepHistoryState__Group__1__Impl
	rule__DeepHistoryState__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__DeepHistoryState__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeepHistoryStateAccess().getHistoryKeyword_1()); }
	'history'
	{ after(grammarAccess.getDeepHistoryStateAccess().getHistoryKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeepHistoryState__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DeepHistoryState__Group__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__DeepHistoryState__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDeepHistoryStateAccess().getNameAssignment_2()); }
	(rule__DeepHistoryState__NameAssignment_2)
	{ after(grammarAccess.getDeepHistoryStateAccess().getNameAssignment_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__CascadeCompositeComponent__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CascadeCompositeComponent__Group__0__Impl
	rule__CascadeCompositeComponent__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCascadeCompositeComponentAccess().getAnnotationsAssignment_0()); }
	(rule__CascadeCompositeComponent__AnnotationsAssignment_0)*
	{ after(grammarAccess.getCascadeCompositeComponentAccess().getAnnotationsAssignment_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CascadeCompositeComponent__Group__1__Impl
	rule__CascadeCompositeComponent__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCascadeCompositeComponentAccess().getCascadeKeyword_1()); }
	'cascade'
	{ after(grammarAccess.getCascadeCompositeComponentAccess().getCascadeKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CascadeCompositeComponent__Group__2__Impl
	rule__CascadeCompositeComponent__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCascadeCompositeComponentAccess().getNameAssignment_2()); }
	(rule__CascadeCompositeComponent__NameAssignment_2)
	{ after(grammarAccess.getCascadeCompositeComponentAccess().getNameAssignment_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CascadeCompositeComponent__Group__3__Impl
	rule__CascadeCompositeComponent__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCascadeCompositeComponentAccess().getGroup_3()); }
	(rule__CascadeCompositeComponent__Group_3__0)?
	{ after(grammarAccess.getCascadeCompositeComponentAccess().getGroup_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CascadeCompositeComponent__Group__4__Impl
	rule__CascadeCompositeComponent__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCascadeCompositeComponentAccess().getGroup_4()); }
	(rule__CascadeCompositeComponent__Group_4__0)?
	{ after(grammarAccess.getCascadeCompositeComponentAccess().getGroup_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CascadeCompositeComponent__Group__5__Impl
	rule__CascadeCompositeComponent__Group__6
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCascadeCompositeComponentAccess().getLeftCurlyBracketKeyword_5()); }
	'{'
	{ after(grammarAccess.getCascadeCompositeComponentAccess().getLeftCurlyBracketKeyword_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group__6
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CascadeCompositeComponent__Group__6__Impl
	rule__CascadeCompositeComponent__Group__7
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group__6__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCascadeCompositeComponentAccess().getAlternatives_6()); }
	(rule__CascadeCompositeComponent__Alternatives_6)*
	{ after(grammarAccess.getCascadeCompositeComponentAccess().getAlternatives_6()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group__7
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CascadeCompositeComponent__Group__7__Impl
	rule__CascadeCompositeComponent__Group__8
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group__7__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCascadeCompositeComponentAccess().getGroup_7()); }
	(rule__CascadeCompositeComponent__Group_7__0)?
	{ after(grammarAccess.getCascadeCompositeComponentAccess().getGroup_7()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group__8
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CascadeCompositeComponent__Group__8__Impl
	rule__CascadeCompositeComponent__Group__9
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group__8__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCascadeCompositeComponentAccess().getGroup_8()); }
	(rule__CascadeCompositeComponent__Group_8__0)?
	{ after(grammarAccess.getCascadeCompositeComponentAccess().getGroup_8()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group__9
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CascadeCompositeComponent__Group__9__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group__9__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCascadeCompositeComponentAccess().getRightCurlyBracketKeyword_9()); }
	'}'
	{ after(grammarAccess.getCascadeCompositeComponentAccess().getRightCurlyBracketKeyword_9()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__CascadeCompositeComponent__Group_3__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CascadeCompositeComponent__Group_3__0__Impl
	rule__CascadeCompositeComponent__Group_3__1
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group_3__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCascadeCompositeComponentAccess().getLeftParenthesisKeyword_3_0()); }
	'('
	{ after(grammarAccess.getCascadeCompositeComponentAccess().getLeftParenthesisKeyword_3_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group_3__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CascadeCompositeComponent__Group_3__1__Impl
	rule__CascadeCompositeComponent__Group_3__2
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group_3__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCascadeCompositeComponentAccess().getGroup_3_1()); }
	(rule__CascadeCompositeComponent__Group_3_1__0)?
	{ after(grammarAccess.getCascadeCompositeComponentAccess().getGroup_3_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group_3__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CascadeCompositeComponent__Group_3__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group_3__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCascadeCompositeComponentAccess().getRightParenthesisKeyword_3_2()); }
	')'
	{ after(grammarAccess.getCascadeCompositeComponentAccess().getRightParenthesisKeyword_3_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__CascadeCompositeComponent__Group_3_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CascadeCompositeComponent__Group_3_1__0__Impl
	rule__CascadeCompositeComponent__Group_3_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group_3_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCascadeCompositeComponentAccess().getParameterDeclarationsAssignment_3_1_0()); }
	(rule__CascadeCompositeComponent__ParameterDeclarationsAssignment_3_1_0)
	{ after(grammarAccess.getCascadeCompositeComponentAccess().getParameterDeclarationsAssignment_3_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group_3_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CascadeCompositeComponent__Group_3_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group_3_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCascadeCompositeComponentAccess().getGroup_3_1_1()); }
	(rule__CascadeCompositeComponent__Group_3_1_1__0)*
	{ after(grammarAccess.getCascadeCompositeComponentAccess().getGroup_3_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__CascadeCompositeComponent__Group_3_1_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CascadeCompositeComponent__Group_3_1_1__0__Impl
	rule__CascadeCompositeComponent__Group_3_1_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group_3_1_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCascadeCompositeComponentAccess().getCommaKeyword_3_1_1_0()); }
	','
	{ after(grammarAccess.getCascadeCompositeComponentAccess().getCommaKeyword_3_1_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group_3_1_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CascadeCompositeComponent__Group_3_1_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group_3_1_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCascadeCompositeComponentAccess().getParameterDeclarationsAssignment_3_1_1_1()); }
	(rule__CascadeCompositeComponent__ParameterDeclarationsAssignment_3_1_1_1)
	{ after(grammarAccess.getCascadeCompositeComponentAccess().getParameterDeclarationsAssignment_3_1_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__CascadeCompositeComponent__Group_4__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CascadeCompositeComponent__Group_4__0__Impl
	rule__CascadeCompositeComponent__Group_4__1
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group_4__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCascadeCompositeComponentAccess().getLeftSquareBracketKeyword_4_0()); }
	'['
	{ after(grammarAccess.getCascadeCompositeComponentAccess().getLeftSquareBracketKeyword_4_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group_4__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CascadeCompositeComponent__Group_4__1__Impl
	rule__CascadeCompositeComponent__Group_4__2
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group_4__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCascadeCompositeComponentAccess().getGroup_4_1()); }
	(rule__CascadeCompositeComponent__Group_4_1__0)?
	{ after(grammarAccess.getCascadeCompositeComponentAccess().getGroup_4_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group_4__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CascadeCompositeComponent__Group_4__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group_4__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCascadeCompositeComponentAccess().getRightSquareBracketKeyword_4_2()); }
	']'
	{ after(grammarAccess.getCascadeCompositeComponentAccess().getRightSquareBracketKeyword_4_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__CascadeCompositeComponent__Group_4_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CascadeCompositeComponent__Group_4_1__0__Impl
	rule__CascadeCompositeComponent__Group_4_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group_4_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCascadeCompositeComponentAccess().getPortsAssignment_4_1_0()); }
	(rule__CascadeCompositeComponent__PortsAssignment_4_1_0)
	{ after(grammarAccess.getCascadeCompositeComponentAccess().getPortsAssignment_4_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group_4_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CascadeCompositeComponent__Group_4_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group_4_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCascadeCompositeComponentAccess().getGroup_4_1_1()); }
	(rule__CascadeCompositeComponent__Group_4_1_1__0)*
	{ after(grammarAccess.getCascadeCompositeComponentAccess().getGroup_4_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__CascadeCompositeComponent__Group_4_1_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CascadeCompositeComponent__Group_4_1_1__0__Impl
	rule__CascadeCompositeComponent__Group_4_1_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group_4_1_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCascadeCompositeComponentAccess().getCommaKeyword_4_1_1_0()); }
	(',')?
	{ after(grammarAccess.getCascadeCompositeComponentAccess().getCommaKeyword_4_1_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group_4_1_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CascadeCompositeComponent__Group_4_1_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group_4_1_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCascadeCompositeComponentAccess().getPortsAssignment_4_1_1_1()); }
	(rule__CascadeCompositeComponent__PortsAssignment_4_1_1_1)
	{ after(grammarAccess.getCascadeCompositeComponentAccess().getPortsAssignment_4_1_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__CascadeCompositeComponent__Group_7__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CascadeCompositeComponent__Group_7__0__Impl
	rule__CascadeCompositeComponent__Group_7__1
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group_7__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCascadeCompositeComponentAccess().getInitialKeyword_7_0()); }
	'initial'
	{ after(grammarAccess.getCascadeCompositeComponentAccess().getInitialKeyword_7_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group_7__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CascadeCompositeComponent__Group_7__1__Impl
	rule__CascadeCompositeComponent__Group_7__2
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group_7__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCascadeCompositeComponentAccess().getExecuteKeyword_7_1()); }
	'execute'
	{ after(grammarAccess.getCascadeCompositeComponentAccess().getExecuteKeyword_7_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group_7__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CascadeCompositeComponent__Group_7__2__Impl
	rule__CascadeCompositeComponent__Group_7__3
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group_7__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCascadeCompositeComponentAccess().getInitialExecutionListAssignment_7_2()); }
	(rule__CascadeCompositeComponent__InitialExecutionListAssignment_7_2)
	{ after(grammarAccess.getCascadeCompositeComponentAccess().getInitialExecutionListAssignment_7_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group_7__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CascadeCompositeComponent__Group_7__3__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group_7__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCascadeCompositeComponentAccess().getGroup_7_3()); }
	(rule__CascadeCompositeComponent__Group_7_3__0)*
	{ after(grammarAccess.getCascadeCompositeComponentAccess().getGroup_7_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__CascadeCompositeComponent__Group_7_3__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CascadeCompositeComponent__Group_7_3__0__Impl
	rule__CascadeCompositeComponent__Group_7_3__1
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group_7_3__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCascadeCompositeComponentAccess().getCommaKeyword_7_3_0()); }
	','
	{ after(grammarAccess.getCascadeCompositeComponentAccess().getCommaKeyword_7_3_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group_7_3__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CascadeCompositeComponent__Group_7_3__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group_7_3__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCascadeCompositeComponentAccess().getInitialExecutionListAssignment_7_3_1()); }
	(rule__CascadeCompositeComponent__InitialExecutionListAssignment_7_3_1)
	{ after(grammarAccess.getCascadeCompositeComponentAccess().getInitialExecutionListAssignment_7_3_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__CascadeCompositeComponent__Group_8__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CascadeCompositeComponent__Group_8__0__Impl
	rule__CascadeCompositeComponent__Group_8__1
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group_8__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCascadeCompositeComponentAccess().getExecuteKeyword_8_0()); }
	'execute'
	{ after(grammarAccess.getCascadeCompositeComponentAccess().getExecuteKeyword_8_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group_8__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CascadeCompositeComponent__Group_8__1__Impl
	rule__CascadeCompositeComponent__Group_8__2
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group_8__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCascadeCompositeComponentAccess().getExecutionListAssignment_8_1()); }
	(rule__CascadeCompositeComponent__ExecutionListAssignment_8_1)
	{ after(grammarAccess.getCascadeCompositeComponentAccess().getExecutionListAssignment_8_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group_8__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CascadeCompositeComponent__Group_8__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group_8__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCascadeCompositeComponentAccess().getGroup_8_2()); }
	(rule__CascadeCompositeComponent__Group_8_2__0)*
	{ after(grammarAccess.getCascadeCompositeComponentAccess().getGroup_8_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__CascadeCompositeComponent__Group_8_2__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CascadeCompositeComponent__Group_8_2__0__Impl
	rule__CascadeCompositeComponent__Group_8_2__1
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group_8_2__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCascadeCompositeComponentAccess().getCommaKeyword_8_2_0()); }
	','
	{ after(grammarAccess.getCascadeCompositeComponentAccess().getCommaKeyword_8_2_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group_8_2__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__CascadeCompositeComponent__Group_8_2__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__Group_8_2__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getCascadeCompositeComponentAccess().getExecutionListAssignment_8_2_1()); }
	(rule__CascadeCompositeComponent__ExecutionListAssignment_8_2_1)
	{ after(grammarAccess.getCascadeCompositeComponentAccess().getExecutionListAssignment_8_2_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__SynchronousCompositeComponent__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousCompositeComponent__Group__0__Impl
	rule__SynchronousCompositeComponent__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousCompositeComponentAccess().getAnnotationsAssignment_0()); }
	(rule__SynchronousCompositeComponent__AnnotationsAssignment_0)*
	{ after(grammarAccess.getSynchronousCompositeComponentAccess().getAnnotationsAssignment_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousCompositeComponent__Group__1__Impl
	rule__SynchronousCompositeComponent__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousCompositeComponentAccess().getSyncKeyword_1()); }
	'sync'
	{ after(grammarAccess.getSynchronousCompositeComponentAccess().getSyncKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousCompositeComponent__Group__2__Impl
	rule__SynchronousCompositeComponent__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousCompositeComponentAccess().getNameAssignment_2()); }
	(rule__SynchronousCompositeComponent__NameAssignment_2)
	{ after(grammarAccess.getSynchronousCompositeComponentAccess().getNameAssignment_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousCompositeComponent__Group__3__Impl
	rule__SynchronousCompositeComponent__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousCompositeComponentAccess().getGroup_3()); }
	(rule__SynchronousCompositeComponent__Group_3__0)?
	{ after(grammarAccess.getSynchronousCompositeComponentAccess().getGroup_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousCompositeComponent__Group__4__Impl
	rule__SynchronousCompositeComponent__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousCompositeComponentAccess().getGroup_4()); }
	(rule__SynchronousCompositeComponent__Group_4__0)?
	{ after(grammarAccess.getSynchronousCompositeComponentAccess().getGroup_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousCompositeComponent__Group__5__Impl
	rule__SynchronousCompositeComponent__Group__6
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousCompositeComponentAccess().getLeftCurlyBracketKeyword_5()); }
	'{'
	{ after(grammarAccess.getSynchronousCompositeComponentAccess().getLeftCurlyBracketKeyword_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__Group__6
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousCompositeComponent__Group__6__Impl
	rule__SynchronousCompositeComponent__Group__7
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__Group__6__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousCompositeComponentAccess().getAlternatives_6()); }
	(rule__SynchronousCompositeComponent__Alternatives_6)*
	{ after(grammarAccess.getSynchronousCompositeComponentAccess().getAlternatives_6()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__Group__7
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousCompositeComponent__Group__7__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__Group__7__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousCompositeComponentAccess().getRightCurlyBracketKeyword_7()); }
	'}'
	{ after(grammarAccess.getSynchronousCompositeComponentAccess().getRightCurlyBracketKeyword_7()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__SynchronousCompositeComponent__Group_3__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousCompositeComponent__Group_3__0__Impl
	rule__SynchronousCompositeComponent__Group_3__1
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__Group_3__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousCompositeComponentAccess().getLeftParenthesisKeyword_3_0()); }
	'('
	{ after(grammarAccess.getSynchronousCompositeComponentAccess().getLeftParenthesisKeyword_3_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__Group_3__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousCompositeComponent__Group_3__1__Impl
	rule__SynchronousCompositeComponent__Group_3__2
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__Group_3__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousCompositeComponentAccess().getGroup_3_1()); }
	(rule__SynchronousCompositeComponent__Group_3_1__0)?
	{ after(grammarAccess.getSynchronousCompositeComponentAccess().getGroup_3_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__Group_3__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousCompositeComponent__Group_3__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__Group_3__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousCompositeComponentAccess().getRightParenthesisKeyword_3_2()); }
	')'
	{ after(grammarAccess.getSynchronousCompositeComponentAccess().getRightParenthesisKeyword_3_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__SynchronousCompositeComponent__Group_3_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousCompositeComponent__Group_3_1__0__Impl
	rule__SynchronousCompositeComponent__Group_3_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__Group_3_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousCompositeComponentAccess().getParameterDeclarationsAssignment_3_1_0()); }
	(rule__SynchronousCompositeComponent__ParameterDeclarationsAssignment_3_1_0)
	{ after(grammarAccess.getSynchronousCompositeComponentAccess().getParameterDeclarationsAssignment_3_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__Group_3_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousCompositeComponent__Group_3_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__Group_3_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousCompositeComponentAccess().getGroup_3_1_1()); }
	(rule__SynchronousCompositeComponent__Group_3_1_1__0)*
	{ after(grammarAccess.getSynchronousCompositeComponentAccess().getGroup_3_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__SynchronousCompositeComponent__Group_3_1_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousCompositeComponent__Group_3_1_1__0__Impl
	rule__SynchronousCompositeComponent__Group_3_1_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__Group_3_1_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousCompositeComponentAccess().getCommaKeyword_3_1_1_0()); }
	','
	{ after(grammarAccess.getSynchronousCompositeComponentAccess().getCommaKeyword_3_1_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__Group_3_1_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousCompositeComponent__Group_3_1_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__Group_3_1_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousCompositeComponentAccess().getParameterDeclarationsAssignment_3_1_1_1()); }
	(rule__SynchronousCompositeComponent__ParameterDeclarationsAssignment_3_1_1_1)
	{ after(grammarAccess.getSynchronousCompositeComponentAccess().getParameterDeclarationsAssignment_3_1_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__SynchronousCompositeComponent__Group_4__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousCompositeComponent__Group_4__0__Impl
	rule__SynchronousCompositeComponent__Group_4__1
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__Group_4__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousCompositeComponentAccess().getLeftSquareBracketKeyword_4_0()); }
	'['
	{ after(grammarAccess.getSynchronousCompositeComponentAccess().getLeftSquareBracketKeyword_4_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__Group_4__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousCompositeComponent__Group_4__1__Impl
	rule__SynchronousCompositeComponent__Group_4__2
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__Group_4__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousCompositeComponentAccess().getGroup_4_1()); }
	(rule__SynchronousCompositeComponent__Group_4_1__0)?
	{ after(grammarAccess.getSynchronousCompositeComponentAccess().getGroup_4_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__Group_4__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousCompositeComponent__Group_4__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__Group_4__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousCompositeComponentAccess().getRightSquareBracketKeyword_4_2()); }
	']'
	{ after(grammarAccess.getSynchronousCompositeComponentAccess().getRightSquareBracketKeyword_4_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__SynchronousCompositeComponent__Group_4_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousCompositeComponent__Group_4_1__0__Impl
	rule__SynchronousCompositeComponent__Group_4_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__Group_4_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousCompositeComponentAccess().getPortsAssignment_4_1_0()); }
	(rule__SynchronousCompositeComponent__PortsAssignment_4_1_0)
	{ after(grammarAccess.getSynchronousCompositeComponentAccess().getPortsAssignment_4_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__Group_4_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousCompositeComponent__Group_4_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__Group_4_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousCompositeComponentAccess().getGroup_4_1_1()); }
	(rule__SynchronousCompositeComponent__Group_4_1_1__0)*
	{ after(grammarAccess.getSynchronousCompositeComponentAccess().getGroup_4_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__SynchronousCompositeComponent__Group_4_1_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousCompositeComponent__Group_4_1_1__0__Impl
	rule__SynchronousCompositeComponent__Group_4_1_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__Group_4_1_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousCompositeComponentAccess().getCommaKeyword_4_1_1_0()); }
	(',')?
	{ after(grammarAccess.getSynchronousCompositeComponentAccess().getCommaKeyword_4_1_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__Group_4_1_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousCompositeComponent__Group_4_1_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__Group_4_1_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousCompositeComponentAccess().getPortsAssignment_4_1_1_1()); }
	(rule__SynchronousCompositeComponent__PortsAssignment_4_1_1_1)
	{ after(grammarAccess.getSynchronousCompositeComponentAccess().getPortsAssignment_4_1_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AsynchronousCompositeComponent__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousCompositeComponent__Group__0__Impl
	rule__AsynchronousCompositeComponent__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousCompositeComponentAccess().getAnnotationsAssignment_0()); }
	(rule__AsynchronousCompositeComponent__AnnotationsAssignment_0)*
	{ after(grammarAccess.getAsynchronousCompositeComponentAccess().getAnnotationsAssignment_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousCompositeComponent__Group__1__Impl
	rule__AsynchronousCompositeComponent__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousCompositeComponentAccess().getAsyncKeyword_1()); }
	'async'
	{ after(grammarAccess.getAsynchronousCompositeComponentAccess().getAsyncKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousCompositeComponent__Group__2__Impl
	rule__AsynchronousCompositeComponent__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousCompositeComponentAccess().getNameAssignment_2()); }
	(rule__AsynchronousCompositeComponent__NameAssignment_2)
	{ after(grammarAccess.getAsynchronousCompositeComponentAccess().getNameAssignment_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousCompositeComponent__Group__3__Impl
	rule__AsynchronousCompositeComponent__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousCompositeComponentAccess().getGroup_3()); }
	(rule__AsynchronousCompositeComponent__Group_3__0)?
	{ after(grammarAccess.getAsynchronousCompositeComponentAccess().getGroup_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousCompositeComponent__Group__4__Impl
	rule__AsynchronousCompositeComponent__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousCompositeComponentAccess().getGroup_4()); }
	(rule__AsynchronousCompositeComponent__Group_4__0)?
	{ after(grammarAccess.getAsynchronousCompositeComponentAccess().getGroup_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousCompositeComponent__Group__5__Impl
	rule__AsynchronousCompositeComponent__Group__6
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousCompositeComponentAccess().getLeftCurlyBracketKeyword_5()); }
	'{'
	{ after(grammarAccess.getAsynchronousCompositeComponentAccess().getLeftCurlyBracketKeyword_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__Group__6
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousCompositeComponent__Group__6__Impl
	rule__AsynchronousCompositeComponent__Group__7
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__Group__6__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousCompositeComponentAccess().getAlternatives_6()); }
	(rule__AsynchronousCompositeComponent__Alternatives_6)*
	{ after(grammarAccess.getAsynchronousCompositeComponentAccess().getAlternatives_6()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__Group__7
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousCompositeComponent__Group__7__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__Group__7__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousCompositeComponentAccess().getRightCurlyBracketKeyword_7()); }
	'}'
	{ after(grammarAccess.getAsynchronousCompositeComponentAccess().getRightCurlyBracketKeyword_7()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AsynchronousCompositeComponent__Group_3__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousCompositeComponent__Group_3__0__Impl
	rule__AsynchronousCompositeComponent__Group_3__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__Group_3__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousCompositeComponentAccess().getLeftParenthesisKeyword_3_0()); }
	'('
	{ after(grammarAccess.getAsynchronousCompositeComponentAccess().getLeftParenthesisKeyword_3_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__Group_3__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousCompositeComponent__Group_3__1__Impl
	rule__AsynchronousCompositeComponent__Group_3__2
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__Group_3__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousCompositeComponentAccess().getGroup_3_1()); }
	(rule__AsynchronousCompositeComponent__Group_3_1__0)?
	{ after(grammarAccess.getAsynchronousCompositeComponentAccess().getGroup_3_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__Group_3__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousCompositeComponent__Group_3__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__Group_3__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousCompositeComponentAccess().getRightParenthesisKeyword_3_2()); }
	')'
	{ after(grammarAccess.getAsynchronousCompositeComponentAccess().getRightParenthesisKeyword_3_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AsynchronousCompositeComponent__Group_3_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousCompositeComponent__Group_3_1__0__Impl
	rule__AsynchronousCompositeComponent__Group_3_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__Group_3_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousCompositeComponentAccess().getParameterDeclarationsAssignment_3_1_0()); }
	(rule__AsynchronousCompositeComponent__ParameterDeclarationsAssignment_3_1_0)
	{ after(grammarAccess.getAsynchronousCompositeComponentAccess().getParameterDeclarationsAssignment_3_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__Group_3_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousCompositeComponent__Group_3_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__Group_3_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousCompositeComponentAccess().getGroup_3_1_1()); }
	(rule__AsynchronousCompositeComponent__Group_3_1_1__0)*
	{ after(grammarAccess.getAsynchronousCompositeComponentAccess().getGroup_3_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AsynchronousCompositeComponent__Group_3_1_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousCompositeComponent__Group_3_1_1__0__Impl
	rule__AsynchronousCompositeComponent__Group_3_1_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__Group_3_1_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousCompositeComponentAccess().getCommaKeyword_3_1_1_0()); }
	','
	{ after(grammarAccess.getAsynchronousCompositeComponentAccess().getCommaKeyword_3_1_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__Group_3_1_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousCompositeComponent__Group_3_1_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__Group_3_1_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousCompositeComponentAccess().getParameterDeclarationsAssignment_3_1_1_1()); }
	(rule__AsynchronousCompositeComponent__ParameterDeclarationsAssignment_3_1_1_1)
	{ after(grammarAccess.getAsynchronousCompositeComponentAccess().getParameterDeclarationsAssignment_3_1_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AsynchronousCompositeComponent__Group_4__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousCompositeComponent__Group_4__0__Impl
	rule__AsynchronousCompositeComponent__Group_4__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__Group_4__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousCompositeComponentAccess().getLeftSquareBracketKeyword_4_0()); }
	'['
	{ after(grammarAccess.getAsynchronousCompositeComponentAccess().getLeftSquareBracketKeyword_4_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__Group_4__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousCompositeComponent__Group_4__1__Impl
	rule__AsynchronousCompositeComponent__Group_4__2
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__Group_4__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousCompositeComponentAccess().getGroup_4_1()); }
	(rule__AsynchronousCompositeComponent__Group_4_1__0)?
	{ after(grammarAccess.getAsynchronousCompositeComponentAccess().getGroup_4_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__Group_4__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousCompositeComponent__Group_4__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__Group_4__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousCompositeComponentAccess().getRightSquareBracketKeyword_4_2()); }
	']'
	{ after(grammarAccess.getAsynchronousCompositeComponentAccess().getRightSquareBracketKeyword_4_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AsynchronousCompositeComponent__Group_4_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousCompositeComponent__Group_4_1__0__Impl
	rule__AsynchronousCompositeComponent__Group_4_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__Group_4_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousCompositeComponentAccess().getPortsAssignment_4_1_0()); }
	(rule__AsynchronousCompositeComponent__PortsAssignment_4_1_0)
	{ after(grammarAccess.getAsynchronousCompositeComponentAccess().getPortsAssignment_4_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__Group_4_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousCompositeComponent__Group_4_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__Group_4_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousCompositeComponentAccess().getGroup_4_1_1()); }
	(rule__AsynchronousCompositeComponent__Group_4_1_1__0)*
	{ after(grammarAccess.getAsynchronousCompositeComponentAccess().getGroup_4_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AsynchronousCompositeComponent__Group_4_1_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousCompositeComponent__Group_4_1_1__0__Impl
	rule__AsynchronousCompositeComponent__Group_4_1_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__Group_4_1_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousCompositeComponentAccess().getCommaKeyword_4_1_1_0()); }
	(',')?
	{ after(grammarAccess.getAsynchronousCompositeComponentAccess().getCommaKeyword_4_1_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__Group_4_1_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousCompositeComponent__Group_4_1_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__Group_4_1_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousCompositeComponentAccess().getPortsAssignment_4_1_1_1()); }
	(rule__AsynchronousCompositeComponent__PortsAssignment_4_1_1_1)
	{ after(grammarAccess.getAsynchronousCompositeComponentAccess().getPortsAssignment_4_1_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ScheduledAsynchronousCompositeComponent__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScheduledAsynchronousCompositeComponent__Group__0__Impl
	rule__ScheduledAsynchronousCompositeComponent__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getAnnotationsAssignment_0()); }
	(rule__ScheduledAsynchronousCompositeComponent__AnnotationsAssignment_0)*
	{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getAnnotationsAssignment_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScheduledAsynchronousCompositeComponent__Group__1__Impl
	rule__ScheduledAsynchronousCompositeComponent__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getScheduledAsyncKeyword_1()); }
	'scheduled-async'
	{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getScheduledAsyncKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScheduledAsynchronousCompositeComponent__Group__2__Impl
	rule__ScheduledAsynchronousCompositeComponent__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getNameAssignment_2()); }
	(rule__ScheduledAsynchronousCompositeComponent__NameAssignment_2)
	{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getNameAssignment_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScheduledAsynchronousCompositeComponent__Group__3__Impl
	rule__ScheduledAsynchronousCompositeComponent__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getGroup_3()); }
	(rule__ScheduledAsynchronousCompositeComponent__Group_3__0)?
	{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getGroup_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScheduledAsynchronousCompositeComponent__Group__4__Impl
	rule__ScheduledAsynchronousCompositeComponent__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getGroup_4()); }
	(rule__ScheduledAsynchronousCompositeComponent__Group_4__0)?
	{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getGroup_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScheduledAsynchronousCompositeComponent__Group__5__Impl
	rule__ScheduledAsynchronousCompositeComponent__Group__6
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getLeftCurlyBracketKeyword_5()); }
	'{'
	{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getLeftCurlyBracketKeyword_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group__6
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScheduledAsynchronousCompositeComponent__Group__6__Impl
	rule__ScheduledAsynchronousCompositeComponent__Group__7
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group__6__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getAlternatives_6()); }
	(rule__ScheduledAsynchronousCompositeComponent__Alternatives_6)*
	{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getAlternatives_6()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group__7
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScheduledAsynchronousCompositeComponent__Group__7__Impl
	rule__ScheduledAsynchronousCompositeComponent__Group__8
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group__7__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getGroup_7()); }
	(rule__ScheduledAsynchronousCompositeComponent__Group_7__0)?
	{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getGroup_7()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group__8
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScheduledAsynchronousCompositeComponent__Group__8__Impl
	rule__ScheduledAsynchronousCompositeComponent__Group__9
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group__8__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getGroup_8()); }
	(rule__ScheduledAsynchronousCompositeComponent__Group_8__0)?
	{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getGroup_8()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group__9
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScheduledAsynchronousCompositeComponent__Group__9__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group__9__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getRightCurlyBracketKeyword_9()); }
	'}'
	{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getRightCurlyBracketKeyword_9()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ScheduledAsynchronousCompositeComponent__Group_3__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScheduledAsynchronousCompositeComponent__Group_3__0__Impl
	rule__ScheduledAsynchronousCompositeComponent__Group_3__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group_3__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getLeftParenthesisKeyword_3_0()); }
	'('
	{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getLeftParenthesisKeyword_3_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group_3__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScheduledAsynchronousCompositeComponent__Group_3__1__Impl
	rule__ScheduledAsynchronousCompositeComponent__Group_3__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group_3__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getGroup_3_1()); }
	(rule__ScheduledAsynchronousCompositeComponent__Group_3_1__0)?
	{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getGroup_3_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group_3__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScheduledAsynchronousCompositeComponent__Group_3__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group_3__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getRightParenthesisKeyword_3_2()); }
	')'
	{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getRightParenthesisKeyword_3_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ScheduledAsynchronousCompositeComponent__Group_3_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScheduledAsynchronousCompositeComponent__Group_3_1__0__Impl
	rule__ScheduledAsynchronousCompositeComponent__Group_3_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group_3_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getParameterDeclarationsAssignment_3_1_0()); }
	(rule__ScheduledAsynchronousCompositeComponent__ParameterDeclarationsAssignment_3_1_0)
	{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getParameterDeclarationsAssignment_3_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group_3_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScheduledAsynchronousCompositeComponent__Group_3_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group_3_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getGroup_3_1_1()); }
	(rule__ScheduledAsynchronousCompositeComponent__Group_3_1_1__0)*
	{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getGroup_3_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ScheduledAsynchronousCompositeComponent__Group_3_1_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScheduledAsynchronousCompositeComponent__Group_3_1_1__0__Impl
	rule__ScheduledAsynchronousCompositeComponent__Group_3_1_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group_3_1_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getCommaKeyword_3_1_1_0()); }
	','
	{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getCommaKeyword_3_1_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group_3_1_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScheduledAsynchronousCompositeComponent__Group_3_1_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group_3_1_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getParameterDeclarationsAssignment_3_1_1_1()); }
	(rule__ScheduledAsynchronousCompositeComponent__ParameterDeclarationsAssignment_3_1_1_1)
	{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getParameterDeclarationsAssignment_3_1_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ScheduledAsynchronousCompositeComponent__Group_4__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScheduledAsynchronousCompositeComponent__Group_4__0__Impl
	rule__ScheduledAsynchronousCompositeComponent__Group_4__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group_4__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getLeftSquareBracketKeyword_4_0()); }
	'['
	{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getLeftSquareBracketKeyword_4_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group_4__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScheduledAsynchronousCompositeComponent__Group_4__1__Impl
	rule__ScheduledAsynchronousCompositeComponent__Group_4__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group_4__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getGroup_4_1()); }
	(rule__ScheduledAsynchronousCompositeComponent__Group_4_1__0)?
	{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getGroup_4_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group_4__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScheduledAsynchronousCompositeComponent__Group_4__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group_4__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getRightSquareBracketKeyword_4_2()); }
	']'
	{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getRightSquareBracketKeyword_4_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ScheduledAsynchronousCompositeComponent__Group_4_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScheduledAsynchronousCompositeComponent__Group_4_1__0__Impl
	rule__ScheduledAsynchronousCompositeComponent__Group_4_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group_4_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getPortsAssignment_4_1_0()); }
	(rule__ScheduledAsynchronousCompositeComponent__PortsAssignment_4_1_0)
	{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getPortsAssignment_4_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group_4_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScheduledAsynchronousCompositeComponent__Group_4_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group_4_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getGroup_4_1_1()); }
	(rule__ScheduledAsynchronousCompositeComponent__Group_4_1_1__0)*
	{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getGroup_4_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ScheduledAsynchronousCompositeComponent__Group_4_1_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScheduledAsynchronousCompositeComponent__Group_4_1_1__0__Impl
	rule__ScheduledAsynchronousCompositeComponent__Group_4_1_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group_4_1_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getCommaKeyword_4_1_1_0()); }
	(',')?
	{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getCommaKeyword_4_1_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group_4_1_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScheduledAsynchronousCompositeComponent__Group_4_1_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group_4_1_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getPortsAssignment_4_1_1_1()); }
	(rule__ScheduledAsynchronousCompositeComponent__PortsAssignment_4_1_1_1)
	{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getPortsAssignment_4_1_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ScheduledAsynchronousCompositeComponent__Group_7__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScheduledAsynchronousCompositeComponent__Group_7__0__Impl
	rule__ScheduledAsynchronousCompositeComponent__Group_7__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group_7__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getInitialKeyword_7_0()); }
	'initial'
	{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getInitialKeyword_7_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group_7__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScheduledAsynchronousCompositeComponent__Group_7__1__Impl
	rule__ScheduledAsynchronousCompositeComponent__Group_7__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group_7__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getExecuteKeyword_7_1()); }
	'execute'
	{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getExecuteKeyword_7_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group_7__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScheduledAsynchronousCompositeComponent__Group_7__2__Impl
	rule__ScheduledAsynchronousCompositeComponent__Group_7__3
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group_7__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getInitialExecutionListAssignment_7_2()); }
	(rule__ScheduledAsynchronousCompositeComponent__InitialExecutionListAssignment_7_2)
	{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getInitialExecutionListAssignment_7_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group_7__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScheduledAsynchronousCompositeComponent__Group_7__3__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group_7__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getGroup_7_3()); }
	(rule__ScheduledAsynchronousCompositeComponent__Group_7_3__0)*
	{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getGroup_7_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ScheduledAsynchronousCompositeComponent__Group_7_3__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScheduledAsynchronousCompositeComponent__Group_7_3__0__Impl
	rule__ScheduledAsynchronousCompositeComponent__Group_7_3__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group_7_3__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getCommaKeyword_7_3_0()); }
	','
	{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getCommaKeyword_7_3_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group_7_3__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScheduledAsynchronousCompositeComponent__Group_7_3__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group_7_3__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getInitialExecutionListAssignment_7_3_1()); }
	(rule__ScheduledAsynchronousCompositeComponent__InitialExecutionListAssignment_7_3_1)
	{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getInitialExecutionListAssignment_7_3_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ScheduledAsynchronousCompositeComponent__Group_8__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScheduledAsynchronousCompositeComponent__Group_8__0__Impl
	rule__ScheduledAsynchronousCompositeComponent__Group_8__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group_8__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getExecuteKeyword_8_0()); }
	'execute'
	{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getExecuteKeyword_8_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group_8__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScheduledAsynchronousCompositeComponent__Group_8__1__Impl
	rule__ScheduledAsynchronousCompositeComponent__Group_8__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group_8__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getExecutionListAssignment_8_1()); }
	(rule__ScheduledAsynchronousCompositeComponent__ExecutionListAssignment_8_1)
	{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getExecutionListAssignment_8_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group_8__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScheduledAsynchronousCompositeComponent__Group_8__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group_8__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getGroup_8_2()); }
	(rule__ScheduledAsynchronousCompositeComponent__Group_8_2__0)*
	{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getGroup_8_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ScheduledAsynchronousCompositeComponent__Group_8_2__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScheduledAsynchronousCompositeComponent__Group_8_2__0__Impl
	rule__ScheduledAsynchronousCompositeComponent__Group_8_2__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group_8_2__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getCommaKeyword_8_2_0()); }
	','
	{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getCommaKeyword_8_2_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group_8_2__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ScheduledAsynchronousCompositeComponent__Group_8_2__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__Group_8_2__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getExecutionListAssignment_8_2_1()); }
	(rule__ScheduledAsynchronousCompositeComponent__ExecutionListAssignment_8_2_1)
	{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getExecutionListAssignment_8_2_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AsynchronousAdapter__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousAdapter__Group__0__Impl
	rule__AsynchronousAdapter__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousAdapterAccess().getAnnotationsAssignment_0()); }
	(rule__AsynchronousAdapter__AnnotationsAssignment_0)*
	{ after(grammarAccess.getAsynchronousAdapterAccess().getAnnotationsAssignment_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousAdapter__Group__1__Impl
	rule__AsynchronousAdapter__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousAdapterAccess().getAdapterKeyword_1()); }
	'adapter'
	{ after(grammarAccess.getAsynchronousAdapterAccess().getAdapterKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousAdapter__Group__2__Impl
	rule__AsynchronousAdapter__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousAdapterAccess().getNameAssignment_2()); }
	(rule__AsynchronousAdapter__NameAssignment_2)
	{ after(grammarAccess.getAsynchronousAdapterAccess().getNameAssignment_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousAdapter__Group__3__Impl
	rule__AsynchronousAdapter__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousAdapterAccess().getGroup_3()); }
	(rule__AsynchronousAdapter__Group_3__0)?
	{ after(grammarAccess.getAsynchronousAdapterAccess().getGroup_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousAdapter__Group__4__Impl
	rule__AsynchronousAdapter__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousAdapterAccess().getOfKeyword_4()); }
	'of'
	{ after(grammarAccess.getAsynchronousAdapterAccess().getOfKeyword_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousAdapter__Group__5__Impl
	rule__AsynchronousAdapter__Group__6
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousAdapterAccess().getWrappedComponentAssignment_5()); }
	(rule__AsynchronousAdapter__WrappedComponentAssignment_5)
	{ after(grammarAccess.getAsynchronousAdapterAccess().getWrappedComponentAssignment_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__Group__6
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousAdapter__Group__6__Impl
	rule__AsynchronousAdapter__Group__7
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__Group__6__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousAdapterAccess().getGroup_6()); }
	(rule__AsynchronousAdapter__Group_6__0)?
	{ after(grammarAccess.getAsynchronousAdapterAccess().getGroup_6()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__Group__7
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousAdapter__Group__7__Impl
	rule__AsynchronousAdapter__Group__8
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__Group__7__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousAdapterAccess().getLeftCurlyBracketKeyword_7()); }
	'{'
	{ after(grammarAccess.getAsynchronousAdapterAccess().getLeftCurlyBracketKeyword_7()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__Group__8
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousAdapter__Group__8__Impl
	rule__AsynchronousAdapter__Group__9
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__Group__8__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousAdapterAccess().getAlternatives_8()); }
	(rule__AsynchronousAdapter__Alternatives_8)*
	{ after(grammarAccess.getAsynchronousAdapterAccess().getAlternatives_8()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__Group__9
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousAdapter__Group__9__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__Group__9__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousAdapterAccess().getRightCurlyBracketKeyword_9()); }
	'}'
	{ after(grammarAccess.getAsynchronousAdapterAccess().getRightCurlyBracketKeyword_9()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AsynchronousAdapter__Group_3__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousAdapter__Group_3__0__Impl
	rule__AsynchronousAdapter__Group_3__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__Group_3__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousAdapterAccess().getLeftParenthesisKeyword_3_0()); }
	'('
	{ after(grammarAccess.getAsynchronousAdapterAccess().getLeftParenthesisKeyword_3_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__Group_3__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousAdapter__Group_3__1__Impl
	rule__AsynchronousAdapter__Group_3__2
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__Group_3__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousAdapterAccess().getGroup_3_1()); }
	(rule__AsynchronousAdapter__Group_3_1__0)?
	{ after(grammarAccess.getAsynchronousAdapterAccess().getGroup_3_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__Group_3__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousAdapter__Group_3__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__Group_3__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousAdapterAccess().getRightParenthesisKeyword_3_2()); }
	')'
	{ after(grammarAccess.getAsynchronousAdapterAccess().getRightParenthesisKeyword_3_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AsynchronousAdapter__Group_3_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousAdapter__Group_3_1__0__Impl
	rule__AsynchronousAdapter__Group_3_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__Group_3_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousAdapterAccess().getParameterDeclarationsAssignment_3_1_0()); }
	(rule__AsynchronousAdapter__ParameterDeclarationsAssignment_3_1_0)
	{ after(grammarAccess.getAsynchronousAdapterAccess().getParameterDeclarationsAssignment_3_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__Group_3_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousAdapter__Group_3_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__Group_3_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousAdapterAccess().getGroup_3_1_1()); }
	(rule__AsynchronousAdapter__Group_3_1_1__0)*
	{ after(grammarAccess.getAsynchronousAdapterAccess().getGroup_3_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AsynchronousAdapter__Group_3_1_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousAdapter__Group_3_1_1__0__Impl
	rule__AsynchronousAdapter__Group_3_1_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__Group_3_1_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousAdapterAccess().getCommaKeyword_3_1_1_0()); }
	','
	{ after(grammarAccess.getAsynchronousAdapterAccess().getCommaKeyword_3_1_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__Group_3_1_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousAdapter__Group_3_1_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__Group_3_1_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousAdapterAccess().getParameterDeclarationsAssignment_3_1_1_1()); }
	(rule__AsynchronousAdapter__ParameterDeclarationsAssignment_3_1_1_1)
	{ after(grammarAccess.getAsynchronousAdapterAccess().getParameterDeclarationsAssignment_3_1_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AsynchronousAdapter__Group_6__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousAdapter__Group_6__0__Impl
	rule__AsynchronousAdapter__Group_6__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__Group_6__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousAdapterAccess().getLeftSquareBracketKeyword_6_0()); }
	'['
	{ after(grammarAccess.getAsynchronousAdapterAccess().getLeftSquareBracketKeyword_6_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__Group_6__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousAdapter__Group_6__1__Impl
	rule__AsynchronousAdapter__Group_6__2
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__Group_6__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousAdapterAccess().getGroup_6_1()); }
	(rule__AsynchronousAdapter__Group_6_1__0)?
	{ after(grammarAccess.getAsynchronousAdapterAccess().getGroup_6_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__Group_6__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousAdapter__Group_6__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__Group_6__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousAdapterAccess().getRightSquareBracketKeyword_6_2()); }
	']'
	{ after(grammarAccess.getAsynchronousAdapterAccess().getRightSquareBracketKeyword_6_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AsynchronousAdapter__Group_6_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousAdapter__Group_6_1__0__Impl
	rule__AsynchronousAdapter__Group_6_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__Group_6_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousAdapterAccess().getPortsAssignment_6_1_0()); }
	(rule__AsynchronousAdapter__PortsAssignment_6_1_0)
	{ after(grammarAccess.getAsynchronousAdapterAccess().getPortsAssignment_6_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__Group_6_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousAdapter__Group_6_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__Group_6_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousAdapterAccess().getGroup_6_1_1()); }
	(rule__AsynchronousAdapter__Group_6_1_1__0)*
	{ after(grammarAccess.getAsynchronousAdapterAccess().getGroup_6_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AsynchronousAdapter__Group_6_1_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousAdapter__Group_6_1_1__0__Impl
	rule__AsynchronousAdapter__Group_6_1_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__Group_6_1_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousAdapterAccess().getCommaKeyword_6_1_1_0()); }
	(',')?
	{ after(grammarAccess.getAsynchronousAdapterAccess().getCommaKeyword_6_1_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__Group_6_1_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousAdapter__Group_6_1_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__Group_6_1_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousAdapterAccess().getPortsAssignment_6_1_1_1()); }
	(rule__AsynchronousAdapter__PortsAssignment_6_1_1_1)
	{ after(grammarAccess.getAsynchronousAdapterAccess().getPortsAssignment_6_1_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ClockDeclaration__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ClockDeclaration__Group__0__Impl
	rule__ClockDeclaration__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ClockDeclaration__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getClockDeclarationAccess().getClockKeyword_0()); }
	'clock'
	{ after(grammarAccess.getClockDeclarationAccess().getClockKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ClockDeclaration__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ClockDeclaration__Group__1__Impl
	rule__ClockDeclaration__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ClockDeclaration__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getClockDeclarationAccess().getNameAssignment_1()); }
	(rule__ClockDeclaration__NameAssignment_1)
	{ after(grammarAccess.getClockDeclarationAccess().getNameAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ClockDeclaration__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ClockDeclaration__Group__2__Impl
	rule__ClockDeclaration__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__ClockDeclaration__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getClockDeclarationAccess().getLeftParenthesisKeyword_2()); }
	'('
	{ after(grammarAccess.getClockDeclarationAccess().getLeftParenthesisKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ClockDeclaration__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ClockDeclaration__Group__3__Impl
	rule__ClockDeclaration__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__ClockDeclaration__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getClockDeclarationAccess().getRateKeyword_3()); }
	'rate'
	{ after(grammarAccess.getClockDeclarationAccess().getRateKeyword_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ClockDeclaration__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ClockDeclaration__Group__4__Impl
	rule__ClockDeclaration__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__ClockDeclaration__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getClockDeclarationAccess().getEqualsSignKeyword_4()); }
	'='
	{ after(grammarAccess.getClockDeclarationAccess().getEqualsSignKeyword_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ClockDeclaration__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ClockDeclaration__Group__5__Impl
	rule__ClockDeclaration__Group__6
;
finally {
	restoreStackSize(stackSize);
}

rule__ClockDeclaration__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getClockDeclarationAccess().getTimeSpecificationAssignment_5()); }
	(rule__ClockDeclaration__TimeSpecificationAssignment_5)
	{ after(grammarAccess.getClockDeclarationAccess().getTimeSpecificationAssignment_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ClockDeclaration__Group__6
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ClockDeclaration__Group__6__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ClockDeclaration__Group__6__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getClockDeclarationAccess().getRightParenthesisKeyword_6()); }
	')'
	{ after(grammarAccess.getClockDeclarationAccess().getRightParenthesisKeyword_6()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__TimeSpecification__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__TimeSpecification__Group__0__Impl
	rule__TimeSpecification__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__TimeSpecification__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getTimeSpecificationAccess().getValueAssignment_0()); }
	(rule__TimeSpecification__ValueAssignment_0)
	{ after(grammarAccess.getTimeSpecificationAccess().getValueAssignment_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__TimeSpecification__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__TimeSpecification__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__TimeSpecification__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getTimeSpecificationAccess().getUnitAssignment_1()); }
	(rule__TimeSpecification__UnitAssignment_1)
	{ after(grammarAccess.getTimeSpecificationAccess().getUnitAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ControlSpecificaiton__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ControlSpecificaiton__Group__0__Impl
	rule__ControlSpecificaiton__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ControlSpecificaiton__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getControlSpecificaitonAccess().getWhenKeyword_0()); }
	'when'
	{ after(grammarAccess.getControlSpecificaitonAccess().getWhenKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ControlSpecificaiton__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ControlSpecificaiton__Group__1__Impl
	rule__ControlSpecificaiton__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ControlSpecificaiton__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getControlSpecificaitonAccess().getTriggerAssignment_1()); }
	(rule__ControlSpecificaiton__TriggerAssignment_1)
	{ after(grammarAccess.getControlSpecificaitonAccess().getTriggerAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ControlSpecificaiton__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ControlSpecificaiton__Group__2__Impl
	rule__ControlSpecificaiton__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__ControlSpecificaiton__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getControlSpecificaitonAccess().getSolidusKeyword_2()); }
	'/'
	{ after(grammarAccess.getControlSpecificaitonAccess().getSolidusKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ControlSpecificaiton__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ControlSpecificaiton__Group__3__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ControlSpecificaiton__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getControlSpecificaitonAccess().getControlFunctionAssignment_3()); }
	(rule__ControlSpecificaiton__ControlFunctionAssignment_3)
	{ after(grammarAccess.getControlSpecificaitonAccess().getControlFunctionAssignment_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__MessageQueue__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MessageQueue__Group__0__Impl
	rule__MessageQueue__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMessageQueueAccess().getQueueKeyword_0()); }
	'queue'
	{ after(grammarAccess.getMessageQueueAccess().getQueueKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MessageQueue__Group__1__Impl
	rule__MessageQueue__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMessageQueueAccess().getNameAssignment_1()); }
	(rule__MessageQueue__NameAssignment_1)
	{ after(grammarAccess.getMessageQueueAccess().getNameAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MessageQueue__Group__2__Impl
	rule__MessageQueue__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMessageQueueAccess().getLeftParenthesisKeyword_2()); }
	'('
	{ after(grammarAccess.getMessageQueueAccess().getLeftParenthesisKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MessageQueue__Group__3__Impl
	rule__MessageQueue__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMessageQueueAccess().getGroup_3()); }
	(rule__MessageQueue__Group_3__0)
	{ after(grammarAccess.getMessageQueueAccess().getGroup_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MessageQueue__Group__4__Impl
	rule__MessageQueue__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMessageQueueAccess().getGroup_4()); }
	(rule__MessageQueue__Group_4__0)
	{ after(grammarAccess.getMessageQueueAccess().getGroup_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MessageQueue__Group__5__Impl
	rule__MessageQueue__Group__6
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMessageQueueAccess().getGroup_5()); }
	(rule__MessageQueue__Group_5__0)?
	{ after(grammarAccess.getMessageQueueAccess().getGroup_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group__6
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MessageQueue__Group__6__Impl
	rule__MessageQueue__Group__7
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group__6__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMessageQueueAccess().getRightParenthesisKeyword_6()); }
	')'
	{ after(grammarAccess.getMessageQueueAccess().getRightParenthesisKeyword_6()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group__7
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MessageQueue__Group__7__Impl
	rule__MessageQueue__Group__8
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group__7__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMessageQueueAccess().getLeftCurlyBracketKeyword_7()); }
	'{'
	{ after(grammarAccess.getMessageQueueAccess().getLeftCurlyBracketKeyword_7()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group__8
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MessageQueue__Group__8__Impl
	rule__MessageQueue__Group__9
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group__8__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMessageQueueAccess().getEventPassingsAssignment_8()); }
	(rule__MessageQueue__EventPassingsAssignment_8)
	{ after(grammarAccess.getMessageQueueAccess().getEventPassingsAssignment_8()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group__9
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MessageQueue__Group__9__Impl
	rule__MessageQueue__Group__10
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group__9__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMessageQueueAccess().getGroup_9()); }
	(rule__MessageQueue__Group_9__0)*
	{ after(grammarAccess.getMessageQueueAccess().getGroup_9()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group__10
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MessageQueue__Group__10__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group__10__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMessageQueueAccess().getRightCurlyBracketKeyword_10()); }
	'}'
	{ after(grammarAccess.getMessageQueueAccess().getRightCurlyBracketKeyword_10()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__MessageQueue__Group_3__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MessageQueue__Group_3__0__Impl
	rule__MessageQueue__Group_3__1
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group_3__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMessageQueueAccess().getPriorityKeyword_3_0()); }
	'priority'
	{ after(grammarAccess.getMessageQueueAccess().getPriorityKeyword_3_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group_3__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MessageQueue__Group_3__1__Impl
	rule__MessageQueue__Group_3__2
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group_3__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMessageQueueAccess().getEqualsSignKeyword_3_1()); }
	'='
	{ after(grammarAccess.getMessageQueueAccess().getEqualsSignKeyword_3_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group_3__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MessageQueue__Group_3__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group_3__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMessageQueueAccess().getPriorityAssignment_3_2()); }
	(rule__MessageQueue__PriorityAssignment_3_2)
	{ after(grammarAccess.getMessageQueueAccess().getPriorityAssignment_3_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__MessageQueue__Group_4__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MessageQueue__Group_4__0__Impl
	rule__MessageQueue__Group_4__1
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group_4__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMessageQueueAccess().getCommaKeyword_4_0()); }
	','
	{ after(grammarAccess.getMessageQueueAccess().getCommaKeyword_4_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group_4__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MessageQueue__Group_4__1__Impl
	rule__MessageQueue__Group_4__2
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group_4__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMessageQueueAccess().getCapacityKeyword_4_1()); }
	'capacity'
	{ after(grammarAccess.getMessageQueueAccess().getCapacityKeyword_4_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group_4__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MessageQueue__Group_4__2__Impl
	rule__MessageQueue__Group_4__3
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group_4__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMessageQueueAccess().getEqualsSignKeyword_4_2()); }
	'='
	{ after(grammarAccess.getMessageQueueAccess().getEqualsSignKeyword_4_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group_4__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MessageQueue__Group_4__3__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group_4__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMessageQueueAccess().getCapacityAssignment_4_3()); }
	(rule__MessageQueue__CapacityAssignment_4_3)
	{ after(grammarAccess.getMessageQueueAccess().getCapacityAssignment_4_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__MessageQueue__Group_5__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MessageQueue__Group_5__0__Impl
	rule__MessageQueue__Group_5__1
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group_5__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMessageQueueAccess().getCommaKeyword_5_0()); }
	','
	{ after(grammarAccess.getMessageQueueAccess().getCommaKeyword_5_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group_5__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MessageQueue__Group_5__1__Impl
	rule__MessageQueue__Group_5__2
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group_5__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMessageQueueAccess().getDiscardKeyword_5_1()); }
	'discard'
	{ after(grammarAccess.getMessageQueueAccess().getDiscardKeyword_5_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group_5__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MessageQueue__Group_5__2__Impl
	rule__MessageQueue__Group_5__3
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group_5__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMessageQueueAccess().getEqualsSignKeyword_5_2()); }
	'='
	{ after(grammarAccess.getMessageQueueAccess().getEqualsSignKeyword_5_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group_5__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MessageQueue__Group_5__3__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group_5__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMessageQueueAccess().getEventDiscardStrategyAssignment_5_3()); }
	(rule__MessageQueue__EventDiscardStrategyAssignment_5_3)
	{ after(grammarAccess.getMessageQueueAccess().getEventDiscardStrategyAssignment_5_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__MessageQueue__Group_9__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MessageQueue__Group_9__0__Impl
	rule__MessageQueue__Group_9__1
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group_9__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMessageQueueAccess().getCommaKeyword_9_0()); }
	','
	{ after(grammarAccess.getMessageQueueAccess().getCommaKeyword_9_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group_9__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MessageQueue__Group_9__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__Group_9__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMessageQueueAccess().getEventPassingsAssignment_9_1()); }
	(rule__MessageQueue__EventPassingsAssignment_9_1)
	{ after(grammarAccess.getMessageQueueAccess().getEventPassingsAssignment_9_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__EventPassing__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__EventPassing__Group__0__Impl
	rule__EventPassing__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__EventPassing__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEventPassingAccess().getSourceAssignment_0()); }
	(rule__EventPassing__SourceAssignment_0)
	{ after(grammarAccess.getEventPassingAccess().getSourceAssignment_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__EventPassing__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__EventPassing__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__EventPassing__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEventPassingAccess().getGroup_1()); }
	(rule__EventPassing__Group_1__0)?
	{ after(grammarAccess.getEventPassingAccess().getGroup_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__EventPassing__Group_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__EventPassing__Group_1__0__Impl
	rule__EventPassing__Group_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__EventPassing__Group_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEventPassingAccess().getHyphenMinusGreaterThanSignKeyword_1_0()); }
	'->'
	{ after(grammarAccess.getEventPassingAccess().getHyphenMinusGreaterThanSignKeyword_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__EventPassing__Group_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__EventPassing__Group_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__EventPassing__Group_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEventPassingAccess().getTargetAssignment_1_1()); }
	(rule__EventPassing__TargetAssignment_1_1)
	{ after(grammarAccess.getEventPassingAccess().getTargetAssignment_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__PortBinding__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__PortBinding__Group__0__Impl
	rule__PortBinding__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__PortBinding__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getPortBindingAccess().getBindKeyword_0()); }
	'bind'
	{ after(grammarAccess.getPortBindingAccess().getBindKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__PortBinding__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__PortBinding__Group__1__Impl
	rule__PortBinding__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__PortBinding__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getPortBindingAccess().getCompositeSystemPortAssignment_1()); }
	(rule__PortBinding__CompositeSystemPortAssignment_1)
	{ after(grammarAccess.getPortBindingAccess().getCompositeSystemPortAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__PortBinding__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__PortBinding__Group__2__Impl
	rule__PortBinding__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__PortBinding__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getPortBindingAccess().getHyphenMinusGreaterThanSignKeyword_2()); }
	'->'
	{ after(grammarAccess.getPortBindingAccess().getHyphenMinusGreaterThanSignKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__PortBinding__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__PortBinding__Group__3__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__PortBinding__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getPortBindingAccess().getInstancePortReferenceAssignment_3()); }
	(rule__PortBinding__InstancePortReferenceAssignment_3)
	{ after(grammarAccess.getPortBindingAccess().getInstancePortReferenceAssignment_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__InstancePortReference__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__InstancePortReference__Group__0__Impl
	rule__InstancePortReference__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__InstancePortReference__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInstancePortReferenceAccess().getInstanceAssignment_0()); }
	(rule__InstancePortReference__InstanceAssignment_0)
	{ after(grammarAccess.getInstancePortReferenceAccess().getInstanceAssignment_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__InstancePortReference__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__InstancePortReference__Group__1__Impl
	rule__InstancePortReference__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__InstancePortReference__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInstancePortReferenceAccess().getFullStopKeyword_1()); }
	'.'
	{ after(grammarAccess.getInstancePortReferenceAccess().getFullStopKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__InstancePortReference__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__InstancePortReference__Group__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__InstancePortReference__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInstancePortReferenceAccess().getPortAssignment_2()); }
	(rule__InstancePortReference__PortAssignment_2)
	{ after(grammarAccess.getInstancePortReferenceAccess().getPortAssignment_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ComponentInstance__Group_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ComponentInstance__Group_1__0__Impl
	rule__ComponentInstance__Group_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstance__Group_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getComponentInstanceAccess().getAsyncKeyword_1_0()); }
	'async'
	{ after(grammarAccess.getComponentInstanceAccess().getAsyncKeyword_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstance__Group_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ComponentInstance__Group_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstance__Group_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getComponentInstanceAccess().getAsynchronousComponentInstanceParserRuleCall_1_1()); }
	ruleAsynchronousComponentInstance
	{ after(grammarAccess.getComponentInstanceAccess().getAsynchronousComponentInstanceParserRuleCall_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ComponentInstanceReferenceExpression__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ComponentInstanceReferenceExpression__Group__0__Impl
	rule__ComponentInstanceReferenceExpression__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceReferenceExpression__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getComponentInstanceReferenceExpressionAccess().getComponentInstanceAssignment_0()); }
	(rule__ComponentInstanceReferenceExpression__ComponentInstanceAssignment_0)
	{ after(grammarAccess.getComponentInstanceReferenceExpressionAccess().getComponentInstanceAssignment_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceReferenceExpression__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ComponentInstanceReferenceExpression__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceReferenceExpression__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getComponentInstanceReferenceExpressionAccess().getGroup_1()); }
	(rule__ComponentInstanceReferenceExpression__Group_1__0)?
	{ after(grammarAccess.getComponentInstanceReferenceExpressionAccess().getGroup_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ComponentInstanceReferenceExpression__Group_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ComponentInstanceReferenceExpression__Group_1__0__Impl
	rule__ComponentInstanceReferenceExpression__Group_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceReferenceExpression__Group_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getComponentInstanceReferenceExpressionAccess().getHyphenMinusGreaterThanSignKeyword_1_0()); }
	'->'
	{ after(grammarAccess.getComponentInstanceReferenceExpressionAccess().getHyphenMinusGreaterThanSignKeyword_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceReferenceExpression__Group_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ComponentInstanceReferenceExpression__Group_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceReferenceExpression__Group_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getComponentInstanceReferenceExpressionAccess().getChildAssignment_1_1()); }
	(rule__ComponentInstanceReferenceExpression__ChildAssignment_1_1)
	{ after(grammarAccess.getComponentInstanceReferenceExpressionAccess().getChildAssignment_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__SynchronousComponentInstance__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousComponentInstance__Group__0__Impl
	rule__SynchronousComponentInstance__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousComponentInstance__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousComponentInstanceAccess().getComponentKeyword_0()); }
	'component'
	{ after(grammarAccess.getSynchronousComponentInstanceAccess().getComponentKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousComponentInstance__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousComponentInstance__Group__1__Impl
	rule__SynchronousComponentInstance__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousComponentInstance__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousComponentInstanceAccess().getNameAssignment_1()); }
	(rule__SynchronousComponentInstance__NameAssignment_1)
	{ after(grammarAccess.getSynchronousComponentInstanceAccess().getNameAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousComponentInstance__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousComponentInstance__Group__2__Impl
	rule__SynchronousComponentInstance__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousComponentInstance__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousComponentInstanceAccess().getColonKeyword_2()); }
	':'
	{ after(grammarAccess.getSynchronousComponentInstanceAccess().getColonKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousComponentInstance__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousComponentInstance__Group__3__Impl
	rule__SynchronousComponentInstance__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousComponentInstance__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousComponentInstanceAccess().getTypeAssignment_3()); }
	(rule__SynchronousComponentInstance__TypeAssignment_3)
	{ after(grammarAccess.getSynchronousComponentInstanceAccess().getTypeAssignment_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousComponentInstance__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousComponentInstance__Group__4__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousComponentInstance__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousComponentInstanceAccess().getGroup_4()); }
	(rule__SynchronousComponentInstance__Group_4__0)?
	{ after(grammarAccess.getSynchronousComponentInstanceAccess().getGroup_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__SynchronousComponentInstance__Group_4__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousComponentInstance__Group_4__0__Impl
	rule__SynchronousComponentInstance__Group_4__1
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousComponentInstance__Group_4__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousComponentInstanceAccess().getLeftParenthesisKeyword_4_0()); }
	'('
	{ after(grammarAccess.getSynchronousComponentInstanceAccess().getLeftParenthesisKeyword_4_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousComponentInstance__Group_4__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousComponentInstance__Group_4__1__Impl
	rule__SynchronousComponentInstance__Group_4__2
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousComponentInstance__Group_4__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousComponentInstanceAccess().getArgumentsAssignment_4_1()); }
	(rule__SynchronousComponentInstance__ArgumentsAssignment_4_1)
	{ after(grammarAccess.getSynchronousComponentInstanceAccess().getArgumentsAssignment_4_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousComponentInstance__Group_4__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousComponentInstance__Group_4__2__Impl
	rule__SynchronousComponentInstance__Group_4__3
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousComponentInstance__Group_4__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousComponentInstanceAccess().getGroup_4_2()); }
	(rule__SynchronousComponentInstance__Group_4_2__0)*
	{ after(grammarAccess.getSynchronousComponentInstanceAccess().getGroup_4_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousComponentInstance__Group_4__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousComponentInstance__Group_4__3__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousComponentInstance__Group_4__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousComponentInstanceAccess().getRightParenthesisKeyword_4_3()); }
	')'
	{ after(grammarAccess.getSynchronousComponentInstanceAccess().getRightParenthesisKeyword_4_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__SynchronousComponentInstance__Group_4_2__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousComponentInstance__Group_4_2__0__Impl
	rule__SynchronousComponentInstance__Group_4_2__1
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousComponentInstance__Group_4_2__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousComponentInstanceAccess().getCommaKeyword_4_2_0()); }
	','
	{ after(grammarAccess.getSynchronousComponentInstanceAccess().getCommaKeyword_4_2_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousComponentInstance__Group_4_2__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousComponentInstance__Group_4_2__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousComponentInstance__Group_4_2__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSynchronousComponentInstanceAccess().getArgumentsAssignment_4_2_1()); }
	(rule__SynchronousComponentInstance__ArgumentsAssignment_4_2_1)
	{ after(grammarAccess.getSynchronousComponentInstanceAccess().getArgumentsAssignment_4_2_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AsynchronousComponentInstance__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousComponentInstance__Group__0__Impl
	rule__AsynchronousComponentInstance__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousComponentInstance__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousComponentInstanceAccess().getComponentKeyword_0()); }
	'component'
	{ after(grammarAccess.getAsynchronousComponentInstanceAccess().getComponentKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousComponentInstance__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousComponentInstance__Group__1__Impl
	rule__AsynchronousComponentInstance__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousComponentInstance__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousComponentInstanceAccess().getNameAssignment_1()); }
	(rule__AsynchronousComponentInstance__NameAssignment_1)
	{ after(grammarAccess.getAsynchronousComponentInstanceAccess().getNameAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousComponentInstance__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousComponentInstance__Group__2__Impl
	rule__AsynchronousComponentInstance__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousComponentInstance__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousComponentInstanceAccess().getColonKeyword_2()); }
	':'
	{ after(grammarAccess.getAsynchronousComponentInstanceAccess().getColonKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousComponentInstance__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousComponentInstance__Group__3__Impl
	rule__AsynchronousComponentInstance__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousComponentInstance__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousComponentInstanceAccess().getTypeAssignment_3()); }
	(rule__AsynchronousComponentInstance__TypeAssignment_3)
	{ after(grammarAccess.getAsynchronousComponentInstanceAccess().getTypeAssignment_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousComponentInstance__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousComponentInstance__Group__4__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousComponentInstance__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousComponentInstanceAccess().getGroup_4()); }
	(rule__AsynchronousComponentInstance__Group_4__0)?
	{ after(grammarAccess.getAsynchronousComponentInstanceAccess().getGroup_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AsynchronousComponentInstance__Group_4__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousComponentInstance__Group_4__0__Impl
	rule__AsynchronousComponentInstance__Group_4__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousComponentInstance__Group_4__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousComponentInstanceAccess().getLeftParenthesisKeyword_4_0()); }
	'('
	{ after(grammarAccess.getAsynchronousComponentInstanceAccess().getLeftParenthesisKeyword_4_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousComponentInstance__Group_4__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousComponentInstance__Group_4__1__Impl
	rule__AsynchronousComponentInstance__Group_4__2
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousComponentInstance__Group_4__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousComponentInstanceAccess().getArgumentsAssignment_4_1()); }
	(rule__AsynchronousComponentInstance__ArgumentsAssignment_4_1)
	{ after(grammarAccess.getAsynchronousComponentInstanceAccess().getArgumentsAssignment_4_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousComponentInstance__Group_4__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousComponentInstance__Group_4__2__Impl
	rule__AsynchronousComponentInstance__Group_4__3
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousComponentInstance__Group_4__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousComponentInstanceAccess().getGroup_4_2()); }
	(rule__AsynchronousComponentInstance__Group_4_2__0)*
	{ after(grammarAccess.getAsynchronousComponentInstanceAccess().getGroup_4_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousComponentInstance__Group_4__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousComponentInstance__Group_4__3__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousComponentInstance__Group_4__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousComponentInstanceAccess().getRightParenthesisKeyword_4_3()); }
	')'
	{ after(grammarAccess.getAsynchronousComponentInstanceAccess().getRightParenthesisKeyword_4_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AsynchronousComponentInstance__Group_4_2__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousComponentInstance__Group_4_2__0__Impl
	rule__AsynchronousComponentInstance__Group_4_2__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousComponentInstance__Group_4_2__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousComponentInstanceAccess().getCommaKeyword_4_2_0()); }
	','
	{ after(grammarAccess.getAsynchronousComponentInstanceAccess().getCommaKeyword_4_2_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousComponentInstance__Group_4_2__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousComponentInstance__Group_4_2__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousComponentInstance__Group_4_2__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAsynchronousComponentInstanceAccess().getArgumentsAssignment_4_2_1()); }
	(rule__AsynchronousComponentInstance__ArgumentsAssignment_4_2_1)
	{ after(grammarAccess.getAsynchronousComponentInstanceAccess().getArgumentsAssignment_4_2_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__SimpleChannel__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SimpleChannel__Group__0__Impl
	rule__SimpleChannel__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__SimpleChannel__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSimpleChannelAccess().getChannelKeyword_0()); }
	'channel'
	{ after(grammarAccess.getSimpleChannelAccess().getChannelKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SimpleChannel__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SimpleChannel__Group__1__Impl
	rule__SimpleChannel__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__SimpleChannel__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSimpleChannelAccess().getLeftSquareBracketKeyword_1()); }
	'['
	{ after(grammarAccess.getSimpleChannelAccess().getLeftSquareBracketKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SimpleChannel__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SimpleChannel__Group__2__Impl
	rule__SimpleChannel__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__SimpleChannel__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSimpleChannelAccess().getProvidedPortAssignment_2()); }
	(rule__SimpleChannel__ProvidedPortAssignment_2)
	{ after(grammarAccess.getSimpleChannelAccess().getProvidedPortAssignment_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SimpleChannel__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SimpleChannel__Group__3__Impl
	rule__SimpleChannel__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__SimpleChannel__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSimpleChannelAccess().getRightSquareBracketKeyword_3()); }
	']'
	{ after(grammarAccess.getSimpleChannelAccess().getRightSquareBracketKeyword_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SimpleChannel__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SimpleChannel__Group__4__Impl
	rule__SimpleChannel__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__SimpleChannel__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSimpleChannelAccess().getOKeyword_4()); }
	'-o)-'
	{ after(grammarAccess.getSimpleChannelAccess().getOKeyword_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SimpleChannel__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SimpleChannel__Group__5__Impl
	rule__SimpleChannel__Group__6
;
finally {
	restoreStackSize(stackSize);
}

rule__SimpleChannel__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSimpleChannelAccess().getLeftSquareBracketKeyword_5()); }
	'['
	{ after(grammarAccess.getSimpleChannelAccess().getLeftSquareBracketKeyword_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SimpleChannel__Group__6
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SimpleChannel__Group__6__Impl
	rule__SimpleChannel__Group__7
;
finally {
	restoreStackSize(stackSize);
}

rule__SimpleChannel__Group__6__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSimpleChannelAccess().getRequiredPortAssignment_6()); }
	(rule__SimpleChannel__RequiredPortAssignment_6)
	{ after(grammarAccess.getSimpleChannelAccess().getRequiredPortAssignment_6()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SimpleChannel__Group__7
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SimpleChannel__Group__7__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__SimpleChannel__Group__7__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSimpleChannelAccess().getRightSquareBracketKeyword_7()); }
	']'
	{ after(grammarAccess.getSimpleChannelAccess().getRightSquareBracketKeyword_7()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__BroadcastChannel__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__BroadcastChannel__Group__0__Impl
	rule__BroadcastChannel__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__BroadcastChannel__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getBroadcastChannelAccess().getChannelKeyword_0()); }
	'channel'
	{ after(grammarAccess.getBroadcastChannelAccess().getChannelKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__BroadcastChannel__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__BroadcastChannel__Group__1__Impl
	rule__BroadcastChannel__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__BroadcastChannel__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getBroadcastChannelAccess().getLeftSquareBracketKeyword_1()); }
	'['
	{ after(grammarAccess.getBroadcastChannelAccess().getLeftSquareBracketKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__BroadcastChannel__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__BroadcastChannel__Group__2__Impl
	rule__BroadcastChannel__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__BroadcastChannel__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getBroadcastChannelAccess().getProvidedPortAssignment_2()); }
	(rule__BroadcastChannel__ProvidedPortAssignment_2)
	{ after(grammarAccess.getBroadcastChannelAccess().getProvidedPortAssignment_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__BroadcastChannel__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__BroadcastChannel__Group__3__Impl
	rule__BroadcastChannel__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__BroadcastChannel__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getBroadcastChannelAccess().getRightSquareBracketKeyword_3()); }
	']'
	{ after(grammarAccess.getBroadcastChannelAccess().getRightSquareBracketKeyword_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__BroadcastChannel__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__BroadcastChannel__Group__4__Impl
	rule__BroadcastChannel__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__BroadcastChannel__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getBroadcastChannelAccess().getOKeyword_4()); }
	'-o)-'
	{ after(grammarAccess.getBroadcastChannelAccess().getOKeyword_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__BroadcastChannel__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__BroadcastChannel__Group__5__Impl
	rule__BroadcastChannel__Group__6
;
finally {
	restoreStackSize(stackSize);
}

rule__BroadcastChannel__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getBroadcastChannelAccess().getLeftSquareBracketKeyword_5()); }
	'['
	{ after(grammarAccess.getBroadcastChannelAccess().getLeftSquareBracketKeyword_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__BroadcastChannel__Group__6
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__BroadcastChannel__Group__6__Impl
	rule__BroadcastChannel__Group__7
;
finally {
	restoreStackSize(stackSize);
}

rule__BroadcastChannel__Group__6__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getBroadcastChannelAccess().getRequiredPortsAssignment_6()); }
	(rule__BroadcastChannel__RequiredPortsAssignment_6)
	{ after(grammarAccess.getBroadcastChannelAccess().getRequiredPortsAssignment_6()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__BroadcastChannel__Group__7
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__BroadcastChannel__Group__7__Impl
	rule__BroadcastChannel__Group__8
;
finally {
	restoreStackSize(stackSize);
}

rule__BroadcastChannel__Group__7__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	(
		{ before(grammarAccess.getBroadcastChannelAccess().getGroup_7()); }
		(rule__BroadcastChannel__Group_7__0)
		{ after(grammarAccess.getBroadcastChannelAccess().getGroup_7()); }
	)
	(
		{ before(grammarAccess.getBroadcastChannelAccess().getGroup_7()); }
		(rule__BroadcastChannel__Group_7__0)*
		{ after(grammarAccess.getBroadcastChannelAccess().getGroup_7()); }
	)
)
;
finally {
	restoreStackSize(stackSize);
}

rule__BroadcastChannel__Group__8
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__BroadcastChannel__Group__8__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__BroadcastChannel__Group__8__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getBroadcastChannelAccess().getRightSquareBracketKeyword_8()); }
	']'
	{ after(grammarAccess.getBroadcastChannelAccess().getRightSquareBracketKeyword_8()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__BroadcastChannel__Group_7__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__BroadcastChannel__Group_7__0__Impl
	rule__BroadcastChannel__Group_7__1
;
finally {
	restoreStackSize(stackSize);
}

rule__BroadcastChannel__Group_7__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getBroadcastChannelAccess().getCommaKeyword_7_0()); }
	','
	{ after(grammarAccess.getBroadcastChannelAccess().getCommaKeyword_7_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__BroadcastChannel__Group_7__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__BroadcastChannel__Group_7__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__BroadcastChannel__Group_7__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getBroadcastChannelAccess().getRequiredPortsAssignment_7_1()); }
	(rule__BroadcastChannel__RequiredPortsAssignment_7_1)
	{ after(grammarAccess.getBroadcastChannelAccess().getRequiredPortsAssignment_7_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__Interface__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Interface__Group__0__Impl
	rule__Interface__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__Interface__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInterfaceAccess().getInterfaceAction_0()); }
	()
	{ after(grammarAccess.getInterfaceAccess().getInterfaceAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Interface__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Interface__Group__1__Impl
	rule__Interface__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__Interface__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInterfaceAccess().getInterfaceKeyword_1()); }
	'interface'
	{ after(grammarAccess.getInterfaceAccess().getInterfaceKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Interface__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Interface__Group__2__Impl
	rule__Interface__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__Interface__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInterfaceAccess().getNameAssignment_2()); }
	(rule__Interface__NameAssignment_2)
	{ after(grammarAccess.getInterfaceAccess().getNameAssignment_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Interface__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Interface__Group__3__Impl
	rule__Interface__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__Interface__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInterfaceAccess().getGroup_3()); }
	(rule__Interface__Group_3__0)?
	{ after(grammarAccess.getInterfaceAccess().getGroup_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Interface__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Interface__Group__4__Impl
	rule__Interface__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__Interface__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInterfaceAccess().getLeftCurlyBracketKeyword_4()); }
	'{'
	{ after(grammarAccess.getInterfaceAccess().getLeftCurlyBracketKeyword_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Interface__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Interface__Group__5__Impl
	rule__Interface__Group__6
;
finally {
	restoreStackSize(stackSize);
}

rule__Interface__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInterfaceAccess().getEventsAssignment_5()); }
	(rule__Interface__EventsAssignment_5)*
	{ after(grammarAccess.getInterfaceAccess().getEventsAssignment_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Interface__Group__6
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Interface__Group__6__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__Interface__Group__6__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInterfaceAccess().getRightCurlyBracketKeyword_6()); }
	'}'
	{ after(grammarAccess.getInterfaceAccess().getRightCurlyBracketKeyword_6()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__Interface__Group_3__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Interface__Group_3__0__Impl
	rule__Interface__Group_3__1
;
finally {
	restoreStackSize(stackSize);
}

rule__Interface__Group_3__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInterfaceAccess().getExtendsKeyword_3_0()); }
	'extends'
	{ after(grammarAccess.getInterfaceAccess().getExtendsKeyword_3_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Interface__Group_3__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Interface__Group_3__1__Impl
	rule__Interface__Group_3__2
;
finally {
	restoreStackSize(stackSize);
}

rule__Interface__Group_3__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInterfaceAccess().getParentsAssignment_3_1()); }
	(rule__Interface__ParentsAssignment_3_1)
	{ after(grammarAccess.getInterfaceAccess().getParentsAssignment_3_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Interface__Group_3__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Interface__Group_3__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__Interface__Group_3__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInterfaceAccess().getGroup_3_2()); }
	(rule__Interface__Group_3_2__0)*
	{ after(grammarAccess.getInterfaceAccess().getGroup_3_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__Interface__Group_3_2__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Interface__Group_3_2__0__Impl
	rule__Interface__Group_3_2__1
;
finally {
	restoreStackSize(stackSize);
}

rule__Interface__Group_3_2__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInterfaceAccess().getCommaKeyword_3_2_0()); }
	','
	{ after(grammarAccess.getInterfaceAccess().getCommaKeyword_3_2_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Interface__Group_3_2__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Interface__Group_3_2__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__Interface__Group_3_2__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInterfaceAccess().getParentsAssignment_3_2_1()); }
	(rule__Interface__ParentsAssignment_3_2_1)
	{ after(grammarAccess.getInterfaceAccess().getParentsAssignment_3_2_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__EventDeclaration__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__EventDeclaration__Group__0__Impl
	rule__EventDeclaration__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__EventDeclaration__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEventDeclarationAccess().getEventDeclarationAction_0()); }
	()
	{ after(grammarAccess.getEventDeclarationAccess().getEventDeclarationAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__EventDeclaration__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__EventDeclaration__Group__1__Impl
	rule__EventDeclaration__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__EventDeclaration__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEventDeclarationAccess().getDirectionAssignment_1()); }
	(rule__EventDeclaration__DirectionAssignment_1)
	{ after(grammarAccess.getEventDeclarationAccess().getDirectionAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__EventDeclaration__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__EventDeclaration__Group__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__EventDeclaration__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEventDeclarationAccess().getEventAssignment_2()); }
	(rule__EventDeclaration__EventAssignment_2)
	{ after(grammarAccess.getEventDeclarationAccess().getEventAssignment_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__Event__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Event__Group__0__Impl
	rule__Event__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__Event__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEventAccess().getEventAction_0()); }
	()
	{ after(grammarAccess.getEventAccess().getEventAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Event__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Event__Group__1__Impl
	rule__Event__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__Event__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEventAccess().getPersistencyAssignment_1()); }
	(rule__Event__PersistencyAssignment_1)?
	{ after(grammarAccess.getEventAccess().getPersistencyAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Event__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Event__Group__2__Impl
	rule__Event__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__Event__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEventAccess().getEventKeyword_2()); }
	'event'
	{ after(grammarAccess.getEventAccess().getEventKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Event__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Event__Group__3__Impl
	rule__Event__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__Event__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEventAccess().getNameAssignment_3()); }
	(rule__Event__NameAssignment_3)
	{ after(grammarAccess.getEventAccess().getNameAssignment_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Event__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Event__Group__4__Impl
	rule__Event__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__Event__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEventAccess().getGroup_4()); }
	(rule__Event__Group_4__0)?
	{ after(grammarAccess.getEventAccess().getGroup_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Event__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Event__Group__5__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__Event__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEventAccess().getGroup_5()); }
	(rule__Event__Group_5__0)?
	{ after(grammarAccess.getEventAccess().getGroup_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__Event__Group_4__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Event__Group_4__0__Impl
	rule__Event__Group_4__1
;
finally {
	restoreStackSize(stackSize);
}

rule__Event__Group_4__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEventAccess().getLeftParenthesisKeyword_4_0()); }
	'('
	{ after(grammarAccess.getEventAccess().getLeftParenthesisKeyword_4_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Event__Group_4__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Event__Group_4__1__Impl
	rule__Event__Group_4__2
;
finally {
	restoreStackSize(stackSize);
}

rule__Event__Group_4__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEventAccess().getParameterDeclarationsAssignment_4_1()); }
	(rule__Event__ParameterDeclarationsAssignment_4_1)
	{ after(grammarAccess.getEventAccess().getParameterDeclarationsAssignment_4_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Event__Group_4__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Event__Group_4__2__Impl
	rule__Event__Group_4__3
;
finally {
	restoreStackSize(stackSize);
}

rule__Event__Group_4__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEventAccess().getGroup_4_2()); }
	(rule__Event__Group_4_2__0)*
	{ after(grammarAccess.getEventAccess().getGroup_4_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Event__Group_4__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Event__Group_4__3__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__Event__Group_4__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEventAccess().getRightParenthesisKeyword_4_3()); }
	')'
	{ after(grammarAccess.getEventAccess().getRightParenthesisKeyword_4_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__Event__Group_4_2__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Event__Group_4_2__0__Impl
	rule__Event__Group_4_2__1
;
finally {
	restoreStackSize(stackSize);
}

rule__Event__Group_4_2__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEventAccess().getCommaKeyword_4_2_0()); }
	','
	{ after(grammarAccess.getEventAccess().getCommaKeyword_4_2_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Event__Group_4_2__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Event__Group_4_2__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__Event__Group_4_2__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEventAccess().getParameterDeclarationsAssignment_4_2_1()); }
	(rule__Event__ParameterDeclarationsAssignment_4_2_1)
	{ after(grammarAccess.getEventAccess().getParameterDeclarationsAssignment_4_2_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__Event__Group_5__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Event__Group_5__0__Impl
	rule__Event__Group_5__1
;
finally {
	restoreStackSize(stackSize);
}

rule__Event__Group_5__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEventAccess().getPriorityKeyword_5_0()); }
	'priority'
	{ after(grammarAccess.getEventAccess().getPriorityKeyword_5_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Event__Group_5__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Event__Group_5__1__Impl
	rule__Event__Group_5__2
;
finally {
	restoreStackSize(stackSize);
}

rule__Event__Group_5__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEventAccess().getEqualsSignKeyword_5_1()); }
	'='
	{ after(grammarAccess.getEventAccess().getEqualsSignKeyword_5_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Event__Group_5__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Event__Group_5__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__Event__Group_5__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEventAccess().getPriorityAssignment_5_2()); }
	(rule__Event__PriorityAssignment_5_2)
	{ after(grammarAccess.getEventAccess().getPriorityAssignment_5_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__InlineStatement__Group_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__InlineStatement__Group_1__0__Impl
	rule__InlineStatement__Group_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__InlineStatement__Group_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInlineStatementAccess().getAlternatives_1_0()); }
	(rule__InlineStatement__Alternatives_1_0)
	{ after(grammarAccess.getInlineStatementAccess().getAlternatives_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__InlineStatement__Group_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__InlineStatement__Group_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__InlineStatement__Group_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInlineStatementAccess().getSemicolonKeyword_1_1()); }
	';'
	{ after(grammarAccess.getInlineStatementAccess().getSemicolonKeyword_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__RaiseEventAction__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RaiseEventAction__Group__0__Impl
	rule__RaiseEventAction__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__RaiseEventAction__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRaiseEventActionAccess().getRaiseKeyword_0()); }
	'raise'
	{ after(grammarAccess.getRaiseEventActionAccess().getRaiseKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RaiseEventAction__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RaiseEventAction__Group__1__Impl
	rule__RaiseEventAction__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__RaiseEventAction__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRaiseEventActionAccess().getPortAssignment_1()); }
	(rule__RaiseEventAction__PortAssignment_1)
	{ after(grammarAccess.getRaiseEventActionAccess().getPortAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RaiseEventAction__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RaiseEventAction__Group__2__Impl
	rule__RaiseEventAction__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__RaiseEventAction__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRaiseEventActionAccess().getFullStopKeyword_2()); }
	'.'
	{ after(grammarAccess.getRaiseEventActionAccess().getFullStopKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RaiseEventAction__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RaiseEventAction__Group__3__Impl
	rule__RaiseEventAction__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__RaiseEventAction__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRaiseEventActionAccess().getEventAssignment_3()); }
	(rule__RaiseEventAction__EventAssignment_3)
	{ after(grammarAccess.getRaiseEventActionAccess().getEventAssignment_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RaiseEventAction__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RaiseEventAction__Group__4__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__RaiseEventAction__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRaiseEventActionAccess().getGroup_4()); }
	(rule__RaiseEventAction__Group_4__0)?
	{ after(grammarAccess.getRaiseEventActionAccess().getGroup_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__RaiseEventAction__Group_4__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RaiseEventAction__Group_4__0__Impl
	rule__RaiseEventAction__Group_4__1
;
finally {
	restoreStackSize(stackSize);
}

rule__RaiseEventAction__Group_4__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRaiseEventActionAccess().getLeftParenthesisKeyword_4_0()); }
	'('
	{ after(grammarAccess.getRaiseEventActionAccess().getLeftParenthesisKeyword_4_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RaiseEventAction__Group_4__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RaiseEventAction__Group_4__1__Impl
	rule__RaiseEventAction__Group_4__2
;
finally {
	restoreStackSize(stackSize);
}

rule__RaiseEventAction__Group_4__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRaiseEventActionAccess().getArgumentsAssignment_4_1()); }
	(rule__RaiseEventAction__ArgumentsAssignment_4_1)
	{ after(grammarAccess.getRaiseEventActionAccess().getArgumentsAssignment_4_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RaiseEventAction__Group_4__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RaiseEventAction__Group_4__2__Impl
	rule__RaiseEventAction__Group_4__3
;
finally {
	restoreStackSize(stackSize);
}

rule__RaiseEventAction__Group_4__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRaiseEventActionAccess().getGroup_4_2()); }
	(rule__RaiseEventAction__Group_4_2__0)*
	{ after(grammarAccess.getRaiseEventActionAccess().getGroup_4_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RaiseEventAction__Group_4__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RaiseEventAction__Group_4__3__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__RaiseEventAction__Group_4__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRaiseEventActionAccess().getRightParenthesisKeyword_4_3()); }
	')'
	{ after(grammarAccess.getRaiseEventActionAccess().getRightParenthesisKeyword_4_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__RaiseEventAction__Group_4_2__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RaiseEventAction__Group_4_2__0__Impl
	rule__RaiseEventAction__Group_4_2__1
;
finally {
	restoreStackSize(stackSize);
}

rule__RaiseEventAction__Group_4_2__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRaiseEventActionAccess().getCommaKeyword_4_2_0()); }
	','
	{ after(grammarAccess.getRaiseEventActionAccess().getCommaKeyword_4_2_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RaiseEventAction__Group_4_2__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RaiseEventAction__Group_4_2__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__RaiseEventAction__Group_4_2__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRaiseEventActionAccess().getArgumentsAssignment_4_2_1()); }
	(rule__RaiseEventAction__ArgumentsAssignment_4_2_1)
	{ after(grammarAccess.getRaiseEventActionAccess().getArgumentsAssignment_4_2_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__SetTimeoutAction__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SetTimeoutAction__Group__0__Impl
	rule__SetTimeoutAction__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__SetTimeoutAction__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSetTimeoutActionAccess().getSetKeyword_0()); }
	'set'
	{ after(grammarAccess.getSetTimeoutActionAccess().getSetKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SetTimeoutAction__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SetTimeoutAction__Group__1__Impl
	rule__SetTimeoutAction__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__SetTimeoutAction__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSetTimeoutActionAccess().getTimeoutDeclarationAssignment_1()); }
	(rule__SetTimeoutAction__TimeoutDeclarationAssignment_1)
	{ after(grammarAccess.getSetTimeoutActionAccess().getTimeoutDeclarationAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SetTimeoutAction__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SetTimeoutAction__Group__2__Impl
	rule__SetTimeoutAction__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__SetTimeoutAction__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSetTimeoutActionAccess().getColonEqualsSignKeyword_2()); }
	':='
	{ after(grammarAccess.getSetTimeoutActionAccess().getColonEqualsSignKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SetTimeoutAction__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SetTimeoutAction__Group__3__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__SetTimeoutAction__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSetTimeoutActionAccess().getTimeAssignment_3()); }
	(rule__SetTimeoutAction__TimeAssignment_3)
	{ after(grammarAccess.getSetTimeoutActionAccess().getTimeAssignment_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ComponentInstanceStateReferenceExpression__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ComponentInstanceStateReferenceExpression__Group__0__Impl
	rule__ComponentInstanceStateReferenceExpression__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceStateReferenceExpression__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getComponentInstanceStateReferenceExpressionAccess().getStateKeyword_0()); }
	'state'
	{ after(grammarAccess.getComponentInstanceStateReferenceExpressionAccess().getStateKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceStateReferenceExpression__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ComponentInstanceStateReferenceExpression__Group__1__Impl
	rule__ComponentInstanceStateReferenceExpression__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceStateReferenceExpression__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getComponentInstanceStateReferenceExpressionAccess().getInstanceAssignment_1()); }
	(rule__ComponentInstanceStateReferenceExpression__InstanceAssignment_1)
	{ after(grammarAccess.getComponentInstanceStateReferenceExpressionAccess().getInstanceAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceStateReferenceExpression__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ComponentInstanceStateReferenceExpression__Group__2__Impl
	rule__ComponentInstanceStateReferenceExpression__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceStateReferenceExpression__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getComponentInstanceStateReferenceExpressionAccess().getFullStopKeyword_2()); }
	'.'
	{ after(grammarAccess.getComponentInstanceStateReferenceExpressionAccess().getFullStopKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceStateReferenceExpression__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ComponentInstanceStateReferenceExpression__Group__3__Impl
	rule__ComponentInstanceStateReferenceExpression__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceStateReferenceExpression__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getComponentInstanceStateReferenceExpressionAccess().getRegionAssignment_3()); }
	(rule__ComponentInstanceStateReferenceExpression__RegionAssignment_3)
	{ after(grammarAccess.getComponentInstanceStateReferenceExpressionAccess().getRegionAssignment_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceStateReferenceExpression__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ComponentInstanceStateReferenceExpression__Group__4__Impl
	rule__ComponentInstanceStateReferenceExpression__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceStateReferenceExpression__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getComponentInstanceStateReferenceExpressionAccess().getFullStopKeyword_4()); }
	'.'
	{ after(grammarAccess.getComponentInstanceStateReferenceExpressionAccess().getFullStopKeyword_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceStateReferenceExpression__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ComponentInstanceStateReferenceExpression__Group__5__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceStateReferenceExpression__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getComponentInstanceStateReferenceExpressionAccess().getStateAssignment_5()); }
	(rule__ComponentInstanceStateReferenceExpression__StateAssignment_5)
	{ after(grammarAccess.getComponentInstanceStateReferenceExpressionAccess().getStateAssignment_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ComponentInstanceVariableReferenceExpression__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ComponentInstanceVariableReferenceExpression__Group__0__Impl
	rule__ComponentInstanceVariableReferenceExpression__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceVariableReferenceExpression__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getComponentInstanceVariableReferenceExpressionAccess().getVariableKeyword_0()); }
	'variable'
	{ after(grammarAccess.getComponentInstanceVariableReferenceExpressionAccess().getVariableKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceVariableReferenceExpression__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ComponentInstanceVariableReferenceExpression__Group__1__Impl
	rule__ComponentInstanceVariableReferenceExpression__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceVariableReferenceExpression__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getComponentInstanceVariableReferenceExpressionAccess().getInstanceAssignment_1()); }
	(rule__ComponentInstanceVariableReferenceExpression__InstanceAssignment_1)
	{ after(grammarAccess.getComponentInstanceVariableReferenceExpressionAccess().getInstanceAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceVariableReferenceExpression__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ComponentInstanceVariableReferenceExpression__Group__2__Impl
	rule__ComponentInstanceVariableReferenceExpression__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceVariableReferenceExpression__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getComponentInstanceVariableReferenceExpressionAccess().getFullStopKeyword_2()); }
	'.'
	{ after(grammarAccess.getComponentInstanceVariableReferenceExpressionAccess().getFullStopKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceVariableReferenceExpression__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ComponentInstanceVariableReferenceExpression__Group__3__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceVariableReferenceExpression__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getComponentInstanceVariableReferenceExpressionAccess().getVariableDeclarationAssignment_3()); }
	(rule__ComponentInstanceVariableReferenceExpression__VariableDeclarationAssignment_3)
	{ after(grammarAccess.getComponentInstanceVariableReferenceExpressionAccess().getVariableDeclarationAssignment_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ComponentInstanceEventReferenceExpression__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ComponentInstanceEventReferenceExpression__Group__0__Impl
	rule__ComponentInstanceEventReferenceExpression__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceEventReferenceExpression__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getComponentInstanceEventReferenceExpressionAccess().getEventKeyword_0()); }
	'event'
	{ after(grammarAccess.getComponentInstanceEventReferenceExpressionAccess().getEventKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceEventReferenceExpression__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ComponentInstanceEventReferenceExpression__Group__1__Impl
	rule__ComponentInstanceEventReferenceExpression__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceEventReferenceExpression__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getComponentInstanceEventReferenceExpressionAccess().getInstanceAssignment_1()); }
	(rule__ComponentInstanceEventReferenceExpression__InstanceAssignment_1)
	{ after(grammarAccess.getComponentInstanceEventReferenceExpressionAccess().getInstanceAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceEventReferenceExpression__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ComponentInstanceEventReferenceExpression__Group__2__Impl
	rule__ComponentInstanceEventReferenceExpression__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceEventReferenceExpression__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getComponentInstanceEventReferenceExpressionAccess().getFullStopKeyword_2()); }
	'.'
	{ after(grammarAccess.getComponentInstanceEventReferenceExpressionAccess().getFullStopKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceEventReferenceExpression__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ComponentInstanceEventReferenceExpression__Group__3__Impl
	rule__ComponentInstanceEventReferenceExpression__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceEventReferenceExpression__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getComponentInstanceEventReferenceExpressionAccess().getPortAssignment_3()); }
	(rule__ComponentInstanceEventReferenceExpression__PortAssignment_3)
	{ after(grammarAccess.getComponentInstanceEventReferenceExpressionAccess().getPortAssignment_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceEventReferenceExpression__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ComponentInstanceEventReferenceExpression__Group__4__Impl
	rule__ComponentInstanceEventReferenceExpression__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceEventReferenceExpression__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getComponentInstanceEventReferenceExpressionAccess().getFullStopKeyword_4()); }
	'.'
	{ after(grammarAccess.getComponentInstanceEventReferenceExpressionAccess().getFullStopKeyword_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceEventReferenceExpression__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ComponentInstanceEventReferenceExpression__Group__5__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceEventReferenceExpression__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getComponentInstanceEventReferenceExpressionAccess().getEventAssignment_5()); }
	(rule__ComponentInstanceEventReferenceExpression__EventAssignment_5)
	{ after(grammarAccess.getComponentInstanceEventReferenceExpressionAccess().getEventAssignment_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ComponentInstanceEventParameterReferenceExpression__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ComponentInstanceEventParameterReferenceExpression__Group__0__Impl
	rule__ComponentInstanceEventParameterReferenceExpression__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceEventParameterReferenceExpression__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getComponentInstanceEventParameterReferenceExpressionAccess().getParameterKeyword_0()); }
	'parameter'
	{ after(grammarAccess.getComponentInstanceEventParameterReferenceExpressionAccess().getParameterKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceEventParameterReferenceExpression__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ComponentInstanceEventParameterReferenceExpression__Group__1__Impl
	rule__ComponentInstanceEventParameterReferenceExpression__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceEventParameterReferenceExpression__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getComponentInstanceEventParameterReferenceExpressionAccess().getInstanceAssignment_1()); }
	(rule__ComponentInstanceEventParameterReferenceExpression__InstanceAssignment_1)
	{ after(grammarAccess.getComponentInstanceEventParameterReferenceExpressionAccess().getInstanceAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceEventParameterReferenceExpression__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ComponentInstanceEventParameterReferenceExpression__Group__2__Impl
	rule__ComponentInstanceEventParameterReferenceExpression__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceEventParameterReferenceExpression__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getComponentInstanceEventParameterReferenceExpressionAccess().getFullStopKeyword_2()); }
	'.'
	{ after(grammarAccess.getComponentInstanceEventParameterReferenceExpressionAccess().getFullStopKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceEventParameterReferenceExpression__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ComponentInstanceEventParameterReferenceExpression__Group__3__Impl
	rule__ComponentInstanceEventParameterReferenceExpression__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceEventParameterReferenceExpression__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getComponentInstanceEventParameterReferenceExpressionAccess().getPortAssignment_3()); }
	(rule__ComponentInstanceEventParameterReferenceExpression__PortAssignment_3)
	{ after(grammarAccess.getComponentInstanceEventParameterReferenceExpressionAccess().getPortAssignment_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceEventParameterReferenceExpression__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ComponentInstanceEventParameterReferenceExpression__Group__4__Impl
	rule__ComponentInstanceEventParameterReferenceExpression__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceEventParameterReferenceExpression__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getComponentInstanceEventParameterReferenceExpressionAccess().getFullStopKeyword_4()); }
	'.'
	{ after(grammarAccess.getComponentInstanceEventParameterReferenceExpressionAccess().getFullStopKeyword_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceEventParameterReferenceExpression__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ComponentInstanceEventParameterReferenceExpression__Group__5__Impl
	rule__ComponentInstanceEventParameterReferenceExpression__Group__6
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceEventParameterReferenceExpression__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getComponentInstanceEventParameterReferenceExpressionAccess().getEventAssignment_5()); }
	(rule__ComponentInstanceEventParameterReferenceExpression__EventAssignment_5)
	{ after(grammarAccess.getComponentInstanceEventParameterReferenceExpressionAccess().getEventAssignment_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceEventParameterReferenceExpression__Group__6
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ComponentInstanceEventParameterReferenceExpression__Group__6__Impl
	rule__ComponentInstanceEventParameterReferenceExpression__Group__7
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceEventParameterReferenceExpression__Group__6__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getComponentInstanceEventParameterReferenceExpressionAccess().getColonColonKeyword_6()); }
	'::'
	{ after(grammarAccess.getComponentInstanceEventParameterReferenceExpressionAccess().getColonColonKeyword_6()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceEventParameterReferenceExpression__Group__7
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ComponentInstanceEventParameterReferenceExpression__Group__7__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceEventParameterReferenceExpression__Group__7__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getComponentInstanceEventParameterReferenceExpressionAccess().getParameterDeclarationAssignment_7()); }
	(rule__ComponentInstanceEventParameterReferenceExpression__ParameterDeclarationAssignment_7)
	{ after(grammarAccess.getComponentInstanceEventParameterReferenceExpressionAccess().getParameterDeclarationAssignment_7()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ProcedureDeclaration__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ProcedureDeclaration__Group__0__Impl
	rule__ProcedureDeclaration__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ProcedureDeclaration__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getProcedureDeclarationAccess().getProcedureKeyword_0()); }
	'procedure'
	{ after(grammarAccess.getProcedureDeclarationAccess().getProcedureKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ProcedureDeclaration__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ProcedureDeclaration__Group__1__Impl
	rule__ProcedureDeclaration__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ProcedureDeclaration__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getProcedureDeclarationAccess().getNameAssignment_1()); }
	(rule__ProcedureDeclaration__NameAssignment_1)
	{ after(grammarAccess.getProcedureDeclarationAccess().getNameAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ProcedureDeclaration__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ProcedureDeclaration__Group__2__Impl
	rule__ProcedureDeclaration__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__ProcedureDeclaration__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getProcedureDeclarationAccess().getGroup_2()); }
	(rule__ProcedureDeclaration__Group_2__0)?
	{ after(grammarAccess.getProcedureDeclarationAccess().getGroup_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ProcedureDeclaration__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ProcedureDeclaration__Group__3__Impl
	rule__ProcedureDeclaration__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__ProcedureDeclaration__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getProcedureDeclarationAccess().getColonKeyword_3()); }
	':'
	{ after(grammarAccess.getProcedureDeclarationAccess().getColonKeyword_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ProcedureDeclaration__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ProcedureDeclaration__Group__4__Impl
	rule__ProcedureDeclaration__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__ProcedureDeclaration__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getProcedureDeclarationAccess().getTypeAssignment_4()); }
	(rule__ProcedureDeclaration__TypeAssignment_4)
	{ after(grammarAccess.getProcedureDeclarationAccess().getTypeAssignment_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ProcedureDeclaration__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ProcedureDeclaration__Group__5__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ProcedureDeclaration__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getProcedureDeclarationAccess().getBodyAssignment_5()); }
	(rule__ProcedureDeclaration__BodyAssignment_5)
	{ after(grammarAccess.getProcedureDeclarationAccess().getBodyAssignment_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ProcedureDeclaration__Group_2__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ProcedureDeclaration__Group_2__0__Impl
	rule__ProcedureDeclaration__Group_2__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ProcedureDeclaration__Group_2__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getProcedureDeclarationAccess().getLeftParenthesisKeyword_2_0()); }
	'('
	{ after(grammarAccess.getProcedureDeclarationAccess().getLeftParenthesisKeyword_2_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ProcedureDeclaration__Group_2__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ProcedureDeclaration__Group_2__1__Impl
	rule__ProcedureDeclaration__Group_2__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ProcedureDeclaration__Group_2__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getProcedureDeclarationAccess().getGroup_2_1()); }
	(rule__ProcedureDeclaration__Group_2_1__0)?
	{ after(grammarAccess.getProcedureDeclarationAccess().getGroup_2_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ProcedureDeclaration__Group_2__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ProcedureDeclaration__Group_2__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ProcedureDeclaration__Group_2__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getProcedureDeclarationAccess().getRightParenthesisKeyword_2_2()); }
	')'
	{ after(grammarAccess.getProcedureDeclarationAccess().getRightParenthesisKeyword_2_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ProcedureDeclaration__Group_2_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ProcedureDeclaration__Group_2_1__0__Impl
	rule__ProcedureDeclaration__Group_2_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ProcedureDeclaration__Group_2_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getProcedureDeclarationAccess().getParameterDeclarationsAssignment_2_1_0()); }
	(rule__ProcedureDeclaration__ParameterDeclarationsAssignment_2_1_0)
	{ after(grammarAccess.getProcedureDeclarationAccess().getParameterDeclarationsAssignment_2_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ProcedureDeclaration__Group_2_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ProcedureDeclaration__Group_2_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ProcedureDeclaration__Group_2_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getProcedureDeclarationAccess().getGroup_2_1_1()); }
	(rule__ProcedureDeclaration__Group_2_1_1__0)*
	{ after(grammarAccess.getProcedureDeclarationAccess().getGroup_2_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ProcedureDeclaration__Group_2_1_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ProcedureDeclaration__Group_2_1_1__0__Impl
	rule__ProcedureDeclaration__Group_2_1_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ProcedureDeclaration__Group_2_1_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getProcedureDeclarationAccess().getCommaKeyword_2_1_1_0()); }
	','
	{ after(grammarAccess.getProcedureDeclarationAccess().getCommaKeyword_2_1_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ProcedureDeclaration__Group_2_1_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ProcedureDeclaration__Group_2_1_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ProcedureDeclaration__Group_2_1_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getProcedureDeclarationAccess().getParameterDeclarationsAssignment_2_1_1_1()); }
	(rule__ProcedureDeclaration__ParameterDeclarationsAssignment_2_1_1_1)
	{ after(grammarAccess.getProcedureDeclarationAccess().getParameterDeclarationsAssignment_2_1_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__Block__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Block__Group__0__Impl
	rule__Block__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__Block__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getBlockAccess().getBlockAction_0()); }
	()
	{ after(grammarAccess.getBlockAccess().getBlockAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Block__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Block__Group__1__Impl
	rule__Block__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__Block__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getBlockAccess().getLeftCurlyBracketKeyword_1()); }
	'{'
	{ after(grammarAccess.getBlockAccess().getLeftCurlyBracketKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Block__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Block__Group__2__Impl
	rule__Block__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__Block__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getBlockAccess().getActionsAssignment_2()); }
	(rule__Block__ActionsAssignment_2)*
	{ after(grammarAccess.getBlockAccess().getActionsAssignment_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__Block__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__Block__Group__3__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__Block__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getBlockAccess().getRightCurlyBracketKeyword_3()); }
	'}'
	{ after(grammarAccess.getBlockAccess().getRightCurlyBracketKeyword_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}


superInlineStatement__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	superInlineStatement__Group__0__Impl
	superInlineStatement__Group__1
;
finally {
	restoreStackSize(stackSize);
}

superInlineStatement__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getActionLanguageInlineStatementAccess().getAlternatives_0()); }
	(superInlineStatement__Alternatives_0)
	{ after(grammarAccess.getActionLanguageInlineStatementAccess().getAlternatives_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

superInlineStatement__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	superInlineStatement__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

superInlineStatement__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getActionLanguageInlineStatementAccess().getSemicolonKeyword_1()); }
	';'
	{ after(grammarAccess.getActionLanguageInlineStatementAccess().getSemicolonKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__EmptyStatement__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__EmptyStatement__Group__0__Impl
	rule__EmptyStatement__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__EmptyStatement__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEmptyStatementAccess().getEmptyStatementAction_0()); }
	()
	{ after(grammarAccess.getEmptyStatementAccess().getEmptyStatementAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__EmptyStatement__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__EmptyStatement__Group__1__Impl
	rule__EmptyStatement__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__EmptyStatement__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEmptyStatementAccess().getNoKeyword_1()); }
	'no'
	{ after(grammarAccess.getEmptyStatementAccess().getNoKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__EmptyStatement__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__EmptyStatement__Group__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__EmptyStatement__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEmptyStatementAccess().getOperationKeyword_2()); }
	'operation'
	{ after(grammarAccess.getEmptyStatementAccess().getOperationKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__IfBranch__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__IfBranch__Group__0__Impl
	rule__IfBranch__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__IfBranch__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getIfBranchAccess().getLeftParenthesisKeyword_0()); }
	'('
	{ after(grammarAccess.getIfBranchAccess().getLeftParenthesisKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__IfBranch__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__IfBranch__Group__1__Impl
	rule__IfBranch__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__IfBranch__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getIfBranchAccess().getGuardAssignment_1()); }
	(rule__IfBranch__GuardAssignment_1)
	{ after(grammarAccess.getIfBranchAccess().getGuardAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__IfBranch__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__IfBranch__Group__2__Impl
	rule__IfBranch__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__IfBranch__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getIfBranchAccess().getRightParenthesisKeyword_2()); }
	')'
	{ after(grammarAccess.getIfBranchAccess().getRightParenthesisKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__IfBranch__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__IfBranch__Group__3__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__IfBranch__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getIfBranchAccess().getActionAssignment_3()); }
	(rule__IfBranch__ActionAssignment_3)
	{ after(grammarAccess.getIfBranchAccess().getActionAssignment_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ElseBranch__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ElseBranch__Group__0__Impl
	rule__ElseBranch__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ElseBranch__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getElseBranchAccess().getGuardAssignment_0()); }
	(rule__ElseBranch__GuardAssignment_0)
	{ after(grammarAccess.getElseBranchAccess().getGuardAssignment_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ElseBranch__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ElseBranch__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ElseBranch__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getElseBranchAccess().getActionAssignment_1()); }
	(rule__ElseBranch__ActionAssignment_1)
	{ after(grammarAccess.getElseBranchAccess().getActionAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__SwitchBranch__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SwitchBranch__Group__0__Impl
	rule__SwitchBranch__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__SwitchBranch__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSwitchBranchAccess().getGuardAssignment_0()); }
	(rule__SwitchBranch__GuardAssignment_0)
	{ after(grammarAccess.getSwitchBranchAccess().getGuardAssignment_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SwitchBranch__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SwitchBranch__Group__1__Impl
	rule__SwitchBranch__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__SwitchBranch__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSwitchBranchAccess().getColonKeyword_1()); }
	':'
	{ after(grammarAccess.getSwitchBranchAccess().getColonKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SwitchBranch__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SwitchBranch__Group__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__SwitchBranch__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSwitchBranchAccess().getActionAssignment_2()); }
	(rule__SwitchBranch__ActionAssignment_2)
	{ after(grammarAccess.getSwitchBranchAccess().getActionAssignment_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__DefaultBranch__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DefaultBranch__Group__0__Impl
	rule__DefaultBranch__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__DefaultBranch__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDefaultBranchAccess().getGuardAssignment_0()); }
	(rule__DefaultBranch__GuardAssignment_0)
	{ after(grammarAccess.getDefaultBranchAccess().getGuardAssignment_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DefaultBranch__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DefaultBranch__Group__1__Impl
	rule__DefaultBranch__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__DefaultBranch__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDefaultBranchAccess().getColonKeyword_1()); }
	':'
	{ after(grammarAccess.getDefaultBranchAccess().getColonKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DefaultBranch__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DefaultBranch__Group__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__DefaultBranch__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDefaultBranchAccess().getActionAssignment_2()); }
	(rule__DefaultBranch__ActionAssignment_2)
	{ after(grammarAccess.getDefaultBranchAccess().getActionAssignment_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ChoiceBranch__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ChoiceBranch__Group__0__Impl
	rule__ChoiceBranch__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ChoiceBranch__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getChoiceBranchAccess().getLeftSquareBracketKeyword_0()); }
	'['
	{ after(grammarAccess.getChoiceBranchAccess().getLeftSquareBracketKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ChoiceBranch__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ChoiceBranch__Group__1__Impl
	rule__ChoiceBranch__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ChoiceBranch__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getChoiceBranchAccess().getGuardAssignment_1()); }
	(rule__ChoiceBranch__GuardAssignment_1)
	{ after(grammarAccess.getChoiceBranchAccess().getGuardAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ChoiceBranch__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ChoiceBranch__Group__2__Impl
	rule__ChoiceBranch__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__ChoiceBranch__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getChoiceBranchAccess().getRightSquareBracketKeyword_2()); }
	']'
	{ after(grammarAccess.getChoiceBranchAccess().getRightSquareBracketKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ChoiceBranch__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ChoiceBranch__Group__3__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ChoiceBranch__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getChoiceBranchAccess().getActionAssignment_3()); }
	(rule__ChoiceBranch__ActionAssignment_3)
	{ after(grammarAccess.getChoiceBranchAccess().getActionAssignment_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__DefaultExpression__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DefaultExpression__Group__0__Impl
	rule__DefaultExpression__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__DefaultExpression__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDefaultExpressionAccess().getDefaultExpressionAction_0()); }
	()
	{ after(grammarAccess.getDefaultExpressionAccess().getDefaultExpressionAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DefaultExpression__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DefaultExpression__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__DefaultExpression__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDefaultExpressionAccess().getDefaultKeyword_1()); }
	'default'
	{ after(grammarAccess.getDefaultExpressionAccess().getDefaultKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__IfStatement__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__IfStatement__Group__0__Impl
	rule__IfStatement__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__IfStatement__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getIfStatementAccess().getIfStatementAction_0()); }
	()
	{ after(grammarAccess.getIfStatementAccess().getIfStatementAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__IfStatement__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__IfStatement__Group__1__Impl
	rule__IfStatement__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__IfStatement__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getIfStatementAccess().getIfKeyword_1()); }
	'if'
	{ after(grammarAccess.getIfStatementAccess().getIfKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__IfStatement__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__IfStatement__Group__2__Impl
	rule__IfStatement__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__IfStatement__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getIfStatementAccess().getConditionalsAssignment_2()); }
	(rule__IfStatement__ConditionalsAssignment_2)
	{ after(grammarAccess.getIfStatementAccess().getConditionalsAssignment_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__IfStatement__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__IfStatement__Group__3__Impl
	rule__IfStatement__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__IfStatement__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getIfStatementAccess().getGroup_3()); }
	(rule__IfStatement__Group_3__0)*
	{ after(grammarAccess.getIfStatementAccess().getGroup_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__IfStatement__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__IfStatement__Group__4__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__IfStatement__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getIfStatementAccess().getConditionalsAssignment_4()); }
	(rule__IfStatement__ConditionalsAssignment_4)?
	{ after(grammarAccess.getIfStatementAccess().getConditionalsAssignment_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__IfStatement__Group_3__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__IfStatement__Group_3__0__Impl
	rule__IfStatement__Group_3__1
;
finally {
	restoreStackSize(stackSize);
}

rule__IfStatement__Group_3__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getIfStatementAccess().getElseifKeyword_3_0()); }
	('elseif')
	{ after(grammarAccess.getIfStatementAccess().getElseifKeyword_3_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__IfStatement__Group_3__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__IfStatement__Group_3__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__IfStatement__Group_3__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getIfStatementAccess().getConditionalsAssignment_3_1()); }
	(rule__IfStatement__ConditionalsAssignment_3_1)
	{ after(grammarAccess.getIfStatementAccess().getConditionalsAssignment_3_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__SwitchStatement__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SwitchStatement__Group__0__Impl
	rule__SwitchStatement__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__SwitchStatement__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSwitchStatementAccess().getSwitchKeyword_0()); }
	'switch'
	{ after(grammarAccess.getSwitchStatementAccess().getSwitchKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SwitchStatement__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SwitchStatement__Group__1__Impl
	rule__SwitchStatement__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__SwitchStatement__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSwitchStatementAccess().getLeftParenthesisKeyword_1()); }
	'('
	{ after(grammarAccess.getSwitchStatementAccess().getLeftParenthesisKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SwitchStatement__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SwitchStatement__Group__2__Impl
	rule__SwitchStatement__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__SwitchStatement__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSwitchStatementAccess().getControlExpressionAssignment_2()); }
	(rule__SwitchStatement__ControlExpressionAssignment_2)
	{ after(grammarAccess.getSwitchStatementAccess().getControlExpressionAssignment_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SwitchStatement__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SwitchStatement__Group__3__Impl
	rule__SwitchStatement__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__SwitchStatement__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSwitchStatementAccess().getRightParenthesisKeyword_3()); }
	')'
	{ after(grammarAccess.getSwitchStatementAccess().getRightParenthesisKeyword_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SwitchStatement__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SwitchStatement__Group__4__Impl
	rule__SwitchStatement__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__SwitchStatement__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSwitchStatementAccess().getLeftCurlyBracketKeyword_4()); }
	'{'
	{ after(grammarAccess.getSwitchStatementAccess().getLeftCurlyBracketKeyword_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SwitchStatement__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SwitchStatement__Group__5__Impl
	rule__SwitchStatement__Group__6
;
finally {
	restoreStackSize(stackSize);
}

rule__SwitchStatement__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSwitchStatementAccess().getGroup_5()); }
	(rule__SwitchStatement__Group_5__0)
	{ after(grammarAccess.getSwitchStatementAccess().getGroup_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SwitchStatement__Group__6
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SwitchStatement__Group__6__Impl
	rule__SwitchStatement__Group__7
;
finally {
	restoreStackSize(stackSize);
}

rule__SwitchStatement__Group__6__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSwitchStatementAccess().getCasesAssignment_6()); }
	(rule__SwitchStatement__CasesAssignment_6)?
	{ after(grammarAccess.getSwitchStatementAccess().getCasesAssignment_6()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SwitchStatement__Group__7
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SwitchStatement__Group__7__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__SwitchStatement__Group__7__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSwitchStatementAccess().getRightCurlyBracketKeyword_7()); }
	'}'
	{ after(grammarAccess.getSwitchStatementAccess().getRightCurlyBracketKeyword_7()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__SwitchStatement__Group_5__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SwitchStatement__Group_5__0__Impl
	rule__SwitchStatement__Group_5__1
;
finally {
	restoreStackSize(stackSize);
}

rule__SwitchStatement__Group_5__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSwitchStatementAccess().getCaseKeyword_5_0()); }
	'case'
	{ after(grammarAccess.getSwitchStatementAccess().getCaseKeyword_5_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SwitchStatement__Group_5__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SwitchStatement__Group_5__1__Impl
	rule__SwitchStatement__Group_5__2
;
finally {
	restoreStackSize(stackSize);
}

rule__SwitchStatement__Group_5__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSwitchStatementAccess().getCasesAssignment_5_1()); }
	(rule__SwitchStatement__CasesAssignment_5_1)
	{ after(grammarAccess.getSwitchStatementAccess().getCasesAssignment_5_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SwitchStatement__Group_5__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SwitchStatement__Group_5__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__SwitchStatement__Group_5__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSwitchStatementAccess().getGroup_5_2()); }
	(rule__SwitchStatement__Group_5_2__0)*
	{ after(grammarAccess.getSwitchStatementAccess().getGroup_5_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__SwitchStatement__Group_5_2__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SwitchStatement__Group_5_2__0__Impl
	rule__SwitchStatement__Group_5_2__1
;
finally {
	restoreStackSize(stackSize);
}

rule__SwitchStatement__Group_5_2__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSwitchStatementAccess().getCaseKeyword_5_2_0()); }
	'case'
	{ after(grammarAccess.getSwitchStatementAccess().getCaseKeyword_5_2_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SwitchStatement__Group_5_2__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SwitchStatement__Group_5_2__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__SwitchStatement__Group_5_2__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSwitchStatementAccess().getCasesAssignment_5_2_1()); }
	(rule__SwitchStatement__CasesAssignment_5_2_1)
	{ after(grammarAccess.getSwitchStatementAccess().getCasesAssignment_5_2_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ForStatement__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ForStatement__Group__0__Impl
	rule__ForStatement__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ForStatement__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getForStatementAccess().getForKeyword_0()); }
	'for'
	{ after(grammarAccess.getForStatementAccess().getForKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ForStatement__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ForStatement__Group__1__Impl
	rule__ForStatement__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ForStatement__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getForStatementAccess().getLeftParenthesisKeyword_1()); }
	'('
	{ after(grammarAccess.getForStatementAccess().getLeftParenthesisKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ForStatement__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ForStatement__Group__2__Impl
	rule__ForStatement__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__ForStatement__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getForStatementAccess().getParameterAssignment_2()); }
	(rule__ForStatement__ParameterAssignment_2)
	{ after(grammarAccess.getForStatementAccess().getParameterAssignment_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ForStatement__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ForStatement__Group__3__Impl
	rule__ForStatement__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__ForStatement__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getForStatementAccess().getInKeyword_3()); }
	'in'
	{ after(grammarAccess.getForStatementAccess().getInKeyword_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ForStatement__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ForStatement__Group__4__Impl
	rule__ForStatement__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__ForStatement__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getForStatementAccess().getRangeAssignment_4()); }
	(rule__ForStatement__RangeAssignment_4)
	{ after(grammarAccess.getForStatementAccess().getRangeAssignment_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ForStatement__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ForStatement__Group__5__Impl
	rule__ForStatement__Group__6
;
finally {
	restoreStackSize(stackSize);
}

rule__ForStatement__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getForStatementAccess().getRightParenthesisKeyword_5()); }
	')'
	{ after(grammarAccess.getForStatementAccess().getRightParenthesisKeyword_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ForStatement__Group__6
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ForStatement__Group__6__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ForStatement__Group__6__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getForStatementAccess().getBodyAssignment_6()); }
	(rule__ForStatement__BodyAssignment_6)
	{ after(grammarAccess.getForStatementAccess().getBodyAssignment_6()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ReturnStatement__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ReturnStatement__Group__0__Impl
	rule__ReturnStatement__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ReturnStatement__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getReturnStatementAccess().getReturnStatementAction_0()); }
	()
	{ after(grammarAccess.getReturnStatementAccess().getReturnStatementAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ReturnStatement__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ReturnStatement__Group__1__Impl
	rule__ReturnStatement__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ReturnStatement__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getReturnStatementAccess().getReturnKeyword_1()); }
	'return'
	{ after(grammarAccess.getReturnStatementAccess().getReturnKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ReturnStatement__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ReturnStatement__Group__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ReturnStatement__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getReturnStatementAccess().getExpressionAssignment_2()); }
	(rule__ReturnStatement__ExpressionAssignment_2)?
	{ after(grammarAccess.getReturnStatementAccess().getExpressionAssignment_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__BreakStatement__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__BreakStatement__Group__0__Impl
	rule__BreakStatement__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__BreakStatement__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getBreakStatementAccess().getBreakStatementAction_0()); }
	()
	{ after(grammarAccess.getBreakStatementAccess().getBreakStatementAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__BreakStatement__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__BreakStatement__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__BreakStatement__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getBreakStatementAccess().getBreakKeyword_1()); }
	'break'
	{ after(grammarAccess.getBreakStatementAccess().getBreakKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ChoiceStatement__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ChoiceStatement__Group__0__Impl
	rule__ChoiceStatement__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ChoiceStatement__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getChoiceStatementAccess().getChoiceKeyword_0()); }
	'choice'
	{ after(grammarAccess.getChoiceStatementAccess().getChoiceKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ChoiceStatement__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ChoiceStatement__Group__1__Impl
	rule__ChoiceStatement__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ChoiceStatement__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getChoiceStatementAccess().getLeftCurlyBracketKeyword_1()); }
	'{'
	{ after(grammarAccess.getChoiceStatementAccess().getLeftCurlyBracketKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ChoiceStatement__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ChoiceStatement__Group__2__Impl
	rule__ChoiceStatement__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__ChoiceStatement__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getChoiceStatementAccess().getGroup_2()); }
	(rule__ChoiceStatement__Group_2__0)
	{ after(grammarAccess.getChoiceStatementAccess().getGroup_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ChoiceStatement__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ChoiceStatement__Group__3__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ChoiceStatement__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getChoiceStatementAccess().getRightCurlyBracketKeyword_3()); }
	'}'
	{ after(grammarAccess.getChoiceStatementAccess().getRightCurlyBracketKeyword_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ChoiceStatement__Group_2__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ChoiceStatement__Group_2__0__Impl
	rule__ChoiceStatement__Group_2__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ChoiceStatement__Group_2__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getChoiceStatementAccess().getBranchKeyword_2_0()); }
	'branch'
	{ after(grammarAccess.getChoiceStatementAccess().getBranchKeyword_2_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ChoiceStatement__Group_2__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ChoiceStatement__Group_2__1__Impl
	rule__ChoiceStatement__Group_2__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ChoiceStatement__Group_2__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getChoiceStatementAccess().getBranchesAssignment_2_1()); }
	(rule__ChoiceStatement__BranchesAssignment_2_1)
	{ after(grammarAccess.getChoiceStatementAccess().getBranchesAssignment_2_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ChoiceStatement__Group_2__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ChoiceStatement__Group_2__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ChoiceStatement__Group_2__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getChoiceStatementAccess().getGroup_2_2()); }
	(rule__ChoiceStatement__Group_2_2__0)*
	{ after(grammarAccess.getChoiceStatementAccess().getGroup_2_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ChoiceStatement__Group_2_2__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ChoiceStatement__Group_2_2__0__Impl
	rule__ChoiceStatement__Group_2_2__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ChoiceStatement__Group_2_2__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getChoiceStatementAccess().getBranchKeyword_2_2_0()); }
	'branch'
	{ after(grammarAccess.getChoiceStatementAccess().getBranchKeyword_2_2_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ChoiceStatement__Group_2_2__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ChoiceStatement__Group_2_2__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ChoiceStatement__Group_2_2__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getChoiceStatementAccess().getBranchesAssignment_2_2_1()); }
	(rule__ChoiceStatement__BranchesAssignment_2_2_1)
	{ after(grammarAccess.getChoiceStatementAccess().getBranchesAssignment_2_2_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AssignmentStatement__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AssignmentStatement__Group__0__Impl
	rule__AssignmentStatement__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AssignmentStatement__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAssignmentStatementAccess().getLhsAssignment_0()); }
	(rule__AssignmentStatement__LhsAssignment_0)
	{ after(grammarAccess.getAssignmentStatementAccess().getLhsAssignment_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AssignmentStatement__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AssignmentStatement__Group__1__Impl
	rule__AssignmentStatement__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__AssignmentStatement__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAssignmentStatementAccess().getColonEqualsSignKeyword_1()); }
	':='
	{ after(grammarAccess.getAssignmentStatementAccess().getColonEqualsSignKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AssignmentStatement__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AssignmentStatement__Group__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AssignmentStatement__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAssignmentStatementAccess().getRhsAssignment_2()); }
	(rule__AssignmentStatement__RhsAssignment_2)
	{ after(grammarAccess.getAssignmentStatementAccess().getRhsAssignment_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AssertionStatement__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AssertionStatement__Group__0__Impl
	rule__AssertionStatement__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AssertionStatement__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAssertionStatementAccess().getAssertKeyword_0()); }
	'assert'
	{ after(grammarAccess.getAssertionStatementAccess().getAssertKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AssertionStatement__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AssertionStatement__Group__1__Impl
	rule__AssertionStatement__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__AssertionStatement__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAssertionStatementAccess().getLeftParenthesisKeyword_1()); }
	'('
	{ after(grammarAccess.getAssertionStatementAccess().getLeftParenthesisKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AssertionStatement__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AssertionStatement__Group__2__Impl
	rule__AssertionStatement__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__AssertionStatement__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAssertionStatementAccess().getAssertionAssignment_2()); }
	(rule__AssertionStatement__AssertionAssignment_2)
	{ after(grammarAccess.getAssertionStatementAccess().getAssertionAssignment_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AssertionStatement__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AssertionStatement__Group__3__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AssertionStatement__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAssertionStatementAccess().getRightParenthesisKeyword_3()); }
	')'
	{ after(grammarAccess.getAssertionStatementAccess().getRightParenthesisKeyword_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AssignableAccessExpression__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AssignableAccessExpression__Group__0__Impl
	rule__AssignableAccessExpression__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AssignableAccessExpression__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAssignableAccessExpressionAccess().getAssignablePrimaryExpressionParserRuleCall_0()); }
	ruleAssignablePrimaryExpression
	{ after(grammarAccess.getAssignableAccessExpressionAccess().getAssignablePrimaryExpressionParserRuleCall_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AssignableAccessExpression__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AssignableAccessExpression__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AssignableAccessExpression__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAssignableAccessExpressionAccess().getAlternatives_1()); }
	(rule__AssignableAccessExpression__Alternatives_1)*
	{ after(grammarAccess.getAssignableAccessExpressionAccess().getAlternatives_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AssignableAccessExpression__Group_1_0__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AssignableAccessExpression__Group_1_0__0__Impl
	rule__AssignableAccessExpression__Group_1_0__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AssignableAccessExpression__Group_1_0__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAssignableAccessExpressionAccess().getArrayAccessExpressionOperandAction_1_0_0()); }
	()
	{ after(grammarAccess.getAssignableAccessExpressionAccess().getArrayAccessExpressionOperandAction_1_0_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AssignableAccessExpression__Group_1_0__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AssignableAccessExpression__Group_1_0__1__Impl
	rule__AssignableAccessExpression__Group_1_0__2
;
finally {
	restoreStackSize(stackSize);
}

rule__AssignableAccessExpression__Group_1_0__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAssignableAccessExpressionAccess().getLeftSquareBracketKeyword_1_0_1()); }
	'['
	{ after(grammarAccess.getAssignableAccessExpressionAccess().getLeftSquareBracketKeyword_1_0_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AssignableAccessExpression__Group_1_0__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AssignableAccessExpression__Group_1_0__2__Impl
	rule__AssignableAccessExpression__Group_1_0__3
;
finally {
	restoreStackSize(stackSize);
}

rule__AssignableAccessExpression__Group_1_0__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAssignableAccessExpressionAccess().getIndexAssignment_1_0_2()); }
	(rule__AssignableAccessExpression__IndexAssignment_1_0_2)
	{ after(grammarAccess.getAssignableAccessExpressionAccess().getIndexAssignment_1_0_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AssignableAccessExpression__Group_1_0__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AssignableAccessExpression__Group_1_0__3__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AssignableAccessExpression__Group_1_0__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAssignableAccessExpressionAccess().getRightSquareBracketKeyword_1_0_3()); }
	']'
	{ after(grammarAccess.getAssignableAccessExpressionAccess().getRightSquareBracketKeyword_1_0_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AssignableAccessExpression__Group_1_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AssignableAccessExpression__Group_1_1__0__Impl
	rule__AssignableAccessExpression__Group_1_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AssignableAccessExpression__Group_1_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAssignableAccessExpressionAccess().getRecordAccessExpressionOperandAction_1_1_0()); }
	()
	{ after(grammarAccess.getAssignableAccessExpressionAccess().getRecordAccessExpressionOperandAction_1_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AssignableAccessExpression__Group_1_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AssignableAccessExpression__Group_1_1__1__Impl
	rule__AssignableAccessExpression__Group_1_1__2
;
finally {
	restoreStackSize(stackSize);
}

rule__AssignableAccessExpression__Group_1_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAssignableAccessExpressionAccess().getFullStopKeyword_1_1_1()); }
	'.'
	{ after(grammarAccess.getAssignableAccessExpressionAccess().getFullStopKeyword_1_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AssignableAccessExpression__Group_1_1__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AssignableAccessExpression__Group_1_1__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AssignableAccessExpression__Group_1_1__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAssignableAccessExpressionAccess().getFieldReferenceAssignment_1_1_2()); }
	(rule__AssignableAccessExpression__FieldReferenceAssignment_1_1_2)
	{ after(grammarAccess.getAssignableAccessExpressionAccess().getFieldReferenceAssignment_1_1_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AssignablePrimaryExpression__Group_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AssignablePrimaryExpression__Group_1__0__Impl
	rule__AssignablePrimaryExpression__Group_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AssignablePrimaryExpression__Group_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAssignablePrimaryExpressionAccess().getLeftParenthesisKeyword_1_0()); }
	'('
	{ after(grammarAccess.getAssignablePrimaryExpressionAccess().getLeftParenthesisKeyword_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AssignablePrimaryExpression__Group_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AssignablePrimaryExpression__Group_1__1__Impl
	rule__AssignablePrimaryExpression__Group_1__2
;
finally {
	restoreStackSize(stackSize);
}

rule__AssignablePrimaryExpression__Group_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAssignablePrimaryExpressionAccess().getAssignableAccessExpressionParserRuleCall_1_1()); }
	ruleAssignableAccessExpression
	{ after(grammarAccess.getAssignablePrimaryExpressionAccess().getAssignableAccessExpressionParserRuleCall_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AssignablePrimaryExpression__Group_1__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AssignablePrimaryExpression__Group_1__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AssignablePrimaryExpression__Group_1__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAssignablePrimaryExpressionAccess().getRightParenthesisKeyword_1_2()); }
	')'
	{ after(grammarAccess.getAssignablePrimaryExpressionAccess().getRightParenthesisKeyword_1_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AssignableDirectReferenceExpression__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AssignableDirectReferenceExpression__Group__0__Impl
	rule__AssignableDirectReferenceExpression__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AssignableDirectReferenceExpression__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAssignableDirectReferenceExpressionAccess().getDirectReferenceExpressionAction_0()); }
	()
	{ after(grammarAccess.getAssignableDirectReferenceExpressionAccess().getDirectReferenceExpressionAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AssignableDirectReferenceExpression__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AssignableDirectReferenceExpression__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AssignableDirectReferenceExpression__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAssignableDirectReferenceExpressionAccess().getDeclarationAssignment_1()); }
	(rule__AssignableDirectReferenceExpression__DeclarationAssignment_1)
	{ after(grammarAccess.getAssignableDirectReferenceExpressionAccess().getDeclarationAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ParameterDeclaration__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ParameterDeclaration__Group__0__Impl
	rule__ParameterDeclaration__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ParameterDeclaration__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getParameterDeclarationAccess().getParameterDeclarationAction_0()); }
	()
	{ after(grammarAccess.getParameterDeclarationAccess().getParameterDeclarationAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ParameterDeclaration__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ParameterDeclaration__Group__1__Impl
	rule__ParameterDeclaration__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ParameterDeclaration__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getParameterDeclarationAccess().getNameAssignment_1()); }
	(rule__ParameterDeclaration__NameAssignment_1)
	{ after(grammarAccess.getParameterDeclarationAccess().getNameAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ParameterDeclaration__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ParameterDeclaration__Group__2__Impl
	rule__ParameterDeclaration__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__ParameterDeclaration__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getParameterDeclarationAccess().getColonKeyword_2()); }
	':'
	{ after(grammarAccess.getParameterDeclarationAccess().getColonKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ParameterDeclaration__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ParameterDeclaration__Group__3__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ParameterDeclaration__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getParameterDeclarationAccess().getTypeAssignment_3()); }
	(rule__ParameterDeclaration__TypeAssignment_3)
	{ after(grammarAccess.getParameterDeclarationAccess().getTypeAssignment_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__FieldDeclaration__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FieldDeclaration__Group__0__Impl
	rule__FieldDeclaration__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__FieldDeclaration__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFieldDeclarationAccess().getFieldDeclarationAction_0()); }
	()
	{ after(grammarAccess.getFieldDeclarationAccess().getFieldDeclarationAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FieldDeclaration__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FieldDeclaration__Group__1__Impl
	rule__FieldDeclaration__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__FieldDeclaration__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFieldDeclarationAccess().getNameAssignment_1()); }
	(rule__FieldDeclaration__NameAssignment_1)
	{ after(grammarAccess.getFieldDeclarationAccess().getNameAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FieldDeclaration__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FieldDeclaration__Group__2__Impl
	rule__FieldDeclaration__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__FieldDeclaration__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFieldDeclarationAccess().getColonKeyword_2()); }
	':'
	{ after(grammarAccess.getFieldDeclarationAccess().getColonKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FieldDeclaration__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FieldDeclaration__Group__3__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__FieldDeclaration__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFieldDeclarationAccess().getTypeAssignment_3()); }
	(rule__FieldDeclaration__TypeAssignment_3)
	{ after(grammarAccess.getFieldDeclarationAccess().getTypeAssignment_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ConstantDeclaration__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ConstantDeclaration__Group__0__Impl
	rule__ConstantDeclaration__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ConstantDeclaration__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getConstantDeclarationAccess().getConstKeyword_0()); }
	'const'
	{ after(grammarAccess.getConstantDeclarationAccess().getConstKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ConstantDeclaration__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ConstantDeclaration__Group__1__Impl
	rule__ConstantDeclaration__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ConstantDeclaration__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getConstantDeclarationAccess().getNameAssignment_1()); }
	(rule__ConstantDeclaration__NameAssignment_1)
	{ after(grammarAccess.getConstantDeclarationAccess().getNameAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ConstantDeclaration__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ConstantDeclaration__Group__2__Impl
	rule__ConstantDeclaration__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__ConstantDeclaration__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getConstantDeclarationAccess().getColonKeyword_2()); }
	':'
	{ after(grammarAccess.getConstantDeclarationAccess().getColonKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ConstantDeclaration__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ConstantDeclaration__Group__3__Impl
	rule__ConstantDeclaration__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__ConstantDeclaration__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getConstantDeclarationAccess().getTypeAssignment_3()); }
	(rule__ConstantDeclaration__TypeAssignment_3)
	{ after(grammarAccess.getConstantDeclarationAccess().getTypeAssignment_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ConstantDeclaration__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ConstantDeclaration__Group__4__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ConstantDeclaration__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getConstantDeclarationAccess().getGroup_4()); }
	(rule__ConstantDeclaration__Group_4__0)?
	{ after(grammarAccess.getConstantDeclarationAccess().getGroup_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ConstantDeclaration__Group_4__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ConstantDeclaration__Group_4__0__Impl
	rule__ConstantDeclaration__Group_4__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ConstantDeclaration__Group_4__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getConstantDeclarationAccess().getColonEqualsSignKeyword_4_0()); }
	':='
	{ after(grammarAccess.getConstantDeclarationAccess().getColonEqualsSignKeyword_4_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ConstantDeclaration__Group_4__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ConstantDeclaration__Group_4__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ConstantDeclaration__Group_4__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getConstantDeclarationAccess().getExpressionAssignment_4_1()); }
	(rule__ConstantDeclaration__ExpressionAssignment_4_1)
	{ after(grammarAccess.getConstantDeclarationAccess().getExpressionAssignment_4_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__VariableDeclaration__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__VariableDeclaration__Group__0__Impl
	rule__VariableDeclaration__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__VariableDeclaration__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getVariableDeclarationAccess().getAnnotationsAssignment_0()); }
	(rule__VariableDeclaration__AnnotationsAssignment_0)*
	{ after(grammarAccess.getVariableDeclarationAccess().getAnnotationsAssignment_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__VariableDeclaration__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__VariableDeclaration__Group__1__Impl
	rule__VariableDeclaration__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__VariableDeclaration__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getVariableDeclarationAccess().getVarKeyword_1()); }
	'var'
	{ after(grammarAccess.getVariableDeclarationAccess().getVarKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__VariableDeclaration__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__VariableDeclaration__Group__2__Impl
	rule__VariableDeclaration__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__VariableDeclaration__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getVariableDeclarationAccess().getNameAssignment_2()); }
	(rule__VariableDeclaration__NameAssignment_2)
	{ after(grammarAccess.getVariableDeclarationAccess().getNameAssignment_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__VariableDeclaration__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__VariableDeclaration__Group__3__Impl
	rule__VariableDeclaration__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__VariableDeclaration__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getVariableDeclarationAccess().getColonKeyword_3()); }
	':'
	{ after(grammarAccess.getVariableDeclarationAccess().getColonKeyword_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__VariableDeclaration__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__VariableDeclaration__Group__4__Impl
	rule__VariableDeclaration__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__VariableDeclaration__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getVariableDeclarationAccess().getTypeAssignment_4()); }
	(rule__VariableDeclaration__TypeAssignment_4)
	{ after(grammarAccess.getVariableDeclarationAccess().getTypeAssignment_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__VariableDeclaration__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__VariableDeclaration__Group__5__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__VariableDeclaration__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getVariableDeclarationAccess().getGroup_5()); }
	(rule__VariableDeclaration__Group_5__0)?
	{ after(grammarAccess.getVariableDeclarationAccess().getGroup_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__VariableDeclaration__Group_5__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__VariableDeclaration__Group_5__0__Impl
	rule__VariableDeclaration__Group_5__1
;
finally {
	restoreStackSize(stackSize);
}

rule__VariableDeclaration__Group_5__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getVariableDeclarationAccess().getColonEqualsSignKeyword_5_0()); }
	':='
	{ after(grammarAccess.getVariableDeclarationAccess().getColonEqualsSignKeyword_5_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__VariableDeclaration__Group_5__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__VariableDeclaration__Group_5__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__VariableDeclaration__Group_5__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getVariableDeclarationAccess().getExpressionAssignment_5_1()); }
	(rule__VariableDeclaration__ExpressionAssignment_5_1)
	{ after(grammarAccess.getVariableDeclarationAccess().getExpressionAssignment_5_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__TransientVariableDeclarationAnnotation__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__TransientVariableDeclarationAnnotation__Group__0__Impl
	rule__TransientVariableDeclarationAnnotation__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__TransientVariableDeclarationAnnotation__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getTransientVariableDeclarationAnnotationAccess().getTransientVariableDeclarationAnnotationAction_0()); }
	()
	{ after(grammarAccess.getTransientVariableDeclarationAnnotationAccess().getTransientVariableDeclarationAnnotationAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__TransientVariableDeclarationAnnotation__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__TransientVariableDeclarationAnnotation__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__TransientVariableDeclarationAnnotation__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getTransientVariableDeclarationAnnotationAccess().getTransientKeyword_1()); }
	'@Transient'
	{ after(grammarAccess.getTransientVariableDeclarationAnnotationAccess().getTransientKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ResettableVariableDeclarationAnnotation__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ResettableVariableDeclarationAnnotation__Group__0__Impl
	rule__ResettableVariableDeclarationAnnotation__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ResettableVariableDeclarationAnnotation__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getResettableVariableDeclarationAnnotationAccess().getResettableVariableDeclarationAnnotationAction_0()); }
	()
	{ after(grammarAccess.getResettableVariableDeclarationAnnotationAccess().getResettableVariableDeclarationAnnotationAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ResettableVariableDeclarationAnnotation__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ResettableVariableDeclarationAnnotation__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ResettableVariableDeclarationAnnotation__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getResettableVariableDeclarationAnnotationAccess().getResettableKeyword_1()); }
	'@Resettable'
	{ after(grammarAccess.getResettableVariableDeclarationAnnotationAccess().getResettableKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__UnremovableVariableDeclarationAnnotation__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__UnremovableVariableDeclarationAnnotation__Group__0__Impl
	rule__UnremovableVariableDeclarationAnnotation__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__UnremovableVariableDeclarationAnnotation__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getUnremovableVariableDeclarationAnnotationAccess().getUnremovableVariableDeclarationAnnotationAction_0()); }
	()
	{ after(grammarAccess.getUnremovableVariableDeclarationAnnotationAccess().getUnremovableVariableDeclarationAnnotationAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__UnremovableVariableDeclarationAnnotation__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__UnremovableVariableDeclarationAnnotation__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__UnremovableVariableDeclarationAnnotation__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getUnremovableVariableDeclarationAnnotationAccess().getUnremovableKeyword_1()); }
	'@Unremovable'
	{ after(grammarAccess.getUnremovableVariableDeclarationAnnotationAccess().getUnremovableKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__InjectedVariableDeclarationAnnotation__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__InjectedVariableDeclarationAnnotation__Group__0__Impl
	rule__InjectedVariableDeclarationAnnotation__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__InjectedVariableDeclarationAnnotation__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInjectedVariableDeclarationAnnotationAccess().getInjectedVariableDeclarationAnnotationAction_0()); }
	()
	{ after(grammarAccess.getInjectedVariableDeclarationAnnotationAccess().getInjectedVariableDeclarationAnnotationAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__InjectedVariableDeclarationAnnotation__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__InjectedVariableDeclarationAnnotation__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__InjectedVariableDeclarationAnnotation__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInjectedVariableDeclarationAnnotationAccess().getInjectedKeyword_1()); }
	'@Injected'
	{ after(grammarAccess.getInjectedVariableDeclarationAnnotationAccess().getInjectedKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__LambdaDeclaration__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__LambdaDeclaration__Group__0__Impl
	rule__LambdaDeclaration__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__LambdaDeclaration__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getLambdaDeclarationAccess().getLambdaKeyword_0()); }
	'lambda'
	{ after(grammarAccess.getLambdaDeclarationAccess().getLambdaKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__LambdaDeclaration__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__LambdaDeclaration__Group__1__Impl
	rule__LambdaDeclaration__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__LambdaDeclaration__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getLambdaDeclarationAccess().getNameAssignment_1()); }
	(rule__LambdaDeclaration__NameAssignment_1)
	{ after(grammarAccess.getLambdaDeclarationAccess().getNameAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__LambdaDeclaration__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__LambdaDeclaration__Group__2__Impl
	rule__LambdaDeclaration__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__LambdaDeclaration__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getLambdaDeclarationAccess().getGroup_2()); }
	(rule__LambdaDeclaration__Group_2__0)?
	{ after(grammarAccess.getLambdaDeclarationAccess().getGroup_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__LambdaDeclaration__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__LambdaDeclaration__Group__3__Impl
	rule__LambdaDeclaration__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__LambdaDeclaration__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getLambdaDeclarationAccess().getColonKeyword_3()); }
	':'
	{ after(grammarAccess.getLambdaDeclarationAccess().getColonKeyword_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__LambdaDeclaration__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__LambdaDeclaration__Group__4__Impl
	rule__LambdaDeclaration__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__LambdaDeclaration__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getLambdaDeclarationAccess().getTypeAssignment_4()); }
	(rule__LambdaDeclaration__TypeAssignment_4)
	{ after(grammarAccess.getLambdaDeclarationAccess().getTypeAssignment_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__LambdaDeclaration__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__LambdaDeclaration__Group__5__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__LambdaDeclaration__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getLambdaDeclarationAccess().getGroup_5()); }
	(rule__LambdaDeclaration__Group_5__0)?
	{ after(grammarAccess.getLambdaDeclarationAccess().getGroup_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__LambdaDeclaration__Group_2__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__LambdaDeclaration__Group_2__0__Impl
	rule__LambdaDeclaration__Group_2__1
;
finally {
	restoreStackSize(stackSize);
}

rule__LambdaDeclaration__Group_2__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getLambdaDeclarationAccess().getLeftParenthesisKeyword_2_0()); }
	'('
	{ after(grammarAccess.getLambdaDeclarationAccess().getLeftParenthesisKeyword_2_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__LambdaDeclaration__Group_2__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__LambdaDeclaration__Group_2__1__Impl
	rule__LambdaDeclaration__Group_2__2
;
finally {
	restoreStackSize(stackSize);
}

rule__LambdaDeclaration__Group_2__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getLambdaDeclarationAccess().getGroup_2_1()); }
	(rule__LambdaDeclaration__Group_2_1__0)?
	{ after(grammarAccess.getLambdaDeclarationAccess().getGroup_2_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__LambdaDeclaration__Group_2__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__LambdaDeclaration__Group_2__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__LambdaDeclaration__Group_2__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getLambdaDeclarationAccess().getRightParenthesisKeyword_2_2()); }
	')'
	{ after(grammarAccess.getLambdaDeclarationAccess().getRightParenthesisKeyword_2_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__LambdaDeclaration__Group_2_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__LambdaDeclaration__Group_2_1__0__Impl
	rule__LambdaDeclaration__Group_2_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__LambdaDeclaration__Group_2_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getLambdaDeclarationAccess().getParameterDeclarationsAssignment_2_1_0()); }
	(rule__LambdaDeclaration__ParameterDeclarationsAssignment_2_1_0)
	{ after(grammarAccess.getLambdaDeclarationAccess().getParameterDeclarationsAssignment_2_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__LambdaDeclaration__Group_2_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__LambdaDeclaration__Group_2_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__LambdaDeclaration__Group_2_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getLambdaDeclarationAccess().getGroup_2_1_1()); }
	(rule__LambdaDeclaration__Group_2_1_1__0)*
	{ after(grammarAccess.getLambdaDeclarationAccess().getGroup_2_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__LambdaDeclaration__Group_2_1_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__LambdaDeclaration__Group_2_1_1__0__Impl
	rule__LambdaDeclaration__Group_2_1_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__LambdaDeclaration__Group_2_1_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getLambdaDeclarationAccess().getCommaKeyword_2_1_1_0()); }
	','
	{ after(grammarAccess.getLambdaDeclarationAccess().getCommaKeyword_2_1_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__LambdaDeclaration__Group_2_1_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__LambdaDeclaration__Group_2_1_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__LambdaDeclaration__Group_2_1_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getLambdaDeclarationAccess().getParameterDeclarationsAssignment_2_1_1_1()); }
	(rule__LambdaDeclaration__ParameterDeclarationsAssignment_2_1_1_1)
	{ after(grammarAccess.getLambdaDeclarationAccess().getParameterDeclarationsAssignment_2_1_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__LambdaDeclaration__Group_5__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__LambdaDeclaration__Group_5__0__Impl
	rule__LambdaDeclaration__Group_5__1
;
finally {
	restoreStackSize(stackSize);
}

rule__LambdaDeclaration__Group_5__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getLambdaDeclarationAccess().getColonEqualsSignKeyword_5_0()); }
	':='
	{ after(grammarAccess.getLambdaDeclarationAccess().getColonEqualsSignKeyword_5_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__LambdaDeclaration__Group_5__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__LambdaDeclaration__Group_5__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__LambdaDeclaration__Group_5__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getLambdaDeclarationAccess().getExpressionAssignment_5_1()); }
	(rule__LambdaDeclaration__ExpressionAssignment_5_1)
	{ after(grammarAccess.getLambdaDeclarationAccess().getExpressionAssignment_5_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__TypeDeclaration__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__TypeDeclaration__Group__0__Impl
	rule__TypeDeclaration__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__TypeDeclaration__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getTypeDeclarationAccess().getTypeKeyword_0()); }
	'type'
	{ after(grammarAccess.getTypeDeclarationAccess().getTypeKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__TypeDeclaration__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__TypeDeclaration__Group__1__Impl
	rule__TypeDeclaration__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__TypeDeclaration__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getTypeDeclarationAccess().getNameAssignment_1()); }
	(rule__TypeDeclaration__NameAssignment_1)
	{ after(grammarAccess.getTypeDeclarationAccess().getNameAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__TypeDeclaration__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__TypeDeclaration__Group__2__Impl
	rule__TypeDeclaration__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__TypeDeclaration__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getTypeDeclarationAccess().getColonKeyword_2()); }
	':'
	{ after(grammarAccess.getTypeDeclarationAccess().getColonKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__TypeDeclaration__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__TypeDeclaration__Group__3__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__TypeDeclaration__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getTypeDeclarationAccess().getTypeAssignment_3()); }
	(rule__TypeDeclaration__TypeAssignment_3)
	{ after(grammarAccess.getTypeDeclarationAccess().getTypeAssignment_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__VoidTypeDefinition__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__VoidTypeDefinition__Group__0__Impl
	rule__VoidTypeDefinition__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__VoidTypeDefinition__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getVoidTypeDefinitionAccess().getVoidTypeDefinitionAction_0()); }
	()
	{ after(grammarAccess.getVoidTypeDefinitionAccess().getVoidTypeDefinitionAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__VoidTypeDefinition__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__VoidTypeDefinition__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__VoidTypeDefinition__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getVoidTypeDefinitionAccess().getVoidKeyword_1()); }
	'void'
	{ after(grammarAccess.getVoidTypeDefinitionAccess().getVoidKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__IntegerTypeDefinition__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__IntegerTypeDefinition__Group__0__Impl
	rule__IntegerTypeDefinition__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__IntegerTypeDefinition__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getIntegerTypeDefinitionAccess().getIntegerTypeDefinitionAction_0()); }
	()
	{ after(grammarAccess.getIntegerTypeDefinitionAccess().getIntegerTypeDefinitionAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__IntegerTypeDefinition__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__IntegerTypeDefinition__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__IntegerTypeDefinition__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getIntegerTypeDefinitionAccess().getIntegerKeyword_1()); }
	'integer'
	{ after(grammarAccess.getIntegerTypeDefinitionAccess().getIntegerKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__BooleanTypeDefinition__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__BooleanTypeDefinition__Group__0__Impl
	rule__BooleanTypeDefinition__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__BooleanTypeDefinition__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getBooleanTypeDefinitionAccess().getBooleanTypeDefinitionAction_0()); }
	()
	{ after(grammarAccess.getBooleanTypeDefinitionAccess().getBooleanTypeDefinitionAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__BooleanTypeDefinition__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__BooleanTypeDefinition__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__BooleanTypeDefinition__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getBooleanTypeDefinitionAccess().getBooleanKeyword_1()); }
	'boolean'
	{ after(grammarAccess.getBooleanTypeDefinitionAccess().getBooleanKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__DecimalTypeDefinition__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DecimalTypeDefinition__Group__0__Impl
	rule__DecimalTypeDefinition__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__DecimalTypeDefinition__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDecimalTypeDefinitionAccess().getDecimalTypeDefinitionAction_0()); }
	()
	{ after(grammarAccess.getDecimalTypeDefinitionAccess().getDecimalTypeDefinitionAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__DecimalTypeDefinition__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__DecimalTypeDefinition__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__DecimalTypeDefinition__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getDecimalTypeDefinitionAccess().getDecimalKeyword_1()); }
	'decimal'
	{ after(grammarAccess.getDecimalTypeDefinitionAccess().getDecimalKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__RationalTypeDefinition__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RationalTypeDefinition__Group__0__Impl
	rule__RationalTypeDefinition__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__RationalTypeDefinition__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRationalTypeDefinitionAccess().getRationalTypeDefinitionAction_0()); }
	()
	{ after(grammarAccess.getRationalTypeDefinitionAccess().getRationalTypeDefinitionAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RationalTypeDefinition__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RationalTypeDefinition__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__RationalTypeDefinition__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRationalTypeDefinitionAccess().getRationalKeyword_1()); }
	'rational'
	{ after(grammarAccess.getRationalTypeDefinitionAccess().getRationalKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ArrayTypeDefinition__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ArrayTypeDefinition__Group__0__Impl
	rule__ArrayTypeDefinition__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ArrayTypeDefinition__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getArrayTypeDefinitionAccess().getLeftSquareBracketKeyword_0()); }
	'['
	{ after(grammarAccess.getArrayTypeDefinitionAccess().getLeftSquareBracketKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ArrayTypeDefinition__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ArrayTypeDefinition__Group__1__Impl
	rule__ArrayTypeDefinition__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ArrayTypeDefinition__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getArrayTypeDefinitionAccess().getSizeAssignment_1()); }
	(rule__ArrayTypeDefinition__SizeAssignment_1)
	{ after(grammarAccess.getArrayTypeDefinitionAccess().getSizeAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ArrayTypeDefinition__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ArrayTypeDefinition__Group__2__Impl
	rule__ArrayTypeDefinition__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__ArrayTypeDefinition__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getArrayTypeDefinitionAccess().getRightSquareBracketKeyword_2()); }
	']'
	{ after(grammarAccess.getArrayTypeDefinitionAccess().getRightSquareBracketKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ArrayTypeDefinition__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ArrayTypeDefinition__Group__3__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ArrayTypeDefinition__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getArrayTypeDefinitionAccess().getElementTypeAssignment_3()); }
	(rule__ArrayTypeDefinition__ElementTypeAssignment_3)
	{ after(grammarAccess.getArrayTypeDefinitionAccess().getElementTypeAssignment_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__IntegerRangeTypeDefinition__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__IntegerRangeTypeDefinition__Group__0__Impl
	rule__IntegerRangeTypeDefinition__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__IntegerRangeTypeDefinition__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getIntegerRangeTypeDefinitionAccess().getIntegerRangeTypeDefinitionAction_0()); }
	()
	{ after(grammarAccess.getIntegerRangeTypeDefinitionAccess().getIntegerRangeTypeDefinitionAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__IntegerRangeTypeDefinition__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__IntegerRangeTypeDefinition__Group__1__Impl
	rule__IntegerRangeTypeDefinition__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__IntegerRangeTypeDefinition__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getIntegerRangeTypeDefinitionAccess().getIntegerKeyword_1()); }
	'integer'
	{ after(grammarAccess.getIntegerRangeTypeDefinitionAccess().getIntegerKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__IntegerRangeTypeDefinition__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__IntegerRangeTypeDefinition__Group__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__IntegerRangeTypeDefinition__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getIntegerRangeTypeDefinitionAccess().getRangeKeyword_2()); }
	'range'
	{ after(grammarAccess.getIntegerRangeTypeDefinitionAccess().getRangeKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__EnumerationTypeDefinition__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__EnumerationTypeDefinition__Group__0__Impl
	rule__EnumerationTypeDefinition__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__EnumerationTypeDefinition__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEnumerationTypeDefinitionAccess().getEnumKeyword_0()); }
	'enum'
	{ after(grammarAccess.getEnumerationTypeDefinitionAccess().getEnumKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__EnumerationTypeDefinition__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__EnumerationTypeDefinition__Group__1__Impl
	rule__EnumerationTypeDefinition__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__EnumerationTypeDefinition__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEnumerationTypeDefinitionAccess().getLeftCurlyBracketKeyword_1()); }
	'{'
	{ after(grammarAccess.getEnumerationTypeDefinitionAccess().getLeftCurlyBracketKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__EnumerationTypeDefinition__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__EnumerationTypeDefinition__Group__2__Impl
	rule__EnumerationTypeDefinition__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__EnumerationTypeDefinition__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEnumerationTypeDefinitionAccess().getLiteralsAssignment_2()); }
	(rule__EnumerationTypeDefinition__LiteralsAssignment_2)
	{ after(grammarAccess.getEnumerationTypeDefinitionAccess().getLiteralsAssignment_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__EnumerationTypeDefinition__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__EnumerationTypeDefinition__Group__3__Impl
	rule__EnumerationTypeDefinition__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__EnumerationTypeDefinition__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEnumerationTypeDefinitionAccess().getGroup_3()); }
	(rule__EnumerationTypeDefinition__Group_3__0)*
	{ after(grammarAccess.getEnumerationTypeDefinitionAccess().getGroup_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__EnumerationTypeDefinition__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__EnumerationTypeDefinition__Group__4__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__EnumerationTypeDefinition__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEnumerationTypeDefinitionAccess().getRightCurlyBracketKeyword_4()); }
	'}'
	{ after(grammarAccess.getEnumerationTypeDefinitionAccess().getRightCurlyBracketKeyword_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__EnumerationTypeDefinition__Group_3__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__EnumerationTypeDefinition__Group_3__0__Impl
	rule__EnumerationTypeDefinition__Group_3__1
;
finally {
	restoreStackSize(stackSize);
}

rule__EnumerationTypeDefinition__Group_3__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEnumerationTypeDefinitionAccess().getCommaKeyword_3_0()); }
	','
	{ after(grammarAccess.getEnumerationTypeDefinitionAccess().getCommaKeyword_3_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__EnumerationTypeDefinition__Group_3__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__EnumerationTypeDefinition__Group_3__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__EnumerationTypeDefinition__Group_3__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEnumerationTypeDefinitionAccess().getLiteralsAssignment_3_1()); }
	(rule__EnumerationTypeDefinition__LiteralsAssignment_3_1)
	{ after(grammarAccess.getEnumerationTypeDefinitionAccess().getLiteralsAssignment_3_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__RecordTypeDefinition__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RecordTypeDefinition__Group__0__Impl
	rule__RecordTypeDefinition__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__RecordTypeDefinition__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRecordTypeDefinitionAccess().getRecordKeyword_0()); }
	'record'
	{ after(grammarAccess.getRecordTypeDefinitionAccess().getRecordKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RecordTypeDefinition__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RecordTypeDefinition__Group__1__Impl
	rule__RecordTypeDefinition__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__RecordTypeDefinition__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRecordTypeDefinitionAccess().getLeftCurlyBracketKeyword_1()); }
	'{'
	{ after(grammarAccess.getRecordTypeDefinitionAccess().getLeftCurlyBracketKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RecordTypeDefinition__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RecordTypeDefinition__Group__2__Impl
	rule__RecordTypeDefinition__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__RecordTypeDefinition__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRecordTypeDefinitionAccess().getFieldDeclarationsAssignment_2()); }
	(rule__RecordTypeDefinition__FieldDeclarationsAssignment_2)
	{ after(grammarAccess.getRecordTypeDefinitionAccess().getFieldDeclarationsAssignment_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RecordTypeDefinition__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RecordTypeDefinition__Group__3__Impl
	rule__RecordTypeDefinition__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__RecordTypeDefinition__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRecordTypeDefinitionAccess().getGroup_3()); }
	(rule__RecordTypeDefinition__Group_3__0)*
	{ after(grammarAccess.getRecordTypeDefinitionAccess().getGroup_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RecordTypeDefinition__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RecordTypeDefinition__Group__4__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__RecordTypeDefinition__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRecordTypeDefinitionAccess().getRightCurlyBracketKeyword_4()); }
	'}'
	{ after(grammarAccess.getRecordTypeDefinitionAccess().getRightCurlyBracketKeyword_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__RecordTypeDefinition__Group_3__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RecordTypeDefinition__Group_3__0__Impl
	rule__RecordTypeDefinition__Group_3__1
;
finally {
	restoreStackSize(stackSize);
}

rule__RecordTypeDefinition__Group_3__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRecordTypeDefinitionAccess().getCommaKeyword_3_0()); }
	','
	{ after(grammarAccess.getRecordTypeDefinitionAccess().getCommaKeyword_3_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RecordTypeDefinition__Group_3__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RecordTypeDefinition__Group_3__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__RecordTypeDefinition__Group_3__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRecordTypeDefinitionAccess().getFieldDeclarationsAssignment_3_1()); }
	(rule__RecordTypeDefinition__FieldDeclarationsAssignment_3_1)
	{ after(grammarAccess.getRecordTypeDefinitionAccess().getFieldDeclarationsAssignment_3_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__SubrangeTypeDefinition__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SubrangeTypeDefinition__Group__0__Impl
	rule__SubrangeTypeDefinition__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__SubrangeTypeDefinition__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSubrangeTypeDefinitionAccess().getSubrangeTypeDefinitionAction_0()); }
	()
	{ after(grammarAccess.getSubrangeTypeDefinitionAccess().getSubrangeTypeDefinitionAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SubrangeTypeDefinition__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SubrangeTypeDefinition__Group__1__Impl
	rule__SubrangeTypeDefinition__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__SubrangeTypeDefinition__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSubrangeTypeDefinitionAccess().getNumberSignLeftSquareBracketKeyword_1()); }
	'#['
	{ after(grammarAccess.getSubrangeTypeDefinitionAccess().getNumberSignLeftSquareBracketKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SubrangeTypeDefinition__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SubrangeTypeDefinition__Group__2__Impl
	rule__SubrangeTypeDefinition__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__SubrangeTypeDefinition__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSubrangeTypeDefinitionAccess().getAlternatives_2()); }
	(rule__SubrangeTypeDefinition__Alternatives_2)
	{ after(grammarAccess.getSubrangeTypeDefinitionAccess().getAlternatives_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SubrangeTypeDefinition__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SubrangeTypeDefinition__Group__3__Impl
	rule__SubrangeTypeDefinition__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__SubrangeTypeDefinition__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSubrangeTypeDefinitionAccess().getToKeyword_3()); }
	'to'
	{ after(grammarAccess.getSubrangeTypeDefinitionAccess().getToKeyword_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SubrangeTypeDefinition__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SubrangeTypeDefinition__Group__4__Impl
	rule__SubrangeTypeDefinition__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__SubrangeTypeDefinition__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSubrangeTypeDefinitionAccess().getAlternatives_4()); }
	(rule__SubrangeTypeDefinition__Alternatives_4)
	{ after(grammarAccess.getSubrangeTypeDefinitionAccess().getAlternatives_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SubrangeTypeDefinition__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SubrangeTypeDefinition__Group__5__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__SubrangeTypeDefinition__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSubrangeTypeDefinitionAccess().getRightSquareBracketKeyword_5()); }
	']'
	{ after(grammarAccess.getSubrangeTypeDefinitionAccess().getRightSquareBracketKeyword_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__SubrangeTypeDefinition__Group_2_0__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SubrangeTypeDefinition__Group_2_0__0__Impl
	rule__SubrangeTypeDefinition__Group_2_0__1
;
finally {
	restoreStackSize(stackSize);
}

rule__SubrangeTypeDefinition__Group_2_0__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSubrangeTypeDefinitionAccess().getHyphenMinusKeyword_2_0_0()); }
	'-'
	{ after(grammarAccess.getSubrangeTypeDefinitionAccess().getHyphenMinusKeyword_2_0_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SubrangeTypeDefinition__Group_2_0__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SubrangeTypeDefinition__Group_2_0__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__SubrangeTypeDefinition__Group_2_0__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSubrangeTypeDefinitionAccess().getInfKeyword_2_0_1()); }
	'inf'
	{ after(grammarAccess.getSubrangeTypeDefinitionAccess().getInfKeyword_2_0_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__FunctionTypeDefinition__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FunctionTypeDefinition__Group__0__Impl
	rule__FunctionTypeDefinition__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__FunctionTypeDefinition__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFunctionTypeDefinitionAccess().getFunctionKeyword_0()); }
	'function'
	{ after(grammarAccess.getFunctionTypeDefinitionAccess().getFunctionKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FunctionTypeDefinition__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FunctionTypeDefinition__Group__1__Impl
	rule__FunctionTypeDefinition__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__FunctionTypeDefinition__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFunctionTypeDefinitionAccess().getLeftParenthesisKeyword_1()); }
	'('
	{ after(grammarAccess.getFunctionTypeDefinitionAccess().getLeftParenthesisKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FunctionTypeDefinition__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FunctionTypeDefinition__Group__2__Impl
	rule__FunctionTypeDefinition__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__FunctionTypeDefinition__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFunctionTypeDefinitionAccess().getGroup_2()); }
	(rule__FunctionTypeDefinition__Group_2__0)?
	{ after(grammarAccess.getFunctionTypeDefinitionAccess().getGroup_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FunctionTypeDefinition__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FunctionTypeDefinition__Group__3__Impl
	rule__FunctionTypeDefinition__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__FunctionTypeDefinition__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFunctionTypeDefinitionAccess().getRightParenthesisKeyword_3()); }
	')'
	{ after(grammarAccess.getFunctionTypeDefinitionAccess().getRightParenthesisKeyword_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FunctionTypeDefinition__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FunctionTypeDefinition__Group__4__Impl
	rule__FunctionTypeDefinition__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__FunctionTypeDefinition__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFunctionTypeDefinitionAccess().getColonKeyword_4()); }
	':'
	{ after(grammarAccess.getFunctionTypeDefinitionAccess().getColonKeyword_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FunctionTypeDefinition__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FunctionTypeDefinition__Group__5__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__FunctionTypeDefinition__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFunctionTypeDefinitionAccess().getReturnTypeAssignment_5()); }
	(rule__FunctionTypeDefinition__ReturnTypeAssignment_5)
	{ after(grammarAccess.getFunctionTypeDefinitionAccess().getReturnTypeAssignment_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__FunctionTypeDefinition__Group_2__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FunctionTypeDefinition__Group_2__0__Impl
	rule__FunctionTypeDefinition__Group_2__1
;
finally {
	restoreStackSize(stackSize);
}

rule__FunctionTypeDefinition__Group_2__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFunctionTypeDefinitionAccess().getParameterTypesAssignment_2_0()); }
	(rule__FunctionTypeDefinition__ParameterTypesAssignment_2_0)
	{ after(grammarAccess.getFunctionTypeDefinitionAccess().getParameterTypesAssignment_2_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FunctionTypeDefinition__Group_2__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FunctionTypeDefinition__Group_2__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__FunctionTypeDefinition__Group_2__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFunctionTypeDefinitionAccess().getGroup_2_1()); }
	(rule__FunctionTypeDefinition__Group_2_1__0)*
	{ after(grammarAccess.getFunctionTypeDefinitionAccess().getGroup_2_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__FunctionTypeDefinition__Group_2_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FunctionTypeDefinition__Group_2_1__0__Impl
	rule__FunctionTypeDefinition__Group_2_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__FunctionTypeDefinition__Group_2_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFunctionTypeDefinitionAccess().getCommaKeyword_2_1_0()); }
	','
	{ after(grammarAccess.getFunctionTypeDefinitionAccess().getCommaKeyword_2_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FunctionTypeDefinition__Group_2_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FunctionTypeDefinition__Group_2_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__FunctionTypeDefinition__Group_2_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFunctionTypeDefinitionAccess().getParameterTypesAssignment_2_1_1()); }
	(rule__FunctionTypeDefinition__ParameterTypesAssignment_2_1_1)
	{ after(grammarAccess.getFunctionTypeDefinitionAccess().getParameterTypesAssignment_2_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ElseExpression__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ElseExpression__Group__0__Impl
	rule__ElseExpression__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ElseExpression__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getElseExpressionAccess().getElseExpressionAction_0()); }
	()
	{ after(grammarAccess.getElseExpressionAccess().getElseExpressionAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ElseExpression__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ElseExpression__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ElseExpression__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getElseExpressionAccess().getElseKeyword_1()); }
	'else'
	{ after(grammarAccess.getElseExpressionAccess().getElseKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__IfThenElseExpression__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__IfThenElseExpression__Group__0__Impl
	rule__IfThenElseExpression__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__IfThenElseExpression__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getIfThenElseExpressionAccess().getImplyExpressionParserRuleCall_0()); }
	ruleImplyExpression
	{ after(grammarAccess.getIfThenElseExpressionAccess().getImplyExpressionParserRuleCall_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__IfThenElseExpression__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__IfThenElseExpression__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__IfThenElseExpression__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getIfThenElseExpressionAccess().getGroup_1()); }
	(rule__IfThenElseExpression__Group_1__0)?
	{ after(grammarAccess.getIfThenElseExpressionAccess().getGroup_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__IfThenElseExpression__Group_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__IfThenElseExpression__Group_1__0__Impl
	rule__IfThenElseExpression__Group_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__IfThenElseExpression__Group_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getIfThenElseExpressionAccess().getIfThenElseExpressionConditionAction_1_0()); }
	()
	{ after(grammarAccess.getIfThenElseExpressionAccess().getIfThenElseExpressionConditionAction_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__IfThenElseExpression__Group_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__IfThenElseExpression__Group_1__1__Impl
	rule__IfThenElseExpression__Group_1__2
;
finally {
	restoreStackSize(stackSize);
}

rule__IfThenElseExpression__Group_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getIfThenElseExpressionAccess().getQuestionMarkKeyword_1_1()); }
	'?'
	{ after(grammarAccess.getIfThenElseExpressionAccess().getQuestionMarkKeyword_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__IfThenElseExpression__Group_1__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__IfThenElseExpression__Group_1__2__Impl
	rule__IfThenElseExpression__Group_1__3
;
finally {
	restoreStackSize(stackSize);
}

rule__IfThenElseExpression__Group_1__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getIfThenElseExpressionAccess().getThenAssignment_1_2()); }
	(rule__IfThenElseExpression__ThenAssignment_1_2)
	{ after(grammarAccess.getIfThenElseExpressionAccess().getThenAssignment_1_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__IfThenElseExpression__Group_1__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__IfThenElseExpression__Group_1__3__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__IfThenElseExpression__Group_1__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getIfThenElseExpressionAccess().getGroup_1_3()); }
	(rule__IfThenElseExpression__Group_1_3__0)
	{ after(grammarAccess.getIfThenElseExpressionAccess().getGroup_1_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__IfThenElseExpression__Group_1_3__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__IfThenElseExpression__Group_1_3__0__Impl
	rule__IfThenElseExpression__Group_1_3__1
;
finally {
	restoreStackSize(stackSize);
}

rule__IfThenElseExpression__Group_1_3__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getIfThenElseExpressionAccess().getColonKeyword_1_3_0()); }
	(':')
	{ after(grammarAccess.getIfThenElseExpressionAccess().getColonKeyword_1_3_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__IfThenElseExpression__Group_1_3__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__IfThenElseExpression__Group_1_3__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__IfThenElseExpression__Group_1_3__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getIfThenElseExpressionAccess().getElseAssignment_1_3_1()); }
	(rule__IfThenElseExpression__ElseAssignment_1_3_1)
	{ after(grammarAccess.getIfThenElseExpressionAccess().getElseAssignment_1_3_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ImplyExpression__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ImplyExpression__Group__0__Impl
	rule__ImplyExpression__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ImplyExpression__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getImplyExpressionAccess().getOrExpressionParserRuleCall_0()); }
	ruleOrExpression
	{ after(grammarAccess.getImplyExpressionAccess().getOrExpressionParserRuleCall_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ImplyExpression__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ImplyExpression__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ImplyExpression__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getImplyExpressionAccess().getGroup_1()); }
	(rule__ImplyExpression__Group_1__0)?
	{ after(grammarAccess.getImplyExpressionAccess().getGroup_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ImplyExpression__Group_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ImplyExpression__Group_1__0__Impl
	rule__ImplyExpression__Group_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ImplyExpression__Group_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getImplyExpressionAccess().getImplyExpressionLeftOperandAction_1_0()); }
	()
	{ after(grammarAccess.getImplyExpressionAccess().getImplyExpressionLeftOperandAction_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ImplyExpression__Group_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ImplyExpression__Group_1__1__Impl
	rule__ImplyExpression__Group_1__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ImplyExpression__Group_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getImplyExpressionAccess().getImplyKeyword_1_1()); }
	'imply'
	{ after(grammarAccess.getImplyExpressionAccess().getImplyKeyword_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ImplyExpression__Group_1__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ImplyExpression__Group_1__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ImplyExpression__Group_1__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getImplyExpressionAccess().getRightOperandAssignment_1_2()); }
	(rule__ImplyExpression__RightOperandAssignment_1_2)
	{ after(grammarAccess.getImplyExpressionAccess().getRightOperandAssignment_1_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__OrExpression__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__OrExpression__Group__0__Impl
	rule__OrExpression__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__OrExpression__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getOrExpressionAccess().getXorExpressionParserRuleCall_0()); }
	ruleXorExpression
	{ after(grammarAccess.getOrExpressionAccess().getXorExpressionParserRuleCall_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__OrExpression__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__OrExpression__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__OrExpression__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getOrExpressionAccess().getGroup_1()); }
	(rule__OrExpression__Group_1__0)?
	{ after(grammarAccess.getOrExpressionAccess().getGroup_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__OrExpression__Group_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__OrExpression__Group_1__0__Impl
	rule__OrExpression__Group_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__OrExpression__Group_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getOrExpressionAccess().getOrExpressionOperandsAction_1_0()); }
	()
	{ after(grammarAccess.getOrExpressionAccess().getOrExpressionOperandsAction_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__OrExpression__Group_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__OrExpression__Group_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__OrExpression__Group_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	(
		{ before(grammarAccess.getOrExpressionAccess().getGroup_1_1()); }
		(rule__OrExpression__Group_1_1__0)
		{ after(grammarAccess.getOrExpressionAccess().getGroup_1_1()); }
	)
	(
		{ before(grammarAccess.getOrExpressionAccess().getGroup_1_1()); }
		(rule__OrExpression__Group_1_1__0)*
		{ after(grammarAccess.getOrExpressionAccess().getGroup_1_1()); }
	)
)
;
finally {
	restoreStackSize(stackSize);
}


rule__OrExpression__Group_1_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__OrExpression__Group_1_1__0__Impl
	rule__OrExpression__Group_1_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__OrExpression__Group_1_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getOrExpressionAccess().getOrKeyword_1_1_0()); }
	'or'
	{ after(grammarAccess.getOrExpressionAccess().getOrKeyword_1_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__OrExpression__Group_1_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__OrExpression__Group_1_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__OrExpression__Group_1_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getOrExpressionAccess().getOperandsAssignment_1_1_1()); }
	(rule__OrExpression__OperandsAssignment_1_1_1)
	{ after(grammarAccess.getOrExpressionAccess().getOperandsAssignment_1_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__XorExpression__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__XorExpression__Group__0__Impl
	rule__XorExpression__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__XorExpression__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getXorExpressionAccess().getAndExpressionParserRuleCall_0()); }
	ruleAndExpression
	{ after(grammarAccess.getXorExpressionAccess().getAndExpressionParserRuleCall_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__XorExpression__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__XorExpression__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__XorExpression__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getXorExpressionAccess().getGroup_1()); }
	(rule__XorExpression__Group_1__0)?
	{ after(grammarAccess.getXorExpressionAccess().getGroup_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__XorExpression__Group_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__XorExpression__Group_1__0__Impl
	rule__XorExpression__Group_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__XorExpression__Group_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getXorExpressionAccess().getXorExpressionOperandsAction_1_0()); }
	()
	{ after(grammarAccess.getXorExpressionAccess().getXorExpressionOperandsAction_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__XorExpression__Group_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__XorExpression__Group_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__XorExpression__Group_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	(
		{ before(grammarAccess.getXorExpressionAccess().getGroup_1_1()); }
		(rule__XorExpression__Group_1_1__0)
		{ after(grammarAccess.getXorExpressionAccess().getGroup_1_1()); }
	)
	(
		{ before(grammarAccess.getXorExpressionAccess().getGroup_1_1()); }
		(rule__XorExpression__Group_1_1__0)*
		{ after(grammarAccess.getXorExpressionAccess().getGroup_1_1()); }
	)
)
;
finally {
	restoreStackSize(stackSize);
}


rule__XorExpression__Group_1_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__XorExpression__Group_1_1__0__Impl
	rule__XorExpression__Group_1_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__XorExpression__Group_1_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getXorExpressionAccess().getXorKeyword_1_1_0()); }
	'xor'
	{ after(grammarAccess.getXorExpressionAccess().getXorKeyword_1_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__XorExpression__Group_1_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__XorExpression__Group_1_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__XorExpression__Group_1_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getXorExpressionAccess().getOperandsAssignment_1_1_1()); }
	(rule__XorExpression__OperandsAssignment_1_1_1)
	{ after(grammarAccess.getXorExpressionAccess().getOperandsAssignment_1_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AndExpression__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AndExpression__Group__0__Impl
	rule__AndExpression__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AndExpression__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAndExpressionAccess().getUnaryLogicExpressionParserRuleCall_0()); }
	ruleUnaryLogicExpression
	{ after(grammarAccess.getAndExpressionAccess().getUnaryLogicExpressionParserRuleCall_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AndExpression__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AndExpression__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AndExpression__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAndExpressionAccess().getGroup_1()); }
	(rule__AndExpression__Group_1__0)?
	{ after(grammarAccess.getAndExpressionAccess().getGroup_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AndExpression__Group_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AndExpression__Group_1__0__Impl
	rule__AndExpression__Group_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AndExpression__Group_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAndExpressionAccess().getAndExpressionOperandsAction_1_0()); }
	()
	{ after(grammarAccess.getAndExpressionAccess().getAndExpressionOperandsAction_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AndExpression__Group_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AndExpression__Group_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AndExpression__Group_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	(
		{ before(grammarAccess.getAndExpressionAccess().getGroup_1_1()); }
		(rule__AndExpression__Group_1_1__0)
		{ after(grammarAccess.getAndExpressionAccess().getGroup_1_1()); }
	)
	(
		{ before(grammarAccess.getAndExpressionAccess().getGroup_1_1()); }
		(rule__AndExpression__Group_1_1__0)*
		{ after(grammarAccess.getAndExpressionAccess().getGroup_1_1()); }
	)
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AndExpression__Group_1_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AndExpression__Group_1_1__0__Impl
	rule__AndExpression__Group_1_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AndExpression__Group_1_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAndExpressionAccess().getAndKeyword_1_1_0()); }
	'and'
	{ after(grammarAccess.getAndExpressionAccess().getAndKeyword_1_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AndExpression__Group_1_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AndExpression__Group_1_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AndExpression__Group_1_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAndExpressionAccess().getOperandsAssignment_1_1_1()); }
	(rule__AndExpression__OperandsAssignment_1_1_1)
	{ after(grammarAccess.getAndExpressionAccess().getOperandsAssignment_1_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__NotExpression__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NotExpression__Group__0__Impl
	rule__NotExpression__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__NotExpression__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNotExpressionAccess().getNotExpressionAction_0()); }
	()
	{ after(grammarAccess.getNotExpressionAccess().getNotExpressionAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NotExpression__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NotExpression__Group__1__Impl
	rule__NotExpression__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__NotExpression__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNotExpressionAccess().getGroup_1()); }
	(rule__NotExpression__Group_1__0)
	{ after(grammarAccess.getNotExpressionAccess().getGroup_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__NotExpression__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NotExpression__Group__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__NotExpression__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNotExpressionAccess().getOperandAssignment_2()); }
	(rule__NotExpression__OperandAssignment_2)
	{ after(grammarAccess.getNotExpressionAccess().getOperandAssignment_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__NotExpression__Group_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__NotExpression__Group_1__0__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__NotExpression__Group_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getNotExpressionAccess().getAlternatives_1_0()); }
	(rule__NotExpression__Alternatives_1_0)
	{ after(grammarAccess.getNotExpressionAccess().getAlternatives_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ForallExpression__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ForallExpression__Group__0__Impl
	rule__ForallExpression__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ForallExpression__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getForallExpressionAccess().getForallExpressionAction_0()); }
	()
	{ after(grammarAccess.getForallExpressionAccess().getForallExpressionAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ForallExpression__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ForallExpression__Group__1__Impl
	rule__ForallExpression__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ForallExpression__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getForallExpressionAccess().getForallKeyword_1()); }
	('forall')
	{ after(grammarAccess.getForallExpressionAccess().getForallKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ForallExpression__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ForallExpression__Group__2__Impl
	rule__ForallExpression__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__ForallExpression__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getForallExpressionAccess().getLeftParenthesisKeyword_2()); }
	'('
	{ after(grammarAccess.getForallExpressionAccess().getLeftParenthesisKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ForallExpression__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ForallExpression__Group__3__Impl
	rule__ForallExpression__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__ForallExpression__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getForallExpressionAccess().getGroup_3()); }
	(rule__ForallExpression__Group_3__0)?
	{ after(grammarAccess.getForallExpressionAccess().getGroup_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ForallExpression__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ForallExpression__Group__4__Impl
	rule__ForallExpression__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__ForallExpression__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getForallExpressionAccess().getRightParenthesisKeyword_4()); }
	')'
	{ after(grammarAccess.getForallExpressionAccess().getRightParenthesisKeyword_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ForallExpression__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ForallExpression__Group__5__Impl
	rule__ForallExpression__Group__6
;
finally {
	restoreStackSize(stackSize);
}

rule__ForallExpression__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getForallExpressionAccess().getColonKeyword_5()); }
	':'
	{ after(grammarAccess.getForallExpressionAccess().getColonKeyword_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ForallExpression__Group__6
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ForallExpression__Group__6__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ForallExpression__Group__6__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getForallExpressionAccess().getOperandAssignment_6()); }
	(rule__ForallExpression__OperandAssignment_6)
	{ after(grammarAccess.getForallExpressionAccess().getOperandAssignment_6()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ForallExpression__Group_3__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ForallExpression__Group_3__0__Impl
	rule__ForallExpression__Group_3__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ForallExpression__Group_3__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getForallExpressionAccess().getParameterDeclarationsAssignment_3_0()); }
	(rule__ForallExpression__ParameterDeclarationsAssignment_3_0)
	{ after(grammarAccess.getForallExpressionAccess().getParameterDeclarationsAssignment_3_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ForallExpression__Group_3__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ForallExpression__Group_3__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ForallExpression__Group_3__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getForallExpressionAccess().getGroup_3_1()); }
	(rule__ForallExpression__Group_3_1__0)*
	{ after(grammarAccess.getForallExpressionAccess().getGroup_3_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ForallExpression__Group_3_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ForallExpression__Group_3_1__0__Impl
	rule__ForallExpression__Group_3_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ForallExpression__Group_3_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getForallExpressionAccess().getCommaKeyword_3_1_0()); }
	','
	{ after(grammarAccess.getForallExpressionAccess().getCommaKeyword_3_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ForallExpression__Group_3_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ForallExpression__Group_3_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ForallExpression__Group_3_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getForallExpressionAccess().getParameterDeclarationsAssignment_3_1_1()); }
	(rule__ForallExpression__ParameterDeclarationsAssignment_3_1_1)
	{ after(grammarAccess.getForallExpressionAccess().getParameterDeclarationsAssignment_3_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ExistsExpression__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ExistsExpression__Group__0__Impl
	rule__ExistsExpression__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ExistsExpression__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getExistsExpressionAccess().getExistsExpressionAction_0()); }
	()
	{ after(grammarAccess.getExistsExpressionAccess().getExistsExpressionAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ExistsExpression__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ExistsExpression__Group__1__Impl
	rule__ExistsExpression__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ExistsExpression__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getExistsExpressionAccess().getExistsKeyword_1()); }
	('exists')
	{ after(grammarAccess.getExistsExpressionAccess().getExistsKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ExistsExpression__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ExistsExpression__Group__2__Impl
	rule__ExistsExpression__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__ExistsExpression__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getExistsExpressionAccess().getLeftParenthesisKeyword_2()); }
	'('
	{ after(grammarAccess.getExistsExpressionAccess().getLeftParenthesisKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ExistsExpression__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ExistsExpression__Group__3__Impl
	rule__ExistsExpression__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__ExistsExpression__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getExistsExpressionAccess().getGroup_3()); }
	(rule__ExistsExpression__Group_3__0)?
	{ after(grammarAccess.getExistsExpressionAccess().getGroup_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ExistsExpression__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ExistsExpression__Group__4__Impl
	rule__ExistsExpression__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__ExistsExpression__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getExistsExpressionAccess().getRightParenthesisKeyword_4()); }
	')'
	{ after(grammarAccess.getExistsExpressionAccess().getRightParenthesisKeyword_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ExistsExpression__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ExistsExpression__Group__5__Impl
	rule__ExistsExpression__Group__6
;
finally {
	restoreStackSize(stackSize);
}

rule__ExistsExpression__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getExistsExpressionAccess().getColonKeyword_5()); }
	':'
	{ after(grammarAccess.getExistsExpressionAccess().getColonKeyword_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ExistsExpression__Group__6
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ExistsExpression__Group__6__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ExistsExpression__Group__6__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getExistsExpressionAccess().getOperandAssignment_6()); }
	(rule__ExistsExpression__OperandAssignment_6)
	{ after(grammarAccess.getExistsExpressionAccess().getOperandAssignment_6()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ExistsExpression__Group_3__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ExistsExpression__Group_3__0__Impl
	rule__ExistsExpression__Group_3__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ExistsExpression__Group_3__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getExistsExpressionAccess().getParameterDeclarationsAssignment_3_0()); }
	(rule__ExistsExpression__ParameterDeclarationsAssignment_3_0)
	{ after(grammarAccess.getExistsExpressionAccess().getParameterDeclarationsAssignment_3_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ExistsExpression__Group_3__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ExistsExpression__Group_3__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ExistsExpression__Group_3__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getExistsExpressionAccess().getGroup_3_1()); }
	(rule__ExistsExpression__Group_3_1__0)*
	{ after(grammarAccess.getExistsExpressionAccess().getGroup_3_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ExistsExpression__Group_3_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ExistsExpression__Group_3_1__0__Impl
	rule__ExistsExpression__Group_3_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ExistsExpression__Group_3_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getExistsExpressionAccess().getCommaKeyword_3_1_0()); }
	','
	{ after(grammarAccess.getExistsExpressionAccess().getCommaKeyword_3_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ExistsExpression__Group_3_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ExistsExpression__Group_3_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ExistsExpression__Group_3_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getExistsExpressionAccess().getParameterDeclarationsAssignment_3_1_1()); }
	(rule__ExistsExpression__ParameterDeclarationsAssignment_3_1_1)
	{ after(grammarAccess.getExistsExpressionAccess().getParameterDeclarationsAssignment_3_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__EquivalenceExpression__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__EquivalenceExpression__Group__0__Impl
	rule__EquivalenceExpression__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__EquivalenceExpression__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEquivalenceExpressionAccess().getComparisionExpressionParserRuleCall_0()); }
	ruleComparisionExpression
	{ after(grammarAccess.getEquivalenceExpressionAccess().getComparisionExpressionParserRuleCall_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__EquivalenceExpression__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__EquivalenceExpression__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__EquivalenceExpression__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEquivalenceExpressionAccess().getAlternatives_1()); }
	(rule__EquivalenceExpression__Alternatives_1)?
	{ after(grammarAccess.getEquivalenceExpressionAccess().getAlternatives_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__EquivalenceExpression__Group_1_0__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__EquivalenceExpression__Group_1_0__0__Impl
	rule__EquivalenceExpression__Group_1_0__1
;
finally {
	restoreStackSize(stackSize);
}

rule__EquivalenceExpression__Group_1_0__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEquivalenceExpressionAccess().getEqualityExpressionLeftOperandAction_1_0_0()); }
	()
	{ after(grammarAccess.getEquivalenceExpressionAccess().getEqualityExpressionLeftOperandAction_1_0_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__EquivalenceExpression__Group_1_0__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__EquivalenceExpression__Group_1_0__1__Impl
	rule__EquivalenceExpression__Group_1_0__2
;
finally {
	restoreStackSize(stackSize);
}

rule__EquivalenceExpression__Group_1_0__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEquivalenceExpressionAccess().getAlternatives_1_0_1()); }
	(rule__EquivalenceExpression__Alternatives_1_0_1)
	{ after(grammarAccess.getEquivalenceExpressionAccess().getAlternatives_1_0_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__EquivalenceExpression__Group_1_0__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__EquivalenceExpression__Group_1_0__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__EquivalenceExpression__Group_1_0__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEquivalenceExpressionAccess().getRightOperandAssignment_1_0_2()); }
	(rule__EquivalenceExpression__RightOperandAssignment_1_0_2)
	{ after(grammarAccess.getEquivalenceExpressionAccess().getRightOperandAssignment_1_0_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__EquivalenceExpression__Group_1_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__EquivalenceExpression__Group_1_1__0__Impl
	rule__EquivalenceExpression__Group_1_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__EquivalenceExpression__Group_1_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEquivalenceExpressionAccess().getInequalityExpressionLeftOperandAction_1_1_0()); }
	()
	{ after(grammarAccess.getEquivalenceExpressionAccess().getInequalityExpressionLeftOperandAction_1_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__EquivalenceExpression__Group_1_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__EquivalenceExpression__Group_1_1__1__Impl
	rule__EquivalenceExpression__Group_1_1__2
;
finally {
	restoreStackSize(stackSize);
}

rule__EquivalenceExpression__Group_1_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEquivalenceExpressionAccess().getAlternatives_1_1_1()); }
	(rule__EquivalenceExpression__Alternatives_1_1_1)
	{ after(grammarAccess.getEquivalenceExpressionAccess().getAlternatives_1_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__EquivalenceExpression__Group_1_1__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__EquivalenceExpression__Group_1_1__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__EquivalenceExpression__Group_1_1__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEquivalenceExpressionAccess().getRightOperandAssignment_1_1_2()); }
	(rule__EquivalenceExpression__RightOperandAssignment_1_1_2)
	{ after(grammarAccess.getEquivalenceExpressionAccess().getRightOperandAssignment_1_1_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ComparisionExpression__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ComparisionExpression__Group__0__Impl
	rule__ComparisionExpression__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ComparisionExpression__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getComparisionExpressionAccess().getAdditiveExpressionParserRuleCall_0()); }
	ruleAdditiveExpression
	{ after(grammarAccess.getComparisionExpressionAccess().getAdditiveExpressionParserRuleCall_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComparisionExpression__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ComparisionExpression__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ComparisionExpression__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getComparisionExpressionAccess().getGroup_1()); }
	(rule__ComparisionExpression__Group_1__0)*
	{ after(grammarAccess.getComparisionExpressionAccess().getGroup_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ComparisionExpression__Group_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ComparisionExpression__Group_1__0__Impl
	rule__ComparisionExpression__Group_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ComparisionExpression__Group_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getComparisionExpressionAccess().getAlternatives_1_0()); }
	(rule__ComparisionExpression__Alternatives_1_0)
	{ after(grammarAccess.getComparisionExpressionAccess().getAlternatives_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComparisionExpression__Group_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ComparisionExpression__Group_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ComparisionExpression__Group_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getComparisionExpressionAccess().getRightOperandAssignment_1_1()); }
	(rule__ComparisionExpression__RightOperandAssignment_1_1)
	{ after(grammarAccess.getComparisionExpressionAccess().getRightOperandAssignment_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ComparisionExpression__Group_1_0_0__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ComparisionExpression__Group_1_0_0__0__Impl
	rule__ComparisionExpression__Group_1_0_0__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ComparisionExpression__Group_1_0_0__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getComparisionExpressionAccess().getGreaterExpressionLeftOperandAction_1_0_0_0()); }
	()
	{ after(grammarAccess.getComparisionExpressionAccess().getGreaterExpressionLeftOperandAction_1_0_0_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComparisionExpression__Group_1_0_0__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ComparisionExpression__Group_1_0_0__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ComparisionExpression__Group_1_0_0__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getComparisionExpressionAccess().getGreaterThanSignKeyword_1_0_0_1()); }
	'>'
	{ after(grammarAccess.getComparisionExpressionAccess().getGreaterThanSignKeyword_1_0_0_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ComparisionExpression__Group_1_0_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ComparisionExpression__Group_1_0_1__0__Impl
	rule__ComparisionExpression__Group_1_0_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ComparisionExpression__Group_1_0_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getComparisionExpressionAccess().getGreaterEqualExpressionLeftOperandAction_1_0_1_0()); }
	()
	{ after(grammarAccess.getComparisionExpressionAccess().getGreaterEqualExpressionLeftOperandAction_1_0_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComparisionExpression__Group_1_0_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ComparisionExpression__Group_1_0_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ComparisionExpression__Group_1_0_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getComparisionExpressionAccess().getGreaterThanSignEqualsSignKeyword_1_0_1_1()); }
	'>='
	{ after(grammarAccess.getComparisionExpressionAccess().getGreaterThanSignEqualsSignKeyword_1_0_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ComparisionExpression__Group_1_0_2__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ComparisionExpression__Group_1_0_2__0__Impl
	rule__ComparisionExpression__Group_1_0_2__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ComparisionExpression__Group_1_0_2__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getComparisionExpressionAccess().getLessExpressionLeftOperandAction_1_0_2_0()); }
	()
	{ after(grammarAccess.getComparisionExpressionAccess().getLessExpressionLeftOperandAction_1_0_2_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComparisionExpression__Group_1_0_2__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ComparisionExpression__Group_1_0_2__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ComparisionExpression__Group_1_0_2__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getComparisionExpressionAccess().getLessThanSignKeyword_1_0_2_1()); }
	'<'
	{ after(grammarAccess.getComparisionExpressionAccess().getLessThanSignKeyword_1_0_2_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ComparisionExpression__Group_1_0_3__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ComparisionExpression__Group_1_0_3__0__Impl
	rule__ComparisionExpression__Group_1_0_3__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ComparisionExpression__Group_1_0_3__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getComparisionExpressionAccess().getLessEqualExpressionLeftOperandAction_1_0_3_0()); }
	()
	{ after(grammarAccess.getComparisionExpressionAccess().getLessEqualExpressionLeftOperandAction_1_0_3_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComparisionExpression__Group_1_0_3__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ComparisionExpression__Group_1_0_3__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ComparisionExpression__Group_1_0_3__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getComparisionExpressionAccess().getLessThanSignEqualsSignKeyword_1_0_3_1()); }
	'<='
	{ after(grammarAccess.getComparisionExpressionAccess().getLessThanSignEqualsSignKeyword_1_0_3_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AdditiveExpression__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AdditiveExpression__Group__0__Impl
	rule__AdditiveExpression__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AdditiveExpression__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAdditiveExpressionAccess().getMultiplicativeExpressionParserRuleCall_0()); }
	ruleMultiplicativeExpression
	{ after(grammarAccess.getAdditiveExpressionAccess().getMultiplicativeExpressionParserRuleCall_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AdditiveExpression__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AdditiveExpression__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AdditiveExpression__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAdditiveExpressionAccess().getAlternatives_1()); }
	(rule__AdditiveExpression__Alternatives_1)*
	{ after(grammarAccess.getAdditiveExpressionAccess().getAlternatives_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AdditiveExpression__Group_1_0__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AdditiveExpression__Group_1_0__0__Impl
	rule__AdditiveExpression__Group_1_0__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AdditiveExpression__Group_1_0__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAdditiveExpressionAccess().getAddExpressionOperandsAction_1_0_0()); }
	()
	{ after(grammarAccess.getAdditiveExpressionAccess().getAddExpressionOperandsAction_1_0_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AdditiveExpression__Group_1_0__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AdditiveExpression__Group_1_0__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AdditiveExpression__Group_1_0__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	(
		{ before(grammarAccess.getAdditiveExpressionAccess().getGroup_1_0_1()); }
		(rule__AdditiveExpression__Group_1_0_1__0)
		{ after(grammarAccess.getAdditiveExpressionAccess().getGroup_1_0_1()); }
	)
	(
		{ before(grammarAccess.getAdditiveExpressionAccess().getGroup_1_0_1()); }
		(rule__AdditiveExpression__Group_1_0_1__0)*
		{ after(grammarAccess.getAdditiveExpressionAccess().getGroup_1_0_1()); }
	)
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AdditiveExpression__Group_1_0_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AdditiveExpression__Group_1_0_1__0__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AdditiveExpression__Group_1_0_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAdditiveExpressionAccess().getGroup_1_0_1_0()); }
	(rule__AdditiveExpression__Group_1_0_1_0__0)
	{ after(grammarAccess.getAdditiveExpressionAccess().getGroup_1_0_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AdditiveExpression__Group_1_0_1_0__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AdditiveExpression__Group_1_0_1_0__0__Impl
	rule__AdditiveExpression__Group_1_0_1_0__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AdditiveExpression__Group_1_0_1_0__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAdditiveExpressionAccess().getPlusSignKeyword_1_0_1_0_0()); }
	'+'
	{ after(grammarAccess.getAdditiveExpressionAccess().getPlusSignKeyword_1_0_1_0_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AdditiveExpression__Group_1_0_1_0__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AdditiveExpression__Group_1_0_1_0__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AdditiveExpression__Group_1_0_1_0__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAdditiveExpressionAccess().getOperandsAssignment_1_0_1_0_1()); }
	(rule__AdditiveExpression__OperandsAssignment_1_0_1_0_1)
	{ after(grammarAccess.getAdditiveExpressionAccess().getOperandsAssignment_1_0_1_0_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AdditiveExpression__Group_1_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AdditiveExpression__Group_1_1__0__Impl
	rule__AdditiveExpression__Group_1_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AdditiveExpression__Group_1_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAdditiveExpressionAccess().getSubtractExpressionLeftOperandAction_1_1_0()); }
	()
	{ after(grammarAccess.getAdditiveExpressionAccess().getSubtractExpressionLeftOperandAction_1_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AdditiveExpression__Group_1_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AdditiveExpression__Group_1_1__1__Impl
	rule__AdditiveExpression__Group_1_1__2
;
finally {
	restoreStackSize(stackSize);
}

rule__AdditiveExpression__Group_1_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAdditiveExpressionAccess().getHyphenMinusKeyword_1_1_1()); }
	'-'
	{ after(grammarAccess.getAdditiveExpressionAccess().getHyphenMinusKeyword_1_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AdditiveExpression__Group_1_1__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AdditiveExpression__Group_1_1__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AdditiveExpression__Group_1_1__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAdditiveExpressionAccess().getRightOperandAssignment_1_1_2()); }
	(rule__AdditiveExpression__RightOperandAssignment_1_1_2)
	{ after(grammarAccess.getAdditiveExpressionAccess().getRightOperandAssignment_1_1_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__MultiplicativeExpression__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MultiplicativeExpression__Group__0__Impl
	rule__MultiplicativeExpression__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__MultiplicativeExpression__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMultiplicativeExpressionAccess().getSignumExpressionParserRuleCall_0()); }
	ruleSignumExpression
	{ after(grammarAccess.getMultiplicativeExpressionAccess().getSignumExpressionParserRuleCall_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__MultiplicativeExpression__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MultiplicativeExpression__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__MultiplicativeExpression__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMultiplicativeExpressionAccess().getAlternatives_1()); }
	(rule__MultiplicativeExpression__Alternatives_1)*
	{ after(grammarAccess.getMultiplicativeExpressionAccess().getAlternatives_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__MultiplicativeExpression__Group_1_0__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MultiplicativeExpression__Group_1_0__0__Impl
	rule__MultiplicativeExpression__Group_1_0__1
;
finally {
	restoreStackSize(stackSize);
}

rule__MultiplicativeExpression__Group_1_0__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMultiplicativeExpressionAccess().getMultiplyExpressionOperandsAction_1_0_0()); }
	()
	{ after(grammarAccess.getMultiplicativeExpressionAccess().getMultiplyExpressionOperandsAction_1_0_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__MultiplicativeExpression__Group_1_0__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MultiplicativeExpression__Group_1_0__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__MultiplicativeExpression__Group_1_0__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	(
		{ before(grammarAccess.getMultiplicativeExpressionAccess().getGroup_1_0_1()); }
		(rule__MultiplicativeExpression__Group_1_0_1__0)
		{ after(grammarAccess.getMultiplicativeExpressionAccess().getGroup_1_0_1()); }
	)
	(
		{ before(grammarAccess.getMultiplicativeExpressionAccess().getGroup_1_0_1()); }
		(rule__MultiplicativeExpression__Group_1_0_1__0)*
		{ after(grammarAccess.getMultiplicativeExpressionAccess().getGroup_1_0_1()); }
	)
)
;
finally {
	restoreStackSize(stackSize);
}


rule__MultiplicativeExpression__Group_1_0_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MultiplicativeExpression__Group_1_0_1__0__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__MultiplicativeExpression__Group_1_0_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMultiplicativeExpressionAccess().getGroup_1_0_1_0()); }
	(rule__MultiplicativeExpression__Group_1_0_1_0__0)
	{ after(grammarAccess.getMultiplicativeExpressionAccess().getGroup_1_0_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__MultiplicativeExpression__Group_1_0_1_0__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MultiplicativeExpression__Group_1_0_1_0__0__Impl
	rule__MultiplicativeExpression__Group_1_0_1_0__1
;
finally {
	restoreStackSize(stackSize);
}

rule__MultiplicativeExpression__Group_1_0_1_0__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMultiplicativeExpressionAccess().getAsteriskKeyword_1_0_1_0_0()); }
	'*'
	{ after(grammarAccess.getMultiplicativeExpressionAccess().getAsteriskKeyword_1_0_1_0_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__MultiplicativeExpression__Group_1_0_1_0__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MultiplicativeExpression__Group_1_0_1_0__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__MultiplicativeExpression__Group_1_0_1_0__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMultiplicativeExpressionAccess().getOperandsAssignment_1_0_1_0_1()); }
	(rule__MultiplicativeExpression__OperandsAssignment_1_0_1_0_1)
	{ after(grammarAccess.getMultiplicativeExpressionAccess().getOperandsAssignment_1_0_1_0_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__MultiplicativeExpression__Group_1_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MultiplicativeExpression__Group_1_1__0__Impl
	rule__MultiplicativeExpression__Group_1_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__MultiplicativeExpression__Group_1_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMultiplicativeExpressionAccess().getAlternatives_1_1_0()); }
	(rule__MultiplicativeExpression__Alternatives_1_1_0)
	{ after(grammarAccess.getMultiplicativeExpressionAccess().getAlternatives_1_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__MultiplicativeExpression__Group_1_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MultiplicativeExpression__Group_1_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__MultiplicativeExpression__Group_1_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMultiplicativeExpressionAccess().getRightOperandAssignment_1_1_1()); }
	(rule__MultiplicativeExpression__RightOperandAssignment_1_1_1)
	{ after(grammarAccess.getMultiplicativeExpressionAccess().getRightOperandAssignment_1_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__MultiplicativeExpression__Group_1_1_0_0__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MultiplicativeExpression__Group_1_1_0_0__0__Impl
	rule__MultiplicativeExpression__Group_1_1_0_0__1
;
finally {
	restoreStackSize(stackSize);
}

rule__MultiplicativeExpression__Group_1_1_0_0__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMultiplicativeExpressionAccess().getDivideExpressionLeftOperandAction_1_1_0_0_0()); }
	()
	{ after(grammarAccess.getMultiplicativeExpressionAccess().getDivideExpressionLeftOperandAction_1_1_0_0_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__MultiplicativeExpression__Group_1_1_0_0__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MultiplicativeExpression__Group_1_1_0_0__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__MultiplicativeExpression__Group_1_1_0_0__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMultiplicativeExpressionAccess().getSolidusKeyword_1_1_0_0_1()); }
	'/'
	{ after(grammarAccess.getMultiplicativeExpressionAccess().getSolidusKeyword_1_1_0_0_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__MultiplicativeExpression__Group_1_1_0_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MultiplicativeExpression__Group_1_1_0_1__0__Impl
	rule__MultiplicativeExpression__Group_1_1_0_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__MultiplicativeExpression__Group_1_1_0_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMultiplicativeExpressionAccess().getDivExpressionLeftOperandAction_1_1_0_1_0()); }
	()
	{ after(grammarAccess.getMultiplicativeExpressionAccess().getDivExpressionLeftOperandAction_1_1_0_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__MultiplicativeExpression__Group_1_1_0_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MultiplicativeExpression__Group_1_1_0_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__MultiplicativeExpression__Group_1_1_0_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMultiplicativeExpressionAccess().getDivKeyword_1_1_0_1_1()); }
	'div'
	{ after(grammarAccess.getMultiplicativeExpressionAccess().getDivKeyword_1_1_0_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__MultiplicativeExpression__Group_1_1_0_2__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MultiplicativeExpression__Group_1_1_0_2__0__Impl
	rule__MultiplicativeExpression__Group_1_1_0_2__1
;
finally {
	restoreStackSize(stackSize);
}

rule__MultiplicativeExpression__Group_1_1_0_2__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMultiplicativeExpressionAccess().getModExpressionLeftOperandAction_1_1_0_2_0()); }
	()
	{ after(grammarAccess.getMultiplicativeExpressionAccess().getModExpressionLeftOperandAction_1_1_0_2_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__MultiplicativeExpression__Group_1_1_0_2__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__MultiplicativeExpression__Group_1_1_0_2__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__MultiplicativeExpression__Group_1_1_0_2__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getMultiplicativeExpressionAccess().getModKeyword_1_1_0_2_1()); }
	'mod'
	{ after(grammarAccess.getMultiplicativeExpressionAccess().getModKeyword_1_1_0_2_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__SignumExpression__Group_0__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SignumExpression__Group_0__0__Impl
	rule__SignumExpression__Group_0__1
;
finally {
	restoreStackSize(stackSize);
}

rule__SignumExpression__Group_0__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSignumExpressionAccess().getUnaryMinusExpressionAction_0_0()); }
	()
	{ after(grammarAccess.getSignumExpressionAccess().getUnaryMinusExpressionAction_0_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SignumExpression__Group_0__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SignumExpression__Group_0__1__Impl
	rule__SignumExpression__Group_0__2
;
finally {
	restoreStackSize(stackSize);
}

rule__SignumExpression__Group_0__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSignumExpressionAccess().getHyphenMinusKeyword_0_1()); }
	('-')
	{ after(grammarAccess.getSignumExpressionAccess().getHyphenMinusKeyword_0_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SignumExpression__Group_0__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SignumExpression__Group_0__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__SignumExpression__Group_0__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSignumExpressionAccess().getOperandAssignment_0_2()); }
	(rule__SignumExpression__OperandAssignment_0_2)
	{ after(grammarAccess.getSignumExpressionAccess().getOperandAssignment_0_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__SignumExpression__Group_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SignumExpression__Group_1__0__Impl
	rule__SignumExpression__Group_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__SignumExpression__Group_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSignumExpressionAccess().getUnaryPlusExpressionAction_1_0()); }
	()
	{ after(grammarAccess.getSignumExpressionAccess().getUnaryPlusExpressionAction_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SignumExpression__Group_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SignumExpression__Group_1__1__Impl
	rule__SignumExpression__Group_1__2
;
finally {
	restoreStackSize(stackSize);
}

rule__SignumExpression__Group_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSignumExpressionAccess().getPlusSignKeyword_1_1()); }
	('+')
	{ after(grammarAccess.getSignumExpressionAccess().getPlusSignKeyword_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__SignumExpression__Group_1__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SignumExpression__Group_1__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__SignumExpression__Group_1__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getSignumExpressionAccess().getOperandAssignment_1_2()); }
	(rule__SignumExpression__OperandAssignment_1_2)
	{ after(grammarAccess.getSignumExpressionAccess().getOperandAssignment_1_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AccessExpression__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AccessExpression__Group__0__Impl
	rule__AccessExpression__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AccessExpression__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAccessExpressionAccess().getPrimaryExpressionParserRuleCall_0()); }
	rulePrimaryExpression
	{ after(grammarAccess.getAccessExpressionAccess().getPrimaryExpressionParserRuleCall_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AccessExpression__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AccessExpression__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AccessExpression__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAccessExpressionAccess().getAlternatives_1()); }
	(rule__AccessExpression__Alternatives_1)*
	{ after(grammarAccess.getAccessExpressionAccess().getAlternatives_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AccessExpression__Group_1_0__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AccessExpression__Group_1_0__0__Impl
	rule__AccessExpression__Group_1_0__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AccessExpression__Group_1_0__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAccessExpressionAccess().getArrayAccessExpressionOperandAction_1_0_0()); }
	()
	{ after(grammarAccess.getAccessExpressionAccess().getArrayAccessExpressionOperandAction_1_0_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AccessExpression__Group_1_0__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AccessExpression__Group_1_0__1__Impl
	rule__AccessExpression__Group_1_0__2
;
finally {
	restoreStackSize(stackSize);
}

rule__AccessExpression__Group_1_0__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAccessExpressionAccess().getLeftSquareBracketKeyword_1_0_1()); }
	'['
	{ after(grammarAccess.getAccessExpressionAccess().getLeftSquareBracketKeyword_1_0_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AccessExpression__Group_1_0__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AccessExpression__Group_1_0__2__Impl
	rule__AccessExpression__Group_1_0__3
;
finally {
	restoreStackSize(stackSize);
}

rule__AccessExpression__Group_1_0__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAccessExpressionAccess().getIndexAssignment_1_0_2()); }
	(rule__AccessExpression__IndexAssignment_1_0_2)
	{ after(grammarAccess.getAccessExpressionAccess().getIndexAssignment_1_0_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AccessExpression__Group_1_0__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AccessExpression__Group_1_0__3__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AccessExpression__Group_1_0__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAccessExpressionAccess().getRightSquareBracketKeyword_1_0_3()); }
	']'
	{ after(grammarAccess.getAccessExpressionAccess().getRightSquareBracketKeyword_1_0_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AccessExpression__Group_1_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AccessExpression__Group_1_1__0__Impl
	rule__AccessExpression__Group_1_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AccessExpression__Group_1_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAccessExpressionAccess().getFunctionAccessExpressionOperandAction_1_1_0()); }
	()
	{ after(grammarAccess.getAccessExpressionAccess().getFunctionAccessExpressionOperandAction_1_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AccessExpression__Group_1_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AccessExpression__Group_1_1__1__Impl
	rule__AccessExpression__Group_1_1__2
;
finally {
	restoreStackSize(stackSize);
}

rule__AccessExpression__Group_1_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAccessExpressionAccess().getLeftParenthesisKeyword_1_1_1()); }
	'('
	{ after(grammarAccess.getAccessExpressionAccess().getLeftParenthesisKeyword_1_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AccessExpression__Group_1_1__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AccessExpression__Group_1_1__2__Impl
	rule__AccessExpression__Group_1_1__3
;
finally {
	restoreStackSize(stackSize);
}

rule__AccessExpression__Group_1_1__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAccessExpressionAccess().getGroup_1_1_2()); }
	(rule__AccessExpression__Group_1_1_2__0)?
	{ after(grammarAccess.getAccessExpressionAccess().getGroup_1_1_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AccessExpression__Group_1_1__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AccessExpression__Group_1_1__3__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AccessExpression__Group_1_1__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAccessExpressionAccess().getRightParenthesisKeyword_1_1_3()); }
	')'
	{ after(grammarAccess.getAccessExpressionAccess().getRightParenthesisKeyword_1_1_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AccessExpression__Group_1_1_2__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AccessExpression__Group_1_1_2__0__Impl
	rule__AccessExpression__Group_1_1_2__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AccessExpression__Group_1_1_2__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAccessExpressionAccess().getArgumentsAssignment_1_1_2_0()); }
	(rule__AccessExpression__ArgumentsAssignment_1_1_2_0)
	{ after(grammarAccess.getAccessExpressionAccess().getArgumentsAssignment_1_1_2_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AccessExpression__Group_1_1_2__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AccessExpression__Group_1_1_2__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AccessExpression__Group_1_1_2__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAccessExpressionAccess().getGroup_1_1_2_1()); }
	(rule__AccessExpression__Group_1_1_2_1__0)*
	{ after(grammarAccess.getAccessExpressionAccess().getGroup_1_1_2_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AccessExpression__Group_1_1_2_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AccessExpression__Group_1_1_2_1__0__Impl
	rule__AccessExpression__Group_1_1_2_1__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AccessExpression__Group_1_1_2_1__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAccessExpressionAccess().getCommaKeyword_1_1_2_1_0()); }
	','
	{ after(grammarAccess.getAccessExpressionAccess().getCommaKeyword_1_1_2_1_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AccessExpression__Group_1_1_2_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AccessExpression__Group_1_1_2_1__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AccessExpression__Group_1_1_2_1__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAccessExpressionAccess().getArgumentsAssignment_1_1_2_1_1()); }
	(rule__AccessExpression__ArgumentsAssignment_1_1_2_1_1)
	{ after(grammarAccess.getAccessExpressionAccess().getArgumentsAssignment_1_1_2_1_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AccessExpression__Group_1_2__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AccessExpression__Group_1_2__0__Impl
	rule__AccessExpression__Group_1_2__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AccessExpression__Group_1_2__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAccessExpressionAccess().getRecordAccessExpressionOperandAction_1_2_0()); }
	()
	{ after(grammarAccess.getAccessExpressionAccess().getRecordAccessExpressionOperandAction_1_2_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AccessExpression__Group_1_2__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AccessExpression__Group_1_2__1__Impl
	rule__AccessExpression__Group_1_2__2
;
finally {
	restoreStackSize(stackSize);
}

rule__AccessExpression__Group_1_2__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAccessExpressionAccess().getFullStopKeyword_1_2_1()); }
	'.'
	{ after(grammarAccess.getAccessExpressionAccess().getFullStopKeyword_1_2_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AccessExpression__Group_1_2__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AccessExpression__Group_1_2__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AccessExpression__Group_1_2__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAccessExpressionAccess().getFieldReferenceAssignment_1_2_2()); }
	(rule__AccessExpression__FieldReferenceAssignment_1_2_2)
	{ after(grammarAccess.getAccessExpressionAccess().getFieldReferenceAssignment_1_2_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__AccessExpression__Group_1_3__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AccessExpression__Group_1_3__0__Impl
	rule__AccessExpression__Group_1_3__1
;
finally {
	restoreStackSize(stackSize);
}

rule__AccessExpression__Group_1_3__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAccessExpressionAccess().getSelectExpressionOperandAction_1_3_0()); }
	()
	{ after(grammarAccess.getAccessExpressionAccess().getSelectExpressionOperandAction_1_3_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AccessExpression__Group_1_3__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AccessExpression__Group_1_3__1__Impl
	rule__AccessExpression__Group_1_3__2
;
finally {
	restoreStackSize(stackSize);
}

rule__AccessExpression__Group_1_3__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAccessExpressionAccess().getHyphenMinusGreaterThanSignKeyword_1_3_1()); }
	'->'
	{ after(grammarAccess.getAccessExpressionAccess().getHyphenMinusGreaterThanSignKeyword_1_3_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__AccessExpression__Group_1_3__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AccessExpression__Group_1_3__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__AccessExpression__Group_1_3__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getAccessExpressionAccess().getSelectKeyword_1_3_2()); }
	'select'
	{ after(grammarAccess.getAccessExpressionAccess().getSelectKeyword_1_3_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__FieldReferenceExpression__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FieldReferenceExpression__Group__0__Impl
	rule__FieldReferenceExpression__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__FieldReferenceExpression__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFieldReferenceExpressionAccess().getFieldReferenceExpressionAction_0()); }
	()
	{ after(grammarAccess.getFieldReferenceExpressionAccess().getFieldReferenceExpressionAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FieldReferenceExpression__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FieldReferenceExpression__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__FieldReferenceExpression__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFieldReferenceExpressionAccess().getFieldDeclarationAssignment_1()); }
	(rule__FieldReferenceExpression__FieldDeclarationAssignment_1)
	{ after(grammarAccess.getFieldReferenceExpressionAccess().getFieldDeclarationAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__InfinityExpression__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__InfinityExpression__Group__0__Impl
	rule__InfinityExpression__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__InfinityExpression__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInfinityExpressionAccess().getInfinityExpressionAction_0()); }
	()
	{ after(grammarAccess.getInfinityExpressionAccess().getInfinityExpressionAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__InfinityExpression__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__InfinityExpression__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__InfinityExpression__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getInfinityExpressionAccess().getInfinityKeyword_1()); }
	'infinity'
	{ after(grammarAccess.getInfinityExpressionAccess().getInfinityKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__RationalLiteralExpression__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RationalLiteralExpression__Group__0__Impl
	rule__RationalLiteralExpression__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__RationalLiteralExpression__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRationalLiteralExpressionAccess().getNumeratorAssignment_0()); }
	(rule__RationalLiteralExpression__NumeratorAssignment_0)
	{ after(grammarAccess.getRationalLiteralExpressionAccess().getNumeratorAssignment_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RationalLiteralExpression__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RationalLiteralExpression__Group__1__Impl
	rule__RationalLiteralExpression__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__RationalLiteralExpression__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRationalLiteralExpressionAccess().getPercentSignKeyword_1()); }
	'%'
	{ after(grammarAccess.getRationalLiteralExpressionAccess().getPercentSignKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RationalLiteralExpression__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RationalLiteralExpression__Group__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__RationalLiteralExpression__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRationalLiteralExpressionAccess().getDenominatorAssignment_2()); }
	(rule__RationalLiteralExpression__DenominatorAssignment_2)
	{ after(grammarAccess.getRationalLiteralExpressionAccess().getDenominatorAssignment_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__TrueExpression__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__TrueExpression__Group__0__Impl
	rule__TrueExpression__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__TrueExpression__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getTrueExpressionAccess().getTrueExpressionAction_0()); }
	()
	{ after(grammarAccess.getTrueExpressionAccess().getTrueExpressionAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__TrueExpression__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__TrueExpression__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__TrueExpression__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getTrueExpressionAccess().getTrueKeyword_1()); }
	'true'
	{ after(grammarAccess.getTrueExpressionAccess().getTrueKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__FalseExpression__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FalseExpression__Group__0__Impl
	rule__FalseExpression__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__FalseExpression__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFalseExpressionAccess().getFalseExpressionAction_0()); }
	()
	{ after(grammarAccess.getFalseExpressionAccess().getFalseExpressionAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FalseExpression__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FalseExpression__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__FalseExpression__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFalseExpressionAccess().getFalseKeyword_1()); }
	'false'
	{ after(grammarAccess.getFalseExpressionAccess().getFalseKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__RecordLiteralExpression__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RecordLiteralExpression__Group__0__Impl
	rule__RecordLiteralExpression__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__RecordLiteralExpression__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRecordLiteralExpressionAccess().getNumberSignKeyword_0()); }
	'#'
	{ after(grammarAccess.getRecordLiteralExpressionAccess().getNumberSignKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RecordLiteralExpression__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RecordLiteralExpression__Group__1__Impl
	rule__RecordLiteralExpression__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__RecordLiteralExpression__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRecordLiteralExpressionAccess().getTypeDeclarationAssignment_1()); }
	(rule__RecordLiteralExpression__TypeDeclarationAssignment_1)
	{ after(grammarAccess.getRecordLiteralExpressionAccess().getTypeDeclarationAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RecordLiteralExpression__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RecordLiteralExpression__Group__2__Impl
	rule__RecordLiteralExpression__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__RecordLiteralExpression__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRecordLiteralExpressionAccess().getLeftCurlyBracketKeyword_2()); }
	'{'
	{ after(grammarAccess.getRecordLiteralExpressionAccess().getLeftCurlyBracketKeyword_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RecordLiteralExpression__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RecordLiteralExpression__Group__3__Impl
	rule__RecordLiteralExpression__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__RecordLiteralExpression__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRecordLiteralExpressionAccess().getFieldAssignmentsAssignment_3()); }
	(rule__RecordLiteralExpression__FieldAssignmentsAssignment_3)
	{ after(grammarAccess.getRecordLiteralExpressionAccess().getFieldAssignmentsAssignment_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RecordLiteralExpression__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RecordLiteralExpression__Group__4__Impl
	rule__RecordLiteralExpression__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__RecordLiteralExpression__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRecordLiteralExpressionAccess().getGroup_4()); }
	(rule__RecordLiteralExpression__Group_4__0)*
	{ after(grammarAccess.getRecordLiteralExpressionAccess().getGroup_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RecordLiteralExpression__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RecordLiteralExpression__Group__5__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__RecordLiteralExpression__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRecordLiteralExpressionAccess().getRightCurlyBracketKeyword_5()); }
	'}'
	{ after(grammarAccess.getRecordLiteralExpressionAccess().getRightCurlyBracketKeyword_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__RecordLiteralExpression__Group_4__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RecordLiteralExpression__Group_4__0__Impl
	rule__RecordLiteralExpression__Group_4__1
;
finally {
	restoreStackSize(stackSize);
}

rule__RecordLiteralExpression__Group_4__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRecordLiteralExpressionAccess().getCommaKeyword_4_0()); }
	','
	{ after(grammarAccess.getRecordLiteralExpressionAccess().getCommaKeyword_4_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__RecordLiteralExpression__Group_4__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__RecordLiteralExpression__Group_4__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__RecordLiteralExpression__Group_4__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getRecordLiteralExpressionAccess().getFieldAssignmentsAssignment_4_1()); }
	(rule__RecordLiteralExpression__FieldAssignmentsAssignment_4_1)
	{ after(grammarAccess.getRecordLiteralExpressionAccess().getFieldAssignmentsAssignment_4_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__FieldAssignment__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FieldAssignment__Group__0__Impl
	rule__FieldAssignment__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__FieldAssignment__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFieldAssignmentAccess().getReferenceAssignment_0()); }
	(rule__FieldAssignment__ReferenceAssignment_0)
	{ after(grammarAccess.getFieldAssignmentAccess().getReferenceAssignment_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FieldAssignment__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FieldAssignment__Group__1__Impl
	rule__FieldAssignment__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__FieldAssignment__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFieldAssignmentAccess().getColonEqualsSignKeyword_1()); }
	':='
	{ after(grammarAccess.getFieldAssignmentAccess().getColonEqualsSignKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__FieldAssignment__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__FieldAssignment__Group__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__FieldAssignment__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getFieldAssignmentAccess().getValueAssignment_2()); }
	(rule__FieldAssignment__ValueAssignment_2)
	{ after(grammarAccess.getFieldAssignmentAccess().getValueAssignment_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__EnumerationLiteralExpression__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__EnumerationLiteralExpression__Group__0__Impl
	rule__EnumerationLiteralExpression__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__EnumerationLiteralExpression__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEnumerationLiteralExpressionAccess().getTypeReferenceAssignment_0()); }
	(rule__EnumerationLiteralExpression__TypeReferenceAssignment_0)
	{ after(grammarAccess.getEnumerationLiteralExpressionAccess().getTypeReferenceAssignment_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__EnumerationLiteralExpression__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__EnumerationLiteralExpression__Group__1__Impl
	rule__EnumerationLiteralExpression__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__EnumerationLiteralExpression__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEnumerationLiteralExpressionAccess().getColonColonKeyword_1()); }
	'::'
	{ after(grammarAccess.getEnumerationLiteralExpressionAccess().getColonColonKeyword_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__EnumerationLiteralExpression__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__EnumerationLiteralExpression__Group__2__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__EnumerationLiteralExpression__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getEnumerationLiteralExpressionAccess().getReferenceAssignment_2()); }
	(rule__EnumerationLiteralExpression__ReferenceAssignment_2)
	{ after(grammarAccess.getEnumerationLiteralExpressionAccess().getReferenceAssignment_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ArrayLiteralExpression__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ArrayLiteralExpression__Group__0__Impl
	rule__ArrayLiteralExpression__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ArrayLiteralExpression__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getArrayLiteralExpressionAccess().getNumberSignLeftSquareBracketKeyword_0()); }
	'#['
	{ after(grammarAccess.getArrayLiteralExpressionAccess().getNumberSignLeftSquareBracketKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ArrayLiteralExpression__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ArrayLiteralExpression__Group__1__Impl
	rule__ArrayLiteralExpression__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__ArrayLiteralExpression__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getArrayLiteralExpressionAccess().getOperandsAssignment_1()); }
	(rule__ArrayLiteralExpression__OperandsAssignment_1)
	{ after(grammarAccess.getArrayLiteralExpressionAccess().getOperandsAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ArrayLiteralExpression__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ArrayLiteralExpression__Group__2__Impl
	rule__ArrayLiteralExpression__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__ArrayLiteralExpression__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getArrayLiteralExpressionAccess().getGroup_2()); }
	(rule__ArrayLiteralExpression__Group_2__0)*
	{ after(grammarAccess.getArrayLiteralExpressionAccess().getGroup_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ArrayLiteralExpression__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ArrayLiteralExpression__Group__3__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ArrayLiteralExpression__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getArrayLiteralExpressionAccess().getRightSquareBracketKeyword_3()); }
	']'
	{ after(grammarAccess.getArrayLiteralExpressionAccess().getRightSquareBracketKeyword_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__ArrayLiteralExpression__Group_2__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ArrayLiteralExpression__Group_2__0__Impl
	rule__ArrayLiteralExpression__Group_2__1
;
finally {
	restoreStackSize(stackSize);
}

rule__ArrayLiteralExpression__Group_2__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getArrayLiteralExpressionAccess().getCommaKeyword_2_0()); }
	','
	{ after(grammarAccess.getArrayLiteralExpressionAccess().getCommaKeyword_2_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__ArrayLiteralExpression__Group_2__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__ArrayLiteralExpression__Group_2__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__ArrayLiteralExpression__Group_2__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getArrayLiteralExpressionAccess().getOperandsAssignment_2_1()); }
	(rule__ArrayLiteralExpression__OperandsAssignment_2_1)
	{ after(grammarAccess.getArrayLiteralExpressionAccess().getOperandsAssignment_2_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__IntegerRangeLiteralExpression__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__IntegerRangeLiteralExpression__Group__0__Impl
	rule__IntegerRangeLiteralExpression__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__IntegerRangeLiteralExpression__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getIntegerRangeLiteralExpressionAccess().getIntegerRangeLiteralExpressionAction_0()); }
	()
	{ after(grammarAccess.getIntegerRangeLiteralExpressionAccess().getIntegerRangeLiteralExpressionAction_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__IntegerRangeLiteralExpression__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__IntegerRangeLiteralExpression__Group__1__Impl
	rule__IntegerRangeLiteralExpression__Group__2
;
finally {
	restoreStackSize(stackSize);
}

rule__IntegerRangeLiteralExpression__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getIntegerRangeLiteralExpressionAccess().getAlternatives_1()); }
	(rule__IntegerRangeLiteralExpression__Alternatives_1)
	{ after(grammarAccess.getIntegerRangeLiteralExpressionAccess().getAlternatives_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__IntegerRangeLiteralExpression__Group__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__IntegerRangeLiteralExpression__Group__2__Impl
	rule__IntegerRangeLiteralExpression__Group__3
;
finally {
	restoreStackSize(stackSize);
}

rule__IntegerRangeLiteralExpression__Group__2__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getIntegerRangeLiteralExpressionAccess().getLeftOperandAssignment_2()); }
	(rule__IntegerRangeLiteralExpression__LeftOperandAssignment_2)
	{ after(grammarAccess.getIntegerRangeLiteralExpressionAccess().getLeftOperandAssignment_2()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__IntegerRangeLiteralExpression__Group__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__IntegerRangeLiteralExpression__Group__3__Impl
	rule__IntegerRangeLiteralExpression__Group__4
;
finally {
	restoreStackSize(stackSize);
}

rule__IntegerRangeLiteralExpression__Group__3__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getIntegerRangeLiteralExpressionAccess().getFullStopFullStopKeyword_3()); }
	'..'
	{ after(grammarAccess.getIntegerRangeLiteralExpressionAccess().getFullStopFullStopKeyword_3()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__IntegerRangeLiteralExpression__Group__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__IntegerRangeLiteralExpression__Group__4__Impl
	rule__IntegerRangeLiteralExpression__Group__5
;
finally {
	restoreStackSize(stackSize);
}

rule__IntegerRangeLiteralExpression__Group__4__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getIntegerRangeLiteralExpressionAccess().getRightOperandAssignment_4()); }
	(rule__IntegerRangeLiteralExpression__RightOperandAssignment_4)
	{ after(grammarAccess.getIntegerRangeLiteralExpressionAccess().getRightOperandAssignment_4()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__IntegerRangeLiteralExpression__Group__5
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__IntegerRangeLiteralExpression__Group__5__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__IntegerRangeLiteralExpression__Group__5__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getIntegerRangeLiteralExpressionAccess().getAlternatives_5()); }
	(rule__IntegerRangeLiteralExpression__Alternatives_5)
	{ after(grammarAccess.getIntegerRangeLiteralExpressionAccess().getAlternatives_5()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__BasicConstraintDefinition__Group__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__BasicConstraintDefinition__Group__0__Impl
	rule__BasicConstraintDefinition__Group__1
;
finally {
	restoreStackSize(stackSize);
}

rule__BasicConstraintDefinition__Group__0__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getBasicConstraintDefinitionAccess().getConstraintKeyword_0()); }
	'constraint'
	{ after(grammarAccess.getBasicConstraintDefinitionAccess().getConstraintKeyword_0()); }
)
;
finally {
	restoreStackSize(stackSize);
}

rule__BasicConstraintDefinition__Group__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__BasicConstraintDefinition__Group__1__Impl
;
finally {
	restoreStackSize(stackSize);
}

rule__BasicConstraintDefinition__Group__1__Impl
	@init {
		int stackSize = keepStackSize();
	}
:
(
	{ before(grammarAccess.getBasicConstraintDefinitionAccess().getExpressionAssignment_1()); }
	(rule__BasicConstraintDefinition__ExpressionAssignment_1)
	{ after(grammarAccess.getBasicConstraintDefinitionAccess().getExpressionAssignment_1()); }
)
;
finally {
	restoreStackSize(stackSize);
}


rule__SynchronousStatechartDefinition__UnorderedGroup_0
	@init {
		int stackSize = keepStackSize();
		getUnorderedGroupHelper().enter(grammarAccess.getSynchronousStatechartDefinitionAccess().getUnorderedGroup_0());
	}
:
	rule__SynchronousStatechartDefinition__UnorderedGroup_0__0
	?
;
finally {
	getUnorderedGroupHelper().leave(grammarAccess.getSynchronousStatechartDefinitionAccess().getUnorderedGroup_0());
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__UnorderedGroup_0__Impl
	@init {
		int stackSize = keepStackSize();
		boolean selected = false;
	}
:
		(
		( 
			{getUnorderedGroupHelper().canSelect(grammarAccess.getSynchronousStatechartDefinitionAccess().getUnorderedGroup_0(), 0)}?=>(
				{
					getUnorderedGroupHelper().select(grammarAccess.getSynchronousStatechartDefinitionAccess().getUnorderedGroup_0(), 0);
				}
				{
					selected = true;
				}
				(
					{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getGroup_0_0()); }
					(rule__SynchronousStatechartDefinition__Group_0_0__0)
					{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getGroup_0_0()); }
				)
			)
		)|
		( 
			{getUnorderedGroupHelper().canSelect(grammarAccess.getSynchronousStatechartDefinitionAccess().getUnorderedGroup_0(), 1)}?=>(
				{
					getUnorderedGroupHelper().select(grammarAccess.getSynchronousStatechartDefinitionAccess().getUnorderedGroup_0(), 1);
				}
				{
					selected = true;
				}
				(
					{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getGroup_0_1()); }
					(rule__SynchronousStatechartDefinition__Group_0_1__0)
					{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getGroup_0_1()); }
				)
			)
		)|
		( 
			{getUnorderedGroupHelper().canSelect(grammarAccess.getSynchronousStatechartDefinitionAccess().getUnorderedGroup_0(), 2)}?=>(
				{
					getUnorderedGroupHelper().select(grammarAccess.getSynchronousStatechartDefinitionAccess().getUnorderedGroup_0(), 2);
				}
				{
					selected = true;
				}
				(
					{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getGroup_0_2()); }
					(rule__SynchronousStatechartDefinition__Group_0_2__0)
					{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getGroup_0_2()); }
				)
			)
		)|
		( 
			{getUnorderedGroupHelper().canSelect(grammarAccess.getSynchronousStatechartDefinitionAccess().getUnorderedGroup_0(), 3)}?=>(
				{
					getUnorderedGroupHelper().select(grammarAccess.getSynchronousStatechartDefinitionAccess().getUnorderedGroup_0(), 3);
				}
				{
					selected = true;
				}
				(
					{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getGroup_0_3()); }
					(rule__SynchronousStatechartDefinition__Group_0_3__0)
					{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getGroup_0_3()); }
				)
			)
		)|
		( 
			{getUnorderedGroupHelper().canSelect(grammarAccess.getSynchronousStatechartDefinitionAccess().getUnorderedGroup_0(), 4)}?=>(
				{
					getUnorderedGroupHelper().select(grammarAccess.getSynchronousStatechartDefinitionAccess().getUnorderedGroup_0(), 4);
				}
				{
					selected = true;
				}
				(
					(
						{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getAlternatives_0_4()); }
						(rule__SynchronousStatechartDefinition__Alternatives_0_4)
						{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getAlternatives_0_4()); }
					)
					(
						{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getAlternatives_0_4()); }
						((rule__SynchronousStatechartDefinition__Alternatives_0_4)=>rule__SynchronousStatechartDefinition__Alternatives_0_4)*
						{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getAlternatives_0_4()); }
					)
				)
			)
		)
		)
;
finally {
	if (selected)
		getUnorderedGroupHelper().returnFromSelection(grammarAccess.getSynchronousStatechartDefinitionAccess().getUnorderedGroup_0());
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__UnorderedGroup_0__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousStatechartDefinition__UnorderedGroup_0__Impl
	rule__SynchronousStatechartDefinition__UnorderedGroup_0__1?
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__UnorderedGroup_0__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousStatechartDefinition__UnorderedGroup_0__Impl
	rule__SynchronousStatechartDefinition__UnorderedGroup_0__2?
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__UnorderedGroup_0__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousStatechartDefinition__UnorderedGroup_0__Impl
	rule__SynchronousStatechartDefinition__UnorderedGroup_0__3?
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__UnorderedGroup_0__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousStatechartDefinition__UnorderedGroup_0__Impl
	rule__SynchronousStatechartDefinition__UnorderedGroup_0__4?
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__UnorderedGroup_0__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__SynchronousStatechartDefinition__UnorderedGroup_0__Impl
;
finally {
	restoreStackSize(stackSize);
}


rule__AsynchronousStatechartDefinition__UnorderedGroup_1
	@init {
		int stackSize = keepStackSize();
		getUnorderedGroupHelper().enter(grammarAccess.getAsynchronousStatechartDefinitionAccess().getUnorderedGroup_1());
	}
:
	rule__AsynchronousStatechartDefinition__UnorderedGroup_1__0
	?
;
finally {
	getUnorderedGroupHelper().leave(grammarAccess.getAsynchronousStatechartDefinitionAccess().getUnorderedGroup_1());
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__UnorderedGroup_1__Impl
	@init {
		int stackSize = keepStackSize();
		boolean selected = false;
	}
:
		(
		( 
			{getUnorderedGroupHelper().canSelect(grammarAccess.getAsynchronousStatechartDefinitionAccess().getUnorderedGroup_1(), 0)}?=>(
				{
					getUnorderedGroupHelper().select(grammarAccess.getAsynchronousStatechartDefinitionAccess().getUnorderedGroup_1(), 0);
				}
				{
					selected = true;
				}
				(
					{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getGroup_1_0()); }
					(rule__AsynchronousStatechartDefinition__Group_1_0__0)
					{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getGroup_1_0()); }
				)
			)
		)|
		( 
			{getUnorderedGroupHelper().canSelect(grammarAccess.getAsynchronousStatechartDefinitionAccess().getUnorderedGroup_1(), 1)}?=>(
				{
					getUnorderedGroupHelper().select(grammarAccess.getAsynchronousStatechartDefinitionAccess().getUnorderedGroup_1(), 1);
				}
				{
					selected = true;
				}
				(
					{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getGroup_1_1()); }
					(rule__AsynchronousStatechartDefinition__Group_1_1__0)
					{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getGroup_1_1()); }
				)
			)
		)|
		( 
			{getUnorderedGroupHelper().canSelect(grammarAccess.getAsynchronousStatechartDefinitionAccess().getUnorderedGroup_1(), 2)}?=>(
				{
					getUnorderedGroupHelper().select(grammarAccess.getAsynchronousStatechartDefinitionAccess().getUnorderedGroup_1(), 2);
				}
				{
					selected = true;
				}
				(
					{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getGroup_1_2()); }
					(rule__AsynchronousStatechartDefinition__Group_1_2__0)
					{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getGroup_1_2()); }
				)
			)
		)|
		( 
			{getUnorderedGroupHelper().canSelect(grammarAccess.getAsynchronousStatechartDefinitionAccess().getUnorderedGroup_1(), 3)}?=>(
				{
					getUnorderedGroupHelper().select(grammarAccess.getAsynchronousStatechartDefinitionAccess().getUnorderedGroup_1(), 3);
				}
				{
					selected = true;
				}
				(
					{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getGroup_1_3()); }
					(rule__AsynchronousStatechartDefinition__Group_1_3__0)
					{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getGroup_1_3()); }
				)
			)
		)|
		( 
			{getUnorderedGroupHelper().canSelect(grammarAccess.getAsynchronousStatechartDefinitionAccess().getUnorderedGroup_1(), 4)}?=>(
				{
					getUnorderedGroupHelper().select(grammarAccess.getAsynchronousStatechartDefinitionAccess().getUnorderedGroup_1(), 4);
				}
				{
					selected = true;
				}
				(
					(
						{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getAlternatives_1_4()); }
						(rule__AsynchronousStatechartDefinition__Alternatives_1_4)
						{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getAlternatives_1_4()); }
					)
					(
						{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getAlternatives_1_4()); }
						((rule__AsynchronousStatechartDefinition__Alternatives_1_4)=>rule__AsynchronousStatechartDefinition__Alternatives_1_4)*
						{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getAlternatives_1_4()); }
					)
				)
			)
		)
		)
;
finally {
	if (selected)
		getUnorderedGroupHelper().returnFromSelection(grammarAccess.getAsynchronousStatechartDefinitionAccess().getUnorderedGroup_1());
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__UnorderedGroup_1__0
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__UnorderedGroup_1__Impl
	rule__AsynchronousStatechartDefinition__UnorderedGroup_1__1?
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__UnorderedGroup_1__1
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__UnorderedGroup_1__Impl
	rule__AsynchronousStatechartDefinition__UnorderedGroup_1__2?
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__UnorderedGroup_1__2
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__UnorderedGroup_1__Impl
	rule__AsynchronousStatechartDefinition__UnorderedGroup_1__3?
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__UnorderedGroup_1__3
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__UnorderedGroup_1__Impl
	rule__AsynchronousStatechartDefinition__UnorderedGroup_1__4?
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__UnorderedGroup_1__4
	@init {
		int stackSize = keepStackSize();
	}
:
	rule__AsynchronousStatechartDefinition__UnorderedGroup_1__Impl
;
finally {
	restoreStackSize(stackSize);
}


rule__FaultExtensionInstructions__ImportsAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getFaultExtensionInstructionsAccess().getImportsPackageCrossReference_1_0()); }
		(
			{ before(grammarAccess.getFaultExtensionInstructionsAccess().getImportsPackageSTRINGTerminalRuleCall_1_0_1()); }
			RULE_STRING
			{ after(grammarAccess.getFaultExtensionInstructionsAccess().getImportsPackageSTRINGTerminalRuleCall_1_0_1()); }
		)
		{ after(grammarAccess.getFaultExtensionInstructionsAccess().getImportsPackageCrossReference_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultExtensionInstructions__NameAssignment_3
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getFaultExtensionInstructionsAccess().getNameIDTerminalRuleCall_3_0()); }
		RULE_ID
		{ after(grammarAccess.getFaultExtensionInstructionsAccess().getNameIDTerminalRuleCall_3_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultExtensionInstructions__ComponentAssignment_5
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getFaultExtensionInstructionsAccess().getComponentComponentCrossReference_5_0()); }
		(
			{ before(grammarAccess.getFaultExtensionInstructionsAccess().getComponentComponentIDTerminalRuleCall_5_0_1()); }
			RULE_ID
			{ after(grammarAccess.getFaultExtensionInstructionsAccess().getComponentComponentIDTerminalRuleCall_5_0_1()); }
		)
		{ after(grammarAccess.getFaultExtensionInstructionsAccess().getComponentComponentCrossReference_5_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultExtensionInstructions__FaultSlicesAssignment_6
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getFaultExtensionInstructionsAccess().getFaultSlicesFaultSliceParserRuleCall_6_0()); }
		ruleFaultSlice
		{ after(grammarAccess.getFaultExtensionInstructionsAccess().getFaultSlicesFaultSliceParserRuleCall_6_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultExtensionInstructions__CommonCausesAssignment_7
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getFaultExtensionInstructionsAccess().getCommonCausesCommonCausesParserRuleCall_7_0()); }
		ruleCommonCauses
		{ after(grammarAccess.getFaultExtensionInstructionsAccess().getCommonCausesCommonCausesParserRuleCall_7_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultSlice__NameAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getFaultSliceAccess().getNameIDTerminalRuleCall_1_0()); }
		RULE_ID
		{ after(grammarAccess.getFaultSliceAccess().getNameIDTerminalRuleCall_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultSlice__AffectedElementsAssignment_3
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getFaultSliceAccess().getAffectedElementsComponentInstanceStateExpressionParserRuleCall_3_0()); }
		ruleComponentInstanceStateExpression
		{ after(grammarAccess.getFaultSliceAccess().getAffectedElementsComponentInstanceStateExpressionParserRuleCall_3_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultSlice__AffectedElementsAssignment_4_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getFaultSliceAccess().getAffectedElementsComponentInstanceStateExpressionParserRuleCall_4_1_0()); }
		ruleComponentInstanceStateExpression
		{ after(grammarAccess.getFaultSliceAccess().getAffectedElementsComponentInstanceStateExpressionParserRuleCall_4_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultSlice__FaultModesAssignment_6
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getFaultSliceAccess().getFaultModesFaultModeParserRuleCall_6_0()); }
		ruleFaultMode
		{ after(grammarAccess.getFaultSliceAccess().getFaultModesFaultModeParserRuleCall_6_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultSlice__GlobalDynamicsAssignment_7
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getFaultSliceAccess().getGlobalDynamicsGlobalDynamicsParserRuleCall_7_0()); }
		ruleGlobalDynamics
		{ after(grammarAccess.getFaultSliceAccess().getGlobalDynamicsGlobalDynamicsParserRuleCall_7_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultMode__NameAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getFaultModeAccess().getNameIDTerminalRuleCall_1_0()); }
		RULE_ID
		{ after(grammarAccess.getFaultModeAccess().getNameIDTerminalRuleCall_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultMode__ProbabilityAssignment_2_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getFaultModeAccess().getProbabilityExpressionParserRuleCall_2_1_0()); }
		ruleExpression
		{ after(grammarAccess.getFaultModeAccess().getProbabilityExpressionParserRuleCall_2_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultMode__LocalDynamicsAssignment_3
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getFaultModeAccess().getLocalDynamicsLocalDynamicsEnumRuleCall_3_0()); }
		ruleLocalDynamics
		{ after(grammarAccess.getFaultModeAccess().getLocalDynamicsLocalDynamicsEnumRuleCall_3_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultMode__EffectAssignment_4
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getFaultModeAccess().getEffectEffectParserRuleCall_4_0()); }
		ruleEffect
		{ after(grammarAccess.getFaultModeAccess().getEffectEffectParserRuleCall_4_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__TermReferenceModeAssignment_2_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getStuckAtEffectAccess().getTermReferenceModeTermReferenceModeEnumRuleCall_2_2_0()); }
		ruleTermReferenceMode
		{ after(grammarAccess.getStuckAtEffectAccess().getTermReferenceModeTermReferenceModeEnumRuleCall_2_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__OccurrenceAssignment_3_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getStuckAtEffectAccess().getOccurrenceOccurrenceEnumRuleCall_3_2_0()); }
		ruleOccurrence
		{ after(grammarAccess.getStuckAtEffectAccess().getOccurrenceOccurrenceEnumRuleCall_3_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__TermAssignment_4_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getStuckAtEffectAccess().getTermExpressionParserRuleCall_4_2_0()); }
		ruleExpression
		{ after(grammarAccess.getStuckAtEffectAccess().getTermExpressionParserRuleCall_4_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__InputAssignment_5_0_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getStuckAtEffectAccess().getInputComponentInstanceStateExpressionParserRuleCall_5_0_2_0()); }
		ruleComponentInstanceStateExpression
		{ after(grammarAccess.getStuckAtEffectAccess().getInputComponentInstanceStateExpressionParserRuleCall_5_0_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__VaroutAssignment_5_1_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getStuckAtEffectAccess().getVaroutComponentInstanceStateExpressionParserRuleCall_5_1_2_0()); }
		ruleComponentInstanceStateExpression
		{ after(grammarAccess.getStuckAtEffectAccess().getVaroutComponentInstanceStateExpressionParserRuleCall_5_1_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__FailureEventAssignment_6_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getStuckAtEffectAccess().getFailureEventComponentInstanceEventReferenceExpressionParserRuleCall_6_2_0()); }
		ruleComponentInstanceEventReferenceExpression
		{ after(grammarAccess.getStuckAtEffectAccess().getFailureEventComponentInstanceEventReferenceExpressionParserRuleCall_6_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtEffect__TemplateAssignment_7
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getStuckAtEffectAccess().getTemplateTemplateParserRuleCall_7_0()); }
		ruleTemplate
		{ after(grammarAccess.getStuckAtEffectAccess().getTemplateTemplateParserRuleCall_7_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__FrozenEffect__InputAssignment_3_0_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getFrozenEffectAccess().getInputComponentInstanceStateExpressionParserRuleCall_3_0_2_0()); }
		ruleComponentInstanceStateExpression
		{ after(grammarAccess.getFrozenEffectAccess().getInputComponentInstanceStateExpressionParserRuleCall_3_0_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__FrozenEffect__VaroutAssignment_3_1_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getFrozenEffectAccess().getVaroutComponentInstanceStateExpressionParserRuleCall_3_1_2_0()); }
		ruleComponentInstanceStateExpression
		{ after(grammarAccess.getFrozenEffectAccess().getVaroutComponentInstanceStateExpressionParserRuleCall_3_1_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__FrozenEffect__FailureEventAssignment_4_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getFrozenEffectAccess().getFailureEventComponentInstanceEventReferenceExpressionParserRuleCall_4_2_0()); }
		ruleComponentInstanceEventReferenceExpression
		{ after(grammarAccess.getFrozenEffectAccess().getFailureEventComponentInstanceEventReferenceExpressionParserRuleCall_4_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__FrozenEffect__TemplateAssignment_5
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getFrozenEffectAccess().getTemplateTemplateParserRuleCall_5_0()); }
		ruleTemplate
		{ after(grammarAccess.getFrozenEffectAccess().getTemplateTemplateParserRuleCall_5_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__TermReferenceModeAssignment_2_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getNonDeterminismEffectAccess().getTermReferenceModeTermReferenceModeEnumRuleCall_2_2_0()); }
		ruleTermReferenceMode
		{ after(grammarAccess.getNonDeterminismEffectAccess().getTermReferenceModeTermReferenceModeEnumRuleCall_2_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__OccurrenceAssignment_3_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getNonDeterminismEffectAccess().getOccurrenceOccurrenceEnumRuleCall_3_2_0()); }
		ruleOccurrence
		{ after(grammarAccess.getNonDeterminismEffectAccess().getOccurrenceOccurrenceEnumRuleCall_3_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__MinimumAssignment_4_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getNonDeterminismEffectAccess().getMinimumExpressionParserRuleCall_4_2_0()); }
		ruleExpression
		{ after(grammarAccess.getNonDeterminismEffectAccess().getMinimumExpressionParserRuleCall_4_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__MaximumAssignment_5_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getNonDeterminismEffectAccess().getMaximumExpressionParserRuleCall_5_2_0()); }
		ruleExpression
		{ after(grammarAccess.getNonDeterminismEffectAccess().getMaximumExpressionParserRuleCall_5_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__InputAssignment_6_0_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getNonDeterminismEffectAccess().getInputComponentInstanceStateExpressionParserRuleCall_6_0_2_0()); }
		ruleComponentInstanceStateExpression
		{ after(grammarAccess.getNonDeterminismEffectAccess().getInputComponentInstanceStateExpressionParserRuleCall_6_0_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__VaroutAssignment_6_1_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getNonDeterminismEffectAccess().getVaroutComponentInstanceStateExpressionParserRuleCall_6_1_2_0()); }
		ruleComponentInstanceStateExpression
		{ after(grammarAccess.getNonDeterminismEffectAccess().getVaroutComponentInstanceStateExpressionParserRuleCall_6_1_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__FailureEventAssignment_7_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getNonDeterminismEffectAccess().getFailureEventComponentInstanceEventReferenceExpressionParserRuleCall_7_2_0()); }
		ruleComponentInstanceEventReferenceExpression
		{ after(grammarAccess.getNonDeterminismEffectAccess().getFailureEventComponentInstanceEventReferenceExpressionParserRuleCall_7_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismEffect__TemplateAssignment_8
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getNonDeterminismEffectAccess().getTemplateTemplateParserRuleCall_8_0()); }
		ruleTemplate
		{ after(grammarAccess.getNonDeterminismEffectAccess().getTemplateTemplateParserRuleCall_8_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__TermReferenceModeAssignment_3_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getNonDeterminismBooleanEffectAccess().getTermReferenceModeTermReferenceModeEnumRuleCall_3_2_0()); }
		ruleTermReferenceMode
		{ after(grammarAccess.getNonDeterminismBooleanEffectAccess().getTermReferenceModeTermReferenceModeEnumRuleCall_3_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__InputAssignment_4_0_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getNonDeterminismBooleanEffectAccess().getInputComponentInstanceStateExpressionParserRuleCall_4_0_2_0()); }
		ruleComponentInstanceStateExpression
		{ after(grammarAccess.getNonDeterminismBooleanEffectAccess().getInputComponentInstanceStateExpressionParserRuleCall_4_0_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__VaroutAssignment_4_1_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getNonDeterminismBooleanEffectAccess().getVaroutComponentInstanceStateExpressionParserRuleCall_4_1_2_0()); }
		ruleComponentInstanceStateExpression
		{ after(grammarAccess.getNonDeterminismBooleanEffectAccess().getVaroutComponentInstanceStateExpressionParserRuleCall_4_1_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__FailureEventAssignment_5_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getNonDeterminismBooleanEffectAccess().getFailureEventComponentInstanceEventReferenceExpressionParserRuleCall_5_2_0()); }
		ruleComponentInstanceEventReferenceExpression
		{ after(grammarAccess.getNonDeterminismBooleanEffectAccess().getFailureEventComponentInstanceEventReferenceExpressionParserRuleCall_5_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__NonDeterminismBooleanEffect__TemplateAssignment_6
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getNonDeterminismBooleanEffectAccess().getTemplateTemplateParserRuleCall_6_0()); }
		ruleTemplate
		{ after(grammarAccess.getNonDeterminismBooleanEffectAccess().getTemplateTemplateParserRuleCall_6_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__OccurrenceAssignment_2_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getConditionalEffectAccess().getOccurrenceOccurrenceEnumRuleCall_2_2_0()); }
		ruleOccurrence
		{ after(grammarAccess.getConditionalEffectAccess().getOccurrenceOccurrenceEnumRuleCall_2_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__ConditionAssignment_3_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getConditionalEffectAccess().getConditionExpressionParserRuleCall_3_2_0()); }
		ruleExpression
		{ after(grammarAccess.getConditionalEffectAccess().getConditionExpressionParserRuleCall_3_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__ThenAssignment_4_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getConditionalEffectAccess().getThenExpressionParserRuleCall_4_2_0()); }
		ruleExpression
		{ after(grammarAccess.getConditionalEffectAccess().getThenExpressionParserRuleCall_4_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__ElseAssignment_5_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getConditionalEffectAccess().getElseExpressionParserRuleCall_5_2_0()); }
		ruleExpression
		{ after(grammarAccess.getConditionalEffectAccess().getElseExpressionParserRuleCall_5_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__InputAssignment_6_0_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getConditionalEffectAccess().getInputComponentInstanceStateExpressionParserRuleCall_6_0_2_0()); }
		ruleComponentInstanceStateExpression
		{ after(grammarAccess.getConditionalEffectAccess().getInputComponentInstanceStateExpressionParserRuleCall_6_0_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__VaroutAssignment_6_1_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getConditionalEffectAccess().getVaroutComponentInstanceStateExpressionParserRuleCall_6_1_2_0()); }
		ruleComponentInstanceStateExpression
		{ after(grammarAccess.getConditionalEffectAccess().getVaroutComponentInstanceStateExpressionParserRuleCall_6_1_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__FailureEventAssignment_7_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getConditionalEffectAccess().getFailureEventComponentInstanceEventReferenceExpressionParserRuleCall_7_2_0()); }
		ruleComponentInstanceEventReferenceExpression
		{ after(grammarAccess.getConditionalEffectAccess().getFailureEventComponentInstanceEventReferenceExpressionParserRuleCall_7_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ConditionalEffect__TemplateAssignment_8
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getConditionalEffectAccess().getTemplateTemplateParserRuleCall_8_0()); }
		ruleTemplate
		{ after(grammarAccess.getConditionalEffectAccess().getTemplateTemplateParserRuleCall_8_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__DeltaAssignment_2_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getRampDownEffectAccess().getDeltaExpressionParserRuleCall_2_2_0()); }
		ruleExpression
		{ after(grammarAccess.getRampDownEffectAccess().getDeltaExpressionParserRuleCall_2_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__BoundAssignment_3_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getRampDownEffectAccess().getBoundExpressionParserRuleCall_3_2_0()); }
		ruleExpression
		{ after(grammarAccess.getRampDownEffectAccess().getBoundExpressionParserRuleCall_3_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__InputAssignment_4_0_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getRampDownEffectAccess().getInputComponentInstanceStateExpressionParserRuleCall_4_0_2_0()); }
		ruleComponentInstanceStateExpression
		{ after(grammarAccess.getRampDownEffectAccess().getInputComponentInstanceStateExpressionParserRuleCall_4_0_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__VaroutAssignment_4_1_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getRampDownEffectAccess().getVaroutComponentInstanceStateExpressionParserRuleCall_4_1_2_0()); }
		ruleComponentInstanceStateExpression
		{ after(grammarAccess.getRampDownEffectAccess().getVaroutComponentInstanceStateExpressionParserRuleCall_4_1_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__FailureEventAssignment_5_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getRampDownEffectAccess().getFailureEventComponentInstanceEventReferenceExpressionParserRuleCall_5_2_0()); }
		ruleComponentInstanceEventReferenceExpression
		{ after(grammarAccess.getRampDownEffectAccess().getFailureEventComponentInstanceEventReferenceExpressionParserRuleCall_5_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__RampDownEffect__TemplateAssignment_6
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getRampDownEffectAccess().getTemplateTemplateParserRuleCall_6_0()); }
		ruleTemplate
		{ after(grammarAccess.getRampDownEffectAccess().getTemplateTemplateParserRuleCall_6_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__InvertedEffect__InputAssignment_3_0_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getInvertedEffectAccess().getInputComponentInstanceStateExpressionParserRuleCall_3_0_2_0()); }
		ruleComponentInstanceStateExpression
		{ after(grammarAccess.getInvertedEffectAccess().getInputComponentInstanceStateExpressionParserRuleCall_3_0_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__InvertedEffect__VaroutAssignment_3_1_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getInvertedEffectAccess().getVaroutComponentInstanceStateExpressionParserRuleCall_3_1_2_0()); }
		ruleComponentInstanceStateExpression
		{ after(grammarAccess.getInvertedEffectAccess().getVaroutComponentInstanceStateExpressionParserRuleCall_3_1_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__InvertedEffect__FailureEventAssignment_4_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getInvertedEffectAccess().getFailureEventComponentInstanceEventReferenceExpressionParserRuleCall_4_2_0()); }
		ruleComponentInstanceEventReferenceExpression
		{ after(grammarAccess.getInvertedEffectAccess().getFailureEventComponentInstanceEventReferenceExpressionParserRuleCall_4_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__InvertedEffect__TemplateAssignment_5
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getInvertedEffectAccess().getTemplateTemplateParserRuleCall_5_0()); }
		ruleTemplate
		{ after(grammarAccess.getInvertedEffectAccess().getTemplateTemplateParserRuleCall_5_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtFixedEffect__InputAssignment_3_0_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getStuckAtFixedEffectAccess().getInputComponentInstanceStateExpressionParserRuleCall_3_0_2_0()); }
		ruleComponentInstanceStateExpression
		{ after(grammarAccess.getStuckAtFixedEffectAccess().getInputComponentInstanceStateExpressionParserRuleCall_3_0_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtFixedEffect__VaroutAssignment_3_1_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getStuckAtFixedEffectAccess().getVaroutComponentInstanceStateExpressionParserRuleCall_3_1_2_0()); }
		ruleComponentInstanceStateExpression
		{ after(grammarAccess.getStuckAtFixedEffectAccess().getVaroutComponentInstanceStateExpressionParserRuleCall_3_1_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtFixedEffect__FailureEventAssignment_4_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getStuckAtFixedEffectAccess().getFailureEventComponentInstanceEventReferenceExpressionParserRuleCall_4_2_0()); }
		ruleComponentInstanceEventReferenceExpression
		{ after(grammarAccess.getStuckAtFixedEffectAccess().getFailureEventComponentInstanceEventReferenceExpressionParserRuleCall_4_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__StuckAtFixedEffect__TemplateAssignment_5
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getStuckAtFixedEffectAccess().getTemplateTemplateParserRuleCall_5_0()); }
		ruleTemplate
		{ after(grammarAccess.getStuckAtFixedEffectAccess().getTemplateTemplateParserRuleCall_5_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__TermReferenceModeAssignment_3_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getRandomEffectAccess().getTermReferenceModeTermReferenceModeEnumRuleCall_3_2_0()); }
		ruleTermReferenceMode
		{ after(grammarAccess.getRandomEffectAccess().getTermReferenceModeTermReferenceModeEnumRuleCall_3_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__InputAssignment_4_0_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getRandomEffectAccess().getInputComponentInstanceStateExpressionParserRuleCall_4_0_2_0()); }
		ruleComponentInstanceStateExpression
		{ after(grammarAccess.getRandomEffectAccess().getInputComponentInstanceStateExpressionParserRuleCall_4_0_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__VaroutAssignment_4_1_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getRandomEffectAccess().getVaroutComponentInstanceStateExpressionParserRuleCall_4_1_2_0()); }
		ruleComponentInstanceStateExpression
		{ after(grammarAccess.getRandomEffectAccess().getVaroutComponentInstanceStateExpressionParserRuleCall_4_1_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__FailureEventAssignment_5_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getRandomEffectAccess().getFailureEventComponentInstanceEventReferenceExpressionParserRuleCall_5_2_0()); }
		ruleComponentInstanceEventReferenceExpression
		{ after(grammarAccess.getRandomEffectAccess().getFailureEventComponentInstanceEventReferenceExpressionParserRuleCall_5_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__RandomEffect__TemplateAssignment_6
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getRandomEffectAccess().getTemplateTemplateParserRuleCall_6_0()); }
		ruleTemplate
		{ after(grammarAccess.getRandomEffectAccess().getTemplateTemplateParserRuleCall_6_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__TermReferenceModeAssignment_3_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getErroneousEffectAccess().getTermReferenceModeTermReferenceModeEnumRuleCall_3_2_0()); }
		ruleTermReferenceMode
		{ after(grammarAccess.getErroneousEffectAccess().getTermReferenceModeTermReferenceModeEnumRuleCall_3_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__InputAssignment_4_0_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getErroneousEffectAccess().getInputComponentInstanceStateExpressionParserRuleCall_4_0_2_0()); }
		ruleComponentInstanceStateExpression
		{ after(grammarAccess.getErroneousEffectAccess().getInputComponentInstanceStateExpressionParserRuleCall_4_0_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__VaroutAssignment_4_1_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getErroneousEffectAccess().getVaroutComponentInstanceStateExpressionParserRuleCall_4_1_2_0()); }
		ruleComponentInstanceStateExpression
		{ after(grammarAccess.getErroneousEffectAccess().getVaroutComponentInstanceStateExpressionParserRuleCall_4_1_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__FailureEventAssignment_5_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getErroneousEffectAccess().getFailureEventComponentInstanceEventReferenceExpressionParserRuleCall_5_2_0()); }
		ruleComponentInstanceEventReferenceExpression
		{ after(grammarAccess.getErroneousEffectAccess().getFailureEventComponentInstanceEventReferenceExpressionParserRuleCall_5_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ErroneousEffect__TemplateAssignment_6
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getErroneousEffectAccess().getTemplateTemplateParserRuleCall_6_0()); }
		ruleTemplate
		{ after(grammarAccess.getErroneousEffectAccess().getTemplateTemplateParserRuleCall_6_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__TermReferenceModeAssignment_2_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getDeltaOutEffectAccess().getTermReferenceModeTermReferenceModeEnumRuleCall_2_2_0()); }
		ruleTermReferenceMode
		{ after(grammarAccess.getDeltaOutEffectAccess().getTermReferenceModeTermReferenceModeEnumRuleCall_2_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__DeltaAssignment_3_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getDeltaOutEffectAccess().getDeltaExpressionParserRuleCall_3_2_0()); }
		ruleExpression
		{ after(grammarAccess.getDeltaOutEffectAccess().getDeltaExpressionParserRuleCall_3_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__InputAssignment_4_0_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getDeltaOutEffectAccess().getInputComponentInstanceStateExpressionParserRuleCall_4_0_2_0()); }
		ruleComponentInstanceStateExpression
		{ after(grammarAccess.getDeltaOutEffectAccess().getInputComponentInstanceStateExpressionParserRuleCall_4_0_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__VaroutAssignment_4_1_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getDeltaOutEffectAccess().getVaroutComponentInstanceStateExpressionParserRuleCall_4_1_2_0()); }
		ruleComponentInstanceStateExpression
		{ after(grammarAccess.getDeltaOutEffectAccess().getVaroutComponentInstanceStateExpressionParserRuleCall_4_1_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__FailureEventAssignment_5_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getDeltaOutEffectAccess().getFailureEventComponentInstanceEventReferenceExpressionParserRuleCall_5_2_0()); }
		ruleComponentInstanceEventReferenceExpression
		{ after(grammarAccess.getDeltaOutEffectAccess().getFailureEventComponentInstanceEventReferenceExpressionParserRuleCall_5_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaOutEffect__TemplateAssignment_6
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getDeltaOutEffectAccess().getTemplateTemplateParserRuleCall_6_0()); }
		ruleTemplate
		{ after(grammarAccess.getDeltaOutEffectAccess().getTemplateTemplateParserRuleCall_6_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__TermReferenceModeAssignment_2_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getDeltaInRandomEffectAccess().getTermReferenceModeTermReferenceModeEnumRuleCall_2_2_0()); }
		ruleTermReferenceMode
		{ after(grammarAccess.getDeltaInRandomEffectAccess().getTermReferenceModeTermReferenceModeEnumRuleCall_2_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__DeltaAssignment_3_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getDeltaInRandomEffectAccess().getDeltaExpressionParserRuleCall_3_2_0()); }
		ruleExpression
		{ after(grammarAccess.getDeltaInRandomEffectAccess().getDeltaExpressionParserRuleCall_3_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__InputAssignment_4_0_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getDeltaInRandomEffectAccess().getInputComponentInstanceStateExpressionParserRuleCall_4_0_2_0()); }
		ruleComponentInstanceStateExpression
		{ after(grammarAccess.getDeltaInRandomEffectAccess().getInputComponentInstanceStateExpressionParserRuleCall_4_0_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__VaroutAssignment_4_1_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getDeltaInRandomEffectAccess().getVaroutComponentInstanceStateExpressionParserRuleCall_4_1_2_0()); }
		ruleComponentInstanceStateExpression
		{ after(grammarAccess.getDeltaInRandomEffectAccess().getVaroutComponentInstanceStateExpressionParserRuleCall_4_1_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__FailureEventAssignment_5_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getDeltaInRandomEffectAccess().getFailureEventComponentInstanceEventReferenceExpressionParserRuleCall_5_2_0()); }
		ruleComponentInstanceEventReferenceExpression
		{ after(grammarAccess.getDeltaInRandomEffectAccess().getFailureEventComponentInstanceEventReferenceExpressionParserRuleCall_5_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInRandomEffect__TemplateAssignment_6
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getDeltaInRandomEffectAccess().getTemplateTemplateParserRuleCall_6_0()); }
		ruleTemplate
		{ after(grammarAccess.getDeltaInRandomEffectAccess().getTemplateTemplateParserRuleCall_6_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__TermReferenceModeAssignment_2_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getDeltaInErroneousEffectAccess().getTermReferenceModeTermReferenceModeEnumRuleCall_2_2_0()); }
		ruleTermReferenceMode
		{ after(grammarAccess.getDeltaInErroneousEffectAccess().getTermReferenceModeTermReferenceModeEnumRuleCall_2_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__DeltaAssignment_3_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getDeltaInErroneousEffectAccess().getDeltaExpressionParserRuleCall_3_2_0()); }
		ruleExpression
		{ after(grammarAccess.getDeltaInErroneousEffectAccess().getDeltaExpressionParserRuleCall_3_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__InputAssignment_4_0_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getDeltaInErroneousEffectAccess().getInputComponentInstanceStateExpressionParserRuleCall_4_0_2_0()); }
		ruleComponentInstanceStateExpression
		{ after(grammarAccess.getDeltaInErroneousEffectAccess().getInputComponentInstanceStateExpressionParserRuleCall_4_0_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__VaroutAssignment_4_1_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getDeltaInErroneousEffectAccess().getVaroutComponentInstanceStateExpressionParserRuleCall_4_1_2_0()); }
		ruleComponentInstanceStateExpression
		{ after(grammarAccess.getDeltaInErroneousEffectAccess().getVaroutComponentInstanceStateExpressionParserRuleCall_4_1_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__FailureEventAssignment_5_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getDeltaInErroneousEffectAccess().getFailureEventComponentInstanceEventReferenceExpressionParserRuleCall_5_2_0()); }
		ruleComponentInstanceEventReferenceExpression
		{ after(grammarAccess.getDeltaInErroneousEffectAccess().getFailureEventComponentInstanceEventReferenceExpressionParserRuleCall_5_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeltaInErroneousEffect__TemplateAssignment_6
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getDeltaInErroneousEffectAccess().getTemplateTemplateParserRuleCall_6_0()); }
		ruleTemplate
		{ after(grammarAccess.getDeltaInErroneousEffectAccess().getTemplateTemplateParserRuleCall_6_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SelfFixTemplate__SelfFixEventAssignment_4_1_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSelfFixTemplateAccess().getSelfFixEventComponentInstanceEventReferenceExpressionParserRuleCall_4_1_2_0()); }
		ruleComponentInstanceEventReferenceExpression
		{ after(grammarAccess.getSelfFixTemplateAccess().getSelfFixEventComponentInstanceEventReferenceExpressionParserRuleCall_4_1_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__GlobalDynamics__TransitionsAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getGlobalDynamicsAccess().getTransitionsFaultTransitionParserRuleCall_2_0()); }
		ruleFaultTransition
		{ after(grammarAccess.getGlobalDynamicsAccess().getTransitionsFaultTransitionParserRuleCall_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultTransition__SourceAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getFaultTransitionAccess().getSourceFaultModeStateReferenceParserRuleCall_2_0()); }
		ruleFaultModeStateReference
		{ after(grammarAccess.getFaultTransitionAccess().getSourceFaultModeStateReferenceParserRuleCall_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultTransition__TargetAssignment_4
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getFaultTransitionAccess().getTargetFaultModeStateReferenceParserRuleCall_4_0()); }
		ruleFaultModeStateReference
		{ after(grammarAccess.getFaultTransitionAccess().getTargetFaultModeStateReferenceParserRuleCall_4_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultTransition__TriggerAssignment_5_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getFaultTransitionAccess().getTriggerFaultTransitionTriggerParserRuleCall_5_1_0()); }
		ruleFaultTransitionTrigger
		{ after(grammarAccess.getFaultTransitionAccess().getTriggerFaultTransitionTriggerParserRuleCall_5_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultTransition__GuardAssignment_6_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getFaultTransitionAccess().getGuardExpressionParserRuleCall_6_1_0()); }
		ruleExpression
		{ after(grammarAccess.getFaultTransitionAccess().getGuardExpressionParserRuleCall_6_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultModeStateReference__FaultModeAssignment_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getFaultModeStateReferenceAccess().getFaultModeFaultModeCrossReference_0_0()); }
		(
			{ before(grammarAccess.getFaultModeStateReferenceAccess().getFaultModeFaultModeIDTerminalRuleCall_0_0_1()); }
			RULE_ID
			{ after(grammarAccess.getFaultModeStateReferenceAccess().getFaultModeFaultModeIDTerminalRuleCall_0_0_1()); }
		)
		{ after(grammarAccess.getFaultModeStateReferenceAccess().getFaultModeFaultModeCrossReference_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultModeStateReference__StateAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getFaultModeStateReferenceAccess().getStateFaultModeStateEnumRuleCall_2_0()); }
		ruleFaultModeState
		{ after(grammarAccess.getFaultModeStateReferenceAccess().getStateFaultModeStateEnumRuleCall_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultTransitionTrigger__FaultModeAssignment_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getFaultTransitionTriggerAccess().getFaultModeFaultModeCrossReference_0_0()); }
		(
			{ before(grammarAccess.getFaultTransitionTriggerAccess().getFaultModeFaultModeIDTerminalRuleCall_0_0_1()); }
			RULE_ID
			{ after(grammarAccess.getFaultTransitionTriggerAccess().getFaultModeFaultModeIDTerminalRuleCall_0_0_1()); }
		)
		{ after(grammarAccess.getFaultTransitionTriggerAccess().getFaultModeFaultModeCrossReference_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__FaultTransitionTrigger__EventAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getFaultTransitionTriggerAccess().getEventFaultEventEnumRuleCall_2_0()); }
		ruleFaultEvent
		{ after(grammarAccess.getFaultTransitionTriggerAccess().getEventFaultEventEnumRuleCall_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauses__CommonCausesAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getCommonCausesAccess().getCommonCausesCommonCauseParserRuleCall_1_0()); }
		ruleCommonCause
		{ after(grammarAccess.getCommonCausesAccess().getCommonCausesCommonCauseParserRuleCall_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCause__NameAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getCommonCauseAccess().getNameIDTerminalRuleCall_1_0()); }
		RULE_ID
		{ after(grammarAccess.getCommonCauseAccess().getNameIDTerminalRuleCall_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCause__ProbabilityAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getCommonCauseAccess().getProbabilityCommonCauseProbabilityParserRuleCall_2_0()); }
		ruleCommonCauseProbability
		{ after(grammarAccess.getCommonCauseAccess().getProbabilityCommonCauseProbabilityParserRuleCall_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCause__ModesAssignment_3
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getCommonCauseAccess().getModesCommonCauseModeParserRuleCall_3_0()); }
		ruleCommonCauseMode
		{ after(grammarAccess.getCommonCauseAccess().getModesCommonCauseModeParserRuleCall_3_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauseProbability__ValueAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getCommonCauseProbabilityAccess().getValueExpressionParserRuleCall_1_0()); }
		ruleExpression
		{ after(grammarAccess.getCommonCauseProbabilityAccess().getValueExpressionParserRuleCall_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauseMode__FaultSliceAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getCommonCauseModeAccess().getFaultSliceFaultSliceCrossReference_1_0()); }
		(
			{ before(grammarAccess.getCommonCauseModeAccess().getFaultSliceFaultSliceIDTerminalRuleCall_1_0_1()); }
			RULE_ID
			{ after(grammarAccess.getCommonCauseModeAccess().getFaultSliceFaultSliceIDTerminalRuleCall_1_0_1()); }
		)
		{ after(grammarAccess.getCommonCauseModeAccess().getFaultSliceFaultSliceCrossReference_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauseMode__FaultModeAssignment_3
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getCommonCauseModeAccess().getFaultModeFaultModeCrossReference_3_0()); }
		(
			{ before(grammarAccess.getCommonCauseModeAccess().getFaultModeFaultModeIDTerminalRuleCall_3_0_1()); }
			RULE_ID
			{ after(grammarAccess.getCommonCauseModeAccess().getFaultModeFaultModeIDTerminalRuleCall_3_0_1()); }
		)
		{ after(grammarAccess.getCommonCauseModeAccess().getFaultModeFaultModeCrossReference_3_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauseMode__RangeAssignment_4
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getCommonCauseModeAccess().getRangeCommonCauseRangeParserRuleCall_4_0()); }
		ruleCommonCauseRange
		{ after(grammarAccess.getCommonCauseModeAccess().getRangeCommonCauseRangeParserRuleCall_4_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauseRange__LowerBoundAssignment_1_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getCommonCauseRangeAccess().getLowerBoundExpressionParserRuleCall_1_1_0()); }
		ruleExpression
		{ after(grammarAccess.getCommonCauseRangeAccess().getLowerBoundExpressionParserRuleCall_1_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauseRange__HigherBoundAssignment_2_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getCommonCauseRangeAccess().getHigherBoundExpressionParserRuleCall_2_1_0()); }
		ruleExpression
		{ after(grammarAccess.getCommonCauseRangeAccess().getHigherBoundExpressionParserRuleCall_2_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauseRange__LowerBoundAssignment_3_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getCommonCauseRangeAccess().getLowerBoundExpressionParserRuleCall_3_1_0()); }
		ruleExpression
		{ after(grammarAccess.getCommonCauseRangeAccess().getLowerBoundExpressionParserRuleCall_3_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__CommonCauseRange__HigherBoundAssignment_3_3
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getCommonCauseRangeAccess().getHigherBoundExpressionParserRuleCall_3_3_0()); }
		ruleExpression
		{ after(grammarAccess.getCommonCauseRangeAccess().getHigherBoundExpressionParserRuleCall_3_3_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__TimeoutDeclaration__NameAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getTimeoutDeclarationAccess().getNameIDTerminalRuleCall_1_0()); }
		RULE_ID
		{ after(grammarAccess.getTimeoutDeclarationAccess().getNameIDTerminalRuleCall_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__EqualTrigger__TypeAssignment_1_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getEqualTriggerAccess().getTypeEqualEnumRuleCall_1_1_0()); }
		ruleEqual
		{ after(grammarAccess.getEqualTriggerAccess().getTypeEqualEnumRuleCall_1_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__EqualTrigger__RightOperandAssignment_1_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getEqualTriggerAccess().getRightOperandEqualTriggerParserRuleCall_1_2_0()); }
		ruleEqualTrigger
		{ after(grammarAccess.getEqualTriggerAccess().getRightOperandEqualTriggerParserRuleCall_1_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ImplyTrigger__TypeAssignment_1_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getImplyTriggerAccess().getTypeImplyEnumRuleCall_1_1_0()); }
		ruleImply
		{ after(grammarAccess.getImplyTriggerAccess().getTypeImplyEnumRuleCall_1_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ImplyTrigger__RightOperandAssignment_1_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getImplyTriggerAccess().getRightOperandImplyTriggerParserRuleCall_1_2_0()); }
		ruleImplyTrigger
		{ after(grammarAccess.getImplyTriggerAccess().getRightOperandImplyTriggerParserRuleCall_1_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__OrTrigger__TypeAssignment_1_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getOrTriggerAccess().getTypeOrEnumRuleCall_1_1_0()); }
		ruleOr
		{ after(grammarAccess.getOrTriggerAccess().getTypeOrEnumRuleCall_1_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__OrTrigger__RightOperandAssignment_1_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getOrTriggerAccess().getRightOperandOrTriggerParserRuleCall_1_2_0()); }
		ruleOrTrigger
		{ after(grammarAccess.getOrTriggerAccess().getRightOperandOrTriggerParserRuleCall_1_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__XorTrigger__TypeAssignment_1_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getXorTriggerAccess().getTypeXorEnumRuleCall_1_1_0()); }
		ruleXor
		{ after(grammarAccess.getXorTriggerAccess().getTypeXorEnumRuleCall_1_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__XorTrigger__RightOperandAssignment_1_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getXorTriggerAccess().getRightOperandXorTriggerParserRuleCall_1_2_0()); }
		ruleXorTrigger
		{ after(grammarAccess.getXorTriggerAccess().getRightOperandXorTriggerParserRuleCall_1_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AndTrigger__TypeAssignment_1_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAndTriggerAccess().getTypeAndEnumRuleCall_1_1_0()); }
		ruleAnd
		{ after(grammarAccess.getAndTriggerAccess().getTypeAndEnumRuleCall_1_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AndTrigger__RightOperandAssignment_1_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAndTriggerAccess().getRightOperandAndTriggerParserRuleCall_1_2_0()); }
		ruleAndTrigger
		{ after(grammarAccess.getAndTriggerAccess().getRightOperandAndTriggerParserRuleCall_1_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__NotTrigger__TypeAssignment_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getNotTriggerAccess().getTypeNotEnumRuleCall_0_0()); }
		ruleNot
		{ after(grammarAccess.getNotTriggerAccess().getTypeNotEnumRuleCall_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__NotTrigger__OperandAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getNotTriggerAccess().getOperandParenthesesTriggerParserRuleCall_1_0()); }
		ruleParenthesesTrigger
		{ after(grammarAccess.getNotTriggerAccess().getOperandParenthesesTriggerParserRuleCall_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__OpaqueTrigger__TriggerAssignment
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getOpaqueTriggerAccess().getTriggerSTRINGTerminalRuleCall_0()); }
		RULE_STRING
		{ after(grammarAccess.getOpaqueTriggerAccess().getTriggerSTRINGTerminalRuleCall_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__EventTrigger__EventReferenceAssignment
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getEventTriggerAccess().getEventReferenceEventReferenceParserRuleCall_0()); }
		ruleEventReference
		{ after(grammarAccess.getEventTriggerAccess().getEventReferenceEventReferenceParserRuleCall_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ClockTickReference__ClockAssignment
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getClockTickReferenceAccess().getClockClockCrossReference_0()); }
		(
			{ before(grammarAccess.getClockTickReferenceAccess().getClockClockIDTerminalRuleCall_0_1()); }
			RULE_ID
			{ after(grammarAccess.getClockTickReferenceAccess().getClockClockIDTerminalRuleCall_0_1()); }
		)
		{ after(grammarAccess.getClockTickReferenceAccess().getClockClockCrossReference_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__TimeoutEventReference__TimeoutAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getTimeoutEventReferenceAccess().getTimeoutTimeoutDeclarationCrossReference_1_0()); }
		(
			{ before(grammarAccess.getTimeoutEventReferenceAccess().getTimeoutTimeoutDeclarationIDTerminalRuleCall_1_0_1()); }
			RULE_ID
			{ after(grammarAccess.getTimeoutEventReferenceAccess().getTimeoutTimeoutDeclarationIDTerminalRuleCall_1_0_1()); }
		)
		{ after(grammarAccess.getTimeoutEventReferenceAccess().getTimeoutTimeoutDeclarationCrossReference_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AnyPortEventReference__PortAssignment_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAnyPortEventReferenceAccess().getPortPortCrossReference_0_0()); }
		(
			{ before(grammarAccess.getAnyPortEventReferenceAccess().getPortPortIDTerminalRuleCall_0_0_1()); }
			RULE_ID
			{ after(grammarAccess.getAnyPortEventReferenceAccess().getPortPortIDTerminalRuleCall_0_0_1()); }
		)
		{ after(grammarAccess.getAnyPortEventReferenceAccess().getPortPortCrossReference_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__PortEventReference__PortAssignment_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getPortEventReferenceAccess().getPortPortCrossReference_0_0()); }
		(
			{ before(grammarAccess.getPortEventReferenceAccess().getPortPortIDTerminalRuleCall_0_0_1()); }
			RULE_ID
			{ after(grammarAccess.getPortEventReferenceAccess().getPortPortIDTerminalRuleCall_0_0_1()); }
		)
		{ after(grammarAccess.getPortEventReferenceAccess().getPortPortCrossReference_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__PortEventReference__EventAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getPortEventReferenceAccess().getEventEventCrossReference_2_0()); }
		(
			{ before(grammarAccess.getPortEventReferenceAccess().getEventEventIDTerminalRuleCall_2_0_1()); }
			RULE_ID
			{ after(grammarAccess.getPortEventReferenceAccess().getEventEventIDTerminalRuleCall_2_0_1()); }
		)
		{ after(grammarAccess.getPortEventReferenceAccess().getEventEventCrossReference_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__Port__NameAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getPortAccess().getNameIDTerminalRuleCall_1_0()); }
		RULE_ID
		{ after(grammarAccess.getPortAccess().getNameIDTerminalRuleCall_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__Port__InterfaceRealizationAssignment_3
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getPortAccess().getInterfaceRealizationInterfaceRealizationParserRuleCall_3_0()); }
		ruleInterfaceRealization
		{ after(grammarAccess.getPortAccess().getInterfaceRealizationInterfaceRealizationParserRuleCall_3_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__InterfaceRealization__RealizationModeAssignment_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getInterfaceRealizationAccess().getRealizationModeRealizationModeEnumRuleCall_0_0()); }
		ruleRealizationMode
		{ after(grammarAccess.getInterfaceRealizationAccess().getRealizationModeRealizationModeEnumRuleCall_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__InterfaceRealization__InterfaceAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getInterfaceRealizationAccess().getInterfaceInterfaceCrossReference_1_0()); }
		(
			{ before(grammarAccess.getInterfaceRealizationAccess().getInterfaceInterfaceIDTerminalRuleCall_1_0_1()); }
			RULE_ID
			{ after(grammarAccess.getInterfaceRealizationAccess().getInterfaceInterfaceIDTerminalRuleCall_1_0_1()); }
		)
		{ after(grammarAccess.getInterfaceRealizationAccess().getInterfaceInterfaceCrossReference_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__SchedulingOrderAssignment_0_0_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getSchedulingOrderSchedulingOrderEnumRuleCall_0_0_2_0()); }
		ruleSchedulingOrder
		{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getSchedulingOrderSchedulingOrderEnumRuleCall_0_0_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__OrthogonalRegionSchedulingOrderAssignment_0_1_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getOrthogonalRegionSchedulingOrderOrthogonalRegionSchedulingOrderEnumRuleCall_0_1_2_0()); }
		ruleOrthogonalRegionSchedulingOrder
		{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getOrthogonalRegionSchedulingOrderOrthogonalRegionSchedulingOrderEnumRuleCall_0_1_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__TransitionPriorityAssignment_0_2_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getTransitionPriorityTransitionPriorityEnumRuleCall_0_2_2_0()); }
		ruleTransitionPriority
		{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getTransitionPriorityTransitionPriorityEnumRuleCall_0_2_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__GuardEvaluationAssignment_0_3_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getGuardEvaluationGuardEvaluationEnumRuleCall_0_3_2_0()); }
		ruleGuardEvaluation
		{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getGuardEvaluationGuardEvaluationEnumRuleCall_0_3_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__AnnotationsAssignment_0_4_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getAnnotationsComponentAnnotationParserRuleCall_0_4_0_0()); }
		ruleComponentAnnotation
		{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getAnnotationsComponentAnnotationParserRuleCall_0_4_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__AnnotationsAssignment_0_4_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getAnnotationsStatechartContractAnnotationParserRuleCall_0_4_1_0()); }
		ruleStatechartContractAnnotation
		{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getAnnotationsStatechartContractAnnotationParserRuleCall_0_4_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__NameAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getNameIDTerminalRuleCall_2_0()); }
		RULE_ID
		{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getNameIDTerminalRuleCall_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__ParameterDeclarationsAssignment_3_1_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_3_1_0_0()); }
		ruleParameterDeclaration
		{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_3_1_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__ParameterDeclarationsAssignment_3_1_1_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_3_1_1_1_0()); }
		ruleParameterDeclaration
		{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_3_1_1_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__PortsAssignment_4_1_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getPortsPortParserRuleCall_4_1_0_0()); }
		rulePort
		{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getPortsPortParserRuleCall_4_1_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__PortsAssignment_4_1_1_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getPortsPortParserRuleCall_4_1_1_1_0()); }
		rulePort
		{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getPortsPortParserRuleCall_4_1_1_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__VariableDeclarationsAssignment_6_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getVariableDeclarationsVariableDeclarationParserRuleCall_6_0_0()); }
		ruleVariableDeclaration
		{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getVariableDeclarationsVariableDeclarationParserRuleCall_6_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__TimeoutDeclarationsAssignment_6_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getTimeoutDeclarationsTimeoutDeclarationParserRuleCall_6_1_0()); }
		ruleTimeoutDeclaration
		{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getTimeoutDeclarationsTimeoutDeclarationParserRuleCall_6_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__RegionsAssignment_7_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getRegionsRegionParserRuleCall_7_0_0()); }
		ruleRegion
		{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getRegionsRegionParserRuleCall_7_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__TransitionsAssignment_7_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getTransitionsTransitionParserRuleCall_7_1_0()); }
		ruleTransition
		{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getTransitionsTransitionParserRuleCall_7_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousStatechartDefinition__FunctionDeclarationsAssignment_8
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSynchronousStatechartDefinitionAccess().getFunctionDeclarationsFunctionDeclarationParserRuleCall_8_0()); }
		ruleFunctionDeclaration
		{ after(grammarAccess.getSynchronousStatechartDefinitionAccess().getFunctionDeclarationsFunctionDeclarationParserRuleCall_8_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__SchedulingOrderAssignment_1_0_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getSchedulingOrderSchedulingOrderEnumRuleCall_1_0_2_0()); }
		ruleSchedulingOrder
		{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getSchedulingOrderSchedulingOrderEnumRuleCall_1_0_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__OrthogonalRegionSchedulingOrderAssignment_1_1_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getOrthogonalRegionSchedulingOrderOrthogonalRegionSchedulingOrderEnumRuleCall_1_1_2_0()); }
		ruleOrthogonalRegionSchedulingOrder
		{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getOrthogonalRegionSchedulingOrderOrthogonalRegionSchedulingOrderEnumRuleCall_1_1_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__TransitionPriorityAssignment_1_2_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getTransitionPriorityTransitionPriorityEnumRuleCall_1_2_2_0()); }
		ruleTransitionPriority
		{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getTransitionPriorityTransitionPriorityEnumRuleCall_1_2_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__GuardEvaluationAssignment_1_3_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getGuardEvaluationGuardEvaluationEnumRuleCall_1_3_2_0()); }
		ruleGuardEvaluation
		{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getGuardEvaluationGuardEvaluationEnumRuleCall_1_3_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__AnnotationsAssignment_1_4_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getAnnotationsComponentAnnotationParserRuleCall_1_4_0_0()); }
		ruleComponentAnnotation
		{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getAnnotationsComponentAnnotationParserRuleCall_1_4_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__AnnotationsAssignment_1_4_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getAnnotationsStatechartContractAnnotationParserRuleCall_1_4_1_0()); }
		ruleStatechartContractAnnotation
		{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getAnnotationsStatechartContractAnnotationParserRuleCall_1_4_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__NameAssignment_3
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getNameIDTerminalRuleCall_3_0()); }
		RULE_ID
		{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getNameIDTerminalRuleCall_3_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__CapacityAssignment_4_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getCapacityExpressionParserRuleCall_4_2_0()); }
		ruleExpression
		{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getCapacityExpressionParserRuleCall_4_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__ParameterDeclarationsAssignment_5_1_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_5_1_0_0()); }
		ruleParameterDeclaration
		{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_5_1_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__ParameterDeclarationsAssignment_5_1_1_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_5_1_1_1_0()); }
		ruleParameterDeclaration
		{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_5_1_1_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__PortsAssignment_6_1_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getPortsPortParserRuleCall_6_1_0_0()); }
		rulePort
		{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getPortsPortParserRuleCall_6_1_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__PortsAssignment_6_1_1_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getPortsPortParserRuleCall_6_1_1_1_0()); }
		rulePort
		{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getPortsPortParserRuleCall_6_1_1_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__VariableDeclarationsAssignment_8_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getVariableDeclarationsVariableDeclarationParserRuleCall_8_0_0()); }
		ruleVariableDeclaration
		{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getVariableDeclarationsVariableDeclarationParserRuleCall_8_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__TimeoutDeclarationsAssignment_8_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getTimeoutDeclarationsTimeoutDeclarationParserRuleCall_8_1_0()); }
		ruleTimeoutDeclaration
		{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getTimeoutDeclarationsTimeoutDeclarationParserRuleCall_8_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__RegionsAssignment_9_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getRegionsRegionParserRuleCall_9_0_0()); }
		ruleRegion
		{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getRegionsRegionParserRuleCall_9_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__TransitionsAssignment_9_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getTransitionsTransitionParserRuleCall_9_1_0()); }
		ruleTransition
		{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getTransitionsTransitionParserRuleCall_9_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousStatechartDefinition__FunctionDeclarationsAssignment_10
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousStatechartDefinitionAccess().getFunctionDeclarationsFunctionDeclarationParserRuleCall_10_0()); }
		ruleFunctionDeclaration
		{ after(grammarAccess.getAsynchronousStatechartDefinitionAccess().getFunctionDeclarationsFunctionDeclarationParserRuleCall_10_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScenarioAllowedWaitingAnnotation__LowerLimitAssignment_3
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getScenarioAllowedWaitingAnnotationAccess().getLowerLimitExpressionParserRuleCall_3_0()); }
		ruleExpression
		{ after(grammarAccess.getScenarioAllowedWaitingAnnotationAccess().getLowerLimitExpressionParserRuleCall_3_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScenarioAllowedWaitingAnnotation__UpperLimitAssignment_5
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getScenarioAllowedWaitingAnnotationAccess().getUpperLimitExpressionParserRuleCall_5_0()); }
		ruleExpression
		{ after(grammarAccess.getScenarioAllowedWaitingAnnotationAccess().getUpperLimitExpressionParserRuleCall_5_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScenarioContractAnnotation__MonitoredComponentAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getScenarioContractAnnotationAccess().getMonitoredComponentComponentCrossReference_2_0()); }
		(
			{ before(grammarAccess.getScenarioContractAnnotationAccess().getMonitoredComponentComponentIDTerminalRuleCall_2_0_1()); }
			RULE_ID
			{ after(grammarAccess.getScenarioContractAnnotationAccess().getMonitoredComponentComponentIDTerminalRuleCall_2_0_1()); }
		)
		{ after(grammarAccess.getScenarioContractAnnotationAccess().getMonitoredComponentComponentCrossReference_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScenarioContractAnnotation__ScenarioTypeAssignment_4
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getScenarioContractAnnotationAccess().getScenarioTypeNotDefinedEventModeEnumRuleCall_4_0()); }
		ruleNotDefinedEventMode
		{ after(grammarAccess.getScenarioContractAnnotationAccess().getScenarioTypeNotDefinedEventModeEnumRuleCall_4_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AdaptiveContractAnnotation__MonitoredComponentAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAdaptiveContractAnnotationAccess().getMonitoredComponentComponentCrossReference_2_0()); }
		(
			{ before(grammarAccess.getAdaptiveContractAnnotationAccess().getMonitoredComponentComponentIDTerminalRuleCall_2_0_1()); }
			RULE_ID
			{ after(grammarAccess.getAdaptiveContractAnnotationAccess().getMonitoredComponentComponentIDTerminalRuleCall_2_0_1()); }
		)
		{ after(grammarAccess.getAdaptiveContractAnnotationAccess().getMonitoredComponentComponentCrossReference_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__Region__NameAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getRegionAccess().getNameIDTerminalRuleCall_1_0()); }
		RULE_ID
		{ after(grammarAccess.getRegionAccess().getNameIDTerminalRuleCall_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__Region__StateNodesAssignment_3
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getRegionAccess().getStateNodesStateNodeParserRuleCall_3_0()); }
		ruleStateNode
		{ after(grammarAccess.getRegionAccess().getStateNodesStateNodeParserRuleCall_3_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__AnnotationsAssignment_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getTransitionAccess().getAnnotationsTransitionAnnotationParserRuleCall_0_0()); }
		ruleTransitionAnnotation
		{ after(grammarAccess.getTransitionAccess().getAnnotationsTransitionAnnotationParserRuleCall_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__PriorityAssignment_2_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getTransitionAccess().getPriorityINTEGERTerminalRuleCall_2_1_0()); }
		RULE_INTEGER
		{ after(grammarAccess.getTransitionAccess().getPriorityINTEGERTerminalRuleCall_2_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__SourceStateAssignment_4
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getTransitionAccess().getSourceStateStateNodeCrossReference_4_0()); }
		(
			{ before(grammarAccess.getTransitionAccess().getSourceStateStateNodeIDTerminalRuleCall_4_0_1()); }
			RULE_ID
			{ after(grammarAccess.getTransitionAccess().getSourceStateStateNodeIDTerminalRuleCall_4_0_1()); }
		)
		{ after(grammarAccess.getTransitionAccess().getSourceStateStateNodeCrossReference_4_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__TargetStateAssignment_6
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getTransitionAccess().getTargetStateStateNodeCrossReference_6_0()); }
		(
			{ before(grammarAccess.getTransitionAccess().getTargetStateStateNodeIDTerminalRuleCall_6_0_1()); }
			RULE_ID
			{ after(grammarAccess.getTransitionAccess().getTargetStateStateNodeIDTerminalRuleCall_6_0_1()); }
		)
		{ after(grammarAccess.getTransitionAccess().getTargetStateStateNodeCrossReference_6_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__TriggerAssignment_7_0_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getTransitionAccess().getTriggerTriggerParserRuleCall_7_0_1_0()); }
		ruleTrigger
		{ after(grammarAccess.getTransitionAccess().getTriggerTriggerParserRuleCall_7_0_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__GuardAssignment_7_1_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getTransitionAccess().getGuardExpressionParserRuleCall_7_1_1_0()); }
		ruleExpression
		{ after(grammarAccess.getTransitionAccess().getGuardExpressionParserRuleCall_7_1_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__Transition__EffectsAssignment_7_2_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getTransitionAccess().getEffectsActionParserRuleCall_7_2_1_0()); }
		ruleAction
		{ after(grammarAccess.getTransitionAccess().getEffectsActionParserRuleCall_7_2_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__TransitionIdAnnotation__NameAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getTransitionIdAnnotationAccess().getNameIDTerminalRuleCall_2_0()); }
		RULE_ID
		{ after(grammarAccess.getTransitionIdAnnotationAccess().getNameIDTerminalRuleCall_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__State__AnnotationsAssignment_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getStateAccess().getAnnotationsStateAnnotationParserRuleCall_0_0()); }
		ruleStateAnnotation
		{ after(grammarAccess.getStateAccess().getAnnotationsStateAnnotationParserRuleCall_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__State__NameAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getStateAccess().getNameIDTerminalRuleCall_2_0()); }
		RULE_ID
		{ after(grammarAccess.getStateAccess().getNameIDTerminalRuleCall_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__State__InvariantsAssignment_3_1_0_0_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getStateAccess().getInvariantsExpressionParserRuleCall_3_1_0_0_1_0()); }
		ruleExpression
		{ after(grammarAccess.getStateAccess().getInvariantsExpressionParserRuleCall_3_1_0_0_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__State__EntryActionsAssignment_3_1_0_1_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getStateAccess().getEntryActionsActionParserRuleCall_3_1_0_1_2_0()); }
		ruleAction
		{ after(grammarAccess.getStateAccess().getEntryActionsActionParserRuleCall_3_1_0_1_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__State__ExitActionsAssignment_3_1_0_2_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getStateAccess().getExitActionsActionParserRuleCall_3_1_0_2_2_0()); }
		ruleAction
		{ after(grammarAccess.getStateAccess().getExitActionsActionParserRuleCall_3_1_0_2_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__State__RegionsAssignment_3_1_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getStateAccess().getRegionsRegionParserRuleCall_3_1_1_0()); }
		ruleRegion
		{ after(grammarAccess.getStateAccess().getRegionsRegionParserRuleCall_3_1_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SpecialStateAnnotation__KindAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSpecialStateAnnotationAccess().getKindSpecialStateKindEnumRuleCall_2_0()); }
		ruleSpecialStateKind
		{ after(grammarAccess.getSpecialStateAnnotationAccess().getKindSpecialStateKindEnumRuleCall_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__StateContractAnnotation__LinkTypeAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getStateContractAnnotationAccess().getLinkTypeLinkTypeEnumRuleCall_1_0()); }
		ruleLinkType
		{ after(grammarAccess.getStateContractAnnotationAccess().getLinkTypeLinkTypeEnumRuleCall_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__StateContractAnnotation__HasHistoryAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getStateContractAnnotationAccess().getHasHistoryHistoryKeyword_2_0()); }
		(
			{ before(grammarAccess.getStateContractAnnotationAccess().getHasHistoryHistoryKeyword_2_0()); }
			'history'
			{ after(grammarAccess.getStateContractAnnotationAccess().getHasHistoryHistoryKeyword_2_0()); }
		)
		{ after(grammarAccess.getStateContractAnnotationAccess().getHasHistoryHistoryKeyword_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__StateContractAnnotation__ContractStatechartAssignment_4
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getStateContractAnnotationAccess().getContractStatechartStatechartDefinitionCrossReference_4_0()); }
		(
			{ before(grammarAccess.getStateContractAnnotationAccess().getContractStatechartStatechartDefinitionIDTerminalRuleCall_4_0_1()); }
			RULE_ID
			{ after(grammarAccess.getStateContractAnnotationAccess().getContractStatechartStatechartDefinitionIDTerminalRuleCall_4_0_1()); }
		)
		{ after(grammarAccess.getStateContractAnnotationAccess().getContractStatechartStatechartDefinitionCrossReference_4_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__StateContractAnnotation__ArgumentsAssignment_5_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getStateContractAnnotationAccess().getArgumentsExpressionParserRuleCall_5_1_0()); }
		ruleExpression
		{ after(grammarAccess.getStateContractAnnotationAccess().getArgumentsExpressionParserRuleCall_5_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__StateContractAnnotation__ArgumentsAssignment_5_2_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getStateContractAnnotationAccess().getArgumentsExpressionParserRuleCall_5_2_1_0()); }
		ruleExpression
		{ after(grammarAccess.getStateContractAnnotationAccess().getArgumentsExpressionParserRuleCall_5_2_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__MissionPhaseStateAnnotation__HistoryAssignment_4_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getMissionPhaseStateAnnotationAccess().getHistoryHistoryEnumRuleCall_4_0_0()); }
		ruleHistory
		{ after(grammarAccess.getMissionPhaseStateAnnotationAccess().getHistoryHistoryEnumRuleCall_4_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__MissionPhaseStateAnnotation__ComponentAssignment_4_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getMissionPhaseStateAnnotationAccess().getComponentComponentInstanceParserRuleCall_4_1_0()); }
		ruleComponentInstance
		{ after(grammarAccess.getMissionPhaseStateAnnotationAccess().getComponentComponentInstanceParserRuleCall_4_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__MissionPhaseStateAnnotation__PortBindingsAssignment_4_3
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getMissionPhaseStateAnnotationAccess().getPortBindingsPortBindingParserRuleCall_4_3_0()); }
		rulePortBinding
		{ after(grammarAccess.getMissionPhaseStateAnnotationAccess().getPortBindingsPortBindingParserRuleCall_4_3_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__MissionPhaseStateAnnotation__VariableBindingsAssignment_4_4
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getMissionPhaseStateAnnotationAccess().getVariableBindingsVariableBindingParserRuleCall_4_4_0()); }
		ruleVariableBinding
		{ after(grammarAccess.getMissionPhaseStateAnnotationAccess().getVariableBindingsVariableBindingParserRuleCall_4_4_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__VariableBinding__StatechartVariableAssignment_0_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getVariableBindingAccess().getStatechartVariableVariableDeclarationCrossReference_0_1_0()); }
		(
			{ before(grammarAccess.getVariableBindingAccess().getStatechartVariableVariableDeclarationIDTerminalRuleCall_0_1_0_1()); }
			RULE_ID
			{ after(grammarAccess.getVariableBindingAccess().getStatechartVariableVariableDeclarationIDTerminalRuleCall_0_1_0_1()); }
		)
		{ after(grammarAccess.getVariableBindingAccess().getStatechartVariableVariableDeclarationCrossReference_0_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__VariableBinding__InstanceVariableReferenceAssignment_0_3
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getVariableBindingAccess().getInstanceVariableReferenceInstanceVariableReferenceParserRuleCall_0_3_0()); }
		ruleInstanceVariableReference
		{ after(grammarAccess.getVariableBindingAccess().getInstanceVariableReferenceInstanceVariableReferenceParserRuleCall_0_3_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__VariableBinding__StatechartVariableAssignment_1_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getVariableBindingAccess().getStatechartVariableVariableDeclarationCrossReference_1_1_0()); }
		(
			{ before(grammarAccess.getVariableBindingAccess().getStatechartVariableVariableDeclarationIDTerminalRuleCall_1_1_0_1()); }
			RULE_ID
			{ after(grammarAccess.getVariableBindingAccess().getStatechartVariableVariableDeclarationIDTerminalRuleCall_1_1_0_1()); }
		)
		{ after(grammarAccess.getVariableBindingAccess().getStatechartVariableVariableDeclarationCrossReference_1_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__VariableBinding__InstanceVariableReferenceAssignment_1_3
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getVariableBindingAccess().getInstanceVariableReferenceInstanceVariableReferenceParserRuleCall_1_3_0()); }
		ruleInstanceVariableReference
		{ after(grammarAccess.getVariableBindingAccess().getInstanceVariableReferenceInstanceVariableReferenceParserRuleCall_1_3_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__InstanceVariableReference__InstanceAssignment_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getInstanceVariableReferenceAccess().getInstanceComponentInstanceCrossReference_0_0()); }
		(
			{ before(grammarAccess.getInstanceVariableReferenceAccess().getInstanceComponentInstanceIDTerminalRuleCall_0_0_1()); }
			RULE_ID
			{ after(grammarAccess.getInstanceVariableReferenceAccess().getInstanceComponentInstanceIDTerminalRuleCall_0_0_1()); }
		)
		{ after(grammarAccess.getInstanceVariableReferenceAccess().getInstanceComponentInstanceCrossReference_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__InstanceVariableReference__VariableAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getInstanceVariableReferenceAccess().getVariableVariableDeclarationCrossReference_2_0()); }
		(
			{ before(grammarAccess.getInstanceVariableReferenceAccess().getVariableVariableDeclarationIDTerminalRuleCall_2_0_1()); }
			RULE_ID
			{ after(grammarAccess.getInstanceVariableReferenceAccess().getVariableVariableDeclarationIDTerminalRuleCall_2_0_1()); }
		)
		{ after(grammarAccess.getInstanceVariableReferenceAccess().getVariableVariableDeclarationCrossReference_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__MergeState__NameAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getMergeStateAccess().getNameIDTerminalRuleCall_1_0()); }
		RULE_ID
		{ after(grammarAccess.getMergeStateAccess().getNameIDTerminalRuleCall_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ChoiceState__NameAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getChoiceStateAccess().getNameIDTerminalRuleCall_1_0()); }
		RULE_ID
		{ after(grammarAccess.getChoiceStateAccess().getNameIDTerminalRuleCall_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__JoinState__NameAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getJoinStateAccess().getNameIDTerminalRuleCall_1_0()); }
		RULE_ID
		{ after(grammarAccess.getJoinStateAccess().getNameIDTerminalRuleCall_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ForkState__NameAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getForkStateAccess().getNameIDTerminalRuleCall_1_0()); }
		RULE_ID
		{ after(grammarAccess.getForkStateAccess().getNameIDTerminalRuleCall_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__InitialState__NameAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getInitialStateAccess().getNameIDTerminalRuleCall_1_0()); }
		RULE_ID
		{ after(grammarAccess.getInitialStateAccess().getNameIDTerminalRuleCall_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ShallowHistoryState__NameAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getShallowHistoryStateAccess().getNameIDTerminalRuleCall_2_0()); }
		RULE_ID
		{ after(grammarAccess.getShallowHistoryStateAccess().getNameIDTerminalRuleCall_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__DeepHistoryState__NameAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getDeepHistoryStateAccess().getNameIDTerminalRuleCall_2_0()); }
		RULE_ID
		{ after(grammarAccess.getDeepHistoryStateAccess().getNameIDTerminalRuleCall_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__AnnotationsAssignment_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getCascadeCompositeComponentAccess().getAnnotationsComponentAnnotationParserRuleCall_0_0()); }
		ruleComponentAnnotation
		{ after(grammarAccess.getCascadeCompositeComponentAccess().getAnnotationsComponentAnnotationParserRuleCall_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__NameAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getCascadeCompositeComponentAccess().getNameIDTerminalRuleCall_2_0()); }
		RULE_ID
		{ after(grammarAccess.getCascadeCompositeComponentAccess().getNameIDTerminalRuleCall_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__ParameterDeclarationsAssignment_3_1_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getCascadeCompositeComponentAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_3_1_0_0()); }
		ruleParameterDeclaration
		{ after(grammarAccess.getCascadeCompositeComponentAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_3_1_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__ParameterDeclarationsAssignment_3_1_1_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getCascadeCompositeComponentAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_3_1_1_1_0()); }
		ruleParameterDeclaration
		{ after(grammarAccess.getCascadeCompositeComponentAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_3_1_1_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__PortsAssignment_4_1_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getCascadeCompositeComponentAccess().getPortsPortParserRuleCall_4_1_0_0()); }
		rulePort
		{ after(grammarAccess.getCascadeCompositeComponentAccess().getPortsPortParserRuleCall_4_1_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__PortsAssignment_4_1_1_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getCascadeCompositeComponentAccess().getPortsPortParserRuleCall_4_1_1_1_0()); }
		rulePort
		{ after(grammarAccess.getCascadeCompositeComponentAccess().getPortsPortParserRuleCall_4_1_1_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__ComponentsAssignment_6_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getCascadeCompositeComponentAccess().getComponentsSynchronousComponentInstanceParserRuleCall_6_0_0()); }
		ruleSynchronousComponentInstance
		{ after(grammarAccess.getCascadeCompositeComponentAccess().getComponentsSynchronousComponentInstanceParserRuleCall_6_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__PortBindingsAssignment_6_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getCascadeCompositeComponentAccess().getPortBindingsPortBindingParserRuleCall_6_1_0()); }
		rulePortBinding
		{ after(grammarAccess.getCascadeCompositeComponentAccess().getPortBindingsPortBindingParserRuleCall_6_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__ChannelsAssignment_6_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getCascadeCompositeComponentAccess().getChannelsChannelParserRuleCall_6_2_0()); }
		ruleChannel
		{ after(grammarAccess.getCascadeCompositeComponentAccess().getChannelsChannelParserRuleCall_6_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__InitialExecutionListAssignment_7_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getCascadeCompositeComponentAccess().getInitialExecutionListAtomicComponentInstanceReferenceExpressionParserRuleCall_7_2_0()); }
		ruleAtomicComponentInstanceReferenceExpression
		{ after(grammarAccess.getCascadeCompositeComponentAccess().getInitialExecutionListAtomicComponentInstanceReferenceExpressionParserRuleCall_7_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__InitialExecutionListAssignment_7_3_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getCascadeCompositeComponentAccess().getInitialExecutionListAtomicComponentInstanceReferenceExpressionParserRuleCall_7_3_1_0()); }
		ruleAtomicComponentInstanceReferenceExpression
		{ after(grammarAccess.getCascadeCompositeComponentAccess().getInitialExecutionListAtomicComponentInstanceReferenceExpressionParserRuleCall_7_3_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__ExecutionListAssignment_8_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getCascadeCompositeComponentAccess().getExecutionListAtomicComponentInstanceReferenceExpressionParserRuleCall_8_1_0()); }
		ruleAtomicComponentInstanceReferenceExpression
		{ after(grammarAccess.getCascadeCompositeComponentAccess().getExecutionListAtomicComponentInstanceReferenceExpressionParserRuleCall_8_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__CascadeCompositeComponent__ExecutionListAssignment_8_2_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getCascadeCompositeComponentAccess().getExecutionListAtomicComponentInstanceReferenceExpressionParserRuleCall_8_2_1_0()); }
		ruleAtomicComponentInstanceReferenceExpression
		{ after(grammarAccess.getCascadeCompositeComponentAccess().getExecutionListAtomicComponentInstanceReferenceExpressionParserRuleCall_8_2_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__AnnotationsAssignment_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSynchronousCompositeComponentAccess().getAnnotationsComponentAnnotationParserRuleCall_0_0()); }
		ruleComponentAnnotation
		{ after(grammarAccess.getSynchronousCompositeComponentAccess().getAnnotationsComponentAnnotationParserRuleCall_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__NameAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSynchronousCompositeComponentAccess().getNameIDTerminalRuleCall_2_0()); }
		RULE_ID
		{ after(grammarAccess.getSynchronousCompositeComponentAccess().getNameIDTerminalRuleCall_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__ParameterDeclarationsAssignment_3_1_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSynchronousCompositeComponentAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_3_1_0_0()); }
		ruleParameterDeclaration
		{ after(grammarAccess.getSynchronousCompositeComponentAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_3_1_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__ParameterDeclarationsAssignment_3_1_1_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSynchronousCompositeComponentAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_3_1_1_1_0()); }
		ruleParameterDeclaration
		{ after(grammarAccess.getSynchronousCompositeComponentAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_3_1_1_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__PortsAssignment_4_1_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSynchronousCompositeComponentAccess().getPortsPortParserRuleCall_4_1_0_0()); }
		rulePort
		{ after(grammarAccess.getSynchronousCompositeComponentAccess().getPortsPortParserRuleCall_4_1_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__PortsAssignment_4_1_1_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSynchronousCompositeComponentAccess().getPortsPortParserRuleCall_4_1_1_1_0()); }
		rulePort
		{ after(grammarAccess.getSynchronousCompositeComponentAccess().getPortsPortParserRuleCall_4_1_1_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__ComponentsAssignment_6_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSynchronousCompositeComponentAccess().getComponentsSynchronousComponentInstanceParserRuleCall_6_0_0()); }
		ruleSynchronousComponentInstance
		{ after(grammarAccess.getSynchronousCompositeComponentAccess().getComponentsSynchronousComponentInstanceParserRuleCall_6_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__PortBindingsAssignment_6_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSynchronousCompositeComponentAccess().getPortBindingsPortBindingParserRuleCall_6_1_0()); }
		rulePortBinding
		{ after(grammarAccess.getSynchronousCompositeComponentAccess().getPortBindingsPortBindingParserRuleCall_6_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousCompositeComponent__ChannelsAssignment_6_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSynchronousCompositeComponentAccess().getChannelsChannelParserRuleCall_6_2_0()); }
		ruleChannel
		{ after(grammarAccess.getSynchronousCompositeComponentAccess().getChannelsChannelParserRuleCall_6_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__AnnotationsAssignment_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousCompositeComponentAccess().getAnnotationsComponentAnnotationParserRuleCall_0_0()); }
		ruleComponentAnnotation
		{ after(grammarAccess.getAsynchronousCompositeComponentAccess().getAnnotationsComponentAnnotationParserRuleCall_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__NameAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousCompositeComponentAccess().getNameIDTerminalRuleCall_2_0()); }
		RULE_ID
		{ after(grammarAccess.getAsynchronousCompositeComponentAccess().getNameIDTerminalRuleCall_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__ParameterDeclarationsAssignment_3_1_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousCompositeComponentAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_3_1_0_0()); }
		ruleParameterDeclaration
		{ after(grammarAccess.getAsynchronousCompositeComponentAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_3_1_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__ParameterDeclarationsAssignment_3_1_1_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousCompositeComponentAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_3_1_1_1_0()); }
		ruleParameterDeclaration
		{ after(grammarAccess.getAsynchronousCompositeComponentAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_3_1_1_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__PortsAssignment_4_1_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousCompositeComponentAccess().getPortsPortParserRuleCall_4_1_0_0()); }
		rulePort
		{ after(grammarAccess.getAsynchronousCompositeComponentAccess().getPortsPortParserRuleCall_4_1_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__PortsAssignment_4_1_1_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousCompositeComponentAccess().getPortsPortParserRuleCall_4_1_1_1_0()); }
		rulePort
		{ after(grammarAccess.getAsynchronousCompositeComponentAccess().getPortsPortParserRuleCall_4_1_1_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__ComponentsAssignment_6_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousCompositeComponentAccess().getComponentsAsynchronousComponentInstanceParserRuleCall_6_0_0()); }
		ruleAsynchronousComponentInstance
		{ after(grammarAccess.getAsynchronousCompositeComponentAccess().getComponentsAsynchronousComponentInstanceParserRuleCall_6_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__PortBindingsAssignment_6_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousCompositeComponentAccess().getPortBindingsPortBindingParserRuleCall_6_1_0()); }
		rulePortBinding
		{ after(grammarAccess.getAsynchronousCompositeComponentAccess().getPortBindingsPortBindingParserRuleCall_6_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousCompositeComponent__ChannelsAssignment_6_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousCompositeComponentAccess().getChannelsChannelParserRuleCall_6_2_0()); }
		ruleChannel
		{ after(grammarAccess.getAsynchronousCompositeComponentAccess().getChannelsChannelParserRuleCall_6_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__AnnotationsAssignment_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getAnnotationsComponentAnnotationParserRuleCall_0_0()); }
		ruleComponentAnnotation
		{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getAnnotationsComponentAnnotationParserRuleCall_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__NameAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getNameIDTerminalRuleCall_2_0()); }
		RULE_ID
		{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getNameIDTerminalRuleCall_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__ParameterDeclarationsAssignment_3_1_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_3_1_0_0()); }
		ruleParameterDeclaration
		{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_3_1_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__ParameterDeclarationsAssignment_3_1_1_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_3_1_1_1_0()); }
		ruleParameterDeclaration
		{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_3_1_1_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__PortsAssignment_4_1_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getPortsPortParserRuleCall_4_1_0_0()); }
		rulePort
		{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getPortsPortParserRuleCall_4_1_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__PortsAssignment_4_1_1_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getPortsPortParserRuleCall_4_1_1_1_0()); }
		rulePort
		{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getPortsPortParserRuleCall_4_1_1_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__ComponentsAssignment_6_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getComponentsAsynchronousComponentInstanceParserRuleCall_6_0_0()); }
		ruleAsynchronousComponentInstance
		{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getComponentsAsynchronousComponentInstanceParserRuleCall_6_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__PortBindingsAssignment_6_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getPortBindingsPortBindingParserRuleCall_6_1_0()); }
		rulePortBinding
		{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getPortBindingsPortBindingParserRuleCall_6_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__ChannelsAssignment_6_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getChannelsChannelParserRuleCall_6_2_0()); }
		ruleChannel
		{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getChannelsChannelParserRuleCall_6_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__InitialExecutionListAssignment_7_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getInitialExecutionListAtomicComponentInstanceReferenceExpressionParserRuleCall_7_2_0()); }
		ruleAtomicComponentInstanceReferenceExpression
		{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getInitialExecutionListAtomicComponentInstanceReferenceExpressionParserRuleCall_7_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__InitialExecutionListAssignment_7_3_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getInitialExecutionListAtomicComponentInstanceReferenceExpressionParserRuleCall_7_3_1_0()); }
		ruleAtomicComponentInstanceReferenceExpression
		{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getInitialExecutionListAtomicComponentInstanceReferenceExpressionParserRuleCall_7_3_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__ExecutionListAssignment_8_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getExecutionListAtomicComponentInstanceReferenceExpressionParserRuleCall_8_1_0()); }
		ruleAtomicComponentInstanceReferenceExpression
		{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getExecutionListAtomicComponentInstanceReferenceExpressionParserRuleCall_8_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ScheduledAsynchronousCompositeComponent__ExecutionListAssignment_8_2_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getExecutionListAtomicComponentInstanceReferenceExpressionParserRuleCall_8_2_1_0()); }
		ruleAtomicComponentInstanceReferenceExpression
		{ after(grammarAccess.getScheduledAsynchronousCompositeComponentAccess().getExecutionListAtomicComponentInstanceReferenceExpressionParserRuleCall_8_2_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__AnnotationsAssignment_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousAdapterAccess().getAnnotationsComponentAnnotationParserRuleCall_0_0()); }
		ruleComponentAnnotation
		{ after(grammarAccess.getAsynchronousAdapterAccess().getAnnotationsComponentAnnotationParserRuleCall_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__NameAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousAdapterAccess().getNameIDTerminalRuleCall_2_0()); }
		RULE_ID
		{ after(grammarAccess.getAsynchronousAdapterAccess().getNameIDTerminalRuleCall_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__ParameterDeclarationsAssignment_3_1_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousAdapterAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_3_1_0_0()); }
		ruleParameterDeclaration
		{ after(grammarAccess.getAsynchronousAdapterAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_3_1_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__ParameterDeclarationsAssignment_3_1_1_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousAdapterAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_3_1_1_1_0()); }
		ruleParameterDeclaration
		{ after(grammarAccess.getAsynchronousAdapterAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_3_1_1_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__WrappedComponentAssignment_5
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousAdapterAccess().getWrappedComponentSynchronousComponentInstanceParserRuleCall_5_0()); }
		ruleSynchronousComponentInstance
		{ after(grammarAccess.getAsynchronousAdapterAccess().getWrappedComponentSynchronousComponentInstanceParserRuleCall_5_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__PortsAssignment_6_1_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousAdapterAccess().getPortsPortParserRuleCall_6_1_0_0()); }
		rulePort
		{ after(grammarAccess.getAsynchronousAdapterAccess().getPortsPortParserRuleCall_6_1_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__PortsAssignment_6_1_1_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousAdapterAccess().getPortsPortParserRuleCall_6_1_1_1_0()); }
		rulePort
		{ after(grammarAccess.getAsynchronousAdapterAccess().getPortsPortParserRuleCall_6_1_1_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__ClocksAssignment_8_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousAdapterAccess().getClocksClockDeclarationParserRuleCall_8_0_0()); }
		ruleClockDeclaration
		{ after(grammarAccess.getAsynchronousAdapterAccess().getClocksClockDeclarationParserRuleCall_8_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__ControlSpecificationsAssignment_8_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousAdapterAccess().getControlSpecificationsControlSpecificaitonParserRuleCall_8_1_0()); }
		ruleControlSpecificaiton
		{ after(grammarAccess.getAsynchronousAdapterAccess().getControlSpecificationsControlSpecificaitonParserRuleCall_8_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousAdapter__MessageQueuesAssignment_8_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousAdapterAccess().getMessageQueuesMessageQueueParserRuleCall_8_2_0()); }
		ruleMessageQueue
		{ after(grammarAccess.getAsynchronousAdapterAccess().getMessageQueuesMessageQueueParserRuleCall_8_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ClockDeclaration__NameAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getClockDeclarationAccess().getNameIDTerminalRuleCall_1_0()); }
		RULE_ID
		{ after(grammarAccess.getClockDeclarationAccess().getNameIDTerminalRuleCall_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ClockDeclaration__TimeSpecificationAssignment_5
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getClockDeclarationAccess().getTimeSpecificationTimeSpecificationParserRuleCall_5_0()); }
		ruleTimeSpecification
		{ after(grammarAccess.getClockDeclarationAccess().getTimeSpecificationTimeSpecificationParserRuleCall_5_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__TimeSpecification__ValueAssignment_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getTimeSpecificationAccess().getValueAdditiveExpressionParserRuleCall_0_0()); }
		ruleAdditiveExpression
		{ after(grammarAccess.getTimeSpecificationAccess().getValueAdditiveExpressionParserRuleCall_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__TimeSpecification__UnitAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getTimeSpecificationAccess().getUnitTimeUnitEnumRuleCall_1_0()); }
		ruleTimeUnit
		{ after(grammarAccess.getTimeSpecificationAccess().getUnitTimeUnitEnumRuleCall_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ControlSpecificaiton__TriggerAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getControlSpecificaitonAccess().getTriggerSimpleTriggerParserRuleCall_1_0()); }
		ruleSimpleTrigger
		{ after(grammarAccess.getControlSpecificaitonAccess().getTriggerSimpleTriggerParserRuleCall_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ControlSpecificaiton__ControlFunctionAssignment_3
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getControlSpecificaitonAccess().getControlFunctionControlFunctionEnumRuleCall_3_0()); }
		ruleControlFunction
		{ after(grammarAccess.getControlSpecificaitonAccess().getControlFunctionControlFunctionEnumRuleCall_3_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__NameAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getMessageQueueAccess().getNameIDTerminalRuleCall_1_0()); }
		RULE_ID
		{ after(grammarAccess.getMessageQueueAccess().getNameIDTerminalRuleCall_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__PriorityAssignment_3_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getMessageQueueAccess().getPriorityINTEGERTerminalRuleCall_3_2_0()); }
		RULE_INTEGER
		{ after(grammarAccess.getMessageQueueAccess().getPriorityINTEGERTerminalRuleCall_3_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__CapacityAssignment_4_3
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getMessageQueueAccess().getCapacityExpressionParserRuleCall_4_3_0()); }
		ruleExpression
		{ after(grammarAccess.getMessageQueueAccess().getCapacityExpressionParserRuleCall_4_3_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__EventDiscardStrategyAssignment_5_3
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getMessageQueueAccess().getEventDiscardStrategyDiscardStrategyEnumRuleCall_5_3_0()); }
		ruleDiscardStrategy
		{ after(grammarAccess.getMessageQueueAccess().getEventDiscardStrategyDiscardStrategyEnumRuleCall_5_3_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__EventPassingsAssignment_8
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getMessageQueueAccess().getEventPassingsEventPassingParserRuleCall_8_0()); }
		ruleEventPassing
		{ after(grammarAccess.getMessageQueueAccess().getEventPassingsEventPassingParserRuleCall_8_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__MessageQueue__EventPassingsAssignment_9_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getMessageQueueAccess().getEventPassingsEventPassingParserRuleCall_9_1_0()); }
		ruleEventPassing
		{ after(grammarAccess.getMessageQueueAccess().getEventPassingsEventPassingParserRuleCall_9_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__EventPassing__SourceAssignment_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getEventPassingAccess().getSourceEventReferenceParserRuleCall_0_0()); }
		ruleEventReference
		{ after(grammarAccess.getEventPassingAccess().getSourceEventReferenceParserRuleCall_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__EventPassing__TargetAssignment_1_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getEventPassingAccess().getTargetEventReferenceParserRuleCall_1_1_0()); }
		ruleEventReference
		{ after(grammarAccess.getEventPassingAccess().getTargetEventReferenceParserRuleCall_1_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__PortBinding__CompositeSystemPortAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getPortBindingAccess().getCompositeSystemPortPortCrossReference_1_0()); }
		(
			{ before(grammarAccess.getPortBindingAccess().getCompositeSystemPortPortIDTerminalRuleCall_1_0_1()); }
			RULE_ID
			{ after(grammarAccess.getPortBindingAccess().getCompositeSystemPortPortIDTerminalRuleCall_1_0_1()); }
		)
		{ after(grammarAccess.getPortBindingAccess().getCompositeSystemPortPortCrossReference_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__PortBinding__InstancePortReferenceAssignment_3
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getPortBindingAccess().getInstancePortReferenceInstancePortReferenceParserRuleCall_3_0()); }
		ruleInstancePortReference
		{ after(grammarAccess.getPortBindingAccess().getInstancePortReferenceInstancePortReferenceParserRuleCall_3_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__InstancePortReference__InstanceAssignment_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getInstancePortReferenceAccess().getInstanceComponentInstanceCrossReference_0_0()); }
		(
			{ before(grammarAccess.getInstancePortReferenceAccess().getInstanceComponentInstanceIDTerminalRuleCall_0_0_1()); }
			RULE_ID
			{ after(grammarAccess.getInstancePortReferenceAccess().getInstanceComponentInstanceIDTerminalRuleCall_0_0_1()); }
		)
		{ after(grammarAccess.getInstancePortReferenceAccess().getInstanceComponentInstanceCrossReference_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__InstancePortReference__PortAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getInstancePortReferenceAccess().getPortPortCrossReference_2_0()); }
		(
			{ before(grammarAccess.getInstancePortReferenceAccess().getPortPortIDTerminalRuleCall_2_0_1()); }
			RULE_ID
			{ after(grammarAccess.getInstancePortReferenceAccess().getPortPortIDTerminalRuleCall_2_0_1()); }
		)
		{ after(grammarAccess.getInstancePortReferenceAccess().getPortPortCrossReference_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AtomicComponentInstanceReferenceExpression__ComponentInstanceAssignment
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAtomicComponentInstanceReferenceExpressionAccess().getComponentInstanceComponentInstanceCrossReference_0()); }
		(
			{ before(grammarAccess.getAtomicComponentInstanceReferenceExpressionAccess().getComponentInstanceComponentInstanceIDTerminalRuleCall_0_1()); }
			RULE_ID
			{ after(grammarAccess.getAtomicComponentInstanceReferenceExpressionAccess().getComponentInstanceComponentInstanceIDTerminalRuleCall_0_1()); }
		)
		{ after(grammarAccess.getAtomicComponentInstanceReferenceExpressionAccess().getComponentInstanceComponentInstanceCrossReference_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceReferenceExpression__ComponentInstanceAssignment_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getComponentInstanceReferenceExpressionAccess().getComponentInstanceComponentInstanceCrossReference_0_0()); }
		(
			{ before(grammarAccess.getComponentInstanceReferenceExpressionAccess().getComponentInstanceComponentInstanceIDTerminalRuleCall_0_0_1()); }
			RULE_ID
			{ after(grammarAccess.getComponentInstanceReferenceExpressionAccess().getComponentInstanceComponentInstanceIDTerminalRuleCall_0_0_1()); }
		)
		{ after(grammarAccess.getComponentInstanceReferenceExpressionAccess().getComponentInstanceComponentInstanceCrossReference_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceReferenceExpression__ChildAssignment_1_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getComponentInstanceReferenceExpressionAccess().getChildComponentInstanceReferenceExpressionParserRuleCall_1_1_0()); }
		ruleComponentInstanceReferenceExpression
		{ after(grammarAccess.getComponentInstanceReferenceExpressionAccess().getChildComponentInstanceReferenceExpressionParserRuleCall_1_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousComponentInstance__NameAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSynchronousComponentInstanceAccess().getNameIDTerminalRuleCall_1_0()); }
		RULE_ID
		{ after(grammarAccess.getSynchronousComponentInstanceAccess().getNameIDTerminalRuleCall_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousComponentInstance__TypeAssignment_3
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSynchronousComponentInstanceAccess().getTypeSynchronousComponentCrossReference_3_0()); }
		(
			{ before(grammarAccess.getSynchronousComponentInstanceAccess().getTypeSynchronousComponentIDTerminalRuleCall_3_0_1()); }
			RULE_ID
			{ after(grammarAccess.getSynchronousComponentInstanceAccess().getTypeSynchronousComponentIDTerminalRuleCall_3_0_1()); }
		)
		{ after(grammarAccess.getSynchronousComponentInstanceAccess().getTypeSynchronousComponentCrossReference_3_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousComponentInstance__ArgumentsAssignment_4_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSynchronousComponentInstanceAccess().getArgumentsExpressionParserRuleCall_4_1_0()); }
		ruleExpression
		{ after(grammarAccess.getSynchronousComponentInstanceAccess().getArgumentsExpressionParserRuleCall_4_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SynchronousComponentInstance__ArgumentsAssignment_4_2_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSynchronousComponentInstanceAccess().getArgumentsExpressionParserRuleCall_4_2_1_0()); }
		ruleExpression
		{ after(grammarAccess.getSynchronousComponentInstanceAccess().getArgumentsExpressionParserRuleCall_4_2_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousComponentInstance__NameAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousComponentInstanceAccess().getNameIDTerminalRuleCall_1_0()); }
		RULE_ID
		{ after(grammarAccess.getAsynchronousComponentInstanceAccess().getNameIDTerminalRuleCall_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousComponentInstance__TypeAssignment_3
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousComponentInstanceAccess().getTypeAsynchronousComponentCrossReference_3_0()); }
		(
			{ before(grammarAccess.getAsynchronousComponentInstanceAccess().getTypeAsynchronousComponentIDTerminalRuleCall_3_0_1()); }
			RULE_ID
			{ after(grammarAccess.getAsynchronousComponentInstanceAccess().getTypeAsynchronousComponentIDTerminalRuleCall_3_0_1()); }
		)
		{ after(grammarAccess.getAsynchronousComponentInstanceAccess().getTypeAsynchronousComponentCrossReference_3_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousComponentInstance__ArgumentsAssignment_4_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousComponentInstanceAccess().getArgumentsExpressionParserRuleCall_4_1_0()); }
		ruleExpression
		{ after(grammarAccess.getAsynchronousComponentInstanceAccess().getArgumentsExpressionParserRuleCall_4_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AsynchronousComponentInstance__ArgumentsAssignment_4_2_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAsynchronousComponentInstanceAccess().getArgumentsExpressionParserRuleCall_4_2_1_0()); }
		ruleExpression
		{ after(grammarAccess.getAsynchronousComponentInstanceAccess().getArgumentsExpressionParserRuleCall_4_2_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SimpleChannel__ProvidedPortAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSimpleChannelAccess().getProvidedPortInstancePortReferenceParserRuleCall_2_0()); }
		ruleInstancePortReference
		{ after(grammarAccess.getSimpleChannelAccess().getProvidedPortInstancePortReferenceParserRuleCall_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SimpleChannel__RequiredPortAssignment_6
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSimpleChannelAccess().getRequiredPortInstancePortReferenceParserRuleCall_6_0()); }
		ruleInstancePortReference
		{ after(grammarAccess.getSimpleChannelAccess().getRequiredPortInstancePortReferenceParserRuleCall_6_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__BroadcastChannel__ProvidedPortAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getBroadcastChannelAccess().getProvidedPortInstancePortReferenceParserRuleCall_2_0()); }
		ruleInstancePortReference
		{ after(grammarAccess.getBroadcastChannelAccess().getProvidedPortInstancePortReferenceParserRuleCall_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__BroadcastChannel__RequiredPortsAssignment_6
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getBroadcastChannelAccess().getRequiredPortsInstancePortReferenceParserRuleCall_6_0()); }
		ruleInstancePortReference
		{ after(grammarAccess.getBroadcastChannelAccess().getRequiredPortsInstancePortReferenceParserRuleCall_6_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__BroadcastChannel__RequiredPortsAssignment_7_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getBroadcastChannelAccess().getRequiredPortsInstancePortReferenceParserRuleCall_7_1_0()); }
		ruleInstancePortReference
		{ after(grammarAccess.getBroadcastChannelAccess().getRequiredPortsInstancePortReferenceParserRuleCall_7_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__Interface__NameAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getInterfaceAccess().getNameIDTerminalRuleCall_2_0()); }
		RULE_ID
		{ after(grammarAccess.getInterfaceAccess().getNameIDTerminalRuleCall_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__Interface__ParentsAssignment_3_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getInterfaceAccess().getParentsInterfaceCrossReference_3_1_0()); }
		(
			{ before(grammarAccess.getInterfaceAccess().getParentsInterfaceIDTerminalRuleCall_3_1_0_1()); }
			RULE_ID
			{ after(grammarAccess.getInterfaceAccess().getParentsInterfaceIDTerminalRuleCall_3_1_0_1()); }
		)
		{ after(grammarAccess.getInterfaceAccess().getParentsInterfaceCrossReference_3_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__Interface__ParentsAssignment_3_2_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getInterfaceAccess().getParentsInterfaceCrossReference_3_2_1_0()); }
		(
			{ before(grammarAccess.getInterfaceAccess().getParentsInterfaceIDTerminalRuleCall_3_2_1_0_1()); }
			RULE_ID
			{ after(grammarAccess.getInterfaceAccess().getParentsInterfaceIDTerminalRuleCall_3_2_1_0_1()); }
		)
		{ after(grammarAccess.getInterfaceAccess().getParentsInterfaceCrossReference_3_2_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__Interface__EventsAssignment_5
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getInterfaceAccess().getEventsEventDeclarationParserRuleCall_5_0()); }
		ruleEventDeclaration
		{ after(grammarAccess.getInterfaceAccess().getEventsEventDeclarationParserRuleCall_5_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__EventDeclaration__DirectionAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getEventDeclarationAccess().getDirectionEventDirectionEnumRuleCall_1_0()); }
		ruleEventDirection
		{ after(grammarAccess.getEventDeclarationAccess().getDirectionEventDirectionEnumRuleCall_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__EventDeclaration__EventAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getEventDeclarationAccess().getEventEventParserRuleCall_2_0()); }
		ruleEvent
		{ after(grammarAccess.getEventDeclarationAccess().getEventEventParserRuleCall_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__Event__PersistencyAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getEventAccess().getPersistencyPersistencyEnumRuleCall_1_0()); }
		rulePersistency
		{ after(grammarAccess.getEventAccess().getPersistencyPersistencyEnumRuleCall_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__Event__NameAssignment_3
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getEventAccess().getNameIDTerminalRuleCall_3_0()); }
		RULE_ID
		{ after(grammarAccess.getEventAccess().getNameIDTerminalRuleCall_3_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__Event__ParameterDeclarationsAssignment_4_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getEventAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_4_1_0()); }
		ruleParameterDeclaration
		{ after(grammarAccess.getEventAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_4_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__Event__ParameterDeclarationsAssignment_4_2_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getEventAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_4_2_1_0()); }
		ruleParameterDeclaration
		{ after(grammarAccess.getEventAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_4_2_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__Event__PriorityAssignment_5_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getEventAccess().getPriorityINTEGERTerminalRuleCall_5_2_0()); }
		RULE_INTEGER
		{ after(grammarAccess.getEventAccess().getPriorityINTEGERTerminalRuleCall_5_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__RaiseEventAction__PortAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getRaiseEventActionAccess().getPortPortCrossReference_1_0()); }
		(
			{ before(grammarAccess.getRaiseEventActionAccess().getPortPortIDTerminalRuleCall_1_0_1()); }
			RULE_ID
			{ after(grammarAccess.getRaiseEventActionAccess().getPortPortIDTerminalRuleCall_1_0_1()); }
		)
		{ after(grammarAccess.getRaiseEventActionAccess().getPortPortCrossReference_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__RaiseEventAction__EventAssignment_3
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getRaiseEventActionAccess().getEventEventCrossReference_3_0()); }
		(
			{ before(grammarAccess.getRaiseEventActionAccess().getEventEventIDTerminalRuleCall_3_0_1()); }
			RULE_ID
			{ after(grammarAccess.getRaiseEventActionAccess().getEventEventIDTerminalRuleCall_3_0_1()); }
		)
		{ after(grammarAccess.getRaiseEventActionAccess().getEventEventCrossReference_3_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__RaiseEventAction__ArgumentsAssignment_4_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getRaiseEventActionAccess().getArgumentsExpressionParserRuleCall_4_1_0()); }
		ruleExpression
		{ after(grammarAccess.getRaiseEventActionAccess().getArgumentsExpressionParserRuleCall_4_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__RaiseEventAction__ArgumentsAssignment_4_2_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getRaiseEventActionAccess().getArgumentsExpressionParserRuleCall_4_2_1_0()); }
		ruleExpression
		{ after(grammarAccess.getRaiseEventActionAccess().getArgumentsExpressionParserRuleCall_4_2_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SetTimeoutAction__TimeoutDeclarationAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSetTimeoutActionAccess().getTimeoutDeclarationTimeoutDeclarationCrossReference_1_0()); }
		(
			{ before(grammarAccess.getSetTimeoutActionAccess().getTimeoutDeclarationTimeoutDeclarationIDTerminalRuleCall_1_0_1()); }
			RULE_ID
			{ after(grammarAccess.getSetTimeoutActionAccess().getTimeoutDeclarationTimeoutDeclarationIDTerminalRuleCall_1_0_1()); }
		)
		{ after(grammarAccess.getSetTimeoutActionAccess().getTimeoutDeclarationTimeoutDeclarationCrossReference_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SetTimeoutAction__TimeAssignment_3
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSetTimeoutActionAccess().getTimeTimeSpecificationParserRuleCall_3_0()); }
		ruleTimeSpecification
		{ after(grammarAccess.getSetTimeoutActionAccess().getTimeTimeSpecificationParserRuleCall_3_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceStateReferenceExpression__InstanceAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getComponentInstanceStateReferenceExpressionAccess().getInstanceComponentInstanceReferenceExpressionParserRuleCall_1_0()); }
		ruleComponentInstanceReferenceExpression
		{ after(grammarAccess.getComponentInstanceStateReferenceExpressionAccess().getInstanceComponentInstanceReferenceExpressionParserRuleCall_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceStateReferenceExpression__RegionAssignment_3
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getComponentInstanceStateReferenceExpressionAccess().getRegionRegionCrossReference_3_0()); }
		(
			{ before(grammarAccess.getComponentInstanceStateReferenceExpressionAccess().getRegionRegionIDTerminalRuleCall_3_0_1()); }
			RULE_ID
			{ after(grammarAccess.getComponentInstanceStateReferenceExpressionAccess().getRegionRegionIDTerminalRuleCall_3_0_1()); }
		)
		{ after(grammarAccess.getComponentInstanceStateReferenceExpressionAccess().getRegionRegionCrossReference_3_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceStateReferenceExpression__StateAssignment_5
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getComponentInstanceStateReferenceExpressionAccess().getStateStateCrossReference_5_0()); }
		(
			{ before(grammarAccess.getComponentInstanceStateReferenceExpressionAccess().getStateStateIDTerminalRuleCall_5_0_1()); }
			RULE_ID
			{ after(grammarAccess.getComponentInstanceStateReferenceExpressionAccess().getStateStateIDTerminalRuleCall_5_0_1()); }
		)
		{ after(grammarAccess.getComponentInstanceStateReferenceExpressionAccess().getStateStateCrossReference_5_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceVariableReferenceExpression__InstanceAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getComponentInstanceVariableReferenceExpressionAccess().getInstanceComponentInstanceReferenceExpressionParserRuleCall_1_0()); }
		ruleComponentInstanceReferenceExpression
		{ after(grammarAccess.getComponentInstanceVariableReferenceExpressionAccess().getInstanceComponentInstanceReferenceExpressionParserRuleCall_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceVariableReferenceExpression__VariableDeclarationAssignment_3
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getComponentInstanceVariableReferenceExpressionAccess().getVariableDeclarationVariableDeclarationCrossReference_3_0()); }
		(
			{ before(grammarAccess.getComponentInstanceVariableReferenceExpressionAccess().getVariableDeclarationVariableDeclarationIDTerminalRuleCall_3_0_1()); }
			RULE_ID
			{ after(grammarAccess.getComponentInstanceVariableReferenceExpressionAccess().getVariableDeclarationVariableDeclarationIDTerminalRuleCall_3_0_1()); }
		)
		{ after(grammarAccess.getComponentInstanceVariableReferenceExpressionAccess().getVariableDeclarationVariableDeclarationCrossReference_3_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceEventReferenceExpression__InstanceAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getComponentInstanceEventReferenceExpressionAccess().getInstanceComponentInstanceReferenceExpressionParserRuleCall_1_0()); }
		ruleComponentInstanceReferenceExpression
		{ after(grammarAccess.getComponentInstanceEventReferenceExpressionAccess().getInstanceComponentInstanceReferenceExpressionParserRuleCall_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceEventReferenceExpression__PortAssignment_3
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getComponentInstanceEventReferenceExpressionAccess().getPortPortCrossReference_3_0()); }
		(
			{ before(grammarAccess.getComponentInstanceEventReferenceExpressionAccess().getPortPortIDTerminalRuleCall_3_0_1()); }
			RULE_ID
			{ after(grammarAccess.getComponentInstanceEventReferenceExpressionAccess().getPortPortIDTerminalRuleCall_3_0_1()); }
		)
		{ after(grammarAccess.getComponentInstanceEventReferenceExpressionAccess().getPortPortCrossReference_3_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceEventReferenceExpression__EventAssignment_5
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getComponentInstanceEventReferenceExpressionAccess().getEventEventCrossReference_5_0()); }
		(
			{ before(grammarAccess.getComponentInstanceEventReferenceExpressionAccess().getEventEventIDTerminalRuleCall_5_0_1()); }
			RULE_ID
			{ after(grammarAccess.getComponentInstanceEventReferenceExpressionAccess().getEventEventIDTerminalRuleCall_5_0_1()); }
		)
		{ after(grammarAccess.getComponentInstanceEventReferenceExpressionAccess().getEventEventCrossReference_5_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceEventParameterReferenceExpression__InstanceAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getComponentInstanceEventParameterReferenceExpressionAccess().getInstanceComponentInstanceReferenceExpressionParserRuleCall_1_0()); }
		ruleComponentInstanceReferenceExpression
		{ after(grammarAccess.getComponentInstanceEventParameterReferenceExpressionAccess().getInstanceComponentInstanceReferenceExpressionParserRuleCall_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceEventParameterReferenceExpression__PortAssignment_3
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getComponentInstanceEventParameterReferenceExpressionAccess().getPortPortCrossReference_3_0()); }
		(
			{ before(grammarAccess.getComponentInstanceEventParameterReferenceExpressionAccess().getPortPortIDTerminalRuleCall_3_0_1()); }
			RULE_ID
			{ after(grammarAccess.getComponentInstanceEventParameterReferenceExpressionAccess().getPortPortIDTerminalRuleCall_3_0_1()); }
		)
		{ after(grammarAccess.getComponentInstanceEventParameterReferenceExpressionAccess().getPortPortCrossReference_3_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceEventParameterReferenceExpression__EventAssignment_5
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getComponentInstanceEventParameterReferenceExpressionAccess().getEventEventCrossReference_5_0()); }
		(
			{ before(grammarAccess.getComponentInstanceEventParameterReferenceExpressionAccess().getEventEventIDTerminalRuleCall_5_0_1()); }
			RULE_ID
			{ after(grammarAccess.getComponentInstanceEventParameterReferenceExpressionAccess().getEventEventIDTerminalRuleCall_5_0_1()); }
		)
		{ after(grammarAccess.getComponentInstanceEventParameterReferenceExpressionAccess().getEventEventCrossReference_5_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComponentInstanceEventParameterReferenceExpression__ParameterDeclarationAssignment_7
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getComponentInstanceEventParameterReferenceExpressionAccess().getParameterDeclarationParameterDeclarationCrossReference_7_0()); }
		(
			{ before(grammarAccess.getComponentInstanceEventParameterReferenceExpressionAccess().getParameterDeclarationParameterDeclarationIDTerminalRuleCall_7_0_1()); }
			RULE_ID
			{ after(grammarAccess.getComponentInstanceEventParameterReferenceExpressionAccess().getParameterDeclarationParameterDeclarationIDTerminalRuleCall_7_0_1()); }
		)
		{ after(grammarAccess.getComponentInstanceEventParameterReferenceExpressionAccess().getParameterDeclarationParameterDeclarationCrossReference_7_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ProcedureDeclaration__NameAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getProcedureDeclarationAccess().getNameIDTerminalRuleCall_1_0()); }
		RULE_ID
		{ after(grammarAccess.getProcedureDeclarationAccess().getNameIDTerminalRuleCall_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ProcedureDeclaration__ParameterDeclarationsAssignment_2_1_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getProcedureDeclarationAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_2_1_0_0()); }
		ruleParameterDeclaration
		{ after(grammarAccess.getProcedureDeclarationAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_2_1_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ProcedureDeclaration__ParameterDeclarationsAssignment_2_1_1_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getProcedureDeclarationAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_2_1_1_1_0()); }
		ruleParameterDeclaration
		{ after(grammarAccess.getProcedureDeclarationAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_2_1_1_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ProcedureDeclaration__TypeAssignment_4
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getProcedureDeclarationAccess().getTypeTypeParserRuleCall_4_0()); }
		ruleType
		{ after(grammarAccess.getProcedureDeclarationAccess().getTypeTypeParserRuleCall_4_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ProcedureDeclaration__BodyAssignment_5
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getProcedureDeclarationAccess().getBodyBlockParserRuleCall_5_0()); }
		ruleBlock
		{ after(grammarAccess.getProcedureDeclarationAccess().getBodyBlockParserRuleCall_5_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__Block__ActionsAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getBlockAccess().getActionsActionParserRuleCall_2_0()); }
		ruleAction
		{ after(grammarAccess.getBlockAccess().getActionsActionParserRuleCall_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__VariableDeclarationStatement__VariableDeclarationAssignment
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getVariableDeclarationStatementAccess().getVariableDeclarationVariableDeclarationParserRuleCall_0()); }
		ruleVariableDeclaration
		{ after(grammarAccess.getVariableDeclarationStatementAccess().getVariableDeclarationVariableDeclarationParserRuleCall_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ConstantDeclarationStatement__ConstantDeclarationAssignment
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getConstantDeclarationStatementAccess().getConstantDeclarationConstantDeclarationParserRuleCall_0()); }
		ruleConstantDeclaration
		{ after(grammarAccess.getConstantDeclarationStatementAccess().getConstantDeclarationConstantDeclarationParserRuleCall_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ExpressionStatement__ExpressionAssignment
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getExpressionStatementAccess().getExpressionExpressionParserRuleCall_0()); }
		ruleExpression
		{ after(grammarAccess.getExpressionStatementAccess().getExpressionExpressionParserRuleCall_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__IfBranch__GuardAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getIfBranchAccess().getGuardExpressionParserRuleCall_1_0()); }
		ruleExpression
		{ after(grammarAccess.getIfBranchAccess().getGuardExpressionParserRuleCall_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__IfBranch__ActionAssignment_3
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getIfBranchAccess().getActionActionParserRuleCall_3_0()); }
		ruleAction
		{ after(grammarAccess.getIfBranchAccess().getActionActionParserRuleCall_3_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ElseBranch__GuardAssignment_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getElseBranchAccess().getGuardElseExpressionParserRuleCall_0_0()); }
		ruleElseExpression
		{ after(grammarAccess.getElseBranchAccess().getGuardElseExpressionParserRuleCall_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ElseBranch__ActionAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getElseBranchAccess().getActionActionParserRuleCall_1_0()); }
		ruleAction
		{ after(grammarAccess.getElseBranchAccess().getActionActionParserRuleCall_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SwitchBranch__GuardAssignment_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSwitchBranchAccess().getGuardExpressionParserRuleCall_0_0()); }
		ruleExpression
		{ after(grammarAccess.getSwitchBranchAccess().getGuardExpressionParserRuleCall_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SwitchBranch__ActionAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSwitchBranchAccess().getActionActionParserRuleCall_2_0()); }
		ruleAction
		{ after(grammarAccess.getSwitchBranchAccess().getActionActionParserRuleCall_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__DefaultBranch__GuardAssignment_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getDefaultBranchAccess().getGuardDefaultExpressionParserRuleCall_0_0()); }
		ruleDefaultExpression
		{ after(grammarAccess.getDefaultBranchAccess().getGuardDefaultExpressionParserRuleCall_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__DefaultBranch__ActionAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getDefaultBranchAccess().getActionActionParserRuleCall_2_0()); }
		ruleAction
		{ after(grammarAccess.getDefaultBranchAccess().getActionActionParserRuleCall_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ChoiceBranch__GuardAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getChoiceBranchAccess().getGuardExpressionParserRuleCall_1_0()); }
		ruleExpression
		{ after(grammarAccess.getChoiceBranchAccess().getGuardExpressionParserRuleCall_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ChoiceBranch__ActionAssignment_3
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getChoiceBranchAccess().getActionActionParserRuleCall_3_0()); }
		ruleAction
		{ after(grammarAccess.getChoiceBranchAccess().getActionActionParserRuleCall_3_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__IfStatement__ConditionalsAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getIfStatementAccess().getConditionalsIfBranchParserRuleCall_2_0()); }
		ruleIfBranch
		{ after(grammarAccess.getIfStatementAccess().getConditionalsIfBranchParserRuleCall_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__IfStatement__ConditionalsAssignment_3_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getIfStatementAccess().getConditionalsIfBranchParserRuleCall_3_1_0()); }
		ruleIfBranch
		{ after(grammarAccess.getIfStatementAccess().getConditionalsIfBranchParserRuleCall_3_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__IfStatement__ConditionalsAssignment_4
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getIfStatementAccess().getConditionalsElseBranchParserRuleCall_4_0()); }
		ruleElseBranch
		{ after(grammarAccess.getIfStatementAccess().getConditionalsElseBranchParserRuleCall_4_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SwitchStatement__ControlExpressionAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSwitchStatementAccess().getControlExpressionExpressionParserRuleCall_2_0()); }
		ruleExpression
		{ after(grammarAccess.getSwitchStatementAccess().getControlExpressionExpressionParserRuleCall_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SwitchStatement__CasesAssignment_5_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSwitchStatementAccess().getCasesSwitchBranchParserRuleCall_5_1_0()); }
		ruleSwitchBranch
		{ after(grammarAccess.getSwitchStatementAccess().getCasesSwitchBranchParserRuleCall_5_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SwitchStatement__CasesAssignment_5_2_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSwitchStatementAccess().getCasesSwitchBranchParserRuleCall_5_2_1_0()); }
		ruleSwitchBranch
		{ after(grammarAccess.getSwitchStatementAccess().getCasesSwitchBranchParserRuleCall_5_2_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SwitchStatement__CasesAssignment_6
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSwitchStatementAccess().getCasesDefaultBranchParserRuleCall_6_0()); }
		ruleDefaultBranch
		{ after(grammarAccess.getSwitchStatementAccess().getCasesDefaultBranchParserRuleCall_6_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ForStatement__ParameterAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getForStatementAccess().getParameterParameterDeclarationParserRuleCall_2_0()); }
		ruleParameterDeclaration
		{ after(grammarAccess.getForStatementAccess().getParameterParameterDeclarationParserRuleCall_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ForStatement__RangeAssignment_4
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getForStatementAccess().getRangeExpressionParserRuleCall_4_0()); }
		ruleExpression
		{ after(grammarAccess.getForStatementAccess().getRangeExpressionParserRuleCall_4_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ForStatement__BodyAssignment_6
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getForStatementAccess().getBodyActionParserRuleCall_6_0()); }
		ruleAction
		{ after(grammarAccess.getForStatementAccess().getBodyActionParserRuleCall_6_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ReturnStatement__ExpressionAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getReturnStatementAccess().getExpressionExpressionParserRuleCall_2_0()); }
		ruleExpression
		{ after(grammarAccess.getReturnStatementAccess().getExpressionExpressionParserRuleCall_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ChoiceStatement__BranchesAssignment_2_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getChoiceStatementAccess().getBranchesChoiceBranchParserRuleCall_2_1_0()); }
		ruleChoiceBranch
		{ after(grammarAccess.getChoiceStatementAccess().getBranchesChoiceBranchParserRuleCall_2_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ChoiceStatement__BranchesAssignment_2_2_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getChoiceStatementAccess().getBranchesChoiceBranchParserRuleCall_2_2_1_0()); }
		ruleChoiceBranch
		{ after(grammarAccess.getChoiceStatementAccess().getBranchesChoiceBranchParserRuleCall_2_2_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AssignmentStatement__LhsAssignment_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAssignmentStatementAccess().getLhsAssignableAccessExpressionParserRuleCall_0_0()); }
		ruleAssignableAccessExpression
		{ after(grammarAccess.getAssignmentStatementAccess().getLhsAssignableAccessExpressionParserRuleCall_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AssignmentStatement__RhsAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAssignmentStatementAccess().getRhsExpressionParserRuleCall_2_0()); }
		ruleExpression
		{ after(grammarAccess.getAssignmentStatementAccess().getRhsExpressionParserRuleCall_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AssertionStatement__AssertionAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAssertionStatementAccess().getAssertionExpressionParserRuleCall_2_0()); }
		ruleExpression
		{ after(grammarAccess.getAssertionStatementAccess().getAssertionExpressionParserRuleCall_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AssignableAccessExpression__IndexAssignment_1_0_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAssignableAccessExpressionAccess().getIndexExpressionParserRuleCall_1_0_2_0()); }
		ruleExpression
		{ after(grammarAccess.getAssignableAccessExpressionAccess().getIndexExpressionParserRuleCall_1_0_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AssignableAccessExpression__FieldReferenceAssignment_1_1_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAssignableAccessExpressionAccess().getFieldReferenceFieldReferenceExpressionParserRuleCall_1_1_2_0()); }
		ruleFieldReferenceExpression
		{ after(grammarAccess.getAssignableAccessExpressionAccess().getFieldReferenceFieldReferenceExpressionParserRuleCall_1_1_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AssignableDirectReferenceExpression__DeclarationAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAssignableDirectReferenceExpressionAccess().getDeclarationValueDeclarationCrossReference_1_0()); }
		(
			{ before(grammarAccess.getAssignableDirectReferenceExpressionAccess().getDeclarationValueDeclarationIDTerminalRuleCall_1_0_1()); }
			RULE_ID
			{ after(grammarAccess.getAssignableDirectReferenceExpressionAccess().getDeclarationValueDeclarationIDTerminalRuleCall_1_0_1()); }
		)
		{ after(grammarAccess.getAssignableDirectReferenceExpressionAccess().getDeclarationValueDeclarationCrossReference_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ParameterDeclaration__NameAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getParameterDeclarationAccess().getNameIDTerminalRuleCall_1_0()); }
		RULE_ID
		{ after(grammarAccess.getParameterDeclarationAccess().getNameIDTerminalRuleCall_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ParameterDeclaration__TypeAssignment_3
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getParameterDeclarationAccess().getTypeTypeParserRuleCall_3_0()); }
		ruleType
		{ after(grammarAccess.getParameterDeclarationAccess().getTypeTypeParserRuleCall_3_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__FieldDeclaration__NameAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getFieldDeclarationAccess().getNameIDTerminalRuleCall_1_0()); }
		RULE_ID
		{ after(grammarAccess.getFieldDeclarationAccess().getNameIDTerminalRuleCall_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__FieldDeclaration__TypeAssignment_3
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getFieldDeclarationAccess().getTypeTypeParserRuleCall_3_0()); }
		ruleType
		{ after(grammarAccess.getFieldDeclarationAccess().getTypeTypeParserRuleCall_3_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ConstantDeclaration__NameAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getConstantDeclarationAccess().getNameIDTerminalRuleCall_1_0()); }
		RULE_ID
		{ after(grammarAccess.getConstantDeclarationAccess().getNameIDTerminalRuleCall_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ConstantDeclaration__TypeAssignment_3
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getConstantDeclarationAccess().getTypeTypeParserRuleCall_3_0()); }
		ruleType
		{ after(grammarAccess.getConstantDeclarationAccess().getTypeTypeParserRuleCall_3_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ConstantDeclaration__ExpressionAssignment_4_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getConstantDeclarationAccess().getExpressionExpressionParserRuleCall_4_1_0()); }
		ruleExpression
		{ after(grammarAccess.getConstantDeclarationAccess().getExpressionExpressionParserRuleCall_4_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__VariableDeclaration__AnnotationsAssignment_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getVariableDeclarationAccess().getAnnotationsVariableDeclarationAnnotationParserRuleCall_0_0()); }
		ruleVariableDeclarationAnnotation
		{ after(grammarAccess.getVariableDeclarationAccess().getAnnotationsVariableDeclarationAnnotationParserRuleCall_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__VariableDeclaration__NameAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getVariableDeclarationAccess().getNameIDTerminalRuleCall_2_0()); }
		RULE_ID
		{ after(grammarAccess.getVariableDeclarationAccess().getNameIDTerminalRuleCall_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__VariableDeclaration__TypeAssignment_4
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getVariableDeclarationAccess().getTypeTypeParserRuleCall_4_0()); }
		ruleType
		{ after(grammarAccess.getVariableDeclarationAccess().getTypeTypeParserRuleCall_4_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__VariableDeclaration__ExpressionAssignment_5_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getVariableDeclarationAccess().getExpressionExpressionParserRuleCall_5_1_0()); }
		ruleExpression
		{ after(grammarAccess.getVariableDeclarationAccess().getExpressionExpressionParserRuleCall_5_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__LambdaDeclaration__NameAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getLambdaDeclarationAccess().getNameIDTerminalRuleCall_1_0()); }
		RULE_ID
		{ after(grammarAccess.getLambdaDeclarationAccess().getNameIDTerminalRuleCall_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__LambdaDeclaration__ParameterDeclarationsAssignment_2_1_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getLambdaDeclarationAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_2_1_0_0()); }
		ruleParameterDeclaration
		{ after(grammarAccess.getLambdaDeclarationAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_2_1_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__LambdaDeclaration__ParameterDeclarationsAssignment_2_1_1_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getLambdaDeclarationAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_2_1_1_1_0()); }
		ruleParameterDeclaration
		{ after(grammarAccess.getLambdaDeclarationAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_2_1_1_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__LambdaDeclaration__TypeAssignment_4
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getLambdaDeclarationAccess().getTypeTypeParserRuleCall_4_0()); }
		ruleType
		{ after(grammarAccess.getLambdaDeclarationAccess().getTypeTypeParserRuleCall_4_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__LambdaDeclaration__ExpressionAssignment_5_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getLambdaDeclarationAccess().getExpressionExpressionParserRuleCall_5_1_0()); }
		ruleExpression
		{ after(grammarAccess.getLambdaDeclarationAccess().getExpressionExpressionParserRuleCall_5_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__TypeDeclaration__NameAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getTypeDeclarationAccess().getNameIDTerminalRuleCall_1_0()); }
		RULE_ID
		{ after(grammarAccess.getTypeDeclarationAccess().getNameIDTerminalRuleCall_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__TypeDeclaration__TypeAssignment_3
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getTypeDeclarationAccess().getTypeTypeParserRuleCall_3_0()); }
		ruleType
		{ after(grammarAccess.getTypeDeclarationAccess().getTypeTypeParserRuleCall_3_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__TypeReference__ReferenceAssignment
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getTypeReferenceAccess().getReferenceTypeDeclarationCrossReference_0()); }
		(
			{ before(grammarAccess.getTypeReferenceAccess().getReferenceTypeDeclarationIDTerminalRuleCall_0_1()); }
			RULE_ID
			{ after(grammarAccess.getTypeReferenceAccess().getReferenceTypeDeclarationIDTerminalRuleCall_0_1()); }
		)
		{ after(grammarAccess.getTypeReferenceAccess().getReferenceTypeDeclarationCrossReference_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ArrayTypeDefinition__SizeAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getArrayTypeDefinitionAccess().getSizeExpressionParserRuleCall_1_0()); }
		ruleExpression
		{ after(grammarAccess.getArrayTypeDefinitionAccess().getSizeExpressionParserRuleCall_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ArrayTypeDefinition__ElementTypeAssignment_3
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getArrayTypeDefinitionAccess().getElementTypeTypeParserRuleCall_3_0()); }
		ruleType
		{ after(grammarAccess.getArrayTypeDefinitionAccess().getElementTypeTypeParserRuleCall_3_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__EnumerationTypeDefinition__LiteralsAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getEnumerationTypeDefinitionAccess().getLiteralsEnumerationLiteralDefinitionParserRuleCall_2_0()); }
		ruleEnumerationLiteralDefinition
		{ after(grammarAccess.getEnumerationTypeDefinitionAccess().getLiteralsEnumerationLiteralDefinitionParserRuleCall_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__EnumerationTypeDefinition__LiteralsAssignment_3_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getEnumerationTypeDefinitionAccess().getLiteralsEnumerationLiteralDefinitionParserRuleCall_3_1_0()); }
		ruleEnumerationLiteralDefinition
		{ after(grammarAccess.getEnumerationTypeDefinitionAccess().getLiteralsEnumerationLiteralDefinitionParserRuleCall_3_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__EnumerationLiteralDefinition__NameAssignment
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getEnumerationLiteralDefinitionAccess().getNameIDTerminalRuleCall_0()); }
		RULE_ID
		{ after(grammarAccess.getEnumerationLiteralDefinitionAccess().getNameIDTerminalRuleCall_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__RecordTypeDefinition__FieldDeclarationsAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getRecordTypeDefinitionAccess().getFieldDeclarationsFieldDeclarationParserRuleCall_2_0()); }
		ruleFieldDeclaration
		{ after(grammarAccess.getRecordTypeDefinitionAccess().getFieldDeclarationsFieldDeclarationParserRuleCall_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__RecordTypeDefinition__FieldDeclarationsAssignment_3_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getRecordTypeDefinitionAccess().getFieldDeclarationsFieldDeclarationParserRuleCall_3_1_0()); }
		ruleFieldDeclaration
		{ after(grammarAccess.getRecordTypeDefinitionAccess().getFieldDeclarationsFieldDeclarationParserRuleCall_3_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SubrangeTypeDefinition__LowerBoundAssignment_2_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSubrangeTypeDefinitionAccess().getLowerBoundExpressionParserRuleCall_2_1_0()); }
		ruleExpression
		{ after(grammarAccess.getSubrangeTypeDefinitionAccess().getLowerBoundExpressionParserRuleCall_2_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SubrangeTypeDefinition__UpperBoundAssignment_4_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSubrangeTypeDefinitionAccess().getUpperBoundExpressionParserRuleCall_4_1_0()); }
		ruleExpression
		{ after(grammarAccess.getSubrangeTypeDefinitionAccess().getUpperBoundExpressionParserRuleCall_4_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__FunctionTypeDefinition__ParameterTypesAssignment_2_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getFunctionTypeDefinitionAccess().getParameterTypesTypeParserRuleCall_2_0_0()); }
		ruleType
		{ after(grammarAccess.getFunctionTypeDefinitionAccess().getParameterTypesTypeParserRuleCall_2_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__FunctionTypeDefinition__ParameterTypesAssignment_2_1_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getFunctionTypeDefinitionAccess().getParameterTypesTypeParserRuleCall_2_1_1_0()); }
		ruleType
		{ after(grammarAccess.getFunctionTypeDefinitionAccess().getParameterTypesTypeParserRuleCall_2_1_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__FunctionTypeDefinition__ReturnTypeAssignment_5
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getFunctionTypeDefinitionAccess().getReturnTypeTypeParserRuleCall_5_0()); }
		ruleType
		{ after(grammarAccess.getFunctionTypeDefinitionAccess().getReturnTypeTypeParserRuleCall_5_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__IfThenElseExpression__ThenAssignment_1_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getIfThenElseExpressionAccess().getThenExpressionParserRuleCall_1_2_0()); }
		ruleExpression
		{ after(grammarAccess.getIfThenElseExpressionAccess().getThenExpressionParserRuleCall_1_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__IfThenElseExpression__ElseAssignment_1_3_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getIfThenElseExpressionAccess().getElseExpressionParserRuleCall_1_3_1_0()); }
		ruleExpression
		{ after(grammarAccess.getIfThenElseExpressionAccess().getElseExpressionParserRuleCall_1_3_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ImplyExpression__RightOperandAssignment_1_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getImplyExpressionAccess().getRightOperandImplyExpressionParserRuleCall_1_2_0()); }
		ruleImplyExpression
		{ after(grammarAccess.getImplyExpressionAccess().getRightOperandImplyExpressionParserRuleCall_1_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__OrExpression__OperandsAssignment_1_1_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getOrExpressionAccess().getOperandsXorExpressionParserRuleCall_1_1_1_0()); }
		ruleXorExpression
		{ after(grammarAccess.getOrExpressionAccess().getOperandsXorExpressionParserRuleCall_1_1_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__XorExpression__OperandsAssignment_1_1_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getXorExpressionAccess().getOperandsAndExpressionParserRuleCall_1_1_1_0()); }
		ruleAndExpression
		{ after(grammarAccess.getXorExpressionAccess().getOperandsAndExpressionParserRuleCall_1_1_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AndExpression__OperandsAssignment_1_1_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAndExpressionAccess().getOperandsUnaryLogicExpressionParserRuleCall_1_1_1_0()); }
		ruleUnaryLogicExpression
		{ after(grammarAccess.getAndExpressionAccess().getOperandsUnaryLogicExpressionParserRuleCall_1_1_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__NotExpression__OperandAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getNotExpressionAccess().getOperandUnaryLogicExpressionParserRuleCall_2_0()); }
		ruleUnaryLogicExpression
		{ after(grammarAccess.getNotExpressionAccess().getOperandUnaryLogicExpressionParserRuleCall_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ForallExpression__ParameterDeclarationsAssignment_3_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getForallExpressionAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_3_0_0()); }
		ruleParameterDeclaration
		{ after(grammarAccess.getForallExpressionAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_3_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ForallExpression__ParameterDeclarationsAssignment_3_1_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getForallExpressionAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_3_1_1_0()); }
		ruleParameterDeclaration
		{ after(grammarAccess.getForallExpressionAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_3_1_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ForallExpression__OperandAssignment_6
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getForallExpressionAccess().getOperandUnaryLogicExpressionParserRuleCall_6_0()); }
		ruleUnaryLogicExpression
		{ after(grammarAccess.getForallExpressionAccess().getOperandUnaryLogicExpressionParserRuleCall_6_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ExistsExpression__ParameterDeclarationsAssignment_3_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getExistsExpressionAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_3_0_0()); }
		ruleParameterDeclaration
		{ after(grammarAccess.getExistsExpressionAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_3_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ExistsExpression__ParameterDeclarationsAssignment_3_1_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getExistsExpressionAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_3_1_1_0()); }
		ruleParameterDeclaration
		{ after(grammarAccess.getExistsExpressionAccess().getParameterDeclarationsParameterDeclarationParserRuleCall_3_1_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ExistsExpression__OperandAssignment_6
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getExistsExpressionAccess().getOperandUnaryLogicExpressionParserRuleCall_6_0()); }
		ruleUnaryLogicExpression
		{ after(grammarAccess.getExistsExpressionAccess().getOperandUnaryLogicExpressionParserRuleCall_6_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__EquivalenceExpression__RightOperandAssignment_1_0_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getEquivalenceExpressionAccess().getRightOperandComparisionExpressionParserRuleCall_1_0_2_0()); }
		ruleComparisionExpression
		{ after(grammarAccess.getEquivalenceExpressionAccess().getRightOperandComparisionExpressionParserRuleCall_1_0_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__EquivalenceExpression__RightOperandAssignment_1_1_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getEquivalenceExpressionAccess().getRightOperandComparisionExpressionParserRuleCall_1_1_2_0()); }
		ruleComparisionExpression
		{ after(grammarAccess.getEquivalenceExpressionAccess().getRightOperandComparisionExpressionParserRuleCall_1_1_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ComparisionExpression__RightOperandAssignment_1_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getComparisionExpressionAccess().getRightOperandAdditiveExpressionParserRuleCall_1_1_0()); }
		ruleAdditiveExpression
		{ after(grammarAccess.getComparisionExpressionAccess().getRightOperandAdditiveExpressionParserRuleCall_1_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AdditiveExpression__OperandsAssignment_1_0_1_0_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAdditiveExpressionAccess().getOperandsMultiplicativeExpressionParserRuleCall_1_0_1_0_1_0()); }
		ruleMultiplicativeExpression
		{ after(grammarAccess.getAdditiveExpressionAccess().getOperandsMultiplicativeExpressionParserRuleCall_1_0_1_0_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AdditiveExpression__RightOperandAssignment_1_1_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAdditiveExpressionAccess().getRightOperandMultiplicativeExpressionParserRuleCall_1_1_2_0()); }
		ruleMultiplicativeExpression
		{ after(grammarAccess.getAdditiveExpressionAccess().getRightOperandMultiplicativeExpressionParserRuleCall_1_1_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__MultiplicativeExpression__OperandsAssignment_1_0_1_0_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getMultiplicativeExpressionAccess().getOperandsSignumExpressionParserRuleCall_1_0_1_0_1_0()); }
		ruleSignumExpression
		{ after(grammarAccess.getMultiplicativeExpressionAccess().getOperandsSignumExpressionParserRuleCall_1_0_1_0_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__MultiplicativeExpression__RightOperandAssignment_1_1_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getMultiplicativeExpressionAccess().getRightOperandSignumExpressionParserRuleCall_1_1_1_0()); }
		ruleSignumExpression
		{ after(grammarAccess.getMultiplicativeExpressionAccess().getRightOperandSignumExpressionParserRuleCall_1_1_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SignumExpression__OperandAssignment_0_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSignumExpressionAccess().getOperandSignumExpressionParserRuleCall_0_2_0()); }
		ruleSignumExpression
		{ after(grammarAccess.getSignumExpressionAccess().getOperandSignumExpressionParserRuleCall_0_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__SignumExpression__OperandAssignment_1_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getSignumExpressionAccess().getOperandSignumExpressionParserRuleCall_1_2_0()); }
		ruleSignumExpression
		{ after(grammarAccess.getSignumExpressionAccess().getOperandSignumExpressionParserRuleCall_1_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AccessExpression__IndexAssignment_1_0_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAccessExpressionAccess().getIndexExpressionParserRuleCall_1_0_2_0()); }
		ruleExpression
		{ after(grammarAccess.getAccessExpressionAccess().getIndexExpressionParserRuleCall_1_0_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AccessExpression__ArgumentsAssignment_1_1_2_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAccessExpressionAccess().getArgumentsExpressionParserRuleCall_1_1_2_0_0()); }
		ruleExpression
		{ after(grammarAccess.getAccessExpressionAccess().getArgumentsExpressionParserRuleCall_1_1_2_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AccessExpression__ArgumentsAssignment_1_1_2_1_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAccessExpressionAccess().getArgumentsExpressionParserRuleCall_1_1_2_1_1_0()); }
		ruleExpression
		{ after(grammarAccess.getAccessExpressionAccess().getArgumentsExpressionParserRuleCall_1_1_2_1_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__AccessExpression__FieldReferenceAssignment_1_2_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getAccessExpressionAccess().getFieldReferenceFieldReferenceExpressionParserRuleCall_1_2_2_0()); }
		ruleFieldReferenceExpression
		{ after(grammarAccess.getAccessExpressionAccess().getFieldReferenceFieldReferenceExpressionParserRuleCall_1_2_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__FieldReferenceExpression__FieldDeclarationAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getFieldReferenceExpressionAccess().getFieldDeclarationFieldDeclarationCrossReference_1_0()); }
		(
			{ before(grammarAccess.getFieldReferenceExpressionAccess().getFieldDeclarationFieldDeclarationIDTerminalRuleCall_1_0_1()); }
			RULE_ID
			{ after(grammarAccess.getFieldReferenceExpressionAccess().getFieldDeclarationFieldDeclarationIDTerminalRuleCall_1_0_1()); }
		)
		{ after(grammarAccess.getFieldReferenceExpressionAccess().getFieldDeclarationFieldDeclarationCrossReference_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__DirectReferenceExpression__DeclarationAssignment
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getDirectReferenceExpressionAccess().getDeclarationValueDeclarationCrossReference_0()); }
		(
			{ before(grammarAccess.getDirectReferenceExpressionAccess().getDeclarationValueDeclarationIDTerminalRuleCall_0_1()); }
			RULE_ID
			{ after(grammarAccess.getDirectReferenceExpressionAccess().getDeclarationValueDeclarationIDTerminalRuleCall_0_1()); }
		)
		{ after(grammarAccess.getDirectReferenceExpressionAccess().getDeclarationValueDeclarationCrossReference_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__IntegerLiteralExpression__ValueAssignment
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getIntegerLiteralExpressionAccess().getValueINTEGERTerminalRuleCall_0()); }
		RULE_INTEGER
		{ after(grammarAccess.getIntegerLiteralExpressionAccess().getValueINTEGERTerminalRuleCall_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__DecimalLiteralExpression__ValueAssignment
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getDecimalLiteralExpressionAccess().getValueDECIMALTerminalRuleCall_0()); }
		RULE_DECIMAL
		{ after(grammarAccess.getDecimalLiteralExpressionAccess().getValueDECIMALTerminalRuleCall_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__RationalLiteralExpression__NumeratorAssignment_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getRationalLiteralExpressionAccess().getNumeratorINTEGERTerminalRuleCall_0_0()); }
		RULE_INTEGER
		{ after(grammarAccess.getRationalLiteralExpressionAccess().getNumeratorINTEGERTerminalRuleCall_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__RationalLiteralExpression__DenominatorAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getRationalLiteralExpressionAccess().getDenominatorINTEGERTerminalRuleCall_2_0()); }
		RULE_INTEGER
		{ after(grammarAccess.getRationalLiteralExpressionAccess().getDenominatorINTEGERTerminalRuleCall_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__RecordLiteralExpression__TypeDeclarationAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getRecordLiteralExpressionAccess().getTypeDeclarationTypeDeclarationCrossReference_1_0()); }
		(
			{ before(grammarAccess.getRecordLiteralExpressionAccess().getTypeDeclarationTypeDeclarationIDTerminalRuleCall_1_0_1()); }
			RULE_ID
			{ after(grammarAccess.getRecordLiteralExpressionAccess().getTypeDeclarationTypeDeclarationIDTerminalRuleCall_1_0_1()); }
		)
		{ after(grammarAccess.getRecordLiteralExpressionAccess().getTypeDeclarationTypeDeclarationCrossReference_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__RecordLiteralExpression__FieldAssignmentsAssignment_3
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getRecordLiteralExpressionAccess().getFieldAssignmentsFieldAssignmentParserRuleCall_3_0()); }
		ruleFieldAssignment
		{ after(grammarAccess.getRecordLiteralExpressionAccess().getFieldAssignmentsFieldAssignmentParserRuleCall_3_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__RecordLiteralExpression__FieldAssignmentsAssignment_4_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getRecordLiteralExpressionAccess().getFieldAssignmentsFieldAssignmentParserRuleCall_4_1_0()); }
		ruleFieldAssignment
		{ after(grammarAccess.getRecordLiteralExpressionAccess().getFieldAssignmentsFieldAssignmentParserRuleCall_4_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__FieldAssignment__ReferenceAssignment_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getFieldAssignmentAccess().getReferenceFieldReferenceExpressionParserRuleCall_0_0()); }
		ruleFieldReferenceExpression
		{ after(grammarAccess.getFieldAssignmentAccess().getReferenceFieldReferenceExpressionParserRuleCall_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__FieldAssignment__ValueAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getFieldAssignmentAccess().getValueExpressionParserRuleCall_2_0()); }
		ruleExpression
		{ after(grammarAccess.getFieldAssignmentAccess().getValueExpressionParserRuleCall_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__EnumerationLiteralExpression__TypeReferenceAssignment_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getEnumerationLiteralExpressionAccess().getTypeReferenceTypeReferenceParserRuleCall_0_0()); }
		ruleTypeReference
		{ after(grammarAccess.getEnumerationLiteralExpressionAccess().getTypeReferenceTypeReferenceParserRuleCall_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__EnumerationLiteralExpression__ReferenceAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getEnumerationLiteralExpressionAccess().getReferenceEnumerationLiteralDefinitionCrossReference_2_0()); }
		(
			{ before(grammarAccess.getEnumerationLiteralExpressionAccess().getReferenceEnumerationLiteralDefinitionIDTerminalRuleCall_2_0_1()); }
			RULE_ID
			{ after(grammarAccess.getEnumerationLiteralExpressionAccess().getReferenceEnumerationLiteralDefinitionIDTerminalRuleCall_2_0_1()); }
		)
		{ after(grammarAccess.getEnumerationLiteralExpressionAccess().getReferenceEnumerationLiteralDefinitionCrossReference_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__OpaqueExpression__ExpressionAssignment
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getOpaqueExpressionAccess().getExpressionSTRINGTerminalRuleCall_0()); }
		RULE_STRING
		{ after(grammarAccess.getOpaqueExpressionAccess().getExpressionSTRINGTerminalRuleCall_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ArrayLiteralExpression__OperandsAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getArrayLiteralExpressionAccess().getOperandsExpressionParserRuleCall_1_0()); }
		ruleExpression
		{ after(grammarAccess.getArrayLiteralExpressionAccess().getOperandsExpressionParserRuleCall_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__ArrayLiteralExpression__OperandsAssignment_2_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getArrayLiteralExpressionAccess().getOperandsExpressionParserRuleCall_2_1_0()); }
		ruleExpression
		{ after(grammarAccess.getArrayLiteralExpressionAccess().getOperandsExpressionParserRuleCall_2_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__IntegerRangeLiteralExpression__LeftInclusiveAssignment_1_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getIntegerRangeLiteralExpressionAccess().getLeftInclusiveLessThanSignPlusSignKeyword_1_0_0()); }
		(
			{ before(grammarAccess.getIntegerRangeLiteralExpressionAccess().getLeftInclusiveLessThanSignPlusSignKeyword_1_0_0()); }
			'<+'
			{ after(grammarAccess.getIntegerRangeLiteralExpressionAccess().getLeftInclusiveLessThanSignPlusSignKeyword_1_0_0()); }
		)
		{ after(grammarAccess.getIntegerRangeLiteralExpressionAccess().getLeftInclusiveLessThanSignPlusSignKeyword_1_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__IntegerRangeLiteralExpression__LeftOperandAssignment_2
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getIntegerRangeLiteralExpressionAccess().getLeftOperandExpressionParserRuleCall_2_0()); }
		ruleExpression
		{ after(grammarAccess.getIntegerRangeLiteralExpressionAccess().getLeftOperandExpressionParserRuleCall_2_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__IntegerRangeLiteralExpression__RightOperandAssignment_4
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getIntegerRangeLiteralExpressionAccess().getRightOperandExpressionParserRuleCall_4_0()); }
		ruleExpression
		{ after(grammarAccess.getIntegerRangeLiteralExpressionAccess().getRightOperandExpressionParserRuleCall_4_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__IntegerRangeLiteralExpression__RightInclusiveAssignment_5_0
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getIntegerRangeLiteralExpressionAccess().getRightInclusivePlusSignGreaterThanSignKeyword_5_0_0()); }
		(
			{ before(grammarAccess.getIntegerRangeLiteralExpressionAccess().getRightInclusivePlusSignGreaterThanSignKeyword_5_0_0()); }
			'+>'
			{ after(grammarAccess.getIntegerRangeLiteralExpressionAccess().getRightInclusivePlusSignGreaterThanSignKeyword_5_0_0()); }
		)
		{ after(grammarAccess.getIntegerRangeLiteralExpressionAccess().getRightInclusivePlusSignGreaterThanSignKeyword_5_0_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

rule__BasicConstraintDefinition__ExpressionAssignment_1
	@init {
		int stackSize = keepStackSize();
	}
:
	(
		{ before(grammarAccess.getBasicConstraintDefinitionAccess().getExpressionExpressionParserRuleCall_1_0()); }
		ruleExpression
		{ after(grammarAccess.getBasicConstraintDefinitionAccess().getExpressionExpressionParserRuleCall_1_0()); }
	)
;
finally {
	restoreStackSize(stackSize);
}

RULE_INT : 'INT has been deactivated';

RULE_INTEGER : '-'? ('0'..'9')+;

RULE_DECIMAL : '-'? ('0'..'9')+ '.' ('0'..'9')+;

RULE_ID : '^'? ('a'..'z'|'A'..'Z'|'_') ('a'..'z'|'A'..'Z'|'_'|'0'..'9')*;

RULE_STRING : ('"' ('\\' .|~(('\\'|'"')))* '"'|'\'' ('\\' .|~(('\\'|'\'')))* '\'');

RULE_ML_COMMENT : '/*' ( options {greedy=false;} : . )*'*/';

RULE_SL_COMMENT : '//' ~(('\n'|'\r'))* ('\r'? '\n')?;

RULE_WS : (' '|'\t'|'\r'|'\n')+;

RULE_ANY_OTHER : .;
